﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace DockSample
{
    public abstract class DetachableLoad : NotifyPropertyChangedBase, ILoad
    {
        protected string _name;
        protected string _oldName;
        protected bool _autoSelect;

        //Possible to attach or detach to different moves?
        protected bool _detachable = true;

        protected MainType _mainType;

        [Category("Application"), Description("The identifying name of this component.")]
        public string Name
        {
            get { return _name; }
            set
            {
                if (value != this._name)
                {
                    this._oldName = this._name;
                    this._name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        [Browsable(false)]
        public string OldName
        {
            get { return _oldName; }
        }

        [Category("Component"), ReadOnly(true), Description("Main type of this component.")]
        public MainType MainType
        {
            get { return _mainType; }
            set
            {
                if (value != this._mainType)
                {
                    this._mainType = value;
                    OnPropertyChanged("MainType");
                }
            }
        }

        [Browsable(false)] //Only included to fulfill the contract with ILoads
        public bool AutoSelect
        {
            get { return _autoSelect; }
        }

        [Browsable(false)] //This load can be selected as active / not active
        public bool Detachable
        {
            get { return _detachable; }
        }
    }
}
