﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Drawing.Design;
using System.Reflection;

namespace DockSample
{
    public class GearBox : GenericLoadComponent, ILoad
    {
        protected MechInterface _outType;
        protected double _outMinDia;
        protected double _outMaxDia;
        protected double _outRadFrc;
        protected double _outAxFrc;

        private new GearBoxBrand _brand = GearBoxBrand.Undefined;
        private GearBoxType _subType = GearBoxType.Undefined;
        
        protected double _gearing = 1;

        public GearBox(string name)
        {
            _name = name;
            _mainType = MainType.GearBox;
        }

        //Empty gearbox
        public GearBox()
        {
            _mainType = MainType.GearBox;
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Mechanical output type.")]
        public MechInterface OutType
        {
            get { return _outType; }
            set
            {
                if (value != this._outType)
                {
                    this._outType = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutType");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Minimum output diameter in millimeters (mm).")]
        public double OutMinDia
        {
            get { return _outMinDia; }
            set
            {
                if (value != this._outMinDia)
                {
                    this._outMinDia = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutMinDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum output diameter in millimeters (mm).")]
        public double OutMaxDia
        {
            get { return _outMaxDia; }
            set
            {
                if (value != this._outMaxDia)
                {
                    this._outMaxDia = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutMaxDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum radial force in newtons (N) on the mechanical output.")]
        public double OutRadFrc
        {
            get { return _outRadFrc; }
            set
            {
                if (value != this._outRadFrc)
                {
                    this._outRadFrc = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutRadFrc");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum axial force in newtons (N) on the mechanical output.")]
        public double OutAxFrc
        {
            get { return _outAxFrc; }
            set
            {
                if (value != this._outAxFrc)
                {
                    this._outAxFrc = value;
                    if (!AutoSelect)
                        OnPropertyChanged("OutAxFrc");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Brand name displayed on component.")]
        public new GearBoxBrand Brand
        {
            get { return _brand; }
            set
            {
                if (value != this._brand)
                {
                    this._brand = value;
                    if (!AutoSelect)
                    OnPropertyChanged("Brand");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Gearbox type.")]
        public new GearBoxType SubType
        {
            get { return _subType; }
            set
            {
                if (value != this._subType)
                {
                    this._subType = value;
                    if (!AutoSelect)
                    OnPropertyChanged("SubType");
                }
            }
        }

        [Category("Main"), ReadOnly(false), Description("Total gearing (no unit).")]
        [Editor(typeof(UIGearingEditor), typeof(UITypeEditor))]
        public double Gearing
        {
            get { return _gearing; }
            set
            {
                if (value != this._gearing)
                {
                    this._gearing = value;
                    if (!AutoSelect)
                    OnPropertyChanged("Gearing");
                }
            }
        }
    }
}
