﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace DockSample
{
    public abstract class GenericLoadComponent : GenericRotatingComponent
    {
        protected double _idleTrq;
        protected double _backlash;
        
        //Mechanical data
        protected MechInterface _inpType;
        protected double _inpMinDia;
        protected double _inpMaxDia;
        protected double _inpRadFrc;
        protected double _inpAxFrc;

        [Category("Main"), ReadOnly(false), Description("Torque in newton meters (Nm) when running without load.")]
        public double IdleTrq
        {
            get { return _idleTrq; }
            set
            {
                if (value != this._idleTrq)
                {
                    this._idleTrq = value;
                    if (!AutoSelect)
                    OnPropertyChanged("IdleTrq");
                }
            }
        }

        [Category("Main"), ReadOnly(false), Description("Maximum backlash in arcminutes (arcmin).")]
        public double Backlash
        {
            get { return _backlash; }
            set
            {
                if (value != this._backlash)
                {
                    this._backlash = value;
                    if (!AutoSelect)
                    OnPropertyChanged("Backlash");
                }
            }
        }

        [Category("Mechanical input"), ReadOnly(false), Description("Mechanical input type.")]
        public MechInterface InpType
        {
            get { return _inpType; }
            set
            {
                if (value != this._inpType)
                {
                    this._inpType = value;
                    if (!AutoSelect)
                    OnPropertyChanged("InpType");
                }
            }
        }

        [Category("Mechanical input"), ReadOnly(false), Description("Minimum input diameter in millimeters (mm).")]
        public double InpMinDia
        {
            get { return _inpMinDia; }
            set
            {
                if (value != this._inpMinDia)
                {
                    this._inpMinDia = value;
                    if (!AutoSelect)
                    OnPropertyChanged("InpMinDia");
                }
            }
        }

        [Category("Mechanical input"), ReadOnly(false), Description("Maximum input diameter in millimeters (mm).")]
        public double InpMaxDia
        {
            get { return _inpMaxDia; }
            set
            {
                if (value != this._inpMaxDia)
                {
                    this._inpMaxDia = value;
                    if (!AutoSelect)
                    OnPropertyChanged("InpMaxDia");
                }
            }
        }

        [Category("Mechanical input"), ReadOnly(false), Description("Maximum radial force in newtons (N) on the mechanical input.")]
        public double InpRadFrc
        {
            get { return _inpRadFrc; }
            set
            {
                if (value != this._inpRadFrc)
                {
                    this._inpRadFrc = value;
                    if (!AutoSelect)
                    OnPropertyChanged("InpRadFrc");
                }
            }
        }

        [Category("Mechanical input"), ReadOnly(false), Description("Maximum axial force in newtons (N) on the mechanical input.")]
        public double InpAxFrc
        {
            get { return _inpAxFrc; }
            set
            {
                if (value != this._inpAxFrc)
                {
                    this._inpAxFrc = value;
                    if (!AutoSelect)
                    OnPropertyChanged("InpAxFrc");
                }
            }
        }
    }
}
