﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Drawing.Design;
using System.Reflection;

namespace DockSample
{
    public abstract class GenericRotatingComponent : GenericStaticComponent
    {
        protected double _inertia;
        
        //Torque data
        protected double _maxTrq;
        protected double _maxTrqDur = 1;
        protected double _trqED = 10;
        protected double _nomTrq;

        //Speed data
        protected double _maxSpd;
        protected double _maxSpdDur = 1;
        protected double _spdED = 10;
        protected double _nomSpd;


        [Category("Main"), ReadOnly(false), Description("Inertia in kilogram times squaremeter (kg•m²).")]
        [Editor(typeof(UIInertiaEditor), typeof(UITypeEditor))]
        public double Inertia
        {
            get
            { return _inertia; }
            set
            {
                if (value != this._inertia)
                {
                    this._inertia = value ;
                    if (!AutoSelect)
                        OnPropertyChanged("Inertia");
                }
            }
        }

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        [Category("Torque"), ReadOnly(false), Description("Maximum torque in newton meters (Nm).")]
        public double MaxTrq
        {
            get { return _maxTrq; }
            set
            {
                if (value != this._maxTrq)
                {
                    this._maxTrq = value;
                    
                        NomTrq = Math.Sqrt((Math.Pow(_maxTrq, 2) * _maxTrqDur) / (_maxTrqDur / (_trqED / 100)));
                        if (!AutoSelect)
                    OnPropertyChanged("MaxTrq");
                }
            }
        }

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        [Category("Torque"), ReadOnly(false), Description("Maximum time in seconds (s) at maximum torque.")]
        public double MaxTrqDur
        {
            get { return _maxTrqDur; }
            set
            {
                if (value != this._maxTrqDur)
                {
                    this._maxTrqDur = value;

                    NomTrq = Math.Sqrt((Math.Pow(_maxTrq, 2) * _maxTrqDur) / (_maxTrqDur / (_trqED / 100)));
                    if (!AutoSelect)
                    OnPropertyChanged("MaxTrqDur");
                }
            }
        }

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        [Category("Torque"), ReadOnly(false), Description("Maximum time at maximum torque in percent (%) of the total cycle time.")]
        public double TrqED
        {
            get { return _trqED; }
            set
            {
                if (value != this._trqED)
                {
                    this._trqED = value;

                        NomTrq = Math.Sqrt((Math.Pow(_maxTrq, 2) * _maxTrqDur) / (_maxTrqDur / (_trqED / 100)));
                        if (!AutoSelect)
                    OnPropertyChanged("TrqED");
                }
            }
        }

        [Category("Torque"), ReadOnly(true), Description("Nominal torque for continous operation in newton meters (Nm).")]
        public double NomTrq
        {
            get { return _nomTrq; }
            set
            {
                if (value != this._nomTrq)
                {
                    this._nomTrq = value;
                    if (!AutoSelect)
                    OnPropertyChanged("NomTrq");
                }
            }
        }

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        [Category("Speed"), ReadOnly(false), Description("Maximum speed in revolutions per minute (rpm).")]
        public double MaxSpd
        {
            get { return _maxSpd; }
            set
            {
                if (value != this._maxSpd)
                {
                    this._maxSpd = value;

                        NomSpd = Math.Sqrt((Math.Pow(_maxSpd, 2) * _maxSpdDur) / (_maxSpdDur / (_spdED / 100)));
                        if (!AutoSelect)
                    OnPropertyChanged("MaxSpd");
                }
            }
        }

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        [Category("Speed"), ReadOnly(false), Description("Maximum time in seconds (s) at maximum speed.")]
        public double MaxSpdDur
        {
            get { return _maxSpdDur; }
            set
            {
                if (value != this._maxSpdDur)
                {
                    this._maxSpdDur = value;

                    NomSpd = Math.Sqrt((Math.Pow(_maxSpd, 2) * _maxSpdDur) / (_maxSpdDur / (_spdED / 100)));
                    if (!AutoSelect)
                    OnPropertyChanged("MaxSpdDur");
                }
            }
        }

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        [Category("Speed"), ReadOnly(false), Description("Maximum time at maximum speed in percent (%) of the total cycle time.")]
        public double SpdED
        {
            get { return _spdED; }
            set
            {
                if (value != this._spdED)
                {
                    this._spdED = value;

                        NomSpd = Math.Sqrt((Math.Pow(_maxSpd, 2) * _maxSpdDur) / (_maxSpdDur / (_spdED / 100)));
                        if (!AutoSelect)
                    OnPropertyChanged("SpdED");
                }
            }
        }

        [Category("Speed"), ReadOnly(true), Description("Nominal speed for continous operation in revolutions per minute (rpm).")]
        public double NomSpd
        {
            get { return _nomSpd; }
            set
            {
                if (value != this._nomSpd)
                {
                    this._nomSpd = value;
                    if (!AutoSelect)
                    OnPropertyChanged("NomSpd");
                }
            }
        }
    }
}
