﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.OleDb;
using System.ComponentModel;
using System.Reflection;
using System.Drawing.Design;

namespace DockSample
{
    public abstract class GenericStaticComponent : NotifyPropertyChangedBase //, ICloneable
    {
        //Object name in program
        protected string _name;
        protected string _oldName;  //used when renaming objects

        //Controlled by user or system?
        protected bool _autoSelect;

        //Possible to attach or detach to different moves?
        protected bool _detachable = false;

        //Component data
        protected string _articleNo = "";
        protected bool _campaign;
        protected bool _sold;
        protected Brand _brand;
        protected MainType _mainType;
        protected string _description = "";
        protected double _price;

        //Main efficiency variable
        protected double _eff = 100;

        //Clone function to create non-reference object copies
        /*public object Clone()
        {
            return this.MemberwiseClone();
        }*/
        
        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        //Function to set the UsedDef property attribute to ReadOnly for those classes that does not implement this functionality
        public void SetAutoSelectTrue()
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(this.GetType())["AutoSelect"];
            ReadOnlyAttribute attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldInfo FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);
        }       
                
        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        public void SetOrResetReadOnlyAttributes(ILoad iLoad)
        {
            //REALLY crappy solution to set the ReadOnlyone attribute at runtime. This is supposed to display or dim the object
            //properties in the PropertyGrid-object when the load is selected as AutoSelect = true or false.
            //Unfortunately it does not work. It affects all instances so the attributes has to be reset when switching
            //between the defined loads. Also attributes that should always be ReadOnly or NOT ReadOnly must be manually reset.
            //The Settings class is also affected if the ReadOnly attributes are not set to false !!!???

            //see: http://stackoverflow.com/questions/2483124/remove-c-sharp-attribute-of-a-property-dynamically
            //and: http://stackoverflow.com/questions/51269/change-attributes-parameter-at-runtime

            bool newValue = iLoad.AutoSelect;

            foreach (PropertyDescriptor property in TypeDescriptor.GetProperties(iLoad))
            {
                //if (property.Name != "AutoSelect")
                //{
                    //Set all properties ReadOnly attribute equal to the value of "AutoSelect"
                    ReadOnlyAttribute attrib = (ReadOnlyAttribute)property.Attributes[typeof(ReadOnlyAttribute)];
                    FieldInfo isReadOnly = attrib.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
                    isReadOnly.SetValue(attrib, newValue);
                //}
            }
            
            //Since the loop above affects all properties make a second round to set those properties that should always
            //be ReadOnly or NOT ReadOnly

            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["Name"];
            ReadOnlyAttribute attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldInfo FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, false);

            descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["AutoSelect"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, false);

            descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["Campaign"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["Sold"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);

            if (!AutoSelect && Campaign)
                FieldIsReadOnly.SetValue(attribute, false);
            else
            {
                FieldIsReadOnly.SetValue(attribute, true);
            }

            descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["MainType"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["NomSpd"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(iLoad.GetType())["NomTrq"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            /*    //Original code to set the ReadOnlyone attribute on one single property
            //---------------------------------------------------------------------
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(this.GetType())["Name"];
            ReadOnlyAttribute attrib = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];

            FieldInfo isReadOnly = attrib.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            isReadOnly.SetValue(attrib, newValue);
            //--------------------------------------------------------------------- */
        } 

        [Category("Application"), ReadOnly(false), Description("The identifying name of this component.")]
        public string Name
        {
            get { return _name; }
            set
            {
                if (value != this._name)
                {
                    this._oldName = this._name;
                    this._name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        [Browsable(false), ReadOnly(false)]
        public string OldName
        {
            get { return _oldName; }
        }

        
        [Category("Application"), ReadOnly(false), Description("Set whether this component is automatically selected by the application or manually by the the user.")]
        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        public bool AutoSelect
        {
            get { return _autoSelect; }
            set
            {
                this._autoSelect = value;

                OnPropertyChanged("AutoSelect");
            }
        }
        

        [Editor(typeof(UIArticleDialog), typeof(UITypeEditor))]
        [Category("Component"), ReadOnly(false), Description("Article no used when ordered by a customer.")]        
        public string ArticleNo
        {
            get { return _articleNo; }
            set
            {
                if (value != this._articleNo)
                {
                    this._articleNo = value;

                        OnPropertyChanged("ArticleNo");
                }
            }
        }

        [Category("Component"), ReadOnly(true), Description("Component condition.")]
        public bool Campaign
        {
            get { return _campaign; }
            set
            {
                if (value != this._campaign)
                {
                    this._campaign = value;

               /*     PropertyDescriptor descriptor = TypeDescriptor.GetProperties(this.GetType())["Sold"];
                    ReadOnlyAttribute attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
                    FieldInfo FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);

                    if (value)
                        FieldIsReadOnly.SetValue(attribute, false);
                    else
                    {
                        FieldIsReadOnly.SetValue(attribute, true);
                    }

                    if (!AutoSelect)
                        OnPropertyChanged("Campaign");
             */   }
            }
        }

        [Category("Component"), ReadOnly(true), Description("Component is sold and cannot be used.")]
        public bool Sold
        {
            get { return _sold; }
            set
            {
                if (value != this._sold)
                {
                    this._sold = value;

                    if (ArticleNo != "")
                        MarkAsSoldInDb();                    

                    if (!AutoSelect)
                        OnPropertyChanged("Sold");
                }
            }
        }

        [Category("Component"), ReadOnly(true), Description("Main type of this component.")]
        public MainType MainType
        {
            get { return _mainType; }
            set
            {
                if (value != this._mainType)
                {
                    this._mainType = value;
                    OnPropertyChanged("MainType");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Brand name displayed on component.")]
        public Brand Brand
        {
            get { return _brand; }
            set
            {
                if (value != this._brand)
                {
                    this._brand = value;
                    if (!AutoSelect)
                        OnPropertyChanged("Brand");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Brief description including model name and main properties not described by other fields.")]
        public string Description
        {
            get { return _description; }
            set
            {
                if (value != this._description)
                {
                    this._description = value;
                    if (!AutoSelect)
                        OnPropertyChanged("Description");
                }
            }
        }

        [Category("Component"), Browsable(false), Description("Price in units.")]
        public double Price
        {
            get { return _price; }
            set
            {
                if (value != this._price)
                {
                    this._price = value;

                    if (!AutoSelect)
                        OnPropertyChanged("Price");
                }
            }
        }

        [Category("Main"), ReadOnly(false), Description("(Efficiency) Total efficiency in percent (%) at nominal operation.")]
        public double Eff
        {
            get { return _eff; }
            set
            {
                if (value != this._eff)
                {
                    this._eff = value;
                    if (!AutoSelect)
                        OnPropertyChanged("Eff");
                }
            }
        }

        [Browsable(false)]
        public bool Detachable
        {
            get { return _detachable; }
        }

        private void MarkAsSoldInDb()
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            //if (!File.Exists(dbPath))
              //  throw new Exception("Database file does not exist!");

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    if (MainType == MainType.Motor)
                        dbDriveSystemsContext.ExecuteCommand
                        ("UPDATE tblMotors SET Sold = {0} WHERE ArticleNo = {1}", Sold, ArticleNo);

                    if (MainType == MainType.GearBox)
                        dbDriveSystemsContext.ExecuteCommand
                        ("UPDATE tblGearBoxes SET Sold = {0} WHERE ArticleNo = {1}", Sold, ArticleNo);
                }
            }
        }
    }
}
