﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Drawing.Design;

namespace DockSample
{
    public class LoadInertia : DetachableLoad
    {
        private double _inertia;
        private Geometry _inertiaGeometry = new Geometry();
        public event EventHandler<RefreshPropertyGridEventArgs> RefreshPropertyGridEvent;

        public LoadInertia(string name)
        {
            _name = name;
            _mainType = MainType.Inertia;

            _inertiaGeometry.PropertyChanged += new PropertyChangedEventHandler(_inertiaGeometry_PropertyChanged);
        }

        void _inertiaGeometry_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (InertiaGeometry.ManualInertiaCalc)
                CalcInertia();
        }

        [Category("Main"), ReadOnly(false), Description("Total inertia (kgm2 x 10-4).")]
        [Editor(typeof(UIInertiaEditor), typeof(UITypeEditor))]
        public double Inertia
        {
            get { return _inertia; }
            set
            {
                if (value != this._inertia)
                {
                    this._inertia = value;
                    OnPropertyChanged("Inertia");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Geometry of rotating part.")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Geometry InertiaGeometry
        {
            get { return _inertiaGeometry; }
            set
            {
                if (value != this._inertiaGeometry)
                {
                    this._inertiaGeometry = value;
                    OnPropertyChanged("InertiaGeometry");
                }
            }
        }

        private void CalcInertia()
        {
            double density = 0;

            switch (InertiaGeometry.Material)
            {
                case Material.Aluminum:
                    density = 2700;
                    break;

                case Material.Plastic:
                    density = 1500;
                    break;

                default:
                    density = 7800; //Material.Steel
                    break;
            }

            //Cylinder volume = PI * radius * radius * cylinderheight

            double totalVolume = Math.PI * Math.Pow(InertiaGeometry.OuterDiameter / 2000, 2) * InertiaGeometry.CylinderHeight / 1000;
            double hollowVolume = Math.PI * Math.Pow(InertiaGeometry.InnerDiameter / 2000, 2) * InertiaGeometry.CylinderHeight / 1000;

            double realVolume = totalVolume - hollowVolume;
            double mass = realVolume * density;

            InertiaGeometry.PartMass = Math.Round(mass, 3);

            InertiaGeometry.PartInertia = mass * (Math.Pow(InertiaGeometry.OuterDiameter / 2000, 2) + Math.Pow(InertiaGeometry.InnerDiameter / 2000, 2)) * 0.5;

            Inertia = InertiaGeometry.PartInertia;

            RaiseRefreshPropertyGridEvent();
        }

        private void RaiseRefreshPropertyGridEvent()
        {
            EventHandler<RefreshPropertyGridEventArgs> handler = RefreshPropertyGridEvent;
            if (handler != null)
                handler(null, new RefreshPropertyGridEventArgs());
        }
    }
}
