﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DockSample
{
    public class LoadResult : NotifyPropertyChangedBase
    {
        private string _name;
        private MainType _mainType;
        private bool _autoSelect;
        private bool _active;
        //Possible to attach or detach to different moves?
        private bool _detachable = false;
        
        private double _gearing;
        private double _accInpGearing;
        private double _accOutGearing;

        private double _inertia;
        protected double _angle;
        protected double _mass;
        private double _idleTorque;

        private double _thrustTorque;
        
        private double _accTorque;
        private bool _accTorqueExists;

        private double _constTorque;
        private bool _constTorqueExists;

        private double _liftTorque;
        
        private double _decTorque;
        private bool _decTorqueExists;
        
        protected double _peakTrq;
        private double _maxTrqDur;
        private double _trqED;
        protected Queue<double> _queueRmsTrq = new Queue<double>();
        protected double _rmsTrqSum;
        protected double _rmsTrqED;        
        
        private double _inpMovePerRev;
        private double _outMovePerRev;
        private bool _inpMovePerRevUnitmm; //false = degrees, true = mm
        private bool _outMovePerRevUnitmm;

        private double _efficiency;

        protected double _peakSpd;
        private double _maxSpdDur;
        private double _spdED; 
        protected Queue<double> _queueRmsSpd = new Queue<double>();
        protected double _rmsSpdSum;
        protected double _rmsSpdED;

        private double _accInpBacklash;
        private double _accOutBacklash;
        private double _backlash;
        private bool _inpBacklashUnitmm; //false = arcmin, true = mm
        private bool _outBacklashUnitmm;

        protected double _regenPwr;
        protected double _regenEnergy;

        protected double _peakReg;
        private double _maxRegDur;
        private double _regED;
        protected Queue<double> _queueRmsReg = new Queue<double>();
        protected double _rmsRegSum;
        protected double _rmsRegED;

        public LoadResult() { }

        public LoadResult(string name, MainType mainType, bool autoSelect)
        {
            _name = name;
            _mainType = mainType;
            _autoSelect = autoSelect;

            _gearing = 1;
            _accInpGearing = 1;
            _accOutGearing = 1;
            
            _efficiency = 1;
            
            _inpMovePerRev = 1;
            _outMovePerRev = 1;
        }

        public string Name
        {
            get { return _name; }
            set
            {
                if (value != this._name)
                {
                    this._name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        public MainType MainType
        {
            get { return _mainType; }
            set
            {
                if (value != _mainType)
                {
                    this._mainType = value;
                    OnPropertyChanged("MainType");
                }
            }
        }

        public bool AutoSelect
        {
            get { return _autoSelect; }
            set
            {
                if (value != this._autoSelect)
                {
                    this._autoSelect = value;
                    OnPropertyChanged("AutoSelect");
                }
            }
        }

        public bool Active
        {
            get { return _active; }
            set
            {
                if (value != this._active)
                {
                    this._active = value;
                    OnPropertyChanged("Active");
                }
            }
        }
        
        public double Gearing
        {
            get { return _gearing; }
            set
            {
                if (value != this._gearing)
                {
                    this._gearing = value;
                    OnPropertyChanged("InpGearing");
                }
            }
        } 

        public double AccInpGearing
        {
            get { return _accInpGearing; }
            set
            {
                if (value != this._accInpGearing)
                {
                    this._accInpGearing = value;
                    OnPropertyChanged("AccInpGearing");
                }
            }
        }

        public double AccOutGearing
        {
            get { return _accOutGearing; }
            set
            {
                if (value != this._accOutGearing)
                {
                    this._accOutGearing = value;
                    OnPropertyChanged("AccOutGearing");
                }
            }
        }

        public double Inertia
        {
            get { return _inertia; }
            set
            {
                if (value != this._inertia)
                {
                    this._inertia = value;
                    OnPropertyChanged("Inertia");
                }
            }
        }

        public double Angle
        {
            get { return _angle; }
            set
            {
                if (value != this._angle)
                {
                    this._angle = value;
                    OnPropertyChanged("Angle");
                }
            }
        }

        public double Mass
        {
            get { return _mass; }
            set
            {
                if (value != this._mass)
                {
                    this._mass = value;
                    OnPropertyChanged("Mass");
                }
            }
        }


        public double IdleTorque
        {
            get { return _idleTorque; }
            set
            {
                if (value != this._idleTorque)
                {
                    this._idleTorque = value;
                    OnPropertyChanged("IdleTorque");
                }
            }
        }

        public double ThrustTorque
        {
            get { return _thrustTorque; }
            set
            {
                if (value != this._thrustTorque)
                {
                    this._thrustTorque = value;
                    OnPropertyChanged("ThustTorque");
                }
            }
        }

        public double AccTorque
        {
            get { return _accTorque; }
            set
            {
                if (value != this._accTorque)
                {
                    this._accTorque = value;
                    OnPropertyChanged("AccTorque");
                }
            }
        }

        public bool AccTorqueExists
        {
            get { return _accTorqueExists; }
            set
            {
                if (value != this._accTorqueExists)
                {
                    this._accTorqueExists = value;
                    OnPropertyChanged("AccTorqueExists");
                }
            }
        }

        public double ConstTorque
        {
            get { return _constTorque; }
            set
            {
                if (value != this._constTorque)
                {
                    this._constTorque = value;
                    OnPropertyChanged("ConstTorque");
                }
            }
        }

        public bool ConstTorqueExists
        {
            get { return _constTorqueExists; }
            set
            {
                if (value != this._constTorqueExists)
                {
                    this._constTorqueExists = value;
                    OnPropertyChanged("ConstTorqueExists");
                }
            }
        }

        public double LiftTorque
        {
            get { return _liftTorque; }
            set
            {
                if (value != this._liftTorque)
                {
                    this._liftTorque = value;
                    OnPropertyChanged("LiftTorque");
                }
            }
        }
        
        public double DecTorque
        {
            get { return _decTorque; }
            set
            {
                if (value != this._decTorque)
                {
                    this._decTorque = value;
                    OnPropertyChanged("DecTorque");
                }
            }
        }

        public bool DecTorqueExists
        {
            get { return _decTorqueExists; }
            set
            {
                if (value != this._decTorqueExists)
                {
                    this._decTorqueExists = value;
                    OnPropertyChanged("DecTorqueExists");
                }
            }
        }


        public double PeakTrq
        {
            get { return _peakTrq; }
            set
            {
                if (value != this._peakTrq)
                {
                    this._peakTrq = value;
                    OnPropertyChanged("PeakTrq");
                }
            }
        }

        public double MaxTrqDur
        {
            get { return _maxTrqDur; }
            set
            {
                if (value != this._maxTrqDur)
                {
                    this._maxTrqDur = value;
                    OnPropertyChanged("MaxTrqDur");
                }
            }
        }

        public double TrqED
        {
            get { return _trqED; }
            set
            {
                if (value != this._trqED)
                {
                    this._trqED = value;
                    OnPropertyChanged("TrqED");
                }
            }
        }

        public Queue<double> QueueRmsTrq
        {
            get { return _queueRmsTrq; }
            set
            {
                if (value != this._queueRmsTrq)
                {
                    this._queueRmsTrq = value;
                    OnPropertyChanged("QueueRmsTrq");
                }
            }
        }
        
        public double RmsTrqSum
        {
            get { return _rmsTrqSum; }
            set
            {
                if (value != this._rmsTrqSum)
                {
                    this._rmsTrqSum = value;
                    OnPropertyChanged("RmsTrqSum");
                }
            }
        }

        public double RmsTrqED
        {
            get { return _rmsTrqED; }
            set
            {
                if (value != this._rmsTrqED)
                {
                    this._rmsTrqED = value;
                    OnPropertyChanged("RmsTrqED");
                }
            }
        }

        public double InpMovePerRev
        {
            get { return _inpMovePerRev; }
            set
            {
                if (value != this._inpMovePerRev)
                {
                    this._inpMovePerRev = value;
                    OnPropertyChanged("InpMovePerRev");
                }
            }
        }

        public double OutMovePerRev
        {
            get { return _outMovePerRev; }
            set
            {
                if (value != this._outMovePerRev)
                {
                    this._outMovePerRev = value;
                    OnPropertyChanged("OutMovePerRev");
                }
            }
        }

        public bool InpMovePerRevUnitmm
        {
            get { return _inpMovePerRevUnitmm; }
            set
            {
                if (value != this._inpMovePerRevUnitmm)
                {
                    this._inpMovePerRevUnitmm = value;
                    OnPropertyChanged("InpMovePerRevUnitmm");
                }
            }
        }

        public bool OutMovePerRevUnitmm
        {
            get { return _outMovePerRevUnitmm; }
            set
            {
                if (value != this._outMovePerRevUnitmm)
                {
                    this._outMovePerRevUnitmm = value;
                    OnPropertyChanged("OutMovePerRevUnitmm");
                }
            }
        }

        public double Efficiency
        {
            get { return _efficiency; }
            set
            {
                if (value != this._efficiency)
                {
                    this._efficiency = value;
                    OnPropertyChanged("Efficiency");
                }
            }
        }

        public double PeakSpd
        {
            get { return _peakSpd; }
            set
            {
                if (value != this._peakSpd)
                {
                    this._peakSpd = value;
                    OnPropertyChanged("PeakSpd");
                }
            }
        }

        public double MaxSpdDur
        {
            get { return _maxSpdDur; }
            set
            {
                if (value != this._maxSpdDur)
                {
                    this._maxSpdDur = value;
                    OnPropertyChanged("MaxSpdDur");
                }
            }
        }

        public double SpdED
        {
            get { return _spdED; }
            set
            {
                if (value != this._spdED)
                {
                    this._spdED = value;
                    OnPropertyChanged("SpdED");
                }
            }
        }

        public Queue<double> QueueRmsSpd
        {
            get { return _queueRmsSpd; }
            set
            {
                if (value != this._queueRmsSpd)
                {
                    this._queueRmsSpd = value;
                    OnPropertyChanged("QueueRmsSpd");
                }
            }
        }

        public double RmsSpdSum
        {
            get { return _rmsSpdSum; }
            set
            {
                if (value != this._rmsSpdSum)
                {
                    this._rmsSpdSum = value;
                    OnPropertyChanged("RmsSpdSum");
                }
            }
        }

        public double RmsSpdED
        {
            get { return _rmsSpdED; }
            set
            {
                if (value != this._rmsSpdED)
                {
                    this._rmsSpdED = value;
                    OnPropertyChanged("RmsSpdED");
                }
            }
        }

        public double Backlash
        {
            get { return _backlash; }
            set
            {
                if (value != this._backlash)
                {
                    this._backlash = value;
                    OnPropertyChanged("Backlash");
                }
            }
        }

        public double AccInpBacklash
        {
            get { return _accInpBacklash; }
            set
            {
                if (value != this._accInpBacklash)
                {
                    this._accInpBacklash = value;
                    OnPropertyChanged("AccInpBacklash");
                }
            }
        }

        public double AccOutBacklash
        {
            get { return _accOutBacklash; }
            set
            {
                if (value != this._accOutBacklash)
                {
                    this._accOutBacklash = value;
                    OnPropertyChanged("AccOutBacklash");
                }
            }
        }

        public bool InpBacklashUnitmm
        {
            get { return _inpBacklashUnitmm; }
            set
            {
                if (value != this._inpBacklashUnitmm)
                {
                    this._inpBacklashUnitmm = value;
                    OnPropertyChanged("InpBacklashUnitmm");
                }
            }
        }

        public bool OutBacklashUnitmm
        {
            get { return _outBacklashUnitmm; }
            set
            {
                if (value != this._outBacklashUnitmm)
                {
                    this._outBacklashUnitmm = value;
                    OnPropertyChanged("OutBacklashUnitmm");
                }
            }
        }

        public double RegenPwr
        {
            get { return _regenPwr; }
            set
            {
                if (value != this._regenPwr)
                {
                    this._regenPwr = value;
                    OnPropertyChanged("RegenPwr");
                }
            }
        }

        public double RegenEnergy
        {
            get { return _regenEnergy; }
            set
            {
                if (value != this._regenEnergy)
                {
                    this._regenEnergy = value;
                    OnPropertyChanged("RegenEnergy");
                }
            }
        }

        public double PeakReg
        {
            get { return _peakReg; }
            set
            {
                if (value != this._peakReg)
                {
                    this._peakReg = value;
                    OnPropertyChanged("PeakReg");
                }
            }
        }

        public double MaxRegDur
        {
            get { return _maxRegDur; }
            set
            {
                if (value != this._maxRegDur)
                {
                    this._maxRegDur = value;
                    OnPropertyChanged("MaxRegDur");
                }
            }
        }

        public double RegED
        {
            get { return _regED; }
            set
            {
                if (value != this._regED)
                {
                    this._regED = value;
                    OnPropertyChanged("RegED");
                }
            }
        }

        public Queue<double> QueueRmsReg
        {
            get { return _queueRmsReg; }
            set
            {
                if (value != this._queueRmsReg)
                {
                    this._queueRmsReg = value;
                    OnPropertyChanged("QueueRmsReg");
                }
            }
        }

        public double RmsRegSum
        {
            get { return _rmsRegSum; }
            set
            {
                if (value != this._rmsRegSum)
                {
                    this._rmsRegSum = value;
                    OnPropertyChanged("RmsRegSum");
                }
            }
        }

        public double RmsRegED
        {
            get { return _rmsRegED; }
            set
            {
                if (value != this._rmsRegED)
                {
                    this._rmsRegED = value;
                    OnPropertyChanged("RmsRegED");
                }
            }
        }
    }
}
