﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace DockSample
{
    public class Mechanics : NotifyPropertyChangedBase
    {
        private bool _manualInertiaCalc;
        private double _outerDiameter;
        private double _innerDiameter;
        private double _cylinderHeight;
        private Material _material = Material.Steel;
        private double _partMass;
        private double _partInertia;

        public Mechanics() {}

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Calculate inertia manually without using mechanical values?.")]
        public bool ManualInertiaCalc
        {
            get { return _manualInertiaCalc; }
            set
            {
                if (value != this._manualInertiaCalc)
                {
                    this._manualInertiaCalc = value;
                    OnPropertyChanged("ManualInertiaCalc");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Outer diameter in millimeters (mm).")]
        public double OuterDiameter
        {
            get { return _outerDiameter; }
            set
            {
                if (value != this._outerDiameter)
                {
                    this._outerDiameter = value;

                    if (_manualInertiaCalc)
                        OnPropertyChanged("OuterDiameter");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Inner diameter in millimeters (mm).")]
        public double InnerDiameter
        {
            get { return _innerDiameter; }
            set
            {
                if (value != this._innerDiameter)
                {
                    this._innerDiameter = value;

                    if (_manualInertiaCalc) 
                        OnPropertyChanged("InnerDiameter");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Cylinder height (or length/widht) in millimeters (mm).")]
        public double CylinderHeight
        {
            get { return _cylinderHeight; }
            set
            {
                if (value != this._cylinderHeight)
                {
                    this._cylinderHeight = value;

                    if (_manualInertiaCalc)
                         OnPropertyChanged("CylinderHeight");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(false), DescriptionAttribute("Type of material. Defines density.")]
        public Material Material
        {
            get { return _material; }
            set
            {
                if (value != this._material)
                {
                    this._material = value;

                    if (_manualInertiaCalc) 
                        OnPropertyChanged("Material");
                }
            }
        }

        [CategoryAttribute("Main"), ReadOnly(true), DescriptionAttribute("Part mass in kilograms (kg).")]
        public double PartMass
        {
            get { return _partMass; }
            set
            {
                if (value != this._partMass)
                    this._partMass = value;
            }
        }

        [CategoryAttribute("Main"), ReadOnly(true), DescriptionAttribute("Part inertia (kgcm²).")]
        public double PartInertia
        {
            get { return _partInertia; }
            set
            {
                if (value != this._partInertia)
                    this._partInertia = value;
            }
        }
    }
}
