﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Reflection;
using System.Drawing.Design;

namespace DockSample
{
    public class Motor : GenericRotatingComponent
    {
        private new string _name;
        private new bool _autoSelect;
        private MotorType _subType;

        //Unique motor data        
        private double _inertiaRatio;
        private double _resolution;
        private double _fldWeakSpd;
        private bool _braked;
        private double _invEff;
        private double _drivePrice;

        //Mechanical data
        protected MechInterface _outType;
        protected double _outMinDia;
        protected double _outMaxDia;
        protected double _outRadFrc;
        protected double _outAxFrc;

        //Constructor
        public Motor()
        {
            _name = "Motor";
            _mainType = MainType.Motor;
        }
        
        /*
        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        //Function to set the UsedDef property attribute to ReadOnly for those classes that does not implement this functionality
        public new void SetAutoSelectTrue()
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(this.GetType())["AutoSelect"];
            ReadOnlyAttribute attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldInfo FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);
        }
        */

        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        public void SetOrResetReadOnlyAttributes(Motor motor)
        {
            //REALLY crappy solution to set the ReadOnlyone attribute at runtime. This is supposed to display or dim the object
            //properties in the PropertyGrid-object when the load is selected as AutoSelect = true or false.
            //Unfortunately it does not work. It affects all instances so the attributes has to be reset when switching
            //between the defined loads. Also attributes that should always be ReadOnly or NOT ReadOnly must be manually reset.
            //The Settings class is also affected if the ReadOnly attributes are not set to false !!!???

            //see: http://stackoverflow.com/questions/2483124/remove-c-sharp-attribute-of-a-property-dynamically
            //and: http://stackoverflow.com/questions/51269/change-attributes-parameter-at-runtime

            bool newValue = motor.AutoSelect;  //?

            foreach (PropertyDescriptor property in TypeDescriptor.GetProperties(motor))
            {
                //Set all properties ReadOnly attribute equal to the value of "AutoSelect"
                ReadOnlyAttribute attrib = (ReadOnlyAttribute)property.Attributes[typeof(ReadOnlyAttribute)];
                FieldInfo isReadOnly = attrib.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
                isReadOnly.SetValue(attrib, newValue);
            }

            //Since the loop above affects all properties make a second round to set those properties that should always
            //be ReadOnly or NOT ReadOnly

            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(motor.GetType())["Name"];
            ReadOnlyAttribute attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldInfo FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(motor.GetType())["AutoSelect"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, false);

            descriptor = TypeDescriptor.GetProperties(motor.GetType())["Campaign"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(motor.GetType())["Sold"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            if (!AutoSelect && Campaign)
                FieldIsReadOnly.SetValue(attribute, false);
            else
            {
                FieldIsReadOnly.SetValue(attribute, true);
            }

            descriptor = TypeDescriptor.GetProperties(motor.GetType())["MainType"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(motor.GetType())["NomSpd"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            descriptor = TypeDescriptor.GetProperties(motor.GetType())["NomTrq"];
            attribute = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];
            FieldIsReadOnly = attribute.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldIsReadOnly.SetValue(attribute, true);

            /*    //Original code to set the ReadOnlyone attribute on one single property
            //---------------------------------------------------------------------
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(this.GetType())["Name"];
            ReadOnlyAttribute attrib = (ReadOnlyAttribute)descriptor.Attributes[typeof(ReadOnlyAttribute)];

            FieldInfo isReadOnly = attrib.GetType().GetField("isReadOnly", BindingFlags.NonPublic | BindingFlags.Instance);
            isReadOnly.SetValue(attrib, newValue);
            //--------------------------------------------------------------------- */
        }
    
        [Category("Application"), ReadOnly(true), Description("The identifying name of this component.")]
        public new string Name
        {
            get { return _name; }
            set
            {
                if (value != this._name)
                {
                    this._oldName = this._name;
                    this._name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        [Category("Application"), ReadOnly(false), Description("Set whether this component is automatically selected by the application or manually by the the user.")]
        [System.ComponentModel.RefreshProperties(RefreshProperties.All)]
        public new bool AutoSelect
        {
            get { return _autoSelect; }
            set
            {
                this._autoSelect = value;

                OnPropertyChanged("AutoSelect");
            }
        }

        [Category("Main"), ReadOnly(false), Description("Recommended highest load to motor inertia ratio (no unit).")]
        public double InertiaRatio
        {
            get { return _inertiaRatio; }
            set
            {
                if (value != this._inertiaRatio)
                {
                    this._inertiaRatio = value;
                    if (!AutoSelect)
                    OnPropertyChanged("InertiaRatio");
                }
            }
        }

        [Category("Precision"), ReadOnly(false), Description("Number of encoder output pulses per motor revolution (no unit).")]
        public double Resolution
        {
            get { return _resolution; }
            set
            {
                if (value != this._resolution)
                {
                    this._resolution = value;
                    if (!AutoSelect)
                    OnPropertyChanged("Resolution");
                }
            }
        }

        [Category("Main"), ReadOnly(false), Description("Maximum speed at nominal torque (rpm). The torque is assumed to decrease linear with increasing speed.")]
        public double FldWeakSpd
        {
            get { return _fldWeakSpd; }
            set
            {
                if (value != this._fldWeakSpd)
                {
                    this._fldWeakSpd = value;
                    if (!AutoSelect)
                    OnPropertyChanged("FldWeakSpd");
                }
            }
        }

        [Category("Main"), ReadOnly(false), Description("Motor natural regenerative consumption due to friction etc in percent [100% = no consumption].")]
        public double InvEff
        {
            get { return _invEff; }
            set
            {
                if (value != this._invEff)
                {
                    this._invEff = value;
                    OnPropertyChanged("InvEff");
                }
            }
        }

        [Category("Main"), ReadOnly(false), Description("The motor is fitted with a mechanical brake.")]
        public bool Braked
        {
            get { return _braked; }
            set
            {
                if (value != this._braked)
                {
                    this._braked = value;
                    if (!AutoSelect)
                    OnPropertyChanged("Braked");
                }
            }
        }

      [Category("Component"), Browsable(false), Description("Typical price of compatible servo drive")]
        public double DrivePrice
        {
            get { return _drivePrice; }
            set
            {
                if (value != this._drivePrice)
                {
                    this._drivePrice = value;
                    if (!AutoSelect)
                        OnPropertyChanged("DrivePrice");
                }
            }
        }

        [Category("Component"), ReadOnly(false), Description("Component sub type.")]
        public MotorType SubType
        {
            get { return _subType; }
            set
            {
                if (value != this._subType)
                {
                    this._subType = value;
                    if (!AutoSelect)
                        OnPropertyChanged("SubType");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Mechanical output type.")]
        public MechInterface OutType
        {
            get { return _outType; }
            set
            {
                if (value != this._outType)
                {
                    this._outType = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutType");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Minimum output diameter in millimeters (mm).")]
        public double OutMinDia
        {
            get { return _outMinDia; }
            set
            {
                if (value != this._outMinDia)
                {
                    this._outMinDia = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutMinDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum output diameter in millimeters (mm).")]
        public double OutMaxDia
        {
            get { return _outMaxDia; }
            set
            {
                if (value != this._outMaxDia)
                {
                    this._outMaxDia = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutMaxDia");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum radial force in newtons (N) on the mechanical output.")]
        public double OutRadFrc
        {
            get { return _outRadFrc; }
            set
            {
                if (value != this._outRadFrc)
                {
                    this._outRadFrc = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutRadFrc");
                }
            }
        }

        [Category("Mechanical output"), ReadOnly(false), Description("Maximum axial force in newtons (N) on the mechanical output.")]
        public double OutAxFrc
        {
            get { return _outAxFrc; }
            set
            {
                if (value != this._outAxFrc)
                {
                    this._outAxFrc = value;
                    if (!AutoSelect)
                    OnPropertyChanged("OutAxFrc");
                }
            }
        }
    }
}
