﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;

namespace DockSample
{
    public class Move : GenericMove
    {
        private bool _refresh;
        private MoveType _moveType;
        private Profile _profile;
        private double _ramp;

        private bool _timeLocked;
        private bool _distanceLocked;
        private bool _distanceInDegrees;
        private bool _finalSpeedLocked;
        private bool _averageAccLocked;

        private ObservableCollection<Sector> _sectorsList = new ObservableCollection<Sector>();
        private LoadResult _totalLoadResults = new LoadResult();
        private SortedList<int, LoadResult> _loadResultsList = new SortedList<int, LoadResult>();
        private MotorResult _motorResults = new MotorResult();

        public Move(string name, MoveType type, Profile profile)
        {
            _name = name;

            _moveType = type;
            _profile = profile;
            _percentSineVel = 25;
            _percentSineAcc = 10;

            if (_timeLocked == false && _distanceLocked == false && 
                _finalSpeedLocked == false && _averageAccLocked == false)
            {
                _timeLocked = true;
                _distanceLocked = true;
            }
        }

        #region Properties

        public bool Refresh
        {
            get { return _refresh; }
            set
            {
                if (value != this._refresh)
                {
                    this._refresh = value;
                    OnPropertyChanged("Refresh");
                }
            }
        }

        public MoveType MoveType
        {
            get { return _moveType; }
            set
            {
                if (value != this._moveType)
                {
                    this._moveType = value;
                    OnPropertyChanged("MoveType");
                }
            }
        }

        public Profile Profile
        {
            get { return _profile; }
            set
            {
                if (value != this._profile)
                {
                    this._profile = value;
                    OnPropertyChanged("Profile");
                }
            }
        }

        public double Ramp
        {
            get { return _ramp; }
            set
            {
                if (value != this._ramp)
                {
                    this._ramp = value;
                    OnPropertyChanged("Ramp");
                }
            }
        }

        public bool TimeLocked
        {
            get { return _timeLocked; }
            set
            {
                if (value != this._timeLocked)
                {
                    this._timeLocked = value;
                    //OnPropertyChanged("TimeLocked");
                }
            }
        }

        public bool DistanceLocked
        {
            get { return _distanceLocked; }
            set
            {
                if (value != this._distanceLocked)
                {
                    this._distanceLocked = value;
                    //OnPropertyChanged("DistanceLocked");
                }
            }
        }

        public bool DistanceInDegrees
        {
            get { return _distanceInDegrees; }
            set
            {
                if (value != this._distanceInDegrees)
                {
                    this._distanceInDegrees = value;
                    OnPropertyChanged("DistanceInDegrees");
                }
            }
        }

        public bool FinalSpeedLocked
        {
            get { return _finalSpeedLocked; }
            set
            {
                if (value != this._finalSpeedLocked)
                {
                    this._finalSpeedLocked = value;
                    //OnPropertyChanged("FinalSpeedLocked");
                }
            }
        }

        public bool AverageAccLocked
        {
            get { return _averageAccLocked; }
            set
            {
                if (value != this._averageAccLocked)
                {
                    this._averageAccLocked = value;
                    //OnPropertyChanged("AverageAccLocked");
                }
            }
        }

        public LoadResult TotalLoadResults
        {
            get { return _totalLoadResults; }
            set
            {
                _totalLoadResults = value;
                OnPropertyChanged("TotalLoadResults");
            }
        }

        public SortedList<int, LoadResult> LoadResultsList
        {
            get { return _loadResultsList; }
        }

        public ObservableCollection<Sector> SectorsList
        {
            get { return _sectorsList; }
        }

        public void Sectors_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null)
                foreach (Sector item in e.NewItems)
                    item.PropertyChanged += Sectors_PropertyChanged;

            if (e.OldItems != null)
                foreach (Sector item in e.OldItems)
                    item.PropertyChanged -= Sectors_PropertyChanged;
        }

        public void Sectors_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            PropertyChangedEventHandler handler = SectorPropertyChanged;
            if (handler != null)
                handler(sender, e);
        }

        public event PropertyChangedEventHandler SectorPropertyChanged;

        public MotorResult MotorResults
        {
            get { return _motorResults; }
            set
            {
                _motorResults = value;
            }
        }

        #endregion

        #region Methods

        public void SwapLoadResults(int loadNr1, int loadNr2)
        {
            //If the load is moved up or down in the moveslist then swap the loadResults.
            //The main reason is to keep the information of which load is selected as active / not active.

            //Create temporary loadResult variables
            LoadResult tempLoadResult1 = LoadResultsList[loadNr1]; ;
            LoadResult tempLoadResult2 = LoadResultsList[loadNr2]; ;

            LoadResultsList[loadNr1] = tempLoadResult2;
            LoadResultsList[loadNr2] = tempLoadResult1;
        }

        public void UpdateLoadResults(SortedList<int, ILoad> loadsList)
        {
            //All loadresults are cleared before recalculated. Save the information
            //of the loads marked as active in each move before clearing the results.

            //This method is called when a load is added, moved or removed.

            //Save the existing LoadResult-records marked as "active"
            SortedList<int, LoadResult> oldLoadResultsList = new SortedList<int, LoadResult>();

            foreach (var varLR in LoadResultsList)
            {
                LoadResult loadResult = varLR.Value;
                int loadResultNr = varLR.Key;

                LoadResult oldLoadResult = new LoadResult();

                oldLoadResult.Name = loadResult.Name;
                oldLoadResult.Active = loadResult.Active;

                //Add the load result
                oldLoadResultsList.Add(loadResultNr, oldLoadResult);
            }

            //Clear the LoadResult list
            LoadResultsList.Clear();

            //Go through each load and create a new loadresult variable
            foreach (var l in loadsList)
            {
                ILoad load = l.Value;
                int loadNr = l.Key;

                LoadResult newLoadResult = new LoadResult();

                ////Go through the old load results and check if the active bit was set
                foreach (var oldLoadResult in oldLoadResultsList)
                {
                    LoadResult ol = oldLoadResult.Value;

                    if (ol.Name == load.Name)
                        if (ol.Active == true)
                            //If the load was active before then set it as active again
                            newLoadResult.Active = true;
                }

                //Add the load result
                LoadResultsList.Add(loadNr, newLoadResult);
            }
        }

        public void ActivateLoadResult(int loadnr)
        {
            //Activate a single loadResult. 
            
            //This method is called when inserting a new load that is always set as active by default.
            foreach (var l in LoadResultsList)
            {
                LoadResult loadResult = l.Value;

                if (l.Key == loadnr)
                    loadResult.Active = true;
            }            
        }

        public void ActivateLoadResults(SortedList<int, ILoad> iLoads)
        {
            //Activate all loadResults
            foreach (var l in iLoads)
            {
                ILoad iLoad = l.Value;
                int loadNr = l.Key;

                //Add a new LoadResult for the new load and set it to active
                LoadResult newLoadResult = new LoadResult(iLoad.Name, iLoad.MainType, iLoad.AutoSelect);
                newLoadResult.Active = true;

                LoadResultsList.Add(l.Key, newLoadResult);
            }
        }

        public void CalcMoveResults()
        {
            int count = 0;
            SectorOffset offset = new SectorOffset();

            //distance = integral of speed
            //average speed = integral of acceleration
            //peak speed = average speed
            //acceleration = integal of jerk

            //jerk = da/dt
            //average acc = dv/dt
            //peak acc = 2 * average acc - initial acc
            //vel = ds/dt

            foreach (Sector sector in SectorsList)
            {
                sector.PercentSineVel = _percentSineVel;
                sector.PercentSineAcc = _percentSineAcc;

                if (MoveType != MoveType.UserDefined) //If triangular, symmetric or asymmetric move
                {
                    if (Profile == Profile.Triangular)
                    {
                        if (count == 0)
                        {
                            //Acceleration
                            sector.InitialSpeed = 0;
                            sector.Time = _time / 2;
                            sector.Distance = _distance / 2;
                            sector.FinalSpeed = (2 * _distance) / _time;
                        }

                        if (count == 1)
                        {
                            //Deceleration
                            sector.InitialSpeed = SectorsList[0].FinalSpeed;
                            sector.Time = _time / 2;
                            sector.Distance = _distance / 2;
                            sector.FinalSpeed = 0;
                        }
                    }

                    if (Profile == Profile.Symmetric)
                    {
                        if (count == 0)
                        {
                            //Acceleration
                            sector.InitialSpeed = 0;
                            sector.Time = _time / 3;
                            sector.FinalSpeed = (3 * _distance) / (2 * _time);
                            sector.Distance = (sector.FinalSpeed * _time / 3) / 2;
                        }

                        if (count == 1)
                        {
                            //Constant speed
                            sector.InitialSpeed = SectorsList[0].FinalSpeed;
                            sector.Time = _time / 3;
                            sector.Distance = sector.InitialSpeed * _time / 3;
                            sector.FinalSpeed = sector.InitialSpeed;
                        }

                        if (count == 2)
                        {
                            //Deceleration
                            sector.InitialSpeed = SectorsList[1].FinalSpeed;
                            sector.Time = _time / 3;
                            sector.FinalSpeed = 0;
                            sector.Distance = (sector.InitialSpeed * _time / 3) / 2;
                        }
                    }

                    if (Profile == Profile.Asymmetric)
                    {
                        
                        double assymetricChangingAccTime = Time * (Ramp / 50.0) / 2.0;
                        double assymetricConstAccTime = Time - (2 * assymetricChangingAccTime);

                        double assymetricFinalSpeed = Distance / ((assymetricChangingAccTime / 2) + assymetricConstAccTime + (assymetricChangingAccTime / 2));                        
                        
                        if (count == 0)
                        {
                            //Acceleration
                            sector.InitialSpeed = 0;
                            sector.Time = assymetricChangingAccTime;

                            sector.FinalSpeed = assymetricFinalSpeed;
                            sector.AverageAcc = (sector.FinalSpeed - sector.InitialSpeed) / assymetricChangingAccTime;

                            sector.Distance = (Math.Pow(sector.FinalSpeed, 2) - Math.Pow(sector.InitialSpeed, 2)) / (2 * sector.AverageAcc);

                            sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                            
                        }

                        if (count == 1)
                        {
                            //Constant speed
                            sector.InitialSpeed = SectorsList[0].FinalSpeed;
                            sector.Time = assymetricConstAccTime;

                            sector.FinalSpeed = assymetricFinalSpeed;
                            sector.AverageAcc = 0;

                            sector.Distance = sector.InitialSpeed * sector.Time;

                            sector.AverageSpeed = sector.InitialSpeed;
                        }

                        if (count == 2)
                        {
                            //Deceleration
                            sector.InitialSpeed = SectorsList[1].FinalSpeed;
                            sector.Time = assymetricChangingAccTime;
                            sector.FinalSpeed = 0;

                            sector.AverageAcc = (sector.FinalSpeed - sector.InitialSpeed) / assymetricChangingAccTime;

                            sector.Distance = (Math.Pow(sector.FinalSpeed, 2) - Math.Pow(sector.InitialSpeed, 2)) / (2 * sector.AverageAcc);

                            sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                        }
                    }

                    //Calculate speed and acceleration
                    sector.AverageSpeed = sector.Distance / sector.Time;
                    sector.AverageAcc = (sector.FinalSpeed - sector.InitialSpeed) / sector.Time;

                    offset = sector.CalcSectorProfile(offset);                    
                }
                else
                {
                    if (MoveType == MoveType.UserDefined)
                    {
                        sector.InitialSpeed = InitialSpeed;
                        sector.Time = _time;
                        sector.Distance = _distance;
                        sector.AverageAcc = _averageAcc;
                        sector.FinalSpeed = _finalSpeed;

                        double diffSpeed = sector.FinalSpeed - sector.InitialSpeed;

                        if (DistanceLocked && AverageAccLocked)
                        {
                            //Calculate time and speed
                            sector.FinalSpeed = Math.Sqrt(Math.Pow(sector.InitialSpeed, 2) + (2 * sector.AverageAcc * sector.Distance));
                            if (sector.InitialSpeed != sector.FinalSpeed)
                                sector.Time = (sector.FinalSpeed - sector.InitialSpeed) / sector.AverageAcc;
                            else
                                sector.Time = sector.Distance / sector.FinalSpeed;
                            sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                        }
                        else
                            if (DistanceLocked && FinalSpeedLocked)
                            {
                                //Calculate time and acceleration 
                                sector.AverageAcc = (Math.Pow(sector.FinalSpeed, 2) - Math.Pow(sector.InitialSpeed, 2)) / (2 * sector.Distance);
                                if (sector.InitialSpeed != sector.FinalSpeed)
                                    sector.Time = (sector.FinalSpeed - sector.InitialSpeed) / sector.AverageAcc;
                                else
                                    sector.Time = sector.Distance / sector.FinalSpeed;
                                sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                            }
                            else
                                if (TimeLocked && AverageAccLocked)
                                {
                                    //Calculate distance and speed
                                    sector.FinalSpeed = sector.InitialSpeed + (sector.AverageAcc * sector.Time);
                                    sector.Distance = (Math.Pow(sector.FinalSpeed, 2) - Math.Pow(sector.InitialSpeed, 2)) / (2 * sector.AverageAcc);
                                    sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                                }
                                else
                                    if (TimeLocked && FinalSpeedLocked)
                                    {
                                        //Calculate distance and acceleration
                                        sector.Distance = ((diffSpeed / 2) * sector.Time) + sector.InitialSpeed * sector.Time;
                                        sector.AverageAcc = diffSpeed / sector.Time;
                                        sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                                    }
                                    else
                                        if (AverageAccLocked && FinalSpeedLocked)
                                        {
                                            //Calculate distance and time
                                            if (sector.InitialSpeed != sector.FinalSpeed)
                                                sector.Time = (sector.FinalSpeed - sector.InitialSpeed) / sector.AverageAcc;
                                            else
                                                sector.Time = sector.Distance / sector.FinalSpeed;

                                            sector.Distance = ((diffSpeed / 2) * sector.Time) + sector.InitialSpeed * sector.Time;
                                            sector.AverageSpeed = ((sector.FinalSpeed - sector.InitialSpeed) / 2) + sector.InitialSpeed;
                                        }
                                        else
                                        {
                                            //Calculate speed and acceleration
                                            sector.AverageSpeed = sector.Distance / sector.Time;
                                            sector.FinalSpeed = 2 * sector.AverageSpeed - sector.InitialSpeed;
                                            sector.AverageAcc = (sector.FinalSpeed - sector.InitialSpeed) / sector.Time;

                                            //If only one checkbox is checked then check distance and time as default

                                            //OBS!! Detta kan ge synkproblem med checkBoxar på frmMove!!!

                                            DistanceLocked = true;
                                            TimeLocked = true;
                                            FinalSpeedLocked = false;
                                            AverageAccLocked = false;
                                        }

                    } 

                    _time = sector.Time;
                    _distance = sector.Distance;
                    _finalSpeed = sector.FinalSpeed;
                    _averageSpeed = sector.AverageSpeed;
                    _averageAcc = sector.AverageAcc;
                    _percentSineVel = sector.PercentSineVel;
                    _percentSineAcc = sector.PercentSineAcc;

                    //Calculate the motion profile for the current sector
                    offset = sector.CalcSectorProfile(offset);
                }
                count++;
            }
        }

        public void CalcLoadResults(SortedList<int, ILoad> iLoads)
        {           
            //Save the existing LoadResult-records marked as "active"
            bool[] activeLoad = new bool[iLoads.Count];

            int count = 0;

            //Create a bit array that is true if the load is active
            foreach (var l in LoadResultsList)
            {
                LoadResult loadResult = l.Value;

                activeLoad[count] = loadResult.Active;
                count++;
            }

            //Clear the LoadResult list
            LoadResultsList.Clear();

            //Save the index of the previous active load
            int prevActLoad = 0;
            bool prevActLoadExists = false;

            //Go through all iLoads to create a new LoadResult.
            foreach (var iL in iLoads)
            {
                ILoad iload = iL.Value;
                int loadNr = LoadResultsList.Count; //This value will increase as the loadresults are created

                //Add a new LoadResult for each active load. 
                //Note: the loadResult object is initialized with preset variables.
                LoadResult loadResult = new LoadResult(iload.Name, iload.MainType, iload.AutoSelect);

                //Set the basic load results (based on the load type) that differs from the values in the constructor.
                switch (iload.MainType)
                {
                    case MainType.GearBox:
                        var gearBox = iload as GearBox;
                        loadResult.Active = true;
                        loadResult.Gearing = gearBox.Gearing;
                        loadResult.AccOutGearing = gearBox.Gearing;
                        loadResult.Inertia = gearBox.Inertia;
                        loadResult.IdleTorque = gearBox.IdleTrq;
                        loadResult.Efficiency = gearBox.Eff / 100;
                        loadResult.InpMovePerRev = 1;
                        loadResult.OutMovePerRev = loadResult.InpMovePerRev / loadResult.Gearing; 
                        loadResult.Backlash = gearBox.Backlash;
                        loadResult.AccOutBacklash = gearBox.Backlash;
                        loadResult.MaxTrqDur = gearBox.MaxTrqDur;
                        loadResult.MaxSpdDur = gearBox.MaxSpdDur;
                        break;

                    case MainType.Coupling:
                        var coupling = iload as Coupling;
                        loadResult.Active = true;
                        loadResult.Inertia = coupling.Inertia;
                        loadResult.IdleTorque = coupling.IdleTrq;
                        loadResult.Efficiency = coupling.Eff / 100;
                        loadResult.Backlash = coupling.Backlash;
                        loadResult.AccOutBacklash = coupling.Backlash;
                        loadResult.MaxTrqDur = coupling.MaxTrqDur;
                        loadResult.MaxSpdDur = coupling.MaxSpdDur;
                        break;

                    case MainType.BallScrew:
                        var ballScrew = iload as BallScrew;
                        loadResult.Active = true;
                        loadResult.Inertia = ballScrew.Inertia;
                        loadResult.Angle = ballScrew.Angle;
                        loadResult.IdleTorque = ballScrew.IdleTrq;
                        loadResult.Efficiency = ballScrew.Eff / 100;
                        loadResult.OutMovePerRev = ballScrew.MovePerRev;
                        loadResult.OutMovePerRevUnitmm = true;
                        loadResult.Backlash = ballScrew.Backlash;
                        loadResult.AccOutBacklash = ballScrew.Backlash;
                        loadResult.OutBacklashUnitmm = true;
                        loadResult.MaxTrqDur = ballScrew.MaxTrqDur;
                        loadResult.MaxSpdDur = ballScrew.MaxSpdDur;
                        break;

                    case MainType.RackAndPinion:
                        var rackAndPinion = iload as RackAndPinion;
                        loadResult.Active = true;
                        loadResult.Inertia = rackAndPinion.Inertia;
                        loadResult.Angle = rackAndPinion.Angle;
                        loadResult.IdleTorque = rackAndPinion.IdleTrq;
                        loadResult.Efficiency = rackAndPinion.Eff / 100;
                        loadResult.OutMovePerRev = rackAndPinion.MovePerRev;
                        loadResult.OutMovePerRevUnitmm = true;
                        loadResult.Backlash = rackAndPinion.Backlash;
                        loadResult.AccOutBacklash = rackAndPinion.Backlash;
                        loadResult.OutBacklashUnitmm = true;
                        loadResult.MaxTrqDur = rackAndPinion.MaxTrqDur;
                        loadResult.MaxSpdDur = rackAndPinion.MaxSpdDur;
                        break;

                    case MainType.TurnTable:
                        var turnTable = iload as TurnTable;
                        loadResult.Active = true;
                        loadResult.Inertia = turnTable.Inertia;
                        loadResult.Angle = turnTable.Angle;
                        loadResult.IdleTorque = turnTable.IdleTrq;
                        loadResult.Efficiency = turnTable.Eff / 100;
                        loadResult.OutMovePerRev = turnTable.MovePerRev;
                        loadResult.OutMovePerRevUnitmm = true;
                        loadResult.Backlash = turnTable.Backlash;
                        loadResult.AccOutBacklash = turnTable.Backlash;
                        loadResult.OutBacklashUnitmm = true;
                        loadResult.MaxTrqDur = turnTable.MaxTrqDur;
                        loadResult.MaxSpdDur = turnTable.MaxSpdDur;
                        break;

                    case MainType.ToothedBelt:
                        var toothedBelt = iload as ToothedBelt;
                        loadResult.Active = true;
                        loadResult.AccOutGearing = 1;
                        loadResult.Inertia = toothedBelt.Inertia;
                        loadResult.Angle = toothedBelt.Angle;
                        loadResult.IdleTorque = toothedBelt.IdleTrq;
                        loadResult.Efficiency = toothedBelt.Eff / 100;
                        loadResult.OutMovePerRev = toothedBelt.MovePerRev;
                        loadResult.OutMovePerRevUnitmm = true;
                        loadResult.Backlash = toothedBelt.Backlash;
                        loadResult.AccOutBacklash = toothedBelt.Backlash;
                        loadResult.OutBacklashUnitmm = true;
                        loadResult.MaxTrqDur = toothedBelt.MaxTrqDur;
                        loadResult.MaxSpdDur = toothedBelt.MaxSpdDur;
                        break;

                    case MainType.Mass:
                        //Check if the current loadResult is checked as active;
                        if (activeLoad[loadNr] == true)
                        {
                            loadResult.Active = true;
                            var mass = iload as LoadMass;
                            
                            loadResult.InpMovePerRevUnitmm = true; //A mass is never defined as rotating
                            loadResult.InpBacklashUnitmm = true;
                            loadResult.Mass = mass.Mass; //Recalculated when the movement per rev is known
                        }
                        break;

                    case MainType.Force:
                        //Check if the current loadResult is checked as active;
                        if (activeLoad[loadNr] == true)
                        {
                            loadResult.Active = true;
                            var force = iload as LoadForce;

                            loadResult.ThrustTorque = force.Force; //ToDo
                        }
                        break;

                    case MainType.Torque:
                        //Check if the current loadResult is checked as active;
                        if (activeLoad[loadNr] == true)
                        {
                            loadResult.Active = true;
                            var torque = iload as LoadTorque;

                            loadResult.IdleTorque = torque.Torque; //ToDo
                        }
                        break;

                    case MainType.Inertia:
                        //Check if the current loadResult is checked as active;
                        if (activeLoad[loadNr] == true)
                        {
                            loadResult.Active = true;
                            var inertia = iload as LoadInertia;

                            loadResult.Inertia = inertia.Inertia;
                        }
                        break;
                }

                //Recalculate those results that depends on the previous load (load0 = previous load for load1)
                if (prevActLoadExists == true)
                {
                    loadResult.AccInpGearing = LoadResultsList[prevActLoad].AccOutGearing;
                    loadResult.AccOutGearing *= loadResult.AccInpGearing;

                    loadResult.AccInpBacklash = LoadResultsList[prevActLoad].AccOutBacklash;
                    loadResult.AccOutBacklash += LoadResultsList[prevActLoad].AccOutBacklash / loadResult.Gearing;

                    loadResult.InpMovePerRev *= LoadResultsList[prevActLoad].OutMovePerRev;
                    loadResult.OutMovePerRev *= loadResult.InpMovePerRev;

                    if (LoadResultsList[prevActLoad].Angle != 0)
                        loadResult.Angle = LoadResultsList[prevActLoad].Angle;

                    if (iload.MainType == MainType.Mass)
                    {
                        loadResult.Inertia = loadResult.Mass * Math.Pow(loadResult.InpMovePerRev * loadResult.AccOutGearing / (2 * Math.PI), 2);
                        loadResult.LiftTorque = (loadResult.Mass * Math.Sin(loadResult.Angle * (Math.PI / 180)) * 9.81 * loadResult.InpMovePerRev * loadResult.AccOutGearing) / (2 * Math.PI);                      
                    }

                    if (iload.MainType == MainType.Force)
                    {
                        loadResult.ThrustTorque = loadResult.ThrustTorque * (loadResult.InpMovePerRev * loadResult.AccOutGearing) / (2 * Math.PI);
                    }
                }

                //Set a flag that a previous load exist
                if (loadResult.Active == true)
                {
                    prevActLoadExists = true;
                    prevActLoad = loadNr;
                    
                    //Save the TotalLoadResults values for those values calculated
                    TotalLoadResults.AccInpGearing = loadResult.AccInpGearing;
                    TotalLoadResults.AccOutGearing = loadResult.AccOutGearing;

                    TotalLoadResults.AccInpBacklash = loadResult.AccInpBacklash;
                    TotalLoadResults.AccOutBacklash = loadResult.AccOutBacklash;

                    TotalLoadResults.InpMovePerRev = loadResult.OutMovePerRev;  //???
                }
                
                //Add the load result to the results list
                LoadResultsList.Add(loadNr, loadResult);
            }                        

            prevActLoad = 0;

            //Go through the loadresults again from the back and recalculate those results that depends on the next load
            for (int i = LoadResultsList.Count - 1; i >= 0; i--)
            {
                //Check if the current loadResult is checked as active;
                if (LoadResultsList[i].Active == true)
                {
                    if (prevActLoad > 0)
                    {
                        LoadResultsList[i].Efficiency *= LoadResultsList[prevActLoad].Efficiency;
                        LoadResultsList[i].Inertia += LoadResultsList[prevActLoad].Inertia / Math.Pow(LoadResultsList[i].Gearing, 2);
                        LoadResultsList[i].IdleTorque += (LoadResultsList[prevActLoad].IdleTorque / LoadResultsList[i].Gearing) / LoadResultsList[i].Efficiency;
                        LoadResultsList[i].LiftTorque += (LoadResultsList[prevActLoad].LiftTorque / LoadResultsList[i].Gearing) / LoadResultsList[i].Efficiency;
                        LoadResultsList[i].ThrustTorque += (LoadResultsList[prevActLoad].ThrustTorque / LoadResultsList[i].Gearing) / LoadResultsList[i].Efficiency;
                    }

                    prevActLoad = i;

                    //Save the TotalLoadResults values for those values calculated
                    TotalLoadResults.Efficiency = LoadResultsList[i].Efficiency;
                    TotalLoadResults.Inertia = LoadResultsList[i].Inertia;
                    TotalLoadResults.IdleTorque = LoadResultsList[i].IdleTorque;
                    TotalLoadResults.LiftTorque = LoadResultsList[i].LiftTorque;
                    TotalLoadResults.ThrustTorque = LoadResultsList[i].ThrustTorque;

                    if (LoadResultsList[i].Angle != 0)
                    TotalLoadResults.Angle = LoadResultsList[i].Angle;
                }
            }

        // - - -  CalcDynamicLoadResults - - - 

            //Calculate those load results that depends on the movement speed or acceleration
            foreach (Sector sector in SectorsList)
            {
                foreach (var loadResult in LoadResultsList)
                {
                    //Calculate the torque for the individual loads
                    LoadResult lR = loadResult.Value;
                    lR.Name = lR.Name; //debug info

                    //Check movement direction
                    bool forwardOrUp = (sector.Distance >= 0);

                    if (Math.Abs(sector.FinalSpeed) > Math.Abs(sector.InitialSpeed)) //acceleration
                    {
                        //Acceleration                        
                        lR.AccTorque = lR.LiftTorque + lR.ThrustTorque +
                            lR.Inertia * (sector.PeakAcc * 2 * Math.PI) / (TotalLoadResults.InpMovePerRev * lR.AccInpGearing * lR.Efficiency);
                        
                        if (forwardOrUp)
                            lR.AccTorque += lR.IdleTorque;
                        else
                            lR.AccTorque -= lR.IdleTorque;
                    }

                    if (sector.FinalSpeed == sector.InitialSpeed) //constant speed
                    {
                        //Constant speed
                        if (sector.Distance != 0)
                            if (forwardOrUp)
                                lR.ConstTorque = lR.LiftTorque + lR.IdleTorque + lR.ThrustTorque;
                            else
                                lR.ConstTorque = lR.LiftTorque - lR.IdleTorque + lR.ThrustTorque;
                    }

                    if (Math.Abs(sector.FinalSpeed) < Math.Abs(sector.InitialSpeed)) //deceleration
                    {
                        //Deceleration
                        lR.DecTorque = lR.LiftTorque + lR.ThrustTorque +
                            lR.Inertia * (sector.PeakAcc * 2 * Math.PI) / (TotalLoadResults.InpMovePerRev * lR.AccInpGearing * lR.Efficiency);

                        if (sector.Distance != 0)
                            if (forwardOrUp)
                                lR.DecTorque += lR.IdleTorque;
                            else
                                lR.DecTorque -= lR.IdleTorque;
                    }
                }

                foreach (SectorElement mp in sector.SectorElementsList)
                {
                    bool forwardOrUp = (sector.Distance >= 0);

                    //Calculate the total torque profile. This profile is offset with the lifting torque
                    mp.Torque = TotalLoadResults.LiftTorque + TotalLoadResults.ThrustTorque +
                        TotalLoadResults.Inertia * (mp.Acc * 2 * Math.PI) / (TotalLoadResults.InpMovePerRev * TotalLoadResults.Efficiency);

                    if (sector.Distance != 0)
                        if (forwardOrUp)
                            mp.Torque += TotalLoadResults.IdleTorque;
                        else
                            mp.Torque -= TotalLoadResults.IdleTorque;

                    //Regenerative energy (including motor???)
                    if ((forwardOrUp && mp.Torque < 0) || (!forwardOrUp && mp.Torque > 0))
                        mp.RegenerativePower = Math.Abs(mp.Torque * (mp.Vel / (TotalLoadResults.InpMovePerRev)));
                    else
                        mp.RegenerativePower = 0;

                    //Regenerative energy
                    //if ((forwardOrUp && torque < 0) || (!forwardOrUp && torque > 0))
                     //   lR.RegenEnergy += Math.Abs(torque * (mp.Vel / (TotalLoadResults.InpMovePerRev * lR.AccInpGearing)) / Settings.MainSettings.Resolution);

                    //if (lR.RegenEnergy > 0)
                      //  lR.RegenEnergy = lR.RegenEnergy;

                    double regPower = Math.Pow(mp.RegenerativePower, 2);

                    //Add the current speed value to the FIFO
                    TotalLoadResults.QueueRmsReg.Enqueue(regPower);

                    //Add the result to a sum for all rms-values for the given duration
                    TotalLoadResults.RmsRegSum += regPower;

                    //Check if the number of items in the FIFO exceeds the time duration
                    if (TotalLoadResults.QueueRmsReg.Count > 5000) //dutyCycleSpd)
                    {
                        if (Math.Sqrt(TotalLoadResults.RmsRegSum / 5000) > TotalLoadResults.RmsRegED)
                            TotalLoadResults.RmsRegED = Math.Sqrt(TotalLoadResults.RmsRegSum / 5000);

                        //Remove the oldest value from the FIFO
                        TotalLoadResults.RmsRegSum -= TotalLoadResults.QueueRmsReg.Peek();
                        TotalLoadResults.QueueRmsReg.Dequeue();
                    }

                    foreach (var loadResult in LoadResultsList)
                    {
                        //Calculate the torque for the individual loads
                        LoadResult lR = loadResult.Value;
                        lR.Name = lR.Name;

                        if (lR.Active == true)
                        {
                            //If the ED-factors are not defined then set conservative values
                            if (lR.MaxTrqDur <= 0)
                                lR.MaxTrqDur = 1;

                            if (lR.MaxSpdDur <= 0)
                                lR.MaxSpdDur = 1;

                            lR.TrqED = 10;
                            lR.SpdED = 10;

                            //The rms-values are based on the duty cycle. During this cycle the rms-value must not exceed the nominal value.
                            double dutyCycleTrq = (lR.MaxTrqDur / (lR.TrqED / 100)) * Settings.MainSettings.Resolution;
                            double dutyCycleSpd = (lR.MaxSpdDur / (lR.SpdED / 100)) * Settings.MainSettings.Resolution;

                            //Torque
                            double torque = lR.LiftTorque + lR.ThrustTorque + 
                                lR.Inertia * (mp.Acc * 2 * Math.PI) / (TotalLoadResults.InpMovePerRev * lR.AccInpGearing * lR.Efficiency);

                            if (sector.Distance != 0) 
                                if (forwardOrUp)
                                    torque += lR.IdleTorque;
                                else
                                    torque -= lR.IdleTorque;
                                                                                    
                            //Save the peak torque value
                            if (Math.Abs(torque) > Math.Abs(lR.PeakTrq))
                                lR.PeakTrq = torque;

                            //Calculate the square of the momentary torque value
                            double temp = Math.Pow(torque, 2);

                            //Add the current torque value to the FIFO
                            lR.QueueRmsTrq.Enqueue(temp);

                            //Add the result to a sum for all rms-values for the given duration
                            lR.RmsTrqSum += temp;

                            //Check if the number of items in the FIFO exceeds the time duration
                            if (lR.QueueRmsTrq.Count > lR.MaxTrqDur * Settings.MainSettings.Resolution)
                            {
                                if (Math.Sqrt(lR.RmsTrqSum / dutyCycleTrq) > lR.RmsTrqED)
                                    lR.RmsTrqED = Math.Sqrt(lR.RmsTrqSum / dutyCycleTrq);

                                //Remove the oldest value from the FIFO
                                lR.RmsTrqSum -= lR.QueueRmsTrq.Peek();
                                lR.QueueRmsTrq.Dequeue();
                            }

                            //Speed
                            double spdRpm = 60 * (mp.Vel / (TotalLoadResults.InpMovePerRev * lR.AccInpGearing));

                            if (Math.Abs(spdRpm) > Math.Abs(lR.PeakSpd))
                                lR.PeakSpd = spdRpm;

                            spdRpm = Math.Pow(spdRpm, 2);

                            //Add the current speed value to the FIFO
                            lR.QueueRmsSpd.Enqueue(spdRpm);

                            //Add the result to a sum for all rms-values for the given duration
                            lR.RmsSpdSum += spdRpm;

                            //Check if the number of items in the FIFO exceeds the time duration
                            if (lR.QueueRmsSpd.Count > dutyCycleSpd)
                            {
                                if (Math.Sqrt(lR.RmsSpdSum / dutyCycleSpd) > lR.RmsSpdED)
                                    lR.RmsSpdED = Math.Sqrt(lR.RmsSpdSum / dutyCycleSpd);

                                //Remove the oldest value from the FIFO
                                lR.RmsSpdSum -= lR.QueueRmsSpd.Peek();
                                lR.QueueRmsSpd.Dequeue();
                            }
                        }
                    }
                }
            }

            //If the total duration for the move did not exceed the allowed ED duration then 
            //recalculate the rms-values for each load in the load results list
            foreach (var loadResult in LoadResultsList)
            {
                //Calculate the torque for the individual loads
                LoadResult lR = loadResult.Value;

                if (lR.Active == true)
                {
                    if (lR.RmsTrqED == 0) 
                        lR.RmsTrqED = Math.Sqrt(lR.RmsTrqSum / lR.QueueRmsTrq.Count);

                    if (lR.RmsSpdED == 0) 
                        lR.RmsSpdED = Math.Sqrt(lR.RmsSpdSum / lR.QueueRmsSpd.Count);

                    if (lR.RmsRegED == 0)
                        lR.RmsRegED = Math.Sqrt(lR.RmsRegSum / lR.QueueRmsReg.Count);
                }
            }

                for (int i = LoadResultsList.Count - 1; i >= 0; i--)
                {
                    //Check if the current loadResult is checked as active;
                    if (LoadResultsList[i].Active == true)
                    {
                        //Save the TotalResults
                        TotalLoadResults.AccTorque = LoadResultsList[i].AccTorque;
                        TotalLoadResults.ConstTorque = LoadResultsList[i].ConstTorque;
                        TotalLoadResults.DecTorque = LoadResultsList[i].DecTorque;
                        TotalLoadResults.PeakTrq = LoadResultsList[i].PeakTrq;
                        TotalLoadResults.RmsTrqED = LoadResultsList[i].RmsTrqED;
                        TotalLoadResults.RmsSpdED = LoadResultsList[i].RmsSpdED;
                        TotalLoadResults.PeakSpd = LoadResultsList[i].PeakSpd;     
                    }
                }
        }

        public List<SectorElement> GetAllSectorElements()
        {
            //Add the sector profiles to a single profile. This is only used to display the resulting
            //graph and table. Do not use this for calculations.

            //Clear the MoveResultsList
            List<SectorElement> SectorElementsList = new List<SectorElement>();

            //Collect LoadResults and Move-data into the MoveResultsList
            double count = 1;
            double accDist = 0;

            foreach (Sector sector in SectorsList)
            {
                foreach (SectorElement sElm in sector.SectorElementsList)
                {
                    SectorElementsList.Add(new SectorElement(count, sElm.Jerk, sElm.Acc, sElm.Vel, 
                        sElm.Pos + accDist, sElm.Torque, sElm.MotorTorqueLimit, sElm.RegenerativePower));
                    count++;
                }
                accDist += sector.Distance;
            }

            return SectorElementsList;
        }

        #endregion
    }
}
