﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DockSample
{
    public class Sector : GenericMove, ICloneable
    {
        private string _parent;
        private int _index;

        private double _averageJerk;

        private double _peakAcc;
        private double _changingAccTime;
        private double _peakJerk;
        private double _changingJerkTime;

        private Jerk[] _jerkArray = new Jerk[8];

        private List<SectorElement> _sectorElementsList = new List<SectorElement>();

        public Sector(string name, string parent, int index)
        {
            _parent = parent;
            _name = name;
            _index = index;

            for (int i = 0; i < 8; i++)
            {
                _jerkArray[i] = new Jerk();
            }
        }

        //Clone function to create non-reference object copies
        public object Clone()
        {
            return this.MemberwiseClone();
        }

        public string Parent
        {
            get { return _parent; }
            set
            {
                if (value != this._parent)
                {
                    this._parent = value;
                }
            }
        }

        public int Index
        {
            get { return _index; }
            set
            {
                if (value != this._index)
                {
                    this._index = value;
                    OnPropertyChanged("Index");
                }
            }
        }

        public double PeakAcc
        {
            get { return _peakAcc; }
            set
            {
                if (value != this._peakAcc)
                {
                    this._peakAcc = value;
                    OnPropertyChanged("PeakAcc");
                }
            }
        }

        public double ChangingAccTime
        {
            get { return _changingAccTime; }
            set
            {
                if (value != this._changingAccTime)
                {
                    this._changingAccTime = value;
                    OnPropertyChanged("ChangingAccTime");
                }
            }
        }

        public double AverageJerk
        {
            get { return _averageJerk; }
            set
            {
                if (value != this._averageJerk)
                {
                    this._averageJerk = value;
                    OnPropertyChanged("AverageJerk");
                }
            }
        }

        public double PeakJerk
        {
            get { return _peakJerk; }
            set
            {
                if (value != this._peakJerk)
                {
                    this._peakJerk = value;
                    OnPropertyChanged("PeakJerk");
                }
            }
        }

        public double ChangingJerkTime
        {
            get { return _changingJerkTime; }
            set
            {
                if (value != this._changingJerkTime)
                {
                    this._changingJerkTime = value;
                    OnPropertyChanged("ChangingJerkTime");
                }
            }
        }

        public List<SectorElement> SectorElementsList
        {
            get { return _sectorElementsList; }
        }

        public SectorOffset CalcSectorProfile(SectorOffset sectorOffset)
        {
            //Calculate the part time of the sector time that the acceleration will be either rising or falling.
            //If sine = 50 the ChangingAccTime will be 0.25 * t 
            //Time = (Rising)ChangingAccTime + (Constant)Time - (2 * ChangingAccTime) + (Falling)ChangingAccTime
            ChangingAccTime = (Time * (PercentSineVel / 100.0)) / 2.0;
            double constAccTime = Time - (2 * ChangingAccTime);

            //PeakAcc = SpeedDiff / (RiseTime/2 + ConstTime + FallTime/2)
            PeakAcc = (FinalSpeed - InitialSpeed) / ((ChangingAccTime / 2) + constAccTime + (ChangingAccTime / 2));

            ChangingJerkTime = (ChangingAccTime * (PercentSineAcc / 100.0)) / 2.0;
            double constantJerkTime = ChangingAccTime - (2 * ChangingJerkTime);

            PeakJerk = PeakAcc / ((ChangingJerkTime / 2) + constantJerkTime + (ChangingJerkTime / 2));

            _jerkArray[1].Time = ChangingJerkTime;
            _jerkArray[1].FinalJerk = PeakJerk;
            _jerkArray[1].AccTime = _jerkArray[1].Time;

            _jerkArray[2].Time = constantJerkTime;
            _jerkArray[2].FinalJerk = PeakJerk;
            _jerkArray[2].AccTime = _jerkArray[2].Time + _jerkArray[1].AccTime;

            _jerkArray[3].Time = ChangingJerkTime;
            _jerkArray[3].FinalJerk = 0;
            _jerkArray[3].AccTime = _jerkArray[3].Time + _jerkArray[2].AccTime;

            //Calculate the jerk array for sector 14
            _jerkArray[4].Time = constAccTime;
            _jerkArray[4].FinalJerk = 0;
            _jerkArray[4].AccTime = _jerkArray[4].Time + _jerkArray[3].AccTime;

            //Calculate the jerk array for sector 15 - 17
            _jerkArray[5].Time = ChangingJerkTime;
            _jerkArray[5].FinalJerk = -1.0 * PeakJerk;
            _jerkArray[5].AccTime = _jerkArray[5].Time + _jerkArray[4].AccTime;

            _jerkArray[6].Time = constantJerkTime;
            _jerkArray[6].FinalJerk = -1.0 * PeakJerk;
            _jerkArray[6].AccTime = _jerkArray[6].Time + _jerkArray[5].AccTime;

            _jerkArray[7].Time = ChangingJerkTime;
            _jerkArray[7].FinalJerk = 0;
            _jerkArray[7].AccTime = _jerkArray[7].Time + _jerkArray[6].AccTime;

            //Reset cumulated variables. Not used for the moment.
            sectorOffset.timetoAdd = 0;
            sectorOffset.valuetoAdd = 0;

            //Calculate the movement data for jerk, acceleration, velocity and distance
            double jerk = sectorOffset.valuetoAdd;
            double acc = 0;
            double vel = InitialSpeed;
            double dist = 0;
            SectorElement sElm;

            _sectorElementsList.Clear();

            int j = 1;

            int nrOfSectorSamples = Convert.ToInt32((Time + sectorOffset.timetoAdd) * Settings.MainSettings.Resolution);

            for (int i = 1; i <= nrOfSectorSamples; i++)
            {
                while (i > (_jerkArray[j].AccTime + sectorOffset.timetoAdd) * Settings.MainSettings.Resolution && j < 7)
                    j++;

                double k = (_jerkArray[j].FinalJerk - _jerkArray[j - 1].FinalJerk) / (_jerkArray[j].Time * Settings.MainSettings.Resolution);
                double offset = _jerkArray[j - 1].FinalJerk;

                double sampleTime = i - _jerkArray[j - 1].AccTime * Settings.MainSettings.Resolution;

                jerk = sampleTime * k + offset;
                acc += jerk / Settings.MainSettings.Resolution;
                vel += acc / Settings.MainSettings.Resolution;
                dist += vel / Settings.MainSettings.Resolution;

                SectorElementsList.Add(sElm = new SectorElement(i, jerk, acc, vel, dist, 0, 0, 0));

                //If this is the last sample and there is any time remaining then calculate a rest to be added to the next sector
                if (i == nrOfSectorSamples)
                    if (Time * Settings.MainSettings.Resolution > i)
                    {
                        sectorOffset.timetoAdd = (Time * Settings.MainSettings.Resolution - nrOfSectorSamples) / Settings.MainSettings.Resolution;
                        sectorOffset.valuetoAdd = (sampleTime + Time * Settings.MainSettings.Resolution - nrOfSectorSamples) * k + offset;
                    }
                    else
                    {
                        sectorOffset.timetoAdd = 0;
                        sectorOffset.valuetoAdd = 0;
                    }
            }

            return sectorOffset;
        }
    }
}
