﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Drawing;
using WeifenLuo.WinFormsUI.Docking;
using System.IO;
using System.Text;
using System.Xml;
using System.Globalization;
using System.Collections.Generic;

namespace DockSample
{
    public class XML
    {
        public static class XMLHandler
                    {
            private const string ConfigFileVersion = "1.0";
            private static string[] CompatibleConfigFileVersions = new string[] { };

            public static void SaveAsXml(string fileName, ApplicationData applicationData, Moves moves, SortedList<int, ILoad> iLoads, Motor motor, SortedList<int, IAccessory> iAccessories)//(DockPanel dockPanel, string fileName, Encoding encoding)
            {
                Encoding encoding = Encoding.Unicode;

                FileStream fs = new FileStream(fileName, FileMode.Create);
                try
                {
                    SaveAsXml(fs, applicationData, moves, iLoads, motor, iAccessories, encoding, false);//(dockPanel, fs, encoding);
                }
                finally
                {
                    fs.Close();
                }
            }

            public static void SaveAsXml(Stream stream, ApplicationData applicationData, Moves moves, SortedList<int, ILoad> iLoads, Motor motor, SortedList<int, IAccessory> iAccessories, Encoding encoding, bool upstream)//(DockPanel dockPanel, Stream stream, Encoding encoding, bool upstream)
            {
                XmlTextWriter xmlOut = new XmlTextWriter(stream, encoding);

                //Use indenting for readability
                xmlOut.Formatting = Formatting.Indented;

                if (!upstream)
                    xmlOut.WriteStartDocument();

                // Always begin file with identification and warning

                xmlOut.WriteComment("Strings.DockPanel_Persistor_XmlFileComment1");
                xmlOut.WriteComment("Strings.DockPanel_Persistor_XmlFileComment2");


                // Associate a version number with the root element so that future version of the code
                // will be able to be backwards compatible or at least recognise out of date versions
                xmlOut.WriteStartElement("BeijerDriveDesignerVer1.0");

                //Application data
                xmlOut.WriteStartElement("ApplicationData");

                xmlOut.WriteAttributeString("Date", applicationData.Date);
                xmlOut.WriteAttributeString("AppName", applicationData.AppName);
                xmlOut.WriteAttributeString("Customer", applicationData.Customer);
                xmlOut.WriteAttributeString("YourRef", applicationData.YourRef);
                xmlOut.WriteAttributeString("OurRef", applicationData.OurRef);
                xmlOut.WriteAttributeString("Description", applicationData.Description);
                //RichTextBox1.SaveFile("MyXml.xml", RichTextBoxStreamType.PlainText)

                xmlOut.WriteEndElement();
                
                //Moves
                xmlOut.WriteStartElement("Moves");
                xmlOut.WriteAttributeString("Count", moves.MovesList.Count.ToString());
            
                foreach (var m in moves.MovesList)
                {
                    Move move = m.Value;

                    xmlOut.WriteStartElement("Move");

                    //Type of move
                    xmlOut.WriteAttributeString("MoveType", move.MoveType.ToString());
                    
                    if (move.MoveType == MoveType.PreDefined)
                        xmlOut.WriteAttributeString("Profile", move.Profile.ToString());
                    else
                        xmlOut.WriteAttributeString("Profile", "UnDefined");

                    //Move parameters
                    xmlOut.WriteAttributeString("Name", move.Name);
                    xmlOut.WriteAttributeString("Time", move.Time.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("Degrees", move.Degrees.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("Distance", move.Distance.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("InitialSpeed", move.InitialSpeed.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("FinalSpeed", move.FinalSpeed.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("AverageAcc", move.AverageAcc.ToString(CultureInfo.InvariantCulture));

                    xmlOut.WriteAttributeString("Ramp", move.Ramp.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("PercentSineVel", move.PercentSineVel.ToString(CultureInfo.InvariantCulture));
                    xmlOut.WriteAttributeString("PercentSineAcc", move.PercentSineAcc.ToString(CultureInfo.InvariantCulture));

                    //Degrees or meters?
                    xmlOut.WriteAttributeString("DistanceInDegrees", move.DistanceInDegrees.ToString());

                    //Move parameters locked
                    xmlOut.WriteAttributeString("TimeLocked", move.TimeLocked.ToString());
                    xmlOut.WriteAttributeString("DistanceLocked", move.DistanceLocked.ToString());
                    xmlOut.WriteAttributeString("FinalSpeedLocked", move.FinalSpeedLocked.ToString());
                    xmlOut.WriteAttributeString("AverageAccLocked", move.AverageAccLocked.ToString());

                    xmlOut.WriteEndElement();

                    //Load Results locked (loads checked as active in this move)
                    //like so: foreach (load in loadslist) ... xmlOut.WriteAttributeString("LoadResult" + loadNr.ToString(), Load.Active.ToString());

                    xmlOut.WriteStartElement("ActiveLoads");
                    xmlOut.WriteAttributeString("LoadCount", move.LoadResultsList.Count.ToString());
                    
                    foreach (var lR in move.LoadResultsList)
                    {
                        xmlOut.WriteStartElement("ActiveLoad");
                        LoadResult loadResult = lR.Value;
                                                
                        xmlOut.WriteAttributeString("LoadName", loadResult.Name.ToString());
                        xmlOut.WriteAttributeString("MainType", loadResult.MainType.ToString()); 
                        xmlOut.WriteAttributeString("AutoSelect", loadResult.AutoSelect.ToString()); 
                        xmlOut.WriteAttributeString("Active", loadResult.Active.ToString());

                        xmlOut.WriteEndElement();
                    }

                    xmlOut.WriteEndElement();
                }

                xmlOut.WriteEndElement();
                
                //iLoads
                xmlOut.WriteStartElement("Loads");
                xmlOut.WriteAttributeString("Count", iLoads.Count.ToString());

                foreach (var iL in iLoads)
                {
                    ILoad iload = iL.Value;

                    xmlOut.WriteStartElement("Load");

                    xmlOut.WriteAttributeString("MainType", iload.MainType.ToString());
                    xmlOut.WriteAttributeString("Name", iload.Name.ToString());
                    xmlOut.WriteAttributeString("AutoSelect", iload.AutoSelect.ToString());

                    //Create the file contents based on the load type
                    switch (iload.MainType)
                    {
                        case MainType.Coupling:
                            var coupling = iload as Coupling;
                            xmlOut.WriteAttributeString("ArticleNo", coupling.ArticleNo.ToString());                            
                            xmlOut.WriteAttributeString("Brand", coupling.Brand.ToString());
                            xmlOut.WriteAttributeString("Description", coupling.Description.ToString());
                            xmlOut.WriteAttributeString("Price", coupling.Price.ToString(CultureInfo.InvariantCulture));

                            //Main data
                            xmlOut.WriteAttributeString("IdleTrq", coupling.IdleTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Inertia", coupling.Inertia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Eff", coupling.Eff.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Backlash", coupling.Backlash.ToString(CultureInfo.InvariantCulture));

                            //Mechanical part data
                            xmlOut.WriteAttributeString("ManualInertiaCalc", coupling.CouplingGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("CylinderHeight", coupling.CouplingGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OuterDiameter", coupling.CouplingGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InnerDiameter", coupling.CouplingGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Material", coupling.CouplingGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("PartMass", coupling.CouplingGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("PartInertia", coupling.CouplingGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));
        
                            //Mechanical input
                            xmlOut.WriteAttributeString("InpType", coupling.InpType.ToString());

                            xmlOut.WriteAttributeString("InpMinDia", coupling.InpMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpMaxDia", coupling.InpMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpRadFrc", coupling.InpRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpAxFrc", coupling.InpAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Mechanical output
                            xmlOut.WriteAttributeString("OutType", coupling.OutType.ToString());

                            xmlOut.WriteAttributeString("OutMinDia", coupling.OutMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OutMaxDia", coupling.OutMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OutRadFrc", coupling.OutRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OutAxFrc", coupling.OutAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Torque data
                            xmlOut.WriteAttributeString("MaxTrq", coupling.MaxTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxTrqDur", coupling.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("TrqED", coupling.TrqED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomTrq", coupling.NomTrq.ToString(CultureInfo.InvariantCulture));

                            //Speed data
                            xmlOut.WriteAttributeString("MaxSpd", coupling.MaxSpd.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxSpdDur", coupling.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("SpdED", coupling.SpdED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomSpd", coupling.NomSpd.ToString(CultureInfo.InvariantCulture));
                            break;

                        case MainType.BallScrew:
                            var ballScrew = iload as BallScrew;

                            xmlOut.WriteAttributeString("ArticleNo", ballScrew.ArticleNo.ToString());
                            xmlOut.WriteAttributeString("Brand", ballScrew.Brand.ToString());
                            xmlOut.WriteAttributeString("Description", ballScrew.Description.ToString());
                            xmlOut.WriteAttributeString("Price", ballScrew.Price.ToString(CultureInfo.InvariantCulture));

                            //Main data
                            xmlOut.WriteAttributeString("MovePerRev", ballScrew.MovePerRev.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("IdleTrq", ballScrew.IdleTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Inertia", ballScrew.Inertia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Eff", ballScrew.Eff.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Backlash", ballScrew.Backlash.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Angle", ballScrew.Angle.ToString(CultureInfo.InvariantCulture));

                            //Mechanical part data
                            xmlOut.WriteAttributeString("ManualInertiaCalc", ballScrew.BallScrewGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("CylinderHeight", ballScrew.BallScrewGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OuterDiameter", ballScrew.BallScrewGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InnerDiameter", ballScrew.BallScrewGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Material", ballScrew.BallScrewGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("PartMass", ballScrew.BallScrewGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("PartInertia", ballScrew.BallScrewGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));
                        
                            //Mechanical input
                            xmlOut.WriteAttributeString("InpType", ballScrew.InpType.ToString());

                            xmlOut.WriteAttributeString("InpMinDia", ballScrew.InpMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpMaxDia", ballScrew.InpMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpRadFrc", ballScrew.InpRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpAxFrc", ballScrew.InpAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Torque data
                            xmlOut.WriteAttributeString("MaxTrq", ballScrew.MaxTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxTrqDur", ballScrew.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("TrqED", ballScrew.TrqED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomTrq", ballScrew.NomTrq.ToString(CultureInfo.InvariantCulture));

                            //Speed data
                            xmlOut.WriteAttributeString("MaxSpd", ballScrew.MaxSpd.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxSpdDur", ballScrew.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("SpdED", ballScrew.SpdED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomSpd", ballScrew.NomSpd.ToString(CultureInfo.InvariantCulture));
                            break;

                        case MainType.TurnTable:
                            var turnTable = iload as TurnTable;

                            xmlOut.WriteAttributeString("ArticleNo", turnTable.ArticleNo.ToString());
                            xmlOut.WriteAttributeString("Brand", turnTable.Brand.ToString());
                            xmlOut.WriteAttributeString("Description", turnTable.Description.ToString());
                            xmlOut.WriteAttributeString("Price", turnTable.Price.ToString(CultureInfo.InvariantCulture));

                            //Main data
                            xmlOut.WriteAttributeString("MovePerRev", turnTable.MovePerRev.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("IdleTrq", turnTable.IdleTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Inertia", turnTable.Inertia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Eff", turnTable.Eff.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Backlash", turnTable.Backlash.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Angle", turnTable.Angle.ToString(CultureInfo.InvariantCulture));

                            //Mechanical part data
                            xmlOut.WriteAttributeString("ManualInertiaCalc", turnTable.TurnTableGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("CylinderHeight", turnTable.TurnTableGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OuterDiameter", turnTable.TurnTableGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InnerDiameter", turnTable.TurnTableGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Material", turnTable.TurnTableGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("PartMass", turnTable.TurnTableGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("PartInertia", turnTable.TurnTableGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));

                            //Mechanical input
                            xmlOut.WriteAttributeString("InpType", turnTable.InpType.ToString());

                            xmlOut.WriteAttributeString("InpMinDia", turnTable.InpMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpMaxDia", turnTable.InpMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpRadFrc", turnTable.InpRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpAxFrc", turnTable.InpAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Torque data
                            xmlOut.WriteAttributeString("MaxTrq", turnTable.MaxTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxTrqDur", turnTable.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("TrqED", turnTable.TrqED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomTrq", turnTable.NomTrq.ToString(CultureInfo.InvariantCulture));

                            //Speed data
                            xmlOut.WriteAttributeString("MaxSpd", turnTable.MaxSpd.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxSpdDur", turnTable.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("SpdED", turnTable.SpdED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomSpd", turnTable.NomSpd.ToString(CultureInfo.InvariantCulture));
                            break;

                        case MainType.RackAndPinion:
                            var rackAndPinion = iload as RackAndPinion;

                            xmlOut.WriteAttributeString("ArticleNo", rackAndPinion.ArticleNo.ToString());
                            xmlOut.WriteAttributeString("Brand", rackAndPinion.Brand.ToString());
                            xmlOut.WriteAttributeString("Description", rackAndPinion.Description.ToString());
                            xmlOut.WriteAttributeString("Price", rackAndPinion.Price.ToString(CultureInfo.InvariantCulture));

                            //Main data
                            xmlOut.WriteAttributeString("MovePerRev", rackAndPinion.MovePerRev.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("IdleTrq", rackAndPinion.IdleTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Inertia", rackAndPinion.Inertia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Eff", rackAndPinion.Eff.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Backlash", rackAndPinion.Backlash.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Angle", rackAndPinion.Angle.ToString(CultureInfo.InvariantCulture));

                            //Mechanical part data
                            xmlOut.WriteAttributeString("ManualInertiaCalc", rackAndPinion.RackAndPinionGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("CylinderHeight", rackAndPinion.RackAndPinionGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OuterDiameter", rackAndPinion.RackAndPinionGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InnerDiameter", rackAndPinion.RackAndPinionGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Material", rackAndPinion.RackAndPinionGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("PartMass", rackAndPinion.RackAndPinionGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("PartInertia", rackAndPinion.RackAndPinionGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));

                            //Mechanical input
                            xmlOut.WriteAttributeString("InpType", rackAndPinion.InpType.ToString());

                            xmlOut.WriteAttributeString("InpMinDia", rackAndPinion.InpMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpMaxDia", rackAndPinion.InpMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpRadFrc", rackAndPinion.InpRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpAxFrc", rackAndPinion.InpAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Torque data
                            xmlOut.WriteAttributeString("MaxTrq", rackAndPinion.MaxTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxTrqDur", rackAndPinion.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("TrqED", rackAndPinion.TrqED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomTrq", rackAndPinion.NomTrq.ToString(CultureInfo.InvariantCulture));

                            //Speed data
                            xmlOut.WriteAttributeString("MaxSpd", rackAndPinion.MaxSpd.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxSpdDur", rackAndPinion.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("SpdED", rackAndPinion.SpdED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomSpd", rackAndPinion.NomSpd.ToString(CultureInfo.InvariantCulture));
                            break;

                        case MainType.ToothedBelt:
                            var toothedBelt = iload as ToothedBelt;

                            xmlOut.WriteAttributeString("ArticleNo", toothedBelt.ArticleNo.ToString());
                            xmlOut.WriteAttributeString("Brand", toothedBelt.Brand.ToString());
                            xmlOut.WriteAttributeString("Description", toothedBelt.Description.ToString());
                            xmlOut.WriteAttributeString("Price", toothedBelt.Price.ToString(CultureInfo.InvariantCulture));

                            //Main data
                            xmlOut.WriteAttributeString("MovePerRev", toothedBelt.MovePerRev.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("IdleTrq", toothedBelt.IdleTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Inertia", toothedBelt.Inertia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Eff", toothedBelt.Eff.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Backlash", toothedBelt.Backlash.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Angle", toothedBelt.Angle.ToString(CultureInfo.InvariantCulture));

                            //Mech data
                            
                            //DriveWheel
                            xmlOut.WriteAttributeString("DriveWheel_ManualInertiaCalc", toothedBelt.DriveWheelGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("DriveWheel_CylinderHeight", toothedBelt.DriveWheelGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("DriveWheel_OuterDiameter", toothedBelt.DriveWheelGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("DriveWheel_InnerDiameter", toothedBelt.DriveWheelGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("DriveWheel_Material", toothedBelt.DriveWheelGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("DriveWheel_PartMass", toothedBelt.DriveWheelGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("DriveWheel_PartInertia", toothedBelt.DriveWheelGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));

                            //LoadWheel
                            xmlOut.WriteAttributeString("LoadWheel_ManualInertiaCalc", toothedBelt.LoadWheelGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("LoadWheel_CylinderHeight", toothedBelt.LoadWheelGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("LoadWheel_OuterDiameter", toothedBelt.LoadWheelGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("LoadWheel_InnerDiameter", toothedBelt.LoadWheelGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("LoadWheel_Material", toothedBelt.LoadWheelGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("LoadWheel_PartMass", toothedBelt.LoadWheelGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("LoadWheel_PartInertia", toothedBelt.LoadWheelGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));
                            
                            //Mechanical input
                            xmlOut.WriteAttributeString("InpType", toothedBelt.InpType.ToString());

                            xmlOut.WriteAttributeString("InpMinDia", toothedBelt.InpMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpMaxDia", toothedBelt.InpMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpRadFrc", toothedBelt.InpRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpAxFrc", toothedBelt.InpAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Torque data
                            xmlOut.WriteAttributeString("MaxTrq", toothedBelt.MaxTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxTrqDur", toothedBelt.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("TrqED", toothedBelt.TrqED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomTrq", toothedBelt.NomTrq.ToString(CultureInfo.InvariantCulture));

                            //Speed data
                            xmlOut.WriteAttributeString("MaxSpd", toothedBelt.MaxSpd.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxSpdDur", toothedBelt.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("SpdED", toothedBelt.SpdED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomSpd", toothedBelt.NomSpd.ToString(CultureInfo.InvariantCulture));
                            break;

                        case MainType.GearBox:
                            var gearBox = iload as GearBox;
                            xmlOut.WriteAttributeString("ArticleNo", gearBox.ArticleNo.ToString());
                            xmlOut.WriteAttributeString("Brand", gearBox.Brand.ToString());
                            xmlOut.WriteAttributeString("Description", gearBox.Description.ToString());
                            xmlOut.WriteAttributeString("Price", gearBox.Price.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Campaign", gearBox.Campaign.ToString(CultureInfo.InvariantCulture));

                            //Main data
                            xmlOut.WriteAttributeString("Gearing", gearBox.Gearing.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("IdleTrq", gearBox.IdleTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Inertia", gearBox.Inertia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Eff", gearBox.Eff.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Backlash", gearBox.Backlash.ToString(CultureInfo.InvariantCulture));

                            //Mechanical input
                            xmlOut.WriteAttributeString("InpType", gearBox.InpType.ToString());

                            xmlOut.WriteAttributeString("InpMinDia", gearBox.InpMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpMaxDia", gearBox.InpMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpRadFrc", gearBox.InpRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InpAxFrc", gearBox.InpAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Mechanical output
                            xmlOut.WriteAttributeString("OutType", gearBox.OutType.ToString());

                            xmlOut.WriteAttributeString("OutMinDia", gearBox.OutMinDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OutMaxDia", gearBox.OutMaxDia.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OutRadFrc", gearBox.OutRadFrc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OutAxFrc", gearBox.OutAxFrc.ToString(CultureInfo.InvariantCulture));

                            //Torque data
                            xmlOut.WriteAttributeString("MaxTrq", gearBox.MaxTrq.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxTrqDur", gearBox.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("TrqED", gearBox.TrqED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomTrq", gearBox.NomTrq.ToString(CultureInfo.InvariantCulture));

                            //Speed data
                            xmlOut.WriteAttributeString("MaxSpd", gearBox.MaxSpd.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("MaxSpdDur", gearBox.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("SpdED", gearBox.SpdED.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("NomSpd", gearBox.NomSpd.ToString(CultureInfo.InvariantCulture));
                            break;

                        case MainType.Mass:
                            var mass = iload as LoadMass;
                            xmlOut.WriteAttributeString("Mass", mass.Mass.ToString(CultureInfo.InvariantCulture));
                            break;

                            case MainType.Inertia:
                            var inertia = iload as LoadInertia;
                            xmlOut.WriteAttributeString("Inertia", inertia.Inertia.ToString(CultureInfo.InvariantCulture));

                            //Mechanical part data
                            xmlOut.WriteAttributeString("ManualInertiaCalc", inertia.InertiaGeometry.ManualInertiaCalc.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("CylinderHeight", inertia.InertiaGeometry.CylinderHeight.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("OuterDiameter", inertia.InertiaGeometry.OuterDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("InnerDiameter", inertia.InertiaGeometry.InnerDiameter.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("Material", inertia.InertiaGeometry.Material.ToString()); //Note!!!
                            xmlOut.WriteAttributeString("PartMass", inertia.InertiaGeometry.PartMass.ToString(CultureInfo.InvariantCulture));
                            xmlOut.WriteAttributeString("PartInertia", inertia.InertiaGeometry.PartInertia.ToString(CultureInfo.InvariantCulture));                            
                            
                            break;
                        
                        case MainType.Force:
                            var force = iload as LoadForce;
                            xmlOut.WriteAttributeString("Force", force.Force.ToString(CultureInfo.InvariantCulture));
                            break;
                    
                        case MainType.Torque:
                            var torque = iload as LoadTorque;
                            xmlOut.WriteAttributeString("Torque", torque.Torque.ToString(CultureInfo.InvariantCulture));
                            break;
                    }

                    xmlOut.WriteEndElement();
                }

                xmlOut.WriteEndElement();

                //Motor
                xmlOut.WriteStartElement("Motor");

                xmlOut.WriteAttributeString("AutoSelect", motor.AutoSelect.ToString());
                xmlOut.WriteAttributeString("ArticleNo", motor.ArticleNo.ToString());
                xmlOut.WriteAttributeString("Brand", motor.Brand.ToString());
                xmlOut.WriteAttributeString("Description", motor.Description.ToString());
                xmlOut.WriteAttributeString("Price", motor.Price.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("Campaign", motor.Campaign.ToString(CultureInfo.InvariantCulture));

                //Main data
                xmlOut.WriteAttributeString("InertiaRatio", motor.InertiaRatio.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("Inertia", motor.Inertia.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("Eff", motor.Eff.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("Resolution", motor.Resolution.ToString(CultureInfo.InvariantCulture));

                //Mechanical output
                xmlOut.WriteAttributeString("OutType", motor.OutType.ToString());

                xmlOut.WriteAttributeString("OutMinDia", motor.OutMinDia.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("OutMaxDia", motor.OutMaxDia.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("OutRadFrc", motor.OutRadFrc.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("OutAxFrc", motor.OutAxFrc.ToString(CultureInfo.InvariantCulture));

                //Torque data
                xmlOut.WriteAttributeString("MaxTrq", motor.MaxTrq.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("MaxTrqDur", motor.MaxTrqDur.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("TrqED", motor.TrqED.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("NomTrq", motor.NomTrq.ToString(CultureInfo.InvariantCulture));

                //Speed data
                xmlOut.WriteAttributeString("MaxSpd", motor.MaxSpd.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("MaxSpdDur", motor.MaxSpdDur.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("SpdED", motor.SpdED.ToString(CultureInfo.InvariantCulture));
                xmlOut.WriteAttributeString("NomSpd", motor.NomSpd.ToString(CultureInfo.InvariantCulture));

                xmlOut.WriteEndElement();

                //iAccessories
                xmlOut.WriteStartElement("Accessories");
                xmlOut.WriteAttributeString("Count", iAccessories.Count.ToString());

                foreach (var acc in iAccessories)
                {
                    IAccessory iAcc = acc.Value;

                    //Create the file contents based on the load type
                    if (iAcc.ArticleNo != "" && iAcc.ArticleNo != null)
                    {
                        xmlOut.WriteStartElement("Accessory");

                        xmlOut.WriteAttributeString("MainType", iAcc.MainType.ToString());
                        xmlOut.WriteAttributeString("ArticleNo", iAcc.ArticleNo.ToString());
                        xmlOut.WriteAttributeString("Description", iAcc.Description.ToString());

                        xmlOut.WriteEndElement();
                    }
                }

                if (!upstream)
                {
                    xmlOut.WriteEndDocument();
                    xmlOut.Close();
                }
                else
                    xmlOut.Flush();
            }

            public static void LoadFromXml(Stream stream, ApplicationData applicationData, Moves moves, SortedList<int, ILoad> iLoads, Motor motor, SortedList<int, IAccessory> iAccessories)//(DockPanel dockPanel, Stream stream, DeserializeDockContent deserializeContent)
            {
                LoadFromXml(stream, applicationData, moves, iLoads, motor, iAccessories, true); //(dockPanel, stream, deserializeContent, true);
            }

            public static void LoadFromXml(Stream stream, ApplicationData applicationData, Moves moves, SortedList<int, ILoad> iLoads, Motor motor, SortedList<int, IAccessory> iAccessories, bool closeStream)//(DockPanel dockPanel, Stream stream, DeserializeDockContent deserializeContent, bool closeStream)
            {
                //if (dockPanel.Contents.Count != 0)
                //throw new InvalidOperationException("Strings.DockPanel_LoadFromXml_AlreadyInitialized");

                XmlTextReader xmlIn = new XmlTextReader(stream);
                xmlIn.WhitespaceHandling = WhitespaceHandling.None;

                //xmlIn.MoveToContent();

                MoveToNextElement(xmlIn);
                MoveToNextElement(xmlIn);
                MoveToNextElement(xmlIn);
                MoveToNextElement(xmlIn);

                while (!xmlIn.Name.Equals("BeijerDriveDesignerVer1.0"))
                {
                    if (!MoveToNextElement(xmlIn))
                        throw new ArgumentException("Strings.DockPanel_LoadFromXml_InvalidXmlFormat");
                }

                //string formatVersion = xmlIn.GetAttribute("FormatVersion");
                //if (!IsFormatVersionValid(formatVersion))
                //    throw new ArgumentException("Strings.DockPanel_LoadFromXml_InvalidFormatVersion");

                //ApplicationData
                MoveToNextElement(xmlIn);
                if (xmlIn.Name != "ApplicationData")
                    throw new ArgumentException("Element 'ApplicationData' not found.");
                //                ContentStruct[] contents = LoadContents(xmlIn);

                applicationData.AppName = Convert.ToString(xmlIn.GetAttribute("AppName"), CultureInfo.InvariantCulture);
                applicationData.Date = Convert.ToString(xmlIn.GetAttribute("Date"), CultureInfo.InvariantCulture);
                applicationData.Customer = Convert.ToString(xmlIn.GetAttribute("Customer"), CultureInfo.InvariantCulture);
                applicationData.YourRef = Convert.ToString(xmlIn.GetAttribute("YourRef"), CultureInfo.InvariantCulture);
                applicationData.OurRef = Convert.ToString(xmlIn.GetAttribute("OurRef"), CultureInfo.InvariantCulture);
                applicationData.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);

                //Moves
                MoveToNextElement(xmlIn);
                if (xmlIn.Name != "Moves")
                    throw new ArgumentException("Element 'Moves' not found.");

                int countOfContents = Convert.ToInt32(xmlIn.GetAttribute("Count"), CultureInfo.InvariantCulture);

                for (int i = 0; i < countOfContents; i++)
                {
                    MoveToNextElement(xmlIn);

                    string strName = Convert.ToString(xmlIn.GetAttribute("Name"), CultureInfo.InvariantCulture);
                    string strMoveType = Convert.ToString(xmlIn.GetAttribute("MoveType"), CultureInfo.InvariantCulture);
                    string strProfile = Convert.ToString(xmlIn.GetAttribute("Profile"), CultureInfo.InvariantCulture);

                    MoveType moveType = MoveType.Undefined;

                    switch (strMoveType)
                    {
                        case ("PreDefined"):
                            moveType = MoveType.PreDefined;
                            break;

                        case ("UserDefined"):
                            moveType = MoveType.UserDefined;
                            break;
                    }

                    Profile profile = Profile.Undefined;

                    switch (strProfile)
                    {
                        case ("Triangular"):
                            profile = Profile.Triangular;
                            break;

                        case ("Symmetric"):
                            profile = Profile.Symmetric;
                            break;

                        case ("Asymmetric"):
                            profile = Profile.Asymmetric;
                            break;
                    }

                    //Add a new move to the moves list
                    Move tempMove = new Move(strName, moveType, profile);

                    tempMove.Time = Convert.ToDouble(xmlIn.GetAttribute("Time"), CultureInfo.InvariantCulture);
                    tempMove.Degrees = Convert.ToDouble(xmlIn.GetAttribute("Degrees"), CultureInfo.InvariantCulture);
                    tempMove.Distance = Convert.ToDouble(xmlIn.GetAttribute("Distance"), CultureInfo.InvariantCulture);
                    tempMove.InitialSpeed = Convert.ToDouble(xmlIn.GetAttribute("InitialSpeed"), CultureInfo.InvariantCulture);
                    tempMove.FinalSpeed = Convert.ToDouble(xmlIn.GetAttribute("FinalSpeed"), CultureInfo.InvariantCulture);
                    tempMove.AverageAcc = Convert.ToDouble(xmlIn.GetAttribute("AverageAcc"), CultureInfo.InvariantCulture);
                    tempMove.Ramp = Convert.ToDouble(xmlIn.GetAttribute("Ramp"), CultureInfo.InvariantCulture);
                    tempMove.PercentSineVel = Convert.ToDouble(xmlIn.GetAttribute("PercentSineVel"), CultureInfo.InvariantCulture);
                    tempMove.PercentSineAcc = Convert.ToDouble(xmlIn.GetAttribute("PercentSineAcc"), CultureInfo.InvariantCulture);

                    tempMove.DistanceInDegrees = Convert.ToBoolean(xmlIn.GetAttribute("DistanceInDegrees"), CultureInfo.InvariantCulture);

                    tempMove.TimeLocked = Convert.ToBoolean(xmlIn.GetAttribute("TimeLocked"), CultureInfo.InvariantCulture);
                    tempMove.DistanceLocked = Convert.ToBoolean(xmlIn.GetAttribute("DistanceLocked"), CultureInfo.InvariantCulture);
                    tempMove.FinalSpeedLocked = Convert.ToBoolean(xmlIn.GetAttribute("FinalSpeedLocked"), CultureInfo.InvariantCulture);
                    tempMove.AverageAccLocked = Convert.ToBoolean(xmlIn.GetAttribute("AverageAccLocked"), CultureInfo.InvariantCulture);

                    //ActiveLoads
                    MoveToNextElement(xmlIn);
                    if (xmlIn.Name != "ActiveLoads")
                        throw new ArgumentException("Element 'ActiveLoads' not found.");

                    int countOfActiveLoads = Convert.ToInt32(xmlIn.GetAttribute("LoadCount"), CultureInfo.InvariantCulture);

                    for (int j = 0; j < countOfActiveLoads; j++)
                    {
                        MoveToNextElement(xmlIn);

                        string loadName = Convert.ToString(xmlIn.GetAttribute("LoadName"), CultureInfo.InvariantCulture);
                        string loadMainType = Convert.ToString(xmlIn.GetAttribute("MainType"), CultureInfo.InvariantCulture);
                        bool autoSelected = Convert.ToBoolean(xmlIn.GetAttribute("AutoSelect"), CultureInfo.InvariantCulture);
                        MainType mt = MainType.Undefined;

                        switch (loadMainType)
                        {
                            case "BallScrew":
                                mt = MainType.BallScrew;
                                break;

                            case "ToothedBelt":
                                mt = MainType.ToothedBelt;
                                break;

                            case "GearBox":
                                mt = MainType.GearBox;
                                break;

                            case "Coupling":
                                mt = MainType.Coupling;
                                break;

                            case "Inertia":
                                mt = MainType.Inertia;
                                break;

                            case "Mass":
                                mt = MainType.Mass;
                                break;
                        }

                        //Add a new loadResult to the move
                        LoadResult loadResult = new LoadResult(loadName, mt, autoSelected);
                        loadResult.Active = Convert.ToBoolean(xmlIn.GetAttribute("Active"), CultureInfo.InvariantCulture);

                        tempMove.LoadResultsList.Add(j, loadResult);
                    }

                    moves.MovesList.Add(i, tempMove);
                }

                //Loads
                MoveToNextElement(xmlIn);
                if (xmlIn.Name != "Loads")
                    throw new ArgumentException("Element 'Moves' not found.");

                countOfContents = Convert.ToInt32(xmlIn.GetAttribute("Count"), CultureInfo.InvariantCulture);

                for (int i = 0; i < countOfContents; i++)
                {
                    MoveToNextElement(xmlIn);

                    string strName = Convert.ToString(xmlIn.GetAttribute("Name"), CultureInfo.InvariantCulture);
                    string strMainType = Convert.ToString(xmlIn.GetAttribute("MainType"), CultureInfo.InvariantCulture);

                    switch (strMainType)
                    {
                        case "BallScrew":
                            //Add a new BallScrew to the list
                            BallScrew ballScrew = new BallScrew(strName);

                            ballScrew.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                            string strBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                            ballScrew.Brand = strBrand;

                            ballScrew.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                            ballScrew.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);

                            //Main data
                            ballScrew.MovePerRev = Convert.ToDouble(xmlIn.GetAttribute("MovePerRev"), CultureInfo.InvariantCulture);
                            ballScrew.IdleTrq = Convert.ToDouble(xmlIn.GetAttribute("IdleTrq"), CultureInfo.InvariantCulture);
                            ballScrew.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                            ballScrew.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                            ballScrew.Backlash = Convert.ToDouble(xmlIn.GetAttribute("Backlash"), CultureInfo.InvariantCulture);
                            ballScrew.Angle = Convert.ToDouble(xmlIn.GetAttribute("Angle"), CultureInfo.InvariantCulture);

                            //Mechanical part data
                            ballScrew.BallScrewGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            ballScrew.BallScrewGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("CylinderHeight"), CultureInfo.InvariantCulture);
                            ballScrew.BallScrewGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("OuterDiameter"), CultureInfo.InvariantCulture);
                            ballScrew.BallScrewGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("InnerDiameter"), CultureInfo.InvariantCulture);

                            string material = Convert.ToString(xmlIn.GetAttribute("Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    ballScrew.BallScrewGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    ballScrew.BallScrewGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    ballScrew.BallScrewGeometry.Material = Material.Plastic;
                                    break;
                            }

                            ballScrew.BallScrewGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("PartMass"), CultureInfo.InvariantCulture);
                            ballScrew.BallScrewGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("PartInertia"), CultureInfo.InvariantCulture);

                            //Mechanical input
                            string strInpType = Convert.ToString(xmlIn.GetAttribute("InpType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    ballScrew.InpType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    ballScrew.InpType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    ballScrew.InpType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    ballScrew.InpType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    ballScrew.InpType = MechInterface.Flange;
                                    break;

                                default:
                                    ballScrew.InpType = MechInterface.Undefined;
                                    break;
                            }

                            ballScrew.InpMinDia = Convert.ToDouble(xmlIn.GetAttribute("InpMinDia"), CultureInfo.InvariantCulture);
                            ballScrew.InpMaxDia = Convert.ToDouble(xmlIn.GetAttribute("InpMaxDia"), CultureInfo.InvariantCulture);
                            ballScrew.InpRadFrc = Convert.ToDouble(xmlIn.GetAttribute("InpRadFrc"), CultureInfo.InvariantCulture);
                            ballScrew.InpAxFrc = Convert.ToDouble(xmlIn.GetAttribute("InpAxFrc"), CultureInfo.InvariantCulture);

                            //Torque data
                            ballScrew.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                            ballScrew.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                            ballScrew.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                            ballScrew.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                            //Speed data
                            ballScrew.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                            ballScrew.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                            ballScrew.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                            ballScrew.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                            var iLoadBallScrew = ballScrew as ILoad;
                            iLoads.Add(i, iLoadBallScrew);
                            break;

                        case "TurnTable":
                            //Add a new BallScrew to the list
                            TurnTable turnTable = new TurnTable(strName);

                            turnTable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                            strBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                            turnTable.Brand = strBrand;

                            turnTable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                            turnTable.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);

                            //Main data
                            turnTable.MovePerRev = Convert.ToDouble(xmlIn.GetAttribute("MovePerRev"), CultureInfo.InvariantCulture);
                            turnTable.IdleTrq = Convert.ToDouble(xmlIn.GetAttribute("IdleTrq"), CultureInfo.InvariantCulture);
                            turnTable.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                            turnTable.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                            turnTable.Backlash = Convert.ToDouble(xmlIn.GetAttribute("Backlash"), CultureInfo.InvariantCulture);
                            turnTable.Angle = Convert.ToDouble(xmlIn.GetAttribute("Angle"), CultureInfo.InvariantCulture);

                            //Mechanical part data
                            turnTable.TurnTableGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            turnTable.TurnTableGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("CylinderHeight"), CultureInfo.InvariantCulture);
                            turnTable.TurnTableGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("OuterDiameter"), CultureInfo.InvariantCulture);
                            turnTable.TurnTableGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("InnerDiameter"), CultureInfo.InvariantCulture);

                            material = Convert.ToString(xmlIn.GetAttribute("Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    turnTable.TurnTableGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    turnTable.TurnTableGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    turnTable.TurnTableGeometry.Material = Material.Plastic;
                                    break;
                            }

                            turnTable.TurnTableGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("PartMass"), CultureInfo.InvariantCulture);
                            turnTable.TurnTableGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("PartInertia"), CultureInfo.InvariantCulture);

                            //Mechanical input
                            strInpType = Convert.ToString(xmlIn.GetAttribute("InpType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    turnTable.InpType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    turnTable.InpType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    turnTable.InpType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    turnTable.InpType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    turnTable.InpType = MechInterface.Flange;
                                    break;

                                default:
                                    turnTable.InpType = MechInterface.Undefined;
                                    break;
                            }

                            turnTable.InpMinDia = Convert.ToDouble(xmlIn.GetAttribute("InpMinDia"), CultureInfo.InvariantCulture);
                            turnTable.InpMaxDia = Convert.ToDouble(xmlIn.GetAttribute("InpMaxDia"), CultureInfo.InvariantCulture);
                            turnTable.InpRadFrc = Convert.ToDouble(xmlIn.GetAttribute("InpRadFrc"), CultureInfo.InvariantCulture);
                            turnTable.InpAxFrc = Convert.ToDouble(xmlIn.GetAttribute("InpAxFrc"), CultureInfo.InvariantCulture);

                            //Torque data
                            turnTable.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                            turnTable.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                            turnTable.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                            turnTable.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                            //Speed data
                            turnTable.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                            turnTable.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                            turnTable.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                            turnTable.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                            var iLoadTurnTable = turnTable as ILoad;
                            iLoads.Add(i, iLoadTurnTable);
                            break;

                        case "RackAndPinion":
                            //Add a new BallScrew to the list
                            RackAndPinion rackAndPinion = new RackAndPinion(strName);

                            rackAndPinion.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                            strBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                            rackAndPinion.Brand = strBrand;

                            rackAndPinion.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                            rackAndPinion.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);

                            //Main data
                            rackAndPinion.MovePerRev = Convert.ToDouble(xmlIn.GetAttribute("MovePerRev"), CultureInfo.InvariantCulture);
                            rackAndPinion.IdleTrq = Convert.ToDouble(xmlIn.GetAttribute("IdleTrq"), CultureInfo.InvariantCulture);
                            rackAndPinion.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                            rackAndPinion.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                            rackAndPinion.Backlash = Convert.ToDouble(xmlIn.GetAttribute("Backlash"), CultureInfo.InvariantCulture);
                            rackAndPinion.Angle = Convert.ToDouble(xmlIn.GetAttribute("Angle"), CultureInfo.InvariantCulture);

                            //Mechanical part data
                            rackAndPinion.RackAndPinionGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            rackAndPinion.RackAndPinionGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("CylinderHeight"), CultureInfo.InvariantCulture);
                            rackAndPinion.RackAndPinionGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("OuterDiameter"), CultureInfo.InvariantCulture);
                            rackAndPinion.RackAndPinionGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("InnerDiameter"), CultureInfo.InvariantCulture);

                            material = Convert.ToString(xmlIn.GetAttribute("Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    rackAndPinion.RackAndPinionGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    rackAndPinion.RackAndPinionGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    rackAndPinion.RackAndPinionGeometry.Material = Material.Plastic;
                                    break;
                            }

                            rackAndPinion.RackAndPinionGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("PartMass"), CultureInfo.InvariantCulture);
                            rackAndPinion.RackAndPinionGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("PartInertia"), CultureInfo.InvariantCulture);

                            //Mechanical input
                            strInpType = Convert.ToString(xmlIn.GetAttribute("InpType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    rackAndPinion.InpType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    rackAndPinion.InpType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    rackAndPinion.InpType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    rackAndPinion.InpType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    rackAndPinion.InpType = MechInterface.Flange;
                                    break;

                                default:
                                    rackAndPinion.InpType = MechInterface.Undefined;
                                    break;
                            }

                            rackAndPinion.InpMinDia = Convert.ToDouble(xmlIn.GetAttribute("InpMinDia"), CultureInfo.InvariantCulture);
                            rackAndPinion.InpMaxDia = Convert.ToDouble(xmlIn.GetAttribute("InpMaxDia"), CultureInfo.InvariantCulture);
                            rackAndPinion.InpRadFrc = Convert.ToDouble(xmlIn.GetAttribute("InpRadFrc"), CultureInfo.InvariantCulture);
                            rackAndPinion.InpAxFrc = Convert.ToDouble(xmlIn.GetAttribute("InpAxFrc"), CultureInfo.InvariantCulture);

                            //Torque data
                            rackAndPinion.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                            rackAndPinion.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                            rackAndPinion.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                            rackAndPinion.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                            //Speed data
                            rackAndPinion.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                            rackAndPinion.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                            rackAndPinion.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                            rackAndPinion.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                            var iLoadRackAndPinion = rackAndPinion as ILoad;
                            iLoads.Add(i, iLoadRackAndPinion);
                            break;

                        case "ToothedBelt":
                            //Add a new ToothedBelt to the list
                            ToothedBelt toothedBelt = new ToothedBelt(strName);

                            toothedBelt.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                            strBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                            toothedBelt.Brand = strBrand;

                            toothedBelt.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                            toothedBelt.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);

                            //Main data
                            toothedBelt.MovePerRev = Convert.ToDouble(xmlIn.GetAttribute("MovePerRev"), CultureInfo.InvariantCulture);
                            toothedBelt.IdleTrq = Convert.ToDouble(xmlIn.GetAttribute("IdleTrq"), CultureInfo.InvariantCulture);
                            toothedBelt.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                            toothedBelt.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                            toothedBelt.Backlash = Convert.ToDouble(xmlIn.GetAttribute("Backlash"), CultureInfo.InvariantCulture);
                            toothedBelt.Angle = Convert.ToDouble(xmlIn.GetAttribute("Angle"), CultureInfo.InvariantCulture);

                            //Mechanical part data                                         

                            //DriveWheel
                            toothedBelt.DriveWheelGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("DriveWheel_ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            toothedBelt.DriveWheelGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("DriveWheel_CylinderHeight"), CultureInfo.InvariantCulture);
                            toothedBelt.DriveWheelGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("DriveWheel_OuterDiameter"), CultureInfo.InvariantCulture);
                            toothedBelt.DriveWheelGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("DriveWheel_InnerDiameter"), CultureInfo.InvariantCulture);

                            material = Convert.ToString(xmlIn.GetAttribute("DriveWheel_Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    toothedBelt.DriveWheelGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    toothedBelt.DriveWheelGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    toothedBelt.DriveWheelGeometry.Material = Material.Plastic;
                                    break;
                            }

                            toothedBelt.DriveWheelGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("DriveWheel_PartMass"), CultureInfo.InvariantCulture);
                            toothedBelt.DriveWheelGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("DriveWheel_PartInertia"), CultureInfo.InvariantCulture);

                            //LoadWheel
                            toothedBelt.LoadWheelGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("LoadWheel_ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            toothedBelt.LoadWheelGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("LoadWheel_CylinderHeight"), CultureInfo.InvariantCulture);
                            toothedBelt.LoadWheelGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("LoadWheel_OuterDiameter"), CultureInfo.InvariantCulture);
                            toothedBelt.LoadWheelGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("LoadWheel_InnerDiameter"), CultureInfo.InvariantCulture);

                            material = Convert.ToString(xmlIn.GetAttribute("LoadWheel_Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    toothedBelt.LoadWheelGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    toothedBelt.LoadWheelGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    toothedBelt.LoadWheelGeometry.Material = Material.Plastic;
                                    break;
                            }

                            toothedBelt.LoadWheelGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("LoadWheel_PartMass"), CultureInfo.InvariantCulture);
                            toothedBelt.LoadWheelGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("LoadWheel_PartInertia"), CultureInfo.InvariantCulture);

                            //Mechanical input
                            strInpType = Convert.ToString(xmlIn.GetAttribute("InpType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    toothedBelt.InpType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    toothedBelt.InpType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    toothedBelt.InpType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    toothedBelt.InpType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    toothedBelt.InpType = MechInterface.Flange;
                                    break;

                                default:
                                    toothedBelt.InpType = MechInterface.Undefined;
                                    break;
                            }

                            toothedBelt.InpMinDia = Convert.ToDouble(xmlIn.GetAttribute("InpMinDia"), CultureInfo.InvariantCulture);
                            toothedBelt.InpMaxDia = Convert.ToDouble(xmlIn.GetAttribute("InpMaxDia"), CultureInfo.InvariantCulture);
                            toothedBelt.InpRadFrc = Convert.ToDouble(xmlIn.GetAttribute("InpRadFrc"), CultureInfo.InvariantCulture);
                            toothedBelt.InpAxFrc = Convert.ToDouble(xmlIn.GetAttribute("InpAxFrc"), CultureInfo.InvariantCulture);

                            //Torque data
                            toothedBelt.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                            toothedBelt.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                            toothedBelt.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                            toothedBelt.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                            //Speed data
                            toothedBelt.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                            toothedBelt.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                            toothedBelt.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                            toothedBelt.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                            var iLoadToothedBelt = toothedBelt as ILoad;
                            iLoads.Add(i, iLoadToothedBelt);
                            break;

                        case "Coupling":
                            //Add a new coupling to the list
                            Coupling coupling = new Coupling(strName);

                            coupling.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                            strBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                            switch (strBrand)
                            {
                                case ("Aluflex"):
                                    coupling.Brand = CouplingBrand.AluFlex;
                                    break;

                                default:
                                    coupling.Brand = CouplingBrand.Undefined;
                                    break;
                            }

                            coupling.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                            coupling.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);

                            //Main data
                            coupling.IdleTrq = Convert.ToDouble(xmlIn.GetAttribute("IdleTrq"), CultureInfo.InvariantCulture);
                            coupling.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                            coupling.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                            coupling.Backlash = Convert.ToDouble(xmlIn.GetAttribute("Backlash"), CultureInfo.InvariantCulture);

                            //LoadWheel
                            coupling.CouplingGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            coupling.CouplingGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("CylinderHeight"), CultureInfo.InvariantCulture);
                            coupling.CouplingGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("OuterDiameter"), CultureInfo.InvariantCulture);
                            coupling.CouplingGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("InnerDiameter"), CultureInfo.InvariantCulture);

                            material = Convert.ToString(xmlIn.GetAttribute("Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    coupling.CouplingGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    coupling.CouplingGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    coupling.CouplingGeometry.Material = Material.Plastic;
                                    break;
                            }

                            coupling.CouplingGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("PartMass"), CultureInfo.InvariantCulture);
                            coupling.CouplingGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("PartInertia"), CultureInfo.InvariantCulture);

                            //Mechanical input
                            strInpType = Convert.ToString(xmlIn.GetAttribute("InpType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    coupling.InpType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    coupling.InpType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    coupling.InpType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    coupling.InpType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    coupling.InpType = MechInterface.Flange;
                                    break;

                                default:
                                    coupling.InpType = MechInterface.Undefined;
                                    break;
                            }

                            coupling.InpMinDia = Convert.ToDouble(xmlIn.GetAttribute("InpMinDia"), CultureInfo.InvariantCulture);
                            coupling.InpMaxDia = Convert.ToDouble(xmlIn.GetAttribute("InpMaxDia"), CultureInfo.InvariantCulture);
                            coupling.InpRadFrc = Convert.ToDouble(xmlIn.GetAttribute("InpRadFrc"), CultureInfo.InvariantCulture);
                            coupling.InpAxFrc = Convert.ToDouble(xmlIn.GetAttribute("InpAxFrc"), CultureInfo.InvariantCulture);

                            //Mechanical output
                            strInpType = Convert.ToString(xmlIn.GetAttribute("OutType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    coupling.OutType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    coupling.OutType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    coupling.OutType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    coupling.OutType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    coupling.OutType = MechInterface.Flange;
                                    break;

                                default:
                                    coupling.OutType = MechInterface.Undefined;
                                    break;
                            }

                            coupling.OutMinDia = Convert.ToDouble(xmlIn.GetAttribute("OutMinDia"), CultureInfo.InvariantCulture);
                            coupling.OutMaxDia = Convert.ToDouble(xmlIn.GetAttribute("OutMaxDia"), CultureInfo.InvariantCulture);
                            coupling.OutRadFrc = Convert.ToDouble(xmlIn.GetAttribute("OutRadFrc"), CultureInfo.InvariantCulture);
                            coupling.OutAxFrc = Convert.ToDouble(xmlIn.GetAttribute("OutAxFrc"), CultureInfo.InvariantCulture);

                            //Torque data
                            coupling.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                            coupling.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                            coupling.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                            coupling.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                            //Speed data
                            coupling.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                            coupling.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                            coupling.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                            coupling.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                            var iLoadCoupling = coupling as ILoad;
                            iLoads.Add(i, iLoadCoupling);
                            break;

                        case "GearBox":
                            //Add a new gearbox to the list
                            GearBox gearBox = new GearBox(strName);

                            gearBox.AutoSelect = Convert.ToBoolean(xmlIn.GetAttribute("AutoSelect"), CultureInfo.InvariantCulture);

                            gearBox.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                            strBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                            switch (strBrand)
                            {
                                case ("Wittenstein"):
                                    gearBox.Brand = GearBoxBrand.Wittenstein;
                                    break;

                                case ("LSIS"):
                                    gearBox.Brand = GearBoxBrand.LSIS;
                                    break;

                                default:
                                    gearBox.Brand = GearBoxBrand.Undefined;
                                    break;
                            }

                            gearBox.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                            gearBox.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);
                            gearBox.Campaign = Convert.ToBoolean(xmlIn.GetAttribute("Campaign"), CultureInfo.InvariantCulture);

                            //Main data
                            gearBox.Gearing = Convert.ToDouble(xmlIn.GetAttribute("Gearing"), CultureInfo.InvariantCulture);
                            gearBox.IdleTrq = Convert.ToDouble(xmlIn.GetAttribute("IdleTrq"), CultureInfo.InvariantCulture);
                            gearBox.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                            gearBox.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                            gearBox.Backlash = Convert.ToDouble(xmlIn.GetAttribute("Backlash"), CultureInfo.InvariantCulture);

                            //Mechanical input
                            strInpType = Convert.ToString(xmlIn.GetAttribute("InpType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    gearBox.InpType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    gearBox.InpType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    gearBox.InpType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    gearBox.InpType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    gearBox.InpType = MechInterface.Flange;
                                    break;

                                default:
                                    gearBox.InpType = MechInterface.Undefined;
                                    break;
                            }

                            gearBox.InpMinDia = Convert.ToDouble(xmlIn.GetAttribute("InpMinDia"), CultureInfo.InvariantCulture);
                            gearBox.InpMaxDia = Convert.ToDouble(xmlIn.GetAttribute("InpMaxDia"), CultureInfo.InvariantCulture);
                            gearBox.InpRadFrc = Convert.ToDouble(xmlIn.GetAttribute("InpRadFrc"), CultureInfo.InvariantCulture);
                            gearBox.InpAxFrc = Convert.ToDouble(xmlIn.GetAttribute("InpAxFrc"), CultureInfo.InvariantCulture);

                            //Mechanical output
                            strInpType = Convert.ToString(xmlIn.GetAttribute("OutType"), CultureInfo.InvariantCulture);

                            switch (strInpType)
                            {
                                case ("SmoothShaft"):
                                    gearBox.OutType = MechInterface.SmoothShaft;
                                    break;

                                case ("KeyedShaft"):
                                    gearBox.OutType = MechInterface.KeyedShaft;
                                    break;

                                case ("SmoothHollow"):
                                    gearBox.OutType = MechInterface.SmoothHollow;
                                    break;

                                case ("KeyedHollow"):
                                    gearBox.OutType = MechInterface.KeyedHollow;
                                    break;

                                case ("Flange"):
                                    gearBox.OutType = MechInterface.Flange;
                                    break;

                                default:
                                    gearBox.OutType = MechInterface.Undefined;
                                    break;
                            }

                            gearBox.OutMinDia = Convert.ToDouble(xmlIn.GetAttribute("OutMinDia"), CultureInfo.InvariantCulture);
                            gearBox.OutMaxDia = Convert.ToDouble(xmlIn.GetAttribute("OutMaxDia"), CultureInfo.InvariantCulture);
                            gearBox.OutRadFrc = Convert.ToDouble(xmlIn.GetAttribute("OutRadFrc"), CultureInfo.InvariantCulture);
                            gearBox.OutAxFrc = Convert.ToDouble(xmlIn.GetAttribute("OutAxFrc"), CultureInfo.InvariantCulture);

                            //Torque data
                            gearBox.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                            gearBox.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                            gearBox.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                            gearBox.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                            //Speed data
                            gearBox.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                            gearBox.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                            gearBox.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                            gearBox.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                            var iLoadGearBox = gearBox as ILoad;
                            iLoads.Add(i, iLoadGearBox);
                            break;

                        case "Mass":
                            //Add a new mass to the list
                            LoadMass mass = new LoadMass(strName);
                            var iLoadMass = mass as ILoad;

                            mass.Mass = Convert.ToDouble(xmlIn.GetAttribute("Mass"), CultureInfo.InvariantCulture);
                            iLoads.Add(i, iLoadMass);
                            break;

                        case "Inertia":
                            LoadInertia inertia = new LoadInertia(strName);
                            var iLoadInertia = inertia as ILoad;

                            inertia.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);

                            //Mech data
                            inertia.InertiaGeometry.ManualInertiaCalc = Convert.ToBoolean(xmlIn.GetAttribute("ManualInertiaCalc"), CultureInfo.InvariantCulture);
                            inertia.InertiaGeometry.CylinderHeight = Convert.ToDouble(xmlIn.GetAttribute("CylinderHeight"), CultureInfo.InvariantCulture);
                            inertia.InertiaGeometry.OuterDiameter = Convert.ToDouble(xmlIn.GetAttribute("OuterDiameter"), CultureInfo.InvariantCulture);
                            inertia.InertiaGeometry.InnerDiameter = Convert.ToDouble(xmlIn.GetAttribute("InnerDiameter"), CultureInfo.InvariantCulture);

                            material = Convert.ToString(xmlIn.GetAttribute("Material"), CultureInfo.InvariantCulture);

                            switch (material)
                            {
                                case "Steel":
                                    inertia.InertiaGeometry.Material = Material.Steel;
                                    break;

                                case "Aluminum":
                                    inertia.InertiaGeometry.Material = Material.Aluminum;
                                    break;

                                case "Plastic":
                                    inertia.InertiaGeometry.Material = Material.Plastic;
                                    break;
                            }

                            inertia.InertiaGeometry.PartMass = Convert.ToDouble(xmlIn.GetAttribute("PartMass"), CultureInfo.InvariantCulture);
                            inertia.InertiaGeometry.PartInertia = Convert.ToDouble(xmlIn.GetAttribute("PartInertia"), CultureInfo.InvariantCulture);

                            iLoads.Add(i, iLoadInertia);
                            break;

                        case "Force":
                            LoadForce force = new LoadForce(strName);
                            force.Force = Convert.ToDouble(xmlIn.GetAttribute("Force"), CultureInfo.InvariantCulture);
                            var iLoadForce = force as ILoad;
                            iLoads.Add(i, iLoadForce);
                            break;

                        case "Torque":
                            LoadTorque torque = new LoadTorque(strName);
                            torque.Torque = Convert.ToDouble(xmlIn.GetAttribute("Torque"), CultureInfo.InvariantCulture);
                            var iLoadTorque = torque as ILoad;
                            iLoads.Add(i, iLoadTorque);
                            break;
                    }
                }

                //Motor
                MoveToNextElement(xmlIn);
                if (xmlIn.Name != "Motor")
                    throw new ArgumentException("Element 'ApplicationData' not found.");
                //                ContentStruct[] contents = LoadContents(xmlIn);

                motor.Name = "Motor";

                motor.AutoSelect = Convert.ToBoolean(xmlIn.GetAttribute("AutoSelect"), CultureInfo.InvariantCulture);
                motor.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                string strMotorBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                motor.Brand = Brand.BSD;

                motor.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                motor.Price = Convert.ToDouble(xmlIn.GetAttribute("Price"), CultureInfo.InvariantCulture);
                motor.Campaign = Convert.ToBoolean(xmlIn.GetAttribute("Campaign"), CultureInfo.InvariantCulture);

                //Main data
                motor.InertiaRatio = Convert.ToDouble(xmlIn.GetAttribute("InertiaRatio"), CultureInfo.InvariantCulture);
                motor.Inertia = Convert.ToDouble(xmlIn.GetAttribute("Inertia"), CultureInfo.InvariantCulture);
                motor.Eff = Convert.ToDouble(xmlIn.GetAttribute("Eff"), CultureInfo.InvariantCulture);
                motor.Resolution = Convert.ToDouble(xmlIn.GetAttribute("Resolution"), CultureInfo.InvariantCulture);

                //Mechanical output
                string strMotorOutType = Convert.ToString(xmlIn.GetAttribute("OutType"), CultureInfo.InvariantCulture);

                switch (strMotorOutType)
                {
                    case ("SmoothShaft"):
                        motor.OutType = MechInterface.SmoothShaft;
                        break;

                    case ("KeyedShaft"):
                        motor.OutType = MechInterface.KeyedShaft;
                        break;

                    case ("SmoothHollow"):
                        motor.OutType = MechInterface.SmoothHollow;
                        break;

                    case ("KeyedHollow"):
                        motor.OutType = MechInterface.KeyedHollow;
                        break;

                    case ("Flange"):
                        motor.OutType = MechInterface.Flange;
                        break;

                    default:
                        motor.OutType = MechInterface.Undefined;
                        break;
                }

                motor.OutMinDia = Convert.ToDouble(xmlIn.GetAttribute("OutMinDia"), CultureInfo.InvariantCulture);
                motor.OutMaxDia = Convert.ToDouble(xmlIn.GetAttribute("OutMaxDia"), CultureInfo.InvariantCulture);
                motor.OutRadFrc = Convert.ToDouble(xmlIn.GetAttribute("OutRadFrc"), CultureInfo.InvariantCulture);
                motor.OutAxFrc = Convert.ToDouble(xmlIn.GetAttribute("OutAxFrc"), CultureInfo.InvariantCulture);

                //Torque data
                motor.MaxTrq = Convert.ToDouble(xmlIn.GetAttribute("MaxTrq"), CultureInfo.InvariantCulture);
                motor.MaxTrqDur = Convert.ToDouble(xmlIn.GetAttribute("MaxTrqDur"), CultureInfo.InvariantCulture);
                motor.TrqED = Convert.ToDouble(xmlIn.GetAttribute("TrqED"), CultureInfo.InvariantCulture);
                motor.NomTrq = Convert.ToDouble(xmlIn.GetAttribute("NomTrq"), CultureInfo.InvariantCulture);

                //Speed data
                motor.MaxSpd = Convert.ToDouble(xmlIn.GetAttribute("MaxSpd"), CultureInfo.InvariantCulture);
                motor.MaxSpdDur = Convert.ToDouble(xmlIn.GetAttribute("MaxSpdDur"), CultureInfo.InvariantCulture);
                motor.SpdED = Convert.ToDouble(xmlIn.GetAttribute("SpdED"), CultureInfo.InvariantCulture);
                motor.NomSpd = Convert.ToDouble(xmlIn.GetAttribute("NomSpd"), CultureInfo.InvariantCulture);

                //Accessories
                MoveToNextElement(xmlIn);
                if (xmlIn.Name == "Accessories")
                {
                    //throw new ArgumentException("Element 'Accessories' not found.");

                    countOfContents = Convert.ToInt32(xmlIn.GetAttribute("Count"), CultureInfo.InvariantCulture);

                    for (int i = 0; i < countOfContents; i++)
                    {
                        MoveToNextElement(xmlIn);

                        //string strName = Convert.ToString(xmlIn.GetAttribute("Name"), CultureInfo.InvariantCulture);
                        string strMainType = Convert.ToString(xmlIn.GetAttribute("MainType"), CultureInfo.InvariantCulture);

                        switch (strMainType)
                        {
                            case "EncoderCable":

                                Cable encoderCable = iAccessories[0] as Cable;

                                encoderCable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                encoderCable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                string strEncoderCableBrand = Convert.ToString(xmlIn.GetAttribute("Brand"), CultureInfo.InvariantCulture);

                                encoderCable.Brand = Brand.BSD;
                                break;

                            case "MotorCable":

                                Cable motorCable = iAccessories[1] as Cable;

                                motorCable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                motorCable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "BrakeCable":

                                Cable brakeCable = iAccessories[2] as Cable;

                                brakeCable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                brakeCable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "Drive":
                                Drive drive = iAccessories[3] as Drive;

                                drive.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                drive.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "BrakeResistor":
                                BrakeResistor brakeResistor = iAccessories[4] as BrakeResistor;

                                brakeResistor.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                brakeResistor.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "EMCFilter":
                                Filter EMCFilter = iAccessories[5] as Filter;

                                EMCFilter.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                EMCFilter.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "DCFilter":
                                Filter DCFilter = iAccessories[6] as Filter;

                                DCFilter.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                DCFilter.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "Battery":
                                Option battery = iAccessories[7] as Option;

                                battery.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                battery.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "Safety":
                                Option safety = iAccessories[8] as Option;

                                safety.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                safety.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "IOCard":
                                Option IOCard = iAccessories[9] as Option;

                                IOCard.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                IOCard.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "IOCable":
                                Option IOCable = iAccessories[10] as Option;

                                IOCable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                IOCable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "BusCable":
                                Option busCable = iAccessories[11] as Option;

                                busCable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                busCable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "Controller":
                                Option controller = iAccessories[12] as Option;

                                controller.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                controller.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "ExtensionCard":
                                Option extensionCard = iAccessories[13] as Option;

                                extensionCard.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                extensionCard.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "ExtensionCable":
                                Option extensionCable = iAccessories[14] as Option;

                                extensionCable.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                extensionCable.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;

                            case "ExtensionTerminal":
                                Option extensionTerminal = iAccessories[15] as Option;

                                extensionTerminal.ArticleNo = Convert.ToString(xmlIn.GetAttribute("ArticleNo"), CultureInfo.InvariantCulture);
                                extensionTerminal.Description = Convert.ToString(xmlIn.GetAttribute("Description"), CultureInfo.InvariantCulture);
                                break;
                        }
                    }
                }

                if (closeStream)
                    xmlIn.Close();
            }

            private static bool MoveToNextElement(XmlTextReader xmlIn)
            {
                if (!xmlIn.Read())
                    return false;

                while (xmlIn.NodeType == XmlNodeType.EndElement)
                {
                    if (!xmlIn.Read())
                        return false;
                }

                return true;
            }

            private static bool IsFormatVersionValid(string formatVersion)
            {
                if (formatVersion == ConfigFileVersion)
                    return true;

                foreach (string s in CompatibleConfigFileVersions)
                    if (s == formatVersion)
                        return true;

                return false;
            }
        }

        public void SaveAsXml(string fileName, ApplicationData applicationData, Moves moves, SortedList<int, ILoad> iLoads, Motor motor, SortedList<int, IAccessory> iAccessories)
        {
            XMLHandler.SaveAsXml(fileName, applicationData, moves, iLoads, motor, iAccessories); //(this, fileName);
        }

        public void LoadFromXml(Stream stream, ApplicationData applicationData, Moves moves, SortedList<int, ILoad> iLoads, Motor motor, SortedList<int, IAccessory> iAccessories) //DeserializeDockContent deserializeContent)
        {
            XMLHandler.LoadFromXml(stream, applicationData, moves, iLoads, motor, iAccessories);//(this, stream, deserializeContent);
        }
     }
}
