﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DockSample
{
    public partial class DlgCreateBallScrew : Form
    {
        private ApplicationData _applicationData;
        private Move _moveToDest;
        private Move _moveReturn;
        private Move _moveWait;
        private BallScrew _ballScrew;
        private LoadMass _mass;
        private LoadForce _force;

        public bool _addReturnMove;
        public bool _addGearBox;
        public bool _addCoupling;      

        public DlgCreateBallScrew()
        {   
            InitializeComponent();
        }
        
        #region Properties

        public ApplicationData ApplicationData
        {
            get { return _applicationData; }
            set { this._applicationData = value; }
        }

        public Move MoveToDest
        {
            get { return _moveToDest; }
            set { this._moveToDest = value; }
        }

        public Move MoveReturn
        {
            get { return _moveReturn; }
            set { this._moveReturn = value; }
        }

        public Move MoveWait
        {
            get { return _moveWait; }
            set { this._moveWait = value; }
        }

        public BallScrew BallScrew
        {
            get { return _ballScrew; }
            set { this._ballScrew = value; }
        }

        public LoadMass Mass
        {
            get { return _mass; }
            set { this._mass = value; }
        }

        public LoadForce Force
        {
            get { return _force; }
            set { this._force = value; }
        }

        #endregion

        #region Events
        
        private void DlgCreateBallScrew_Load(object sender, EventArgs e)
        {
            //Appdata
            ApplicationData = new ApplicationData();
            ApplicationData.Date = ApplicationData.Date = System.DateTime.Today.ToString("yyyy-MM-dd");
            textBox_Date.Text = ApplicationData.Date;

            //Move to destination
            MoveToDest = new Move("MoveToDest", MoveType.PreDefined, Profile.Symmetric);
            textBox_MoveName.Text = MoveToDest.Name;
            radioButton_Symmetric.Checked = true;

            MoveToDest.Time = 0;
            textBox_MoveTime.Text = MoveToDest.Time.ToString();
            textBox_MoveTimeBasic.Text = MoveToDest.Time.ToString();

            MoveToDest.Distance = 0;
            textBox_MoveDistance.Text = MoveToDest.Distance.ToString();
            textBox_MoveDistanceBasic.Text = MoveToDest.Distance.ToString();

            //Return move
            _addReturnMove = true; 
            checkBox_IncludeRevMov.Checked = _addReturnMove; 
            
            MoveReturn = new Move("MoveReturn", MoveType.PreDefined, Profile.Symmetric);
                        
            //Wait time
            MoveWait = new Move("MoveWait", MoveType.UserDefined, Profile.Undefined);

            MoveWait.Time = 1;
            textBox_WaitTime.Text = MoveWait.Time.ToString();

            //Component
            BallScrew = new BallScrew("MyBallScrew");

            textBox_ComponentName.Text = BallScrew.Name;
            
            BallScrew.BallScrewGeometry.ManualInertiaCalc = true;
            BallScrew.BallScrewGeometry.Material = Material.Steel;

            textBox_MovePerRev.Text = BallScrew.MovePerRev.ToString();
            textBox_MovePerRevBasic.Text = BallScrew.MovePerRev.ToString();

            BallScrew.IdleTrq = 0.3;
            textBox_IdleTorque.Text = BallScrew.IdleTrq.ToString();

            BallScrew.Backlash = 10;
            textBox_Backlash.Text = BallScrew.Backlash.ToString();

            BallScrew.BallScrewGeometry.OuterDiameter = 30; //30 mm
            textBox_OuterDiameter.Text = BallScrew.BallScrewGeometry.OuterDiameter.ToString();
            textBox_InnerDiameter.Text = BallScrew.BallScrewGeometry.InnerDiameter.ToString();
            textBox_CylinderHeight.Text = BallScrew.BallScrewGeometry.CylinderHeight.ToString();

            BallScrew.PropertyChanged +=new PropertyChangedEventHandler(BallScrew_PropertyChanged);

            //load the ComboBox_Material with valid items
            comboBox_Material.Items.Add(Material.Steel.ToString());
            comboBox_Material.Items.Add(Material.Aluminum.ToString());
            comboBox_Material.Items.Add(Material.Plastic.ToString());

            comboBox_Material.SelectedItem = comboBox_Material.Items[0]; //Steel
            
            //Load
            Mass = new LoadMass("MyMass");
            textBox_LoadName.Text = Mass.Name;
            textBox_LoadMass.Text = Mass.Mass.ToString();
            textBox_LoadMassBasic.Text = Mass.Mass.ToString();
            
            //Additional data
            radioButton_Horizontal.Checked = true;
            radioButton_HorizontalBasic.Checked = true;
            
            _addCoupling = true;
            checkBox_Coupling.Checked = _addCoupling;

            _addGearBox = true;
            checkBox_UseGearBox.Checked = _addGearBox;

            Force = new LoadForce("MyForce");
            textBox_ForceName.Text = Force.Name;
            textBox_LoadForce.Text = Force.Force.ToString();
            textBox_ForceNameBasic.Text = Force.Name;
            textBox_LoadForceBasic.Text = Force.Force.ToString();
        }

        private void BallScrew_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            BallScrew ballScrew = (BallScrew)sender;

            //Catch events
            switch (e.PropertyName)
            {
                case "Inertia":
                    textBox_Mass.Text = ballScrew.BallScrewGeometry.PartMass.ToString();
                    textBox_Inertia.Text = Math.Round(ballScrew.BallScrewGeometry.PartInertia * 10000, 3).ToString();
                    break;
            }
        }

        //Application data
        private void textBox_Date_Validated(object sender, EventArgs e)
        {
            if (ApplicationData.Date != textBox_Date.Text)
                ApplicationData.Date = textBox_Date.Text;
        }

        private void textBox_Customer_Validated(object sender, EventArgs e)
        {
             ApplicationData.Customer = textBox_Customer.Text;
        }

        private void textBox_OurRef_Validated(object sender, EventArgs e)
        {
            ApplicationData.OurRef = textBox_OurRef.Text;
        }

        private void textBox_Description_Validated(object sender, EventArgs e)
        {
            ApplicationData.Description = textBox_Description.Text;
        }

        private void textBox_AppName_Validated(object sender, EventArgs e)
        {
            ApplicationData.AppName = textBox_AppName.Text;
        }

        private void textBox_YourRef_Validated(object sender, EventArgs e)
        {
            ApplicationData.YourRef = textBox_YourRef.Text;
        }

        //Move to destination
        private void textBox_MoveName_Validated(object sender, EventArgs e)
        {
            MoveToDest.Name = textBox_MoveName.Text;
        }

        private void textBox_MoveTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

            UpdateMoveTime(value);
        }

        private void textBox_MoveTime_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

                UpdateMoveTime(value);
            }
        }

        private void textBox_MoveTimeBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTimeBasic.Text), 3));

            UpdateMoveTime(value);
        }
        
        private void textBox_MoveTimeBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTimeBasic.Text), 3));

                UpdateMoveTime(value);
            }
        }

        private void UpdateMoveTime(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_MoveTime.Text = value.ToString();
            textBox_MoveTimeBasic.Text = value.ToString();

            MoveToDest.Time = value;

            if (value > 0)
            {
                textBox_MoveTime.BackColor = SystemColors.Window;
                textBox_MoveTimeBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_MoveTime.BackColor = Color.Gold;
                textBox_MoveTimeBasic.BackColor = Color.Gold;
            }
        }

        private void textBox_MoveDistance_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

            UpdateDistance(value);
        }
        
        private void textBox_MoveDistance_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

                UpdateDistance(value);
            }
        }

        private void textBox_MoveDistanceBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Round(validate_textBox(textBox_MoveDistanceBasic.Text), 3);

            UpdateDistance(value);
        }

        private void textBox_MoveDistanceBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Round(validate_textBox(textBox_MoveDistanceBasic.Text), 3);

                UpdateDistance(value);
            }
        }

        private void UpdateDistance(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_MoveDistance.Text = value.ToString();
            textBox_MoveDistanceBasic.Text = value.ToString();

            MoveToDest.Distance = value / 1000;

            if (value != 0)
            {
                textBox_MoveDistance.BackColor = SystemColors.Window;
                textBox_MoveDistanceBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_MoveDistance.BackColor = Color.Gold;
                textBox_MoveDistanceBasic.BackColor = Color.Gold;
            }

            BallScrew.BallScrewGeometry.CylinderHeight = MoveToDest.Distance * 1000 + 300;
            textBox_CylinderHeight.Text = BallScrew.BallScrewGeometry.CylinderHeight.ToString();
        }

        private void radioButton_Triangular_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Triangular.Checked)
            {
                MoveToDest.Profile = Profile.Triangular;
                radioButton_Symmetric.Checked = false;
            }
            else
            {
                MoveToDest.Profile = Profile.Symmetric;
                radioButton_Symmetric.Checked = true;
            }
        }

        private void radioButton_Symmetric_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Symmetric.Checked)
            {
                MoveToDest.Profile = Profile.Symmetric;
                radioButton_Triangular.Checked = false;
            }
            else
            {
                MoveToDest.Profile = Profile.Triangular;
                radioButton_Triangular.Checked = true;
            }
        }

        //MoveReturn
        private void checkBox_IncludeRevMov_CheckStateChanged(object sender, EventArgs e)
        {
            if (checkBox_IncludeRevMov.Checked)
                _addReturnMove = true;
            else
                _addReturnMove = false;
        }

        //MoveWait
        private void textBox_WaitTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_WaitTime.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_WaitTime.Text = value.ToString();

            MoveWait.Time = value;
        }

        //BallScrew        
        private void textBox_ComponentName_Validated(object sender, EventArgs e)
        {
            BallScrew.Name = textBox_ComponentName.Text;
        }

        private void textBox_IdleTorque_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_IdleTorque.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_IdleTorque.Text = value.ToString();

            BallScrew.IdleTrq = value;
        }

        private void textBox_MovePerRev_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRev.Text), 3));

            UpdateMovePerRev(value);
        }

        private void textBox_MovePerRev_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRev.Text), 3));

                UpdateMovePerRev(value);
            }
        }

        private void textBox_MovePerRevBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRevBasic.Text), 3));

            UpdateMovePerRev(value);
        }

        private void textBox_MovePerRevBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRevBasic.Text), 3));

                UpdateMovePerRev(value);
            }
        }

        private void UpdateMovePerRev(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_MovePerRev.Text = value.ToString();
            textBox_MovePerRevBasic.Text = value.ToString();

            BallScrew.MovePerRev = value / 1000;

            if (value > 0)
            {
                textBox_MovePerRev.BackColor = SystemColors.Window;
                textBox_MovePerRevBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_MovePerRev.BackColor = Color.Gold;
                textBox_MovePerRevBasic.BackColor = Color.Gold;
            }
        }

        private void textBox_CylinderHeight_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_CylinderHeight.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_CylinderHeight.Text = value.ToString();

            BallScrew.BallScrewGeometry.CylinderHeight = value;
        }

        private void textBox_CylinderHeight_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_CylinderHeight.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_CylinderHeight.Text = value.ToString();

                BallScrew.BallScrewGeometry.CylinderHeight = value;
            }
        }

        private void textBox_OuterDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_OuterDiameter.Text = value.ToString();

            BallScrew.BallScrewGeometry.OuterDiameter = value;
        }

        private void textBox_OuterDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_OuterDiameter.Text = value.ToString();

                BallScrew.BallScrewGeometry.OuterDiameter = value;
            }
        }

        private void textBox_InnerDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_InnerDiameter.Text = value.ToString();

            BallScrew.BallScrewGeometry.InnerDiameter = value;
        }

        private void textBox_InnerDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_InnerDiameter.Text = value.ToString();

                BallScrew.BallScrewGeometry.InnerDiameter = value;
            }
        }

        private void comboBox_Material_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Material.SelectedIndex == 0)
                BallScrew.BallScrewGeometry.Material = Material.Steel;

            if (comboBox_Material.SelectedIndex == 1)
                BallScrew.BallScrewGeometry.Material = Material.Aluminum;

            if (comboBox_Material.SelectedIndex == 2)
                BallScrew.BallScrewGeometry.Material = Material.Plastic;
        }

        private void radioButton_Horisontal_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Horizontal.Checked)
            {
                BallScrew.Angle = 0;
                radioButton_HorizontalBasic.Checked = true;
                radioButton_Vertical.Checked = false;
                radioButton_VerticalBasic.Checked = false;
            }
            else
            {
                BallScrew.Angle = 90;
                radioButton_HorizontalBasic.Checked = false;
                radioButton_Vertical.Checked = true;
                radioButton_VerticalBasic.Checked = true;
            }
        }

        private void radioButton_HorisontalBasic_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_HorizontalBasic.Checked)
            {
                BallScrew.Angle = 0;
                radioButton_Horizontal.Checked = true;
                radioButton_Vertical.Checked = false;
                radioButton_VerticalBasic.Checked = false;

            }
            else
            {
                BallScrew.Angle = 90;
                radioButton_Horizontal.Checked = false;
                radioButton_Vertical.Checked = true;
                radioButton_VerticalBasic.Checked = true;
            }
        }

        private void radioButton_Vertical_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Vertical.Checked)
            {
                BallScrew.Angle = 90;
                radioButton_VerticalBasic.Checked = true;
                radioButton_Horizontal.Checked = false;
                radioButton_HorizontalBasic.Checked = false;
            }
            else
            {
                BallScrew.Angle = 0;
                radioButton_VerticalBasic.Checked = false;
                radioButton_Horizontal.Checked = true;
                radioButton_HorizontalBasic.Checked = true;
            }
        }

        private void radioButton_VerticalBasic_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_VerticalBasic.Checked)
            {
                BallScrew.Angle = 90;
                radioButton_Vertical.Checked = true;
                radioButton_Horizontal.Checked = false;
                radioButton_HorizontalBasic.Checked = false;
            }
            else
            {
                BallScrew.Angle = 0;
                radioButton_Vertical.Checked = false;
                radioButton_Horizontal.Checked = true;
                radioButton_HorizontalBasic.Checked = true;
            }
        }

        //Mass
        private void textBox_LoadName_Validated(object sender, EventArgs e)
        {
            Mass.Name = textBox_LoadName.Text;
        }

        private void textBox_LoadMass_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMass.Text), 3));

            UpdateLoadMass(value);
        }

        private void textBox_LoadMass_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMass.Text), 3));

                UpdateLoadMass(value);
            }
        }

        private void textBox_LoadMassBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMassBasic.Text), 3));

            UpdateLoadMass(value);
        }

        private void textBox_LoadMassBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMassBasic.Text), 3));

                UpdateLoadMass(value);
            }
        }

        private void UpdateLoadMass(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_LoadMass.Text = value.ToString();
            textBox_LoadMassBasic.Text = value.ToString();

            Mass.Mass = value;

            if (value > 0)
            {
                textBox_LoadMass.BackColor = SystemColors.Window;
                textBox_LoadMassBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_LoadMass.BackColor = Color.Gold;
                textBox_LoadMassBasic.BackColor = Color.Gold;
            }
        }

        //Additional
        private void checkBox_UseGearBox_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_UseGearBox.Checked)
                _addGearBox = true;
            else
                _addGearBox = false;
        }

        private void checkBox_Coupling_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_Coupling.Checked)
                _addCoupling = true;
            else
                _addCoupling = false;
        }

        private void textBox_LoadForce_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadForce.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_LoadForce.Text = value.ToString();
            textBox_LoadForceBasic.Text = value.ToString();

            Force.Force = value;
        }

        private void textBox_ForceName_Validating(object sender, CancelEventArgs e)
        {
            textBox_ForceNameBasic.Text = textBox_ForceName.Text;
            Force.Name = textBox_ForceName.Text;
        }

        private void textBox_LoadForceBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadForceBasic.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_LoadForce.Text = value.ToString();
            textBox_LoadForceBasic.Text = value.ToString();

            Force.Force = value;
        }

        private void textBox_ForceNameBasic_Validating(object sender, CancelEventArgs e)
        {
            textBox_ForceName.Text = textBox_ForceNameBasic.Text;
            Force.Name = textBox_ForceNameBasic.Text;
        }

        //Create return data
        private void button_Create_Click(object sender, EventArgs e)
        {
            bool invalidData = false;

            if (MoveToDest.Time <= 0)
            {
                textBox_MoveTime.BackColor = Color.Salmon;
                textBox_MoveTimeBasic.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (MoveToDest.Distance == 0)
            {
                textBox_MoveDistance.BackColor = Color.Salmon;
                textBox_MoveDistanceBasic.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (BallScrew.MovePerRev <= 0)
            {
                textBox_MovePerRev.BackColor = Color.Salmon;
                textBox_MovePerRevBasic.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (Mass.Mass <= 0)
            {
                textBox_LoadMass.BackColor = Color.Salmon;
                textBox_LoadMassBasic.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (!invalidData)
            {
                this.DialogResult = DialogResult.OK;
                Close();
            }
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        #endregion

        #region Methods

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        #endregion        

    }
}
