﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DockSample
{
    public partial class DlgCreateLinearUnit : Form
    {
        public string appName;
        public string date;
        public string customer;
        public string ourRef;
        public string yourRef;
        public string description;

        public string moveToDestName;
        public Profile moveToDestProfile;
        public double moveToDestTime;
        public double moveToDestDistance;

        public bool addReturnMove;
        public string moveReturnName;

        public string moveWaitName;
        public double moveWaitTime;

        public string ballScrewName;
        public double ballScrewPitch = 97;
        public double ballScrewInertia;

        public string massName;
        public double mass;

        public bool horisontal;
        public bool useGearBox;
        public bool useCoupling;

        public DlgCreateLinearUnit()
        {
            InitializeComponent();
        }

        private void DlgCreateLinearUnit_Load(object sender, EventArgs e)
        {
            moveToDestProfile = Profile.Symmetric;
            addReturnMove = true;
            useCoupling = true;
            useGearBox = true;
            horisontal = true;

            textBox_Date.Text = System.DateTime.Today.ToString("yyyy-MM-dd");

            date = textBox_Date.Text;
            ballScrewName = textBox_BallScrewName.Text;
            moveToDestName = textBox_MoveName.Text;
            moveWaitName = textBox_WaitName.Text;
        }

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        private void button_Create_Click(object sender, EventArgs e)
        {
            bool invalidData = false;

            /*          _applicationData.AppName = textBox_AppName.Text;
                      _applicationData.Date = textBox_Date.Text;
                      _applicationData.Customer = textBox_Customer.Text;
                      _applicationData.OurRef = textBox_OurRef.Text;
                      _applicationData.YourRef = textBox_YourRef.Text;
                      _applicationData.Description = textBox_Descr.Text;
            
                      _moveToDest.Name = textBox_MoveName.Text;

                      if (radioButton_Symmetric.Checked == true)
                          _moveToDest.Profile = Profile.Symmetric;
                      else
                          _moveToDest.Profile = Profile.Triangular;

                      _moveToDest.Time = Convert.ToDouble(textBox_MoveTime.Text);
                      _moveToDest.Distance = Convert.ToDouble(textBox_MoveDistance.Text);
            
                      _waitMove.Name = textBox_WaitName.Text;
                      _waitMove.Time = Convert.ToDouble(textBox_WaitTime.Text);

                      _loadMass.Name = textBox_LoadName.Text;
                      _loadMass.Mass = Convert.ToDouble(textBox_Mass.Text);

                      _ballScrew.MovePerRev = Convert.ToDouble(textBox_Pitch.Text);
           */
            this.DialogResult = DialogResult.OK;
            Close();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        private void textBox_MoveTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_MoveTime.Text = value.ToString();

            moveToDestTime = value;
        }

        private void textBox_MoveDistance_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_MoveDistance.Text = value.ToString();

            moveToDestDistance = value;

            textBox_BallScrewLength.Text = (value + 0.5).ToString();

            CalculateInertia();
        }

        private void textBox_WaitTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_WaitTime.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_WaitTime.Text = value.ToString();

            moveWaitTime = value;
        }

        private void textBox_Pitch_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_Pitch.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_Pitch.Text = value.ToString();

            ballScrewPitch = value;
        }

        private void textBox_Mass_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_Mass.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_Mass.Text = value.ToString();

            mass = value;
        }

        private void textBox_Date_Validated(object sender, EventArgs e)
        {
            date = textBox_Date.Text;
        }

        private void textBox_Customer_Validated(object sender, EventArgs e)
        {
            customer = textBox_Customer.Text;
        }

        private void textBox_OurRef_Validated(object sender, EventArgs e)
        {
            ourRef = textBox_OurRef.Text;
        }

        private void textBox_Descr_Validated(object sender, EventArgs e)
        {
            description = textBox_Descr.Text;
        }

        private void textBox_AppName_Validated(object sender, EventArgs e)
        {
            appName = textBox_AppName.Text;
        }

        private void textBox_YourRef_Validated(object sender, EventArgs e)
        {
            yourRef = textBox_YourRef.Text;
        }

        private void textBox_MoveName_Validated(object sender, EventArgs e)
        {
            moveToDestName = textBox_MoveName.Text;
        }

        private void radioButton_Triangular_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Triangular.Checked)
                moveToDestProfile = Profile.Triangular;
            else
                moveToDestProfile = Profile.Symmetric;
        }

        private void radioButton_Symmetric_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Symmetric.Checked)
                moveToDestProfile = Profile.Symmetric;
            else
                moveToDestProfile = Profile.Triangular;
        }

        private void checkBox_IncludeRevMov_CheckStateChanged(object sender, EventArgs e)
        {
            if (checkBox_IncludeRevMov.Checked)
                addReturnMove = true;
            else
                addReturnMove = false;
        }

        private void textBox_WaitName_Validated(object sender, EventArgs e)
        {
            moveWaitName = textBox_WaitName.Text;
        }

        private void textBox_BallScrewName_Validating(object sender, CancelEventArgs e)
        {
            ballScrewName = textBox_BallScrewName.Text;
        }

        private void textBox_BallScrewLength_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_BallScrewLength.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_BallScrewLength.Text = value.ToString();

            CalculateInertia();
        }

        private void textBox_OuterDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_OuterDiameter.Text = value.ToString();

            CalculateInertia();
        }

        private void textBox_Density_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_Density.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_Density.Text = value.ToString();

            CalculateInertia();
        }

        private void textBox_InnerDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_InnerDiameter.Text = value.ToString();

            CalculateInertia();
        }

        private void textBox_LoadName_Validated(object sender, EventArgs e)
        {
            massName = textBox_LoadName.Text;
        }

        private void radioButton_Horisontal_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Horisontal.Checked)
                horisontal = true;
            else
                horisontal = false;
        }

        private void radioButton_Vertical_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Vertical.Checked)
                horisontal = false;
            else
                horisontal = true;
        }

        private void checkBox_UseGearBox_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_UseGearBox.Checked)
                useGearBox = true;
            else
                useGearBox = false;
        }

        private void checkBox_Coupling_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_Coupling.Checked)
                useCoupling = true;
            else
                useCoupling = false;
        }

        private void CalculateInertia()
        {
            double value;

            if (double.TryParse(textBox_BallScrewLength.Text, out value) == false)
                return;

            double length = Convert.ToDouble(textBox_BallScrewLength.Text);
            double density = Convert.ToDouble(textBox_Density.Text);
            double outerDiameter = Convert.ToDouble(textBox_OuterDiameter.Text);
            double innerDiameter = Convert.ToDouble(textBox_Mass.Text);


            double totalVolume = Math.PI * Math.Pow(outerDiameter / 2000, 2) * length / 1000;
            double hollowVolume = Math.PI * Math.Pow(innerDiameter / 2000, 2) * length / 1000;

            double realVolume = totalVolume - hollowVolume;
            double mass = realVolume * density;

            textBox_Weight.Text = Math.Round(mass, 3).ToString();

            ballScrewInertia = mass * (Math.Pow(outerDiameter / 2000, 2) + Math.Pow(innerDiameter / 2000, 2)) * 0.5;
            
            textBox_Inertia.Text = ballScrewInertia.ToString();
        }
    }
}
