﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DockSample
{
    public partial class DlgCreateRackAndPinion : Form
    {
        private ApplicationData _applicationData;
        private Move _moveToDest;
        private Move _moveReturn;
        private Move _moveWait;
        private RackAndPinion _rackAndPinion;
        private LoadMass _mass;

        public bool _addReturnMove;
        public bool _addGearBox;
        public bool _addCoupling;      

        public DlgCreateRackAndPinion()
        {   
            InitializeComponent();
        }
        
        #region Properties

        public ApplicationData ApplicationData
        {
            get { return _applicationData; }
            set { this._applicationData = value; }
        }

        public Move MoveToDest
        {
            get { return _moveToDest; }
            set { this._moveToDest = value; }
        }

        public Move MoveReturn
        {
            get { return _moveReturn; }
            set { this._moveReturn = value; }
        }

        public Move MoveWait
        {
            get { return _moveWait; }
            set { this._moveWait = value; }
        }

        public RackAndPinion RackAndPinion
        {
            get { return _rackAndPinion; }
            set { this._rackAndPinion = value; }
        }

        public LoadMass Mass
        {
            get { return _mass; }
            set { this._mass = value; }
        }

        #endregion

        #region Events
        
        private void DlgCreateRackAndPinion_Load(object sender, EventArgs e)
        {
            //Appdata
            ApplicationData = new ApplicationData();
            ApplicationData.Date = ApplicationData.Date = System.DateTime.Today.ToString("yyyy-MM-dd");
            textBox_Date.Text = ApplicationData.Date;

            //Move to destination
            MoveToDest = new Move("MoveToDest", MoveType.PreDefined, Profile.Symmetric);
            textBox_MoveName.Text = MoveToDest.Name;
            radioButton_Symmetric.Checked = true;

            MoveToDest.Time = 0;
            textBox_MoveTime.Text = MoveToDest.Time.ToString();

            MoveToDest.Distance = 0;
            textBox_MoveDistance.Text = MoveToDest.Distance.ToString();

            //Return move
            _addReturnMove = true; 
            checkBox_IncludeRevMov.Checked = _addReturnMove; 
            
            MoveReturn = new Move("MoveReturn", MoveType.PreDefined, Profile.Symmetric);
                        
            //Wait time
            MoveWait = new Move("MoveWait", MoveType.UserDefined, Profile.Undefined);

            MoveWait.Time = 1;
            textBox_WaitTime.Text = MoveWait.Time.ToString();

            //Component
            RackAndPinion = new RackAndPinion("MyRackAndPinion");

            textBox_ComponentName.Text = RackAndPinion.Name;

            RackAndPinion.RackAndPinionGeometry.ManualInertiaCalc = true;
            RackAndPinion.RackAndPinionGeometry.Material = Material.Steel;

            textBox_MovePerRev.Text = RackAndPinion.MovePerRev.ToString();

            RackAndPinion.IdleTrq = 0.15;
            textBox_IdleTorque.Text = RackAndPinion.IdleTrq.ToString();

            RackAndPinion.RackAndPinionGeometry.OuterDiameter = 30; //30 mm
            textBox_OuterDiameter.Text = RackAndPinion.RackAndPinionGeometry.OuterDiameter.ToString();
            textBox_InnerDiameter.Text = RackAndPinion.RackAndPinionGeometry.InnerDiameter.ToString();
            textBox_CylinderHeight.Text = RackAndPinion.RackAndPinionGeometry.CylinderHeight.ToString();

            RackAndPinion.PropertyChanged += new PropertyChangedEventHandler(RackAndPinion_PropertyChanged);

            //load the ComboBox_Material with valid items
            comboBox_Material.Items.Add(Material.Steel.ToString());
            comboBox_Material.Items.Add(Material.Aluminum.ToString());
            comboBox_Material.Items.Add(Material.Plastic.ToString());

            comboBox_Material.SelectedItem = comboBox_Material.Items[0]; //Steel
            
            //Load
            Mass = new LoadMass("MyMass");
            textBox_LoadName.Text = Mass.Name;
            textBox_LoadMass.Text = Mass.Mass.ToString();
            
            //Additional data
            radioButton_Horisontal.Checked = true;
            
            _addCoupling = false;
            checkBox_Coupling.Checked = _addCoupling;

            _addGearBox = true;
            checkBox_UseGearBox.Checked = _addGearBox;
        }

        private void RackAndPinion_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            RackAndPinion rackAndPinion = (RackAndPinion)sender;

            //Catch events
            switch (e.PropertyName)
            {
                case "Inertia":
                    textBox_Mass.Text = rackAndPinion.RackAndPinionGeometry.PartMass.ToString();
                    textBox_Inertia.Text = Math.Round(rackAndPinion.RackAndPinionGeometry.PartInertia * 10000, 3).ToString();
                    break;
            }
        }

        //Application data
        private void textBox_Date_Validated(object sender, EventArgs e)
        {
            if (ApplicationData.Date != textBox_Date.Text)
                ApplicationData.Date = textBox_Date.Text;
        }

        private void textBox_Customer_Validated(object sender, EventArgs e)
        {
             ApplicationData.Customer = textBox_Customer.Text;
        }

        private void textBox_OurRef_Validated(object sender, EventArgs e)
        {
            ApplicationData.OurRef = textBox_OurRef.Text;
        }

        private void textBox_Description_Validated(object sender, EventArgs e)
        {
            ApplicationData.Description = textBox_Description.Text;
        }

        private void textBox_AppName_Validated(object sender, EventArgs e)
        {
            ApplicationData.AppName = textBox_AppName.Text;
        }

        private void textBox_YourRef_Validated(object sender, EventArgs e)
        {
            ApplicationData.YourRef = textBox_YourRef.Text;
        }

        //Move to destination
        private void textBox_MoveName_Validated(object sender, EventArgs e)
        {
            MoveToDest.Name = textBox_MoveName.Text;
        }

        private void textBox_MoveTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_MoveTime.Text = value.ToString();

            MoveToDest.Time = value;

            if (value > 0)
                textBox_MoveTime.BackColor = SystemColors.Window;
            else
                textBox_MoveTime.BackColor = Color.Gold;
        }

        private void textBox_MoveTime_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_MoveTime.Text = value.ToString();

                MoveToDest.Time = value;

                if (value > 0)
                    textBox_MoveTime.BackColor = SystemColors.Window;
                else
                    textBox_MoveTime.BackColor = Color.Gold;
            }
        }

        private void textBox_MoveDistance_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_MoveDistance.Text = value.ToString();

            MoveToDest.Distance = value / 1000;

            if (value != 0)
                textBox_MoveDistance.BackColor = SystemColors.Window;
            else
                textBox_MoveDistance.BackColor = Color.Gold;
        }

        private void textBox_MoveDistance_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_MoveDistance.Text = value.ToString();

                MoveToDest.Distance = value / 1000;

                if (value != 0)
                    textBox_MoveDistance.BackColor = SystemColors.Window;
                else
                    textBox_MoveDistance.BackColor = Color.Gold;
            }
        }

        private void radioButton_Triangular_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Triangular.Checked)
                MoveToDest.Profile = Profile.Triangular;
            else
                MoveToDest.Profile = Profile.Symmetric;
        }

        private void radioButton_Symmetric_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Symmetric.Checked)
                MoveToDest.Profile = Profile.Symmetric;
            else
                MoveToDest.Profile = Profile.Triangular;
        }

        //MoveReturn
        private void checkBox_IncludeRevMov_CheckStateChanged(object sender, EventArgs e)
        {
            if (checkBox_IncludeRevMov.Checked)
                _addReturnMove = true;
            else
                _addReturnMove = false;
        }

        //MoveWait
        private void textBox_WaitTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_WaitTime.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_WaitTime.Text = value.ToString();

            MoveWait.Time = value;
        }

        //BallScrew        
        private void textBox_ComponentName_Validated(object sender, EventArgs e)
        {
            RackAndPinion.Name = textBox_ComponentName.Text;
        }

        private void textBox_IdleTorque_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_IdleTorque.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_IdleTorque.Text = value.ToString();

            RackAndPinion.IdleTrq = value;
        }

        private void textBox_MovePerRev_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRev.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_MovePerRev.Text = value.ToString();

            RackAndPinion.MovePerRev = value / 1000;

            if (value > 0)
                textBox_MovePerRev.BackColor = SystemColors.Window;
            else
                textBox_MovePerRev.BackColor = Color.Gold;

            RackAndPinion.RackAndPinionGeometry.OuterDiameter = RackAndPinion.MovePerRev * 1000 / Math.PI;
            textBox_OuterDiameter.Text = Math.Round(RackAndPinion.RackAndPinionGeometry.OuterDiameter, 3).ToString();
        }

        private void textBox_MovePerRev_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRev.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_MovePerRev.Text = value.ToString();

                RackAndPinion.MovePerRev = value / 1000;

                if (value > 0)
                    textBox_MovePerRev.BackColor = SystemColors.Window;
                else
                    textBox_MovePerRev.BackColor = Color.Gold;

                RackAndPinion.RackAndPinionGeometry.OuterDiameter = RackAndPinion.MovePerRev * 1000 / Math.PI;
                textBox_OuterDiameter.Text = Math.Round(RackAndPinion.RackAndPinionGeometry.OuterDiameter, 3).ToString();
            }
        }

        private void textBox_CylinderHeight_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_CylinderHeight.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_CylinderHeight.Text = value.ToString();

            RackAndPinion.RackAndPinionGeometry.CylinderHeight = value;
        }

        private void textBox_CylinderHeight_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_CylinderHeight.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_CylinderHeight.Text = value.ToString();

                RackAndPinion.RackAndPinionGeometry.CylinderHeight = value;
            }
        }

        private void textBox_OuterDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_OuterDiameter.Text = value.ToString();

            RackAndPinion.RackAndPinionGeometry.OuterDiameter = value;

            RackAndPinion.MovePerRev = (Math.PI * value) / 1000;
            textBox_MovePerRev.Text = Math.Round(RackAndPinion.MovePerRev * 1000, 3).ToString();
        }

        private void textBox_OuterDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_OuterDiameter.Text = value.ToString();

                RackAndPinion.RackAndPinionGeometry.OuterDiameter = value;

                RackAndPinion.MovePerRev = (Math.PI * value) / 1000;
                textBox_MovePerRev.Text = Math.Round(RackAndPinion.MovePerRev * 1000, 3).ToString();
            }
        }

        private void textBox_InnerDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_InnerDiameter.Text = value.ToString();

            RackAndPinion.RackAndPinionGeometry.InnerDiameter = value;
        }

        private void textBox_InnerDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_InnerDiameter.Text = value.ToString();

                RackAndPinion.RackAndPinionGeometry.InnerDiameter = value;
            }
        }

        private void comboBox_Material_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Material.SelectedIndex == 0)
                RackAndPinion.RackAndPinionGeometry.Material = Material.Steel;

            if (comboBox_Material.SelectedIndex == 1)
                RackAndPinion.RackAndPinionGeometry.Material = Material.Aluminum;

            if (comboBox_Material.SelectedIndex == 2)
                RackAndPinion.RackAndPinionGeometry.Material = Material.Plastic;
        }

        private void radioButton_Horisontal_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Horisontal.Checked)
                RackAndPinion.Angle = 0;
            else
                RackAndPinion.Angle = 90;
        }

        private void radioButton_Vertical_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Vertical.Checked)
                RackAndPinion.Angle = 90;
            else
                RackAndPinion.Angle = 0;
        }

        //Mass
        private void textBox_LoadName_Validated(object sender, EventArgs e)
        {
            Mass.Name = textBox_LoadName.Text;
        }

        private void textBox_LoadMass_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMass.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_LoadMass.Text = value.ToString();

            Mass.Mass = value;

            if (value > 0)
                textBox_LoadMass.BackColor = SystemColors.Window;
            else
                textBox_LoadMass.BackColor = Color.Gold;
        }

        private void textBox_LoadMass_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMass.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_LoadMass.Text = value.ToString();

                Mass.Mass = value;

                if (value > 0)
                    textBox_LoadMass.BackColor = SystemColors.Window;
                else
                    textBox_LoadMass.BackColor = Color.Gold;
            }
        }

        //Additional
        private void checkBox_UseGearBox_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_UseGearBox.Checked)
                _addGearBox = true;
            else
                _addGearBox = false;
        }

        private void checkBox_Coupling_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_Coupling.Checked)
                _addCoupling = true;
            else
                _addCoupling = false;
        }

        //Create return data
        private void button_Create_Click(object sender, EventArgs e)
        {
            bool invalidData = false;

            if (MoveToDest.Time <= 0)
            {
                textBox_MoveTime.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (MoveToDest.Distance == 0)
            {
                textBox_MoveDistance.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (RackAndPinion.MovePerRev <= 0)
            {
                textBox_MovePerRev.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (Mass.Mass <= 0)
            {
                textBox_LoadMass.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (!invalidData)
            {
                this.DialogResult = DialogResult.OK;
                Close();
            }
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        #endregion

        #region Methods

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        #endregion
    }
}
