﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DockSample
{
    public partial class DlgCreateToothedBelt : Form
    {
        private ApplicationData _applicationData;
        private Move _moveToDest;
        private Move _moveReturn;
        private Move _moveWait;
        private ToothedBelt _toothedBelt;
        private LoadMass _mass;

        public bool _addReturnMove;
        public bool _addGearBox;
        public bool _addCoupling;

        private double beltLength;
        private double beltWidth;
        private double beltThickness;
        private double beltDensity;
        private double beltMass;

        public DlgCreateToothedBelt()
        {   
            InitializeComponent();
        }
        
        #region Properties

        public ApplicationData ApplicationData
        {
            get { return _applicationData; }
            set { this._applicationData = value; }
        }

        public Move MoveToDest
        {
            get { return _moveToDest; }
            set { this._moveToDest = value; }
        }

        public Move MoveReturn
        {
            get { return _moveReturn; }
            set { this._moveReturn = value; }
        }

        public Move MoveWait
        {
            get { return _moveWait; }
            set { this._moveWait = value; }
        }

        public ToothedBelt ToothedBelt
        {
            get { return _toothedBelt; }
            set { this._toothedBelt= value; }
        }

        public LoadMass Mass
        {
            get { return _mass; }
            set { this._mass = value; }
        }
        #endregion

        #region Events
        
        private void DlgCreateToothedBelt_Load(object sender, EventArgs e)
        {
            //Appdata
            ApplicationData = new ApplicationData();
            ApplicationData.Date = ApplicationData.Date = System.DateTime.Today.ToString("yyyy-MM-dd");
            textBox_Date.Text = ApplicationData.Date;

            //Move to destination
            MoveToDest = new Move("MoveToDest", MoveType.PreDefined, Profile.Symmetric);
            textBox_MoveName.Text = MoveToDest.Name;
            radioButton_Symmetric.Checked = true;

            MoveToDest.Time = 0;
            textBox_MoveTime.Text = MoveToDest.Time.ToString();

            MoveToDest.Distance = 0;
            textBox_MoveDistance.Text = MoveToDest.Distance.ToString();

            //Return move
            _addReturnMove = true; 
            checkBox_IncludeRevMov.Checked = _addReturnMove; 
            
            MoveReturn = new Move("MoveReturn", MoveType.PreDefined, Profile.Symmetric);
                        
            //Wait time
            MoveWait = new Move("MoveWait", MoveType.UserDefined, Profile.Undefined);

            MoveWait.Time = 1;
            textBox_WaitTime.Text = MoveWait.Time.ToString();

            //Component
            ToothedBelt = new ToothedBelt("MyToothedBelt");

            textBox_ComponentName.Text = ToothedBelt.Name;

            ToothedBelt.DriveWheelGeometry.ManualInertiaCalc = true;
            ToothedBelt.DriveWheelGeometry.Material = Material.Aluminum;

            //textBox_Gearing.Text = ToothedBelt.Gearing.ToString();

            ToothedBelt.IdleTrq = 0.3;
            textBox_IdleTorque.Text = ToothedBelt.IdleTrq.ToString();

            textBox_MovePerRev.Text = ToothedBelt.MovePerRev.ToString();

            textBox_OuterDiameter.Text = ToothedBelt.DriveWheelGeometry.OuterDiameter.ToString();
            textBox_InnerDiameter.Text = ToothedBelt.DriveWheelGeometry.InnerDiameter.ToString();
            
            ToothedBelt.DriveWheelGeometry.CylinderHeight = 30;
            textBox_CylinderHeight.Text = ToothedBelt.DriveWheelGeometry.CylinderHeight.ToString();

            ToothedBelt.PropertyChanged += new PropertyChangedEventHandler(ToothedBelt_PropertyChanged);

            //load the ComboBox_Material with valid items
            comboBox_Material.Items.Add(Material.Steel.ToString());
            comboBox_Material.Items.Add(Material.Aluminum.ToString());
            comboBox_Material.Items.Add(Material.Plastic.ToString());

            comboBox_Material.SelectedIndex = 1;
            
            //Load wheel
            ToothedBelt.LoadWheelGeometry.ManualInertiaCalc = true;
            ToothedBelt.LoadWheelGeometry.Material = Material.Aluminum;

            textBox_LoadWheelOuterDiameter.Text = ToothedBelt.LoadWheelGeometry.OuterDiameter.ToString();
            textBox_LoadWheelInnerDiameter.Text = ToothedBelt.LoadWheelGeometry.InnerDiameter.ToString();

            ToothedBelt.LoadWheelGeometry.CylinderHeight = 30;
            textBox_LoadWheelCylinderHeight.Text = ToothedBelt.LoadWheelGeometry.CylinderHeight.ToString();

            //load the ComboBox_Material with valid items
            comboBox_LoadWheelMaterial.Items.Add(Material.Steel.ToString());
            comboBox_LoadWheelMaterial.Items.Add(Material.Aluminum.ToString());
            comboBox_LoadWheelMaterial.Items.Add(Material.Plastic.ToString());

            comboBox_LoadWheelMaterial.SelectedIndex = 1; //Aluminum

            //Belt
            beltWidth = 30; //30 mm
            beltThickness = 5; //5 mm
            beltDensity = 2000; //kg / m3

            //Load
            Mass = new LoadMass("MyMass");
            textBox_LoadName.Text = Mass.Name;
            textBox_LoadMass.Text = Mass.Mass.ToString();
            
            //Additional data
            radioButton_Horizontal.Checked = true;
            
            _addCoupling = true;
            checkBox_Coupling.Checked = _addCoupling;

            _addGearBox = true;
            checkBox_UseGearBox.Checked = _addGearBox;
        }

        private void ToothedBelt_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            ToothedBelt toothedBelt = (ToothedBelt)sender;

            //Catch events
            switch (e.PropertyName)
            {
                case "Inertia":
                    textBox_Mass.Text = Math.Round(toothedBelt.DriveWheelGeometry.PartMass, 3).ToString();
                    textBox_LoadWheelMass.Text = Math.Round(toothedBelt.LoadWheelGeometry.PartMass, 3).ToString();

                    textBox_Inertia.Text = Math.Round(toothedBelt.DriveWheelGeometry.PartInertia * 10000, 3).ToString();
                    textBox_LoadWheelInertia.Text = Math.Round(toothedBelt.LoadWheelGeometry.PartInertia * 10000, 3).ToString();
                    textBox_TotalInertia.Text = Math.Round((toothedBelt.Inertia * 10000), 3).ToString();

                    break;

                //case "Gearing":
                    //textBox_Gearing.Text = toothedBelt.Gearing.ToString();
                    //break;
            }
        }

        //Application data
        private void textBox_Date_Validated(object sender, EventArgs e)
        {
            if (ApplicationData.Date != textBox_Date.Text)
                ApplicationData.Date = textBox_Date.Text;
        }

        private void textBox_Customer_Validated(object sender, EventArgs e)
        {
             ApplicationData.Customer = textBox_Customer.Text;
        }

        private void textBox_OurRef_Validated(object sender, EventArgs e)
        {
            ApplicationData.OurRef = textBox_OurRef.Text;
        }

        private void textBox_Description_Validated(object sender, EventArgs e)
        {
            ApplicationData.Description = textBox_Description.Text;
        }

        private void textBox_AppName_Validated(object sender, EventArgs e)
        {
            ApplicationData.AppName = textBox_AppName.Text;
        }

        private void textBox_YourRef_Validated(object sender, EventArgs e)
        {
            ApplicationData.YourRef = textBox_YourRef.Text;
        }

        //Move to destination
        private void textBox_MoveName_Validated(object sender, EventArgs e)
        {
            MoveToDest.Name = textBox_MoveName.Text;
        }

        private void textBox_MoveTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

            UpdateMoveTime(value);
        }

        private void textBox_MoveTime_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTime.Text), 3));

                UpdateMoveTime(value);
            }
        }

        private void textBox_MoveTimeBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTimeBasic.Text), 3));

            UpdateMoveTime(value);
        }

        private void textBox_MoveTimeBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_MoveTimeBasic.Text), 3));

                UpdateMoveTime(value);
            }
        }

        private void UpdateMoveTime(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_MoveTime.Text = value.ToString();
            textBox_MoveTimeBasic.Text = value.ToString();

            MoveToDest.Time = value;

            if (value > 0)
            {
                textBox_MoveTime.BackColor = SystemColors.Window;
                textBox_MoveTimeBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_MoveTime.BackColor = Color.Gold;
                textBox_MoveTimeBasic.BackColor = Color.Gold;
            }
        }

        private void textBox_MoveDistance_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

            UpdateDistance(value);            
        }

        private void textBox_MoveDistance_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Round(validate_textBox(textBox_MoveDistance.Text), 3);

                UpdateDistance(value);
            }
        }

        private void textBox_MoveDistanceBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Round(validate_textBox(textBox_MoveDistanceBasic.Text), 3);

            UpdateDistance(value);
        }

        private void textBox_MoveDistanceBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Round(validate_textBox(textBox_MoveDistanceBasic.Text), 3);

                UpdateDistance(value);
            }
        }

        private void UpdateDistance(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_MoveDistance.Text = value.ToString();
            textBox_MoveDistanceBasic.Text = value.ToString();

            MoveToDest.Distance = value / 1000;

            if (value != 0)
            {
                textBox_MoveDistance.BackColor = SystemColors.Window;
                textBox_MoveDistanceBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_MoveDistance.BackColor = Color.Gold;
                textBox_MoveDistanceBasic.BackColor = Color.Gold;
            }

            beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin

            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        private void radioButton_Triangular_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Triangular.Checked)
            {
                MoveToDest.Profile = Profile.Triangular;
                radioButton_Symmetric.Checked = false;
            }
            else
            {
                MoveToDest.Profile = Profile.Symmetric;
                radioButton_Symmetric.Checked = true;
            }
        }

        private void radioButton_Symmetric_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Symmetric.Checked)
            {
                MoveToDest.Profile = Profile.Symmetric;
                radioButton_Triangular.Checked = false;
            }
            else
            {
                MoveToDest.Profile = Profile.Triangular;
                radioButton_Triangular.Checked = true;
            }
        }

        //MoveReturn
        private void checkBox_IncludeRevMov_CheckStateChanged(object sender, EventArgs e)
        {
            if (checkBox_IncludeRevMov.Checked)
                _addReturnMove = true;
            else
                _addReturnMove = false;
        }

        //MoveWait
        private void textBox_WaitTime_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_WaitTime.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_WaitTime.Text = value.ToString();

            MoveWait.Time = value;
        }

        //ToothedBelt        
        private void textBox_ComponentName_Validated(object sender, EventArgs e)
        {
            ToothedBelt.Name = textBox_ComponentName.Text;
        }

        private void textBox_IdleTorque_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_IdleTorque.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_IdleTorque.Text = value.ToString();

            ToothedBelt.IdleTrq = value;
        }

        private void textBox_MovePerRev_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRev.Text), 3));

            UpdateMovePerRev(value);
        }

        private void textBox_MovePerRev_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRev.Text), 3));

                UpdateMovePerRev(value);
            }
        }

        private void textBox_MovePerRevBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRevBasic.Text), 3));

            UpdateMovePerRev(value);
        }

        private void textBox_MovePerRevBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                double value = Math.Abs(Math.Round(validate_textBox(textBox_MovePerRevBasic.Text), 3));

                UpdateMovePerRev(value);
            }
        }

        private void UpdateMovePerRev(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_MovePerRev.Text = value.ToString();
            textBox_MovePerRevBasic.Text = value.ToString();

            ToothedBelt.MovePerRev = value / 1000;

            if (value > 0)
            {
                textBox_MovePerRev.BackColor = SystemColors.Window;
                textBox_MovePerRevBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_MovePerRev.BackColor = Color.Gold;
                textBox_MovePerRevBasic.BackColor = Color.Gold;
            }

            ToothedBelt.LoadWheelGeometry.OuterDiameter = value / Math.PI;
            textBox_LoadWheelOuterDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.OuterDiameter, 3).ToString();

            ToothedBelt.DriveWheelGeometry.OuterDiameter = value / Math.PI;
            textBox_OuterDiameter.Text = Math.Round(ToothedBelt.DriveWheelGeometry.OuterDiameter, 3).ToString();
            textBox_DiameterBasic.Text = textBox_OuterDiameter.Text;

            beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        //Drive wheel
        private void textBox_CylinderHeight_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_CylinderHeight.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_CylinderHeight.Text = value.ToString();

            ToothedBelt.DriveWheelGeometry.CylinderHeight = value;

            ToothedBelt.LoadWheelGeometry.CylinderHeight = ToothedBelt.DriveWheelGeometry.CylinderHeight;
            textBox_LoadWheelCylinderHeight.Text = Math.Round(ToothedBelt.LoadWheelGeometry.CylinderHeight, 3).ToString();

            if (ToothedBelt.LoadWheelGeometry.CylinderHeight < ToothedBelt.DriveWheelGeometry.CylinderHeight)
                beltWidth = ToothedBelt.LoadWheelGeometry.CylinderHeight;
            else
                beltWidth = ToothedBelt.DriveWheelGeometry.CylinderHeight;

            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        private void textBox_CylinderHeight_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_CylinderHeight.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_CylinderHeight.Text = value.ToString();

                ToothedBelt.DriveWheelGeometry.CylinderHeight = value;

                ToothedBelt.LoadWheelGeometry.CylinderHeight = ToothedBelt.DriveWheelGeometry.CylinderHeight;
                textBox_LoadWheelCylinderHeight.Text = Math.Round(ToothedBelt.LoadWheelGeometry.CylinderHeight, 3).ToString();

                if (ToothedBelt.LoadWheelGeometry.CylinderHeight < ToothedBelt.DriveWheelGeometry.CylinderHeight)
                    beltWidth = ToothedBelt.LoadWheelGeometry.CylinderHeight;
                else
                    beltWidth = ToothedBelt.DriveWheelGeometry.CylinderHeight;

                beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

                textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
            }
        }

        private void textBox_OuterDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_OuterDiameter.Text = value.ToString();
            textBox_DiameterBasic.Text = textBox_OuterDiameter.Text;

            ToothedBelt.DriveWheelGeometry.OuterDiameter = value;

            ToothedBelt.LoadWheelGeometry.OuterDiameter = ToothedBelt.DriveWheelGeometry.OuterDiameter;
            textBox_LoadWheelOuterDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.OuterDiameter, 3).ToString();

            ToothedBelt.MovePerRev = (Math.PI * value) / 1000;
            textBox_MovePerRev.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();
            textBox_MovePerRevBasic.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();

            beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        private void textBox_OuterDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_OuterDiameter.Text = value.ToString();
                textBox_DiameterBasic.Text = textBox_OuterDiameter.Text;

                ToothedBelt.DriveWheelGeometry.OuterDiameter = value;

                ToothedBelt.LoadWheelGeometry.OuterDiameter = ToothedBelt.DriveWheelGeometry.OuterDiameter;
                textBox_LoadWheelOuterDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.OuterDiameter, 3).ToString();

                ToothedBelt.MovePerRev = (Math.PI * value) / 1000;
                textBox_MovePerRev.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();
                textBox_MovePerRevBasic.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();

                beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
                beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

                textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
            }
        }

        private void textBox_InnerDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_InnerDiameter.Text = value.ToString();

            ToothedBelt.DriveWheelGeometry.InnerDiameter = value;

            ToothedBelt.LoadWheelGeometry.InnerDiameter = ToothedBelt.DriveWheelGeometry.InnerDiameter;
            textBox_LoadWheelInnerDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.InnerDiameter, 3).ToString();
        }

        private void textBox_InnerDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_InnerDiameter.Text = value.ToString();

                ToothedBelt.DriveWheelGeometry.InnerDiameter = value;

                ToothedBelt.LoadWheelGeometry.InnerDiameter = ToothedBelt.DriveWheelGeometry.InnerDiameter;
                textBox_LoadWheelInnerDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.InnerDiameter, 3).ToString();
            }
        }

        private void comboBox_Material_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Material.SelectedIndex == 0)
                ToothedBelt.DriveWheelGeometry.Material = Material.Steel;

            if (comboBox_Material.SelectedIndex == 1)
                ToothedBelt.DriveWheelGeometry.Material = Material.Aluminum;

            if (comboBox_Material.SelectedIndex == 2)
                ToothedBelt.DriveWheelGeometry.Material = Material.Plastic;
        }

        //Load wheel
        private void textBox_LoadWheelCylinderHeight_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadWheelCylinderHeight.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_LoadWheelCylinderHeight.Text = value.ToString();

            ToothedBelt.LoadWheelGeometry.CylinderHeight = value;

            if (ToothedBelt.LoadWheelGeometry.CylinderHeight < ToothedBelt.DriveWheelGeometry.CylinderHeight)
                beltWidth = ToothedBelt.LoadWheelGeometry.CylinderHeight;
            else
                beltWidth = ToothedBelt.DriveWheelGeometry.CylinderHeight;

            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        private void textBox_LoadWheelCylinderHeight_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadWheelCylinderHeight.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_LoadWheelCylinderHeight.Text = value.ToString();

                ToothedBelt.LoadWheelGeometry.CylinderHeight = value;

                if (ToothedBelt.LoadWheelGeometry.CylinderHeight < ToothedBelt.DriveWheelGeometry.CylinderHeight)
                    beltWidth = ToothedBelt.LoadWheelGeometry.CylinderHeight;
                else
                    beltWidth = ToothedBelt.DriveWheelGeometry.CylinderHeight;

                beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

                textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
            }
        }

        private void textBox_LoadWheelOuterDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadWheelOuterDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_LoadWheelOuterDiameter.Text = value.ToString();

            ToothedBelt.LoadWheelGeometry.OuterDiameter = value;

            ToothedBelt.MovePerRev = (Math.PI * value) / 1000;
            textBox_MovePerRev.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();

            beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        private void textBox_LoadWheelOuterDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadWheelOuterDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_LoadWheelOuterDiameter.Text = value.ToString();

                ToothedBelt.LoadWheelGeometry.OuterDiameter = value;

                ToothedBelt.MovePerRev = (Math.PI * value) / 1000;
                textBox_MovePerRev.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();

                beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
                beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

                textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
            }
        }

        private void textBox_LoadWheelInnerDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadWheelInnerDiameter.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_LoadWheelInnerDiameter.Text = value.ToString();

            ToothedBelt.LoadWheelGeometry.InnerDiameter = value;
        }

        private void textBox_LoadWheelInnerDiameter_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadWheelInnerDiameter.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_LoadWheelInnerDiameter.Text = value.ToString();

                ToothedBelt.LoadWheelGeometry.InnerDiameter = value;
            }
        }

        private void comboBox_LoadWheelMaterial_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_LoadWheelMaterial.SelectedIndex == 0)
                ToothedBelt.LoadWheelGeometry.Material = Material.Steel;

            if (comboBox_LoadWheelMaterial.SelectedIndex == 1)
                ToothedBelt.LoadWheelGeometry.Material = Material.Aluminum;

            if (comboBox_LoadWheelMaterial.SelectedIndex == 2)
                ToothedBelt.LoadWheelGeometry.Material = Material.Plastic;
        }

        private void radioButton_Horisontal_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Horizontal.Checked)
            {
                ToothedBelt.Angle = 0;
                radioButton_HorizontalBasic.Checked = true;
                radioButton_Vertical.Checked = false;
                radioButton_VerticalBasic.Checked = false;
            }
            else
            {
                ToothedBelt.Angle = 90;
                radioButton_HorizontalBasic.Checked = false;
                radioButton_Vertical.Checked = true;
                radioButton_VerticalBasic.Checked = true;
            }
        }

        private void radioButton_HorisontalBasic_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_HorizontalBasic.Checked)
            {
                ToothedBelt.Angle = 0;
                radioButton_Horizontal.Checked = true;
                radioButton_Vertical.Checked = false;
                radioButton_VerticalBasic.Checked = false;

            }
            else
            {
                ToothedBelt.Angle = 90;
                radioButton_Horizontal.Checked = false;
                radioButton_Vertical.Checked = true;
                radioButton_VerticalBasic.Checked = true;
            }
        }

        private void radioButton_Vertical_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_Vertical.Checked)
            {
                ToothedBelt.Angle = 90;
                radioButton_VerticalBasic.Checked = true;
                radioButton_Horizontal.Checked = false;
                radioButton_HorizontalBasic.Checked = false;
            }
            else
            {
                ToothedBelt.Angle = 0;
                radioButton_VerticalBasic.Checked = false;
                radioButton_Horizontal.Checked = true;
                radioButton_HorizontalBasic.Checked = true;
            }
        }

        private void radioButton_VerticalBasic_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_VerticalBasic.Checked)
            {
                ToothedBelt.Angle = 90;
                radioButton_Vertical.Checked = true;
                radioButton_Horizontal.Checked = false;
                radioButton_HorizontalBasic.Checked = false;
            }
            else
            {
                ToothedBelt.Angle = 0;
                radioButton_Vertical.Checked = false;
                radioButton_Horizontal.Checked = true;
                radioButton_HorizontalBasic.Checked = true;
            }
        }

        //Mass
        private void textBox_LoadName_Validated(object sender, EventArgs e)
        {
            Mass.Name = textBox_LoadName.Text;
        }

        private void textBox_LoadName_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                ToothedBelt.Name = textBox_LoadName.Text;
            }
        }

        private void textBox_LoadMass_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMass.Text), 3));

            UpdateLoadMass(value);
        }

        private void textBox_LoadMass_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMass.Text), 3));

                UpdateLoadMass(value);
            }
        }

        private void textBox_LoadMassBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMassBasic.Text), 3));

            UpdateLoadMass(value);
        }

        private void textBox_LoadMassBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_LoadMassBasic.Text), 3));

                UpdateLoadMass(value);
            }
        }

        private void textBox_DiameterBasic_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double value = Math.Abs(Math.Round(validate_textBox(textBox_DiameterBasic.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_DiameterBasic.Text = value.ToString();
            textBox_OuterDiameter.Text = textBox_DiameterBasic.Text;

            ToothedBelt.DriveWheelGeometry.OuterDiameter = value;

            ToothedBelt.LoadWheelGeometry.OuterDiameter = ToothedBelt.DriveWheelGeometry.OuterDiameter;
            textBox_LoadWheelOuterDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.OuterDiameter, 3).ToString();

            ToothedBelt.MovePerRev = (Math.PI * value) / 1000;
            textBox_MovePerRev.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();
            textBox_MovePerRevBasic.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();

            beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
            beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

            textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
        }

        private void textBox_DiameterBasic_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double value = Math.Abs(Math.Round(validate_textBox(textBox_DiameterBasic.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_DiameterBasic.Text = value.ToString();
                textBox_OuterDiameter.Text = textBox_DiameterBasic.Text;

                ToothedBelt.DriveWheelGeometry.OuterDiameter = value;

                ToothedBelt.LoadWheelGeometry.OuterDiameter = ToothedBelt.DriveWheelGeometry.OuterDiameter;
                textBox_LoadWheelOuterDiameter.Text = Math.Round(ToothedBelt.LoadWheelGeometry.OuterDiameter, 3).ToString();

                ToothedBelt.MovePerRev = (Math.PI * value) / 1000;
                textBox_MovePerRev.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();
                textBox_MovePerRevBasic.Text = Math.Round(ToothedBelt.MovePerRev * 1000, 3).ToString();

                beltLength = 2 * 1000 * MoveToDest.Distance + ToothedBelt.DriveWheelGeometry.OuterDiameter * Math.PI + ToothedBelt.LoadWheelGeometry.OuterDiameter * Math.PI; //One wheel circumference margin
                beltMass = ((beltWidth / 1000) * (beltThickness / 1000) * (beltLength / 1000)) * beltDensity;

                textBox_BeltMass.Text = Math.Round(beltMass, 3).ToString();
            }
        }

        private void UpdateLoadMass(double value)
        {
            //Update the textBox if the validate-function did any formatting
            textBox_LoadMass.Text = value.ToString();
            textBox_LoadMassBasic.Text = value.ToString();

            Mass.Mass = value;

            if (value > 0)
            {
                textBox_LoadMass.BackColor = SystemColors.Window;
                textBox_LoadMassBasic.BackColor = SystemColors.Window;
            }
            else
            {
                textBox_LoadMass.BackColor = Color.Gold;
                textBox_LoadMassBasic.BackColor = Color.Gold;
            }
        }

        //Additional
        private void checkBox_UseGearBox_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_UseGearBox.Checked)
                _addGearBox = true;
            else
                _addGearBox = false;
        }

        private void checkBox_Coupling_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_Coupling.Checked)
                _addCoupling = true;
            else
                _addCoupling = false;
        }

        //Create return data
        private void button_Create_Click(object sender, EventArgs e)
        {
            bool invalidData = false;

            if (MoveToDest.Time <= 0)
            {
                textBox_MoveTime.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (MoveToDest.Distance == 0)
            {
                textBox_MoveDistance.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (ToothedBelt.MovePerRev <= 0)
            {
                textBox_MovePerRev.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (checkBox_AddBeltMass.Checked)
                Mass.Mass += beltMass;

            if (Mass.Mass <= 0)
            {
                textBox_LoadMass.BackColor = Color.Salmon;
                invalidData = true;
            }

            if (!invalidData)
            {
                this.DialogResult = DialogResult.OK;
                Close();
            }
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        #endregion

        #region Methods

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        #endregion

    }
}
