﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.Data.OleDb;

namespace DockSample
{
    public partial class FrmAccessories : DockContent
    {
        private Motor _motor;
        private Drive _drive;
        private BrakeResistor _brakeResistor;
        private SortedList<int, IAccessory> _iAccessories;
        private Moves _moves;

        private string selectedMotor;

        public FrmAccessories(Motor motor, SortedList<int, IAccessory> iAccessories, Moves moves)
        {
            _motor = motor as Motor;
            _iAccessories = iAccessories as SortedList<int, IAccessory>;
            _moves = moves as Moves;

            _drive = new Drive();
            _brakeResistor = new BrakeResistor();

            InitializeComponent();
        }

        private void FrmAccessories_Load(object sender, EventArgs e)
        {
            _motor.PropertyChanged += new PropertyChangedEventHandler(Motor_PropertyChanged);

            UpdateComboBoxValues();
        }

        private void FrmAccessories_Activated(object sender, EventArgs e)
        {
            UpdateComboBoxValues();
            
            if (selectedMotor != _motor.ArticleNo)
            {
                UpdateMotorRelatedComboBoxItemsFromDB();
                selectedMotor = _motor.ArticleNo;
            }

            if (_moves.MovesList.Count > 0)
                CalculateRegenerativeResults();
        }
        private void Motor_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "ArticleNo")
                if (!_motor.AutoSelect)
                {
                    UpdateMotorRelatedComboBoxItemsFromDB();
                    selectedMotor = _motor.ArticleNo;
                }
        }

        private void UpdateComboBoxValues()
        {
            foreach (var acc in _iAccessories)
            {
                IAccessory iAcc = acc.Value;

                if (iAcc.ArticleNo != "" && iAcc.ArticleNo != null)
                    switch (iAcc.MainType)
                    {
                        case MainType.EncoderCable:
                            comboBox_EncoderCable.Text = iAcc.Description;
                            break;

                        case MainType.MotorCable:
                            comboBox_MotorCable.Text = iAcc.Description;
                            break;

                        case MainType.BrakeCable:
                            comboBox_BrakeCable.Text = iAcc.Description;
                            break;

                        case MainType.Drive:
                            comboBox_Drive.Text = iAcc.Description;
                            break;

                        case MainType.BrakeResistor:
                            comboBox_BrakeResistor.Text = iAcc.Description;
                            break;

                        case MainType.EMCFilter:
                            comboBox_EMCFilter.Text = iAcc.Description;
                            break;

                        case MainType.DCFilter:
                            comboBox_DCFilter.Text = iAcc.Description;
                            break;

                        case MainType.Safety:
                            comboBox_Safety.Text = iAcc.Description;
                            break;

                        case MainType.Battery:
                            comboBox_Battery.Text = iAcc.Description;
                            break;

                        case MainType.IOCard:
                            comboBox_IOCard.Text = iAcc.Description;
                            break;

                        case MainType.IOCable:
                            comboBox_IOCable.Text = iAcc.Description;
                            break;

                        case MainType.BusCable:
                            comboBox_BusCable.Text = iAcc.Description;
                            break;

                        case MainType.Controller:
                            comboBox_Controller.Text = iAcc.Description;
                            break;

                        case MainType.ExtensionCard:
                            comboBox_ExtensionCard.Text = iAcc.Description;
                            break;

                        case MainType.ExtensionCable:
                            comboBox_ExtensionCable.Text = iAcc.Description;
                            break;

                        case MainType.ExtensionTerminal:
                            comboBox_ExtensionTerminal.Text = iAcc.Description;
                            break;
                    }
            }
        }

        private double FormatValue(double raw)
        {
            //Hm........?
            if (Math.Abs(raw) < 10)
                return Math.Round(raw, 2);

            if (Math.Abs(raw) < 100)
                return Math.Round(raw, 1);
            else
                return Math.Round(raw, 0);
        }

        private void UpdateMotorRelatedComboBoxItemsFromDB()
        {
            //Display the selected motor
            textBox_Motor.Text = _motor.Description;

            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //flag to check if the current combobox value is valid
                    bool valueExists;

                    // - - -  DRIVES - - - 

                    //Query all drives in the database that can be used with this motor
                    var qryDrivesPerMotor =
                                from tblDrives myDrives in dbDriveSystemsContext.tblDrives
                                join tblDriveAndMotorCombinations myDriveAndMotorCombinations
                                    in dbDriveSystemsContext.tblDriveAndMotorCombinations
                                on myDrives.ArticleNo equals myDriveAndMotorCombinations.DriveArticleNo
                                where myDriveAndMotorCombinations.MotorArticleNo == _motor.ArticleNo
                                orderby myDrives.ArticleNo
                                select myDrives;

                    comboBox_Drive.Items.Clear();

                    //Reselect if the ComboBox text exists in the items list
                    foreach (var drive in qryDrivesPerMotor)
                        //Check if the drive fulfills the search criteria(s)
                        if (((Settings.MainSettings.ConditionFilter == Condition.OnlyNew && !drive.Campaign) ||
                            (Settings.MainSettings.ConditionFilter == Condition.NewAndCampaign && !drive.Sold) ||
                            (Settings.MainSettings.ConditionFilter == Condition.OnlyCampaign && drive.Campaign && !drive.Sold)))
                        {
                            comboBox_Drive.Items.Add(new ComboBoxItem(drive.Description, drive.ArticleNo));
                            if (comboBox_Drive.Text == drive.Description)
                                comboBox_Drive.SelectedIndex = comboBox_Drive.Items.IndexOf(drive.Description);
                        }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_Drive.Text == "" || comboBox_Drive.Text == null)
                    {
                        if (comboBox_Drive.Items.Count > 0)
                            comboBox_Drive.BackColor = Color.Gold;
                        else
                            comboBox_Drive.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_Drive.Items)
                            if (itm.ToString() == comboBox_Drive.Text)
                            {
                                UpdateDriveRelatedComboBoxItemsFromDB(itm.HiddenValue);
                                valueExists = true;
                            }

                        if (valueExists)
                            comboBox_Drive.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_Drive.BackColor = Color.Salmon;
                    }

                    // - - -  CABLES - - - 

                    //Query all encoder cables in the database that can be used with this motor
                    var qryCablesPerMotor =
                                from tblCables myCables in dbDriveSystemsContext.tblCables
                                join tblMotorAndCableCombinations myMotorAndCablesCombinations
                                    in dbDriveSystemsContext.tblMotorAndCableCombinations
                                on myCables.ArticleNo equals myMotorAndCablesCombinations.CableArticleNo
                                where myMotorAndCablesCombinations.MotorArticleNo == _motor.ArticleNo
                                orderby myCables.ArticleNo
                                select myCables;

                    //Add cable items
                    comboBox_EncoderCable.Items.Clear();
                    comboBox_MotorCable.Items.Clear();
                    comboBox_BrakeCable.Items.Clear();

                    //Reselect if the ComboBox text exists in the items list
                    foreach (var cable in qryCablesPerMotor)
                    {
                        switch (cable.SubType)
                        {
                            case "EncoderCable":
                                comboBox_EncoderCable.Items.Add(new ComboBoxItem(cable.Description, cable.ArticleNo));
                                if (comboBox_EncoderCable.Text == cable.Description)
                                    comboBox_EncoderCable.SelectedIndex = comboBox_EncoderCable.Items.IndexOf(cable.Description);
                                break;

                            case "MotorCable":
                                comboBox_MotorCable.Items.Add(new ComboBoxItem(cable.Description, cable.ArticleNo));
                                if (comboBox_MotorCable.Text == cable.Description)
                                    comboBox_MotorCable.SelectedIndex = comboBox_MotorCable.Items.IndexOf(cable.Description);
                                break;

                            case "BrakeCable":
                                if (_motor.Braked)
                                {
                                    comboBox_BrakeCable.Items.Add(new ComboBoxItem(cable.Description, cable.ArticleNo));
                                    if (comboBox_BrakeCable.Text == cable.Description)
                                        comboBox_BrakeCable.SelectedIndex = comboBox_BrakeCable.Items.IndexOf(cable.Description);
                                }
                                break;
                        }
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_EncoderCable.Text == "" || comboBox_EncoderCable.Text == null)
                    {
                        if (comboBox_EncoderCable.Items.Count > 0)
                            comboBox_EncoderCable.BackColor = Color.Gold;
                        else
                            comboBox_EncoderCable.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_EncoderCable.Items)
                            if (itm.ToString() == comboBox_EncoderCable.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_EncoderCable.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_EncoderCable.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_MotorCable.Text == "" || comboBox_MotorCable.Text == null)
                    {
                        if (comboBox_MotorCable.Items.Count > 0)
                            comboBox_MotorCable.BackColor = Color.Gold;
                        else
                            comboBox_MotorCable.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_MotorCable.Items)
                            if (itm.ToString() == comboBox_MotorCable.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_MotorCable.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_MotorCable.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_BrakeCable.Text == "" || comboBox_BrakeCable.Text == null)
                    {
                        if (comboBox_BrakeCable.Items.Count > 0)
                            comboBox_BrakeCable.BackColor = Color.Gold;
                        else
                            comboBox_BrakeCable.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_BrakeCable.Items)
                            if (itm.ToString() == comboBox_BrakeCable.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_BrakeCable.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_BrakeCable.BackColor = Color.Salmon;
                    }
                }
            }
        }

        private void UpdateDriveRelatedComboBoxItemsFromDB(string driveArticleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //flag to check if the current combobox value is valid
                    bool valueExists;

                    // - - -  FILTERS - - - 

                    //Query all filters in the database that can be used with this drive
                    var qryFiltersPerDrive =
                                from tblFilters myFilters in dbDriveSystemsContext.tblFilters
                                join tblDriveAndFilterCombinations myDriveAndFilterCombinations
                                    in dbDriveSystemsContext.tblDriveAndFilterCombinations
                                on myFilters.ArticleNo equals myDriveAndFilterCombinations.FilterArticleNo
                                where myDriveAndFilterCombinations.DriveArticleNo == driveArticleNo
                                orderby myFilters.ArticleNo
                                select myFilters;

                    comboBox_EMCFilter.Items.Clear();
                    comboBox_DCFilter.Items.Clear();

                    //Reselect if the ComboBox text exists in the items list
                    foreach (var filter in qryFiltersPerDrive)
                    {
                        switch (filter.SubType)
                        {
                            case "EMC-filter":
                                comboBox_EMCFilter.Items.Add(new ComboBoxItem(filter.Description, filter.ArticleNo));
                                if (comboBox_EMCFilter.Text == filter.Description)
                                    comboBox_EMCFilter.SelectedIndex = comboBox_EMCFilter.Items.IndexOf(filter.Description);
                                break;

                            case "DC-filter":
                                comboBox_DCFilter.Items.Add(new ComboBoxItem(filter.Description, filter.ArticleNo));
                                if (comboBox_DCFilter.Text == filter.Description)
                                    comboBox_DCFilter.SelectedIndex = comboBox_DCFilter.Items.IndexOf(filter.Description);
                                break;
                        }
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_EMCFilter.Text == "" || comboBox_EMCFilter.Text == null)
                    {
                        if (comboBox_EMCFilter.Items.Count > 0)
                            comboBox_EMCFilter.BackColor = Color.Gold;
                        else
                            comboBox_EMCFilter.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_EMCFilter.Items)
                            if (itm.ToString() == comboBox_EMCFilter.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_EMCFilter.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_EMCFilter.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_DCFilter.Text == "" || comboBox_DCFilter.Text == null)
                    {
                        if (comboBox_DCFilter.Items.Count > 0)
                            comboBox_DCFilter.BackColor = Color.Gold;
                        else
                            comboBox_DCFilter.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_DCFilter.Items)
                            if (itm.ToString() == comboBox_DCFilter.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_DCFilter.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_DCFilter.BackColor = Color.Salmon;
                    }

                    // - - -  BRAKE RESISTORS - - - 

                    //Query all brake resistors in the database that can be used with this drive
                    var qryBrakeResistorsPerDrive =
                                from tblBrakeResistors myBrakeResistors in dbDriveSystemsContext.tblBrakeResistors
                                join tblDriveAndBrakeResistorCombinations myDriveAndBrakeResistorCombinations
                                    in dbDriveSystemsContext.tblDriveAndBrakeResistorCombinations
                                on myBrakeResistors.ArticleNo equals myDriveAndBrakeResistorCombinations.BrakeResistorArticleNo
                                where myDriveAndBrakeResistorCombinations.DriveArticleNo == driveArticleNo
                                orderby myBrakeResistors.ArticleNo
                                select myBrakeResistors;

                    comboBox_BrakeResistor.Items.Clear();

                    //Reselect if the ComboBox text exists in the items list
                    foreach (var resistor in qryBrakeResistorsPerDrive)
                    {
                        comboBox_BrakeResistor.Items.Add(new ComboBoxItem(resistor.Description, resistor.ArticleNo));
                        if (comboBox_BrakeResistor.Text == resistor.Description)
                            comboBox_BrakeResistor.SelectedIndex = comboBox_Drive.Items.IndexOf(resistor.Description);
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_BrakeResistor.Text == "" || comboBox_BrakeResistor.Text == null)
                    {
                        if (comboBox_BrakeResistor.Items.Count > 0)
                            comboBox_BrakeResistor.BackColor = Color.Gold;
                        else
                            comboBox_BrakeResistor.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_BrakeResistor.Items)
                            if (itm.ToString() == comboBox_BrakeResistor.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_BrakeResistor.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_BrakeResistor.BackColor = Color.Salmon;
                    }

                    // - - -  OPTIONS - - - 

                    //Query all options in the database that can be used with this drive
                    var qryOptionsPerDrive =
                                from tblOptions myOptions in dbDriveSystemsContext.tblOptions
                                join tblDriveAndOptionsCombinations myDriveAndOptionsCombinations
                                    in dbDriveSystemsContext.tblDriveAndOptionsCombinations
                                on myOptions.ArticleNo equals myDriveAndOptionsCombinations.OptionArticleNo
                                where myDriveAndOptionsCombinations.DriveArticleNo == driveArticleNo
                                orderby myOptions.ArticleNo
                                select myOptions;

                    comboBox_Battery.Items.Clear();
                    comboBox_Safety.Items.Clear();
                    comboBox_IOCard.Items.Clear();
                    comboBox_IOCable.Items.Clear();
                    comboBox_ExtensionCard.Items.Clear();
                    comboBox_ExtensionCable.Items.Clear();
                    comboBox_ExtensionTerminal.Items.Clear();
                    comboBox_BusCable.Items.Clear();
                    comboBox_Controller.Items.Clear();

                    //Reselect if the ComboBox text exists in the items list
                    foreach (var option in qryOptionsPerDrive)
                    {
                        switch (option.SubType)
                        {
                            case "Safety":
                                comboBox_Safety.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_Safety.Text == option.Description)
                                    comboBox_Safety.SelectedIndex = comboBox_Safety.Items.IndexOf(option.Description);
                                break;

                            case "Bus-cable":
                                comboBox_BusCable.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_BusCable.Text == option.Description)
                                    comboBox_BusCable.SelectedIndex = comboBox_BusCable.Items.IndexOf(option.Description);
                                break;

                            case "Controller":
                                comboBox_Controller.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_Controller.Text == option.Description)
                                    comboBox_Controller.SelectedIndex = comboBox_Controller.Items.IndexOf(option.Description);
                                break;

                            case "Extension-card":
                                comboBox_ExtensionCard.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_ExtensionCard.Text == option.Description)
                                    comboBox_ExtensionCard.SelectedIndex = comboBox_ExtensionCard.Items.IndexOf(option.Description);
                                break;

                            case "Extension-cable":
                                comboBox_ExtensionCable.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_ExtensionCable.Text == option.Description)
                                    comboBox_ExtensionCable.SelectedIndex = comboBox_ExtensionCable.Items.IndexOf(option.Description);
                                break;

                            case "Extension-terminal":
                                comboBox_ExtensionTerminal.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_ExtensionTerminal.Text == option.Description)
                                    comboBox_ExtensionTerminal.SelectedIndex = comboBox_ExtensionTerminal.Items.IndexOf(option.Description);
                                break;

                            case "I/O-card":
                                comboBox_IOCard.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_IOCard.Text == option.Description)
                                    comboBox_IOCard.SelectedIndex = comboBox_IOCard.Items.IndexOf(option.Description);
                                break;

                            case "I/O-cable":
                                comboBox_IOCable.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_IOCable.Text == option.Description)
                                    comboBox_IOCable.SelectedIndex = comboBox_IOCable.Items.IndexOf(option.Description);
                                break;

                            case "Battery":
                                comboBox_Battery.Items.Add(new ComboBoxItem(option.Description, option.ArticleNo));
                                if (comboBox_Battery.Text == option.Description)
                                    comboBox_Battery.SelectedIndex = comboBox_Battery.Items.IndexOf(option.Description);
                                break;
                        }
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_Safety.Text == "" || comboBox_Safety.Text == null)
                    {
                        if (comboBox_Safety.Items.Count > 0)
                            comboBox_Safety.BackColor = Color.Gold;
                        else
                            comboBox_Safety.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_Safety.Items)
                            if (itm.ToString() == comboBox_Safety.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_Safety.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_Safety.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_BusCable.Text == "" || comboBox_BusCable.Text == null)
                    {
                        if (comboBox_BusCable.Items.Count > 0)
                            comboBox_BusCable.BackColor = Color.Gold;
                        else
                            comboBox_BusCable.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_BusCable.Items)
                            if (itm.ToString() == comboBox_BusCable.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_BusCable.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_BusCable.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_Controller.Text == "" || comboBox_Controller.Text == null)
                    {
                        if (comboBox_Controller.Items.Count > 0)
                            comboBox_Controller.BackColor = Color.Gold;
                        else
                            comboBox_Controller.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_Controller.Items)
                            if (itm.ToString() == comboBox_Controller.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_Controller.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_Controller.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_ExtensionCard.Text == "" || comboBox_ExtensionCard.Text == null)
                    {
                        if (comboBox_ExtensionCard.Items.Count > 0)
                            comboBox_ExtensionCard.BackColor = Color.Gold;
                        else
                            comboBox_ExtensionCard.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_ExtensionCard.Items)
                            if (itm.ToString() == comboBox_ExtensionCard.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_ExtensionCard.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_ExtensionCard.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_ExtensionCable.Text == "" || comboBox_ExtensionCable.Text == null)
                    {
                        if (comboBox_ExtensionCable.Items.Count > 0)
                            comboBox_ExtensionCable.BackColor = Color.Gold;
                        else
                            comboBox_ExtensionCable.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_ExtensionCable.Items)
                            if (itm.ToString() == comboBox_ExtensionCable.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_ExtensionCable.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_ExtensionCable.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_ExtensionTerminal.Text == "" || comboBox_ExtensionTerminal.Text == null)
                    {
                        if (comboBox_ExtensionTerminal.Items.Count > 0)
                            comboBox_ExtensionTerminal.BackColor = Color.Gold;
                        else
                            comboBox_ExtensionTerminal.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_ExtensionTerminal.Items)
                            if (itm.ToString() == comboBox_ExtensionTerminal.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_ExtensionTerminal.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_ExtensionTerminal.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_IOCard.Text == "" || comboBox_IOCard.Text == null)
                    {
                        if (comboBox_IOCard.Items.Count > 0)
                            comboBox_IOCard.BackColor = Color.Gold;
                        else
                            comboBox_IOCard.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_IOCard.Items)
                            if (itm.ToString() == comboBox_IOCard.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_IOCard.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_IOCard.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_IOCable.Text == "" || comboBox_IOCable.Text == null)
                    {
                        if (comboBox_IOCable.Items.Count > 0)
                            comboBox_IOCable.BackColor = Color.Gold;
                        else
                            comboBox_IOCable.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_IOCable.Items)
                            if (itm.ToString() == comboBox_IOCable.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_IOCable.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_IOCable.BackColor = Color.Salmon;
                    }

                    //Set color ´gold' if the ComboBox contains items but none has been selected
                    if (comboBox_Battery.Text == "" || comboBox_Battery.Text == null)
                    {
                        if (comboBox_Battery.Items.Count > 0)
                            comboBox_Battery.BackColor = Color.Gold;
                        else
                            comboBox_Battery.BackColor = System.Drawing.SystemColors.Window;
                    }
                    else
                    {
                        //If the ComboBox has a value then check if it is valid
                        valueExists = false;

                        foreach (ComboBoxItem itm in comboBox_Battery.Items)
                            if (itm.ToString() == comboBox_Battery.Text)
                                valueExists = true;

                        if (valueExists)
                            comboBox_Battery.BackColor = System.Drawing.SystemColors.Window;
                        else
                            comboBox_Battery.BackColor = Color.Salmon;
                    }
                }
            }
        }

        private void comboBox_EncoderCable_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Cable encoderCable = _iAccessories[0] as Cable;
                encoderCable.ArticleNo = null;

                comboBox_EncoderCable.SelectedIndex = -1;

                if (comboBox_EncoderCable.Items.Count > 0)
                    comboBox_EncoderCable.BackColor = Color.Gold;
                else
                    comboBox_EncoderCable.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_EncoderCable_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_EncoderCable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_EncoderCable.SelectedIndex != -1)
            {
                if (comboBox_EncoderCable.Text != "" && comboBox_EncoderCable.Text != null)
                {
                    Cable encoderCable = _iAccessories[0] as Cable;
                    encoderCable.ArticleNo = ((ComboBoxItem)comboBox_EncoderCable.SelectedItem).HiddenValue;
                    encoderCable.Description = comboBox_EncoderCable.Text;
                }
                comboBox_EncoderCable.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_MotorCable_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Cable motorCable = _iAccessories[1] as Cable;
                motorCable.ArticleNo = null;

                comboBox_MotorCable.SelectedIndex = -1;

                if (comboBox_MotorCable.Items.Count > 0)
                    comboBox_MotorCable.BackColor = Color.Gold;
                else
                    comboBox_MotorCable.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_MotorCable_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_MotorCable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_MotorCable.SelectedIndex != -1)
            {
                if (comboBox_MotorCable.Text != "" && comboBox_MotorCable.Text != null)
                {
                    Cable motorCable = _iAccessories[1] as Cable;
                    motorCable.ArticleNo = ((ComboBoxItem)comboBox_MotorCable.SelectedItem).HiddenValue;
                    motorCable.Description = comboBox_MotorCable.Text;
                }
                comboBox_MotorCable.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_BrakeCable_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Cable brakeCable = _iAccessories[2] as Cable;
                brakeCable.ArticleNo = null;

                comboBox_BrakeCable.SelectedIndex = -1;

                if (comboBox_BrakeCable.Items.Count > 0)
                    comboBox_BrakeCable.BackColor = Color.Gold;
                else
                    comboBox_BrakeCable.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_BrakeCable_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_BrakeCable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_BrakeCable.SelectedIndex != -1)
            {
                if (comboBox_BrakeCable.Text != "" && comboBox_BrakeCable.Text != null)
                {
                    Cable brakeCable = _iAccessories[2] as Cable;
                    brakeCable.ArticleNo = ((ComboBoxItem)comboBox_BrakeCable.SelectedItem).HiddenValue;
                    brakeCable.Description = comboBox_BrakeCable.Text;
                }
                comboBox_BrakeCable.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_Drive_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Drive drive = _iAccessories[3] as Drive;
                drive.ArticleNo = null;

                comboBox_Drive.SelectedIndex = -1;

                if (comboBox_Drive.Items.Count > 0)
                    comboBox_Drive.BackColor = Color.Gold;
                else
                    comboBox_Drive.BackColor = System.Drawing.SystemColors.Window;
                
                if (_moves.MovesList.Count > 0)
                    CalculateRegenerativeResults();
            }
        }

        private void comboBox_Drive_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_Drive_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Drive.SelectedIndex != -1)
            {
                if (comboBox_Drive.Text != "" && comboBox_Drive.Text != null)
                {
                    Drive drive = _iAccessories[3] as Drive;
                    drive.ArticleNo = ((ComboBoxItem)comboBox_Drive.SelectedItem).HiddenValue;
                    drive.Description = comboBox_Drive.Text;

                    UpdateDriveRelatedComboBoxItemsFromDB(drive.ArticleNo);

                    ReadDriveDataFromDb(drive.ArticleNo);

                    if (_moves.MovesList.Count > 0)
                        CalculateRegenerativeResults();
                }
                comboBox_Drive.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_BrakeResistor_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                BrakeResistor brakeResistor = _iAccessories[4] as BrakeResistor;
                brakeResistor.ArticleNo = null;

                comboBox_BrakeResistor.SelectedIndex = -1;

                if (comboBox_BrakeResistor.Items.Count > 0)
                    comboBox_BrakeResistor.BackColor = Color.Gold;
                else
                    comboBox_BrakeResistor.BackColor = System.Drawing.SystemColors.Window;

                if (_moves.MovesList.Count > 0) 
                    CalculateRegenerativeResults();
            }
        }

        private void comboBox_BrakeResistor_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_BrakeResistor_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (comboBox_BrakeResistor.SelectedIndex != -1)
            {
                if (comboBox_BrakeResistor.Text != "" && comboBox_BrakeResistor.Text != null)
                {
                    BrakeResistor brakeResistor = _iAccessories[4] as BrakeResistor;
                    brakeResistor.ArticleNo = ((ComboBoxItem)comboBox_BrakeResistor.SelectedItem).HiddenValue;
                    brakeResistor.Description = comboBox_BrakeResistor.Text;

                    ReadBrakeResistorDataFromDb(brakeResistor.ArticleNo);

                    if (_moves.MovesList.Count > 0) 
                        CalculateRegenerativeResults();
                }
                comboBox_BrakeResistor.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_EMCFilter_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Filter EMCFilter = _iAccessories[5] as Filter;
                EMCFilter.ArticleNo = null;

                comboBox_EMCFilter.SelectedIndex = -1;

                if (comboBox_EMCFilter.Items.Count > 0)
                    comboBox_EMCFilter.BackColor = Color.Gold;
                else
                    comboBox_EMCFilter.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_EMCFilter_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_EMCFilter_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_EMCFilter.SelectedIndex != -1)
            {
                if (comboBox_EMCFilter.Text != "" && comboBox_EMCFilter.Text != null)
                {
                    Filter EMCFilter = _iAccessories[5] as Filter;
                    EMCFilter.ArticleNo = ((ComboBoxItem)comboBox_EMCFilter.SelectedItem).HiddenValue;
                    EMCFilter.Description = comboBox_EMCFilter.Text;
                }
                comboBox_EMCFilter.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_DCFilter_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Filter DCFilter = _iAccessories[6] as Filter;
                DCFilter.ArticleNo = null;

                comboBox_DCFilter.SelectedIndex = -1;

                if (comboBox_DCFilter.Items.Count > 0)
                    comboBox_DCFilter.BackColor = Color.Gold;
                else
                    comboBox_DCFilter.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_DCFilter_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_DCFilter_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_DCFilter.SelectedIndex != -1)
            {
                if (comboBox_DCFilter.Text != "" && comboBox_DCFilter.Text != null)
                {
                    Filter DCFilter = _iAccessories[6] as Filter;
                    DCFilter.ArticleNo = ((ComboBoxItem)comboBox_DCFilter.SelectedItem).HiddenValue;
                    DCFilter.Description = comboBox_DCFilter.Text;
                }
                comboBox_DCFilter.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_Battery_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option Battery = _iAccessories[7] as Option;
                Battery.ArticleNo = null;

                comboBox_Battery.SelectedIndex = -1;

                if (comboBox_Battery.Items.Count > 0)
                    comboBox_Battery.BackColor = Color.Gold;
                else
                    comboBox_Battery.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_Battery_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_Battery_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Battery.SelectedIndex != -1)
            {
                if (comboBox_Battery.Text != "" && comboBox_Battery.Text != null)
                {
                    Option Battery = _iAccessories[7] as Option;
                    Battery.ArticleNo = ((ComboBoxItem)comboBox_Battery.SelectedItem).HiddenValue;
                    Battery.Description = comboBox_Battery.Text;
                }
                comboBox_Battery.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_Safety_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option Safety = _iAccessories[8] as Option;
                Safety.ArticleNo = null;

                comboBox_Safety.SelectedIndex = -1;

                if (comboBox_Safety.Items.Count > 0)
                    comboBox_Safety.BackColor = Color.Gold;
                else
                    comboBox_Safety.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_Safety_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_Safety_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Safety.SelectedIndex != -1)
            {
                if (comboBox_Safety.Text != "" && comboBox_Safety.Text != null)
                {
                    Option Safety = _iAccessories[8] as Option;
                    Safety.ArticleNo = ((ComboBoxItem)comboBox_Safety.SelectedItem).HiddenValue;
                    Safety.Description = comboBox_Safety.Text;
                }
                comboBox_Safety.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_IOCard_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option IOCard = _iAccessories[9] as Option;
                IOCard.ArticleNo = null;

                comboBox_IOCard.SelectedIndex = -1;

                if (comboBox_IOCard.Items.Count > 0)
                    comboBox_IOCard.BackColor = Color.Gold;
                else
                    comboBox_IOCard.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_IOCard_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_IOCard_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_IOCard.SelectedIndex != -1)
            {
                if (comboBox_IOCard.Text != "" && comboBox_IOCard.Text != null)
                {
                    Option IOCard = _iAccessories[9] as Option;
                    IOCard.ArticleNo = ((ComboBoxItem)comboBox_IOCard.SelectedItem).HiddenValue;
                    IOCard.Description = comboBox_IOCard.Text;
                }
                comboBox_IOCard.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_IOCable_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option IOCable = _iAccessories[10] as Option;
                IOCable.ArticleNo = null;

                comboBox_IOCable.SelectedIndex = -1;

                if (comboBox_IOCable.Items.Count > 0)
                    comboBox_IOCable.BackColor = Color.Gold;
                else
                    comboBox_IOCable.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_IOCable_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_IOCable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_IOCable.SelectedIndex != -1)
            {
                if (comboBox_IOCable.Text != "" && comboBox_IOCable.Text != null)
                {
                    Option IOCable = _iAccessories[10] as Option;
                    IOCable.ArticleNo = ((ComboBoxItem)comboBox_IOCable.SelectedItem).HiddenValue;
                    IOCable.Description = comboBox_IOCable.Text;
                }
                comboBox_IOCable.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_BusCable_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option BusCable = _iAccessories[11] as Option;
                BusCable.ArticleNo = null;

                comboBox_BusCable.SelectedIndex = -1;

                if (comboBox_BusCable.Items.Count > 0)
                    comboBox_BusCable.BackColor = Color.Gold;
                else
                    comboBox_BusCable.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_BusCable_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_BusCable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_BusCable.SelectedIndex != -1)
            {
                if (comboBox_BusCable.Text != "" && comboBox_BusCable.Text != null)
                {
                    Option BusCable = _iAccessories[11] as Option;
                    BusCable.ArticleNo = ((ComboBoxItem)comboBox_BusCable.SelectedItem).HiddenValue;
                    BusCable.Description = comboBox_BusCable.Text;
                }
                comboBox_BusCable.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_Controller_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option Controller = _iAccessories[12] as Option;
                Controller.ArticleNo = null;

                comboBox_Controller.SelectedIndex = -1;

                if (comboBox_Controller.Items.Count > 0)
                    comboBox_Controller.BackColor = Color.Gold;
                else
                    comboBox_Controller.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_Controller_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_Controller_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_Controller.SelectedIndex != -1)
            {
                if (comboBox_Controller.Text != "" && comboBox_Controller.Text != null)
                {
                    Option Controller = _iAccessories[12] as Option;
                    Controller.ArticleNo = ((ComboBoxItem)comboBox_Controller.SelectedItem).HiddenValue;
                    Controller.Description = comboBox_Controller.Text;
                }
                comboBox_Controller.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_ExtensionCard_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option ExtensionCard = _iAccessories[13] as Option;
                ExtensionCard.ArticleNo = null;

                comboBox_ExtensionCard.SelectedIndex = -1;

                if (comboBox_ExtensionCard.Items.Count > 0)
                    comboBox_ExtensionCard.BackColor = Color.Gold;
                else
                    comboBox_ExtensionCard.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_ExtensionCard_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_ExtensionCard_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_ExtensionCard.SelectedIndex != -1)
            {
                if (comboBox_ExtensionCard.Text != "" && comboBox_ExtensionCard.Text != null)
                {
                    Option ExtensionCard = _iAccessories[13] as Option;
                    ExtensionCard.ArticleNo = ((ComboBoxItem)comboBox_ExtensionCard.SelectedItem).HiddenValue;
                    ExtensionCard.Description = comboBox_ExtensionCard.Text;
                }
                comboBox_ExtensionCard.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_ExtensionCable_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option ExtensionCable = _iAccessories[14] as Option;
                ExtensionCable.ArticleNo = null;

                comboBox_ExtensionCable.SelectedIndex = -1;

                if (comboBox_ExtensionCable.Items.Count > 0)
                    comboBox_ExtensionCable.BackColor = Color.Gold;
                else
                    comboBox_ExtensionCable.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_ExtensionCable_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_ExtensionCable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_ExtensionCable.SelectedIndex != -1)
            {
                if (comboBox_ExtensionCable.Text != "" && comboBox_ExtensionCable.Text != null)
                {
                    Option ExtensionCable = _iAccessories[14] as Option;
                    ExtensionCable.ArticleNo = ((ComboBoxItem)comboBox_ExtensionCable.SelectedItem).HiddenValue;
                    ExtensionCable.Description = comboBox_ExtensionCable.Text;
                }
                comboBox_ExtensionCable.BackColor = SystemColors.Window;
            }
        }

        private void comboBox_ExtensionTerminal_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                Option ExtensionTerminal = _iAccessories[15] as Option;
                ExtensionTerminal.ArticleNo = null;

                comboBox_ExtensionTerminal.SelectedIndex = -1;

                if (comboBox_ExtensionTerminal.Items.Count > 0)
                    comboBox_ExtensionTerminal.BackColor = Color.Gold;
                else
                    comboBox_ExtensionTerminal.BackColor = System.Drawing.SystemColors.Window;
            }
        }

        private void comboBox_ExtensionTerminal_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        private void comboBox_ExtensionTerminal_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox_ExtensionTerminal.SelectedIndex != -1)
            {
                if (comboBox_ExtensionTerminal.Text != "" && comboBox_ExtensionTerminal.Text != null)
                {
                    Option ExtensionTerminal = _iAccessories[15] as Option;
                    ExtensionTerminal.ArticleNo = ((ComboBoxItem)comboBox_ExtensionTerminal.SelectedItem).HiddenValue;
                    ExtensionTerminal.Description = comboBox_ExtensionTerminal.Text;
                }
                comboBox_ExtensionTerminal.BackColor = SystemColors.Window;
            }
        }

        private void ReadDriveDataFromDb(string driveArticleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {

                    //Query all gearboxes in the database that can be used with this motor
                    var qryDrives =
                        from tblDrives myDrive in dbDriveSystemsContext.tblDrives
                        where myDrive.ArticleNo == driveArticleNo
                        select myDrive;

                    foreach (var dBDrive in qryDrives) //Should only be one item
                    {
                        #region ReadDriveData

                        _drive.ArticleNo = dBDrive.ArticleNo;
                        _drive.CapCharge = dBDrive.CapacitorCharge;
                        _drive.IntBrkRes.MaxPwr = dBDrive.IntBrakeResMaxPwr;
                        _drive.IntBrkRes.MaxPwrDur = dBDrive.IntBrakeResDur;
                        _drive.IntBrkRes.PwrED = dBDrive.IntBrakeResED;
                        _drive.IntBrkRes.Resistance = dBDrive.Resistance;

                        #endregion
                    }

                    _drive.IntBrkRes.NomPwr = Math.Sqrt((Math.Pow(_drive.IntBrkRes.MaxPwr, 2) * _drive.IntBrkRes.MaxPwrDur) / (_drive.IntBrkRes.MaxPwrDur / (_drive.IntBrkRes.PwrED / 100)));
                }
            }
        }

        private void ReadBrakeResistorDataFromDb(string brakeResistorArticleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {

                    //Query all gearboxes in the database that can be used with this motor
                    var qryBrakeResistors =
                        from tblBrakeResistors myBrakeResistor in dbDriveSystemsContext.tblBrakeResistors
                        where myBrakeResistor.ArticleNo == brakeResistorArticleNo
                        select myBrakeResistor;

                    foreach (var dBBrakeResistor in qryBrakeResistors) //Should only be one item
                    {
                        #region ReadBrakeResistorData

                        _brakeResistor.ArticleNo = dBBrakeResistor.ArticleNo;
                        _brakeResistor.MaxPwr = dBBrakeResistor.MaxPwr;
                        _brakeResistor.MaxPwrDur = dBBrakeResistor.MaxPwrDur;
                        _brakeResistor.PwrED = dBBrakeResistor.PwrED;
                        _brakeResistor.Resistance = dBBrakeResistor.Resistance;

                        #endregion
                    }

                    _brakeResistor.NomPwr = Math.Sqrt((Math.Pow(_brakeResistor.MaxPwr, 2) * _brakeResistor.MaxPwrDur) / (_brakeResistor.MaxPwrDur / (_brakeResistor.PwrED / 100)));
                }
            }
        }

        private void CalculateRegenerativeResults()
        {
            bool useInternal = false;
            bool useExternal = false;

            double nomPercent;
            double peakPercent;

            double truePower;
            double truePercent;

            double dutyCycleReg;
            double nominalPower;
            double maximumPower;
            double resistance;

            //Internal or external brake resistor?
            if (comboBox_Drive.Text != "" && comboBox_Drive.Text != null)
                if (comboBox_Drive.BackColor != Color.Salmon)
                    useInternal = true;

            if (comboBox_BrakeResistor.Text != "" && comboBox_BrakeResistor.Text != null)
                if (comboBox_BrakeResistor.BackColor != Color.Salmon)
                    if (useInternal)
                    {
                        useExternal = true;
                        useInternal = false;
                    }

            if (useInternal)
            {
                dutyCycleReg = _drive.IntBrkRes.MaxPwrDur * 1000;
                nominalPower = _drive.IntBrkRes.NomPwr;
                maximumPower = _drive.IntBrkRes.MaxPwr;
                resistance = _drive.IntBrkRes.Resistance;
            }
            else
            {
                dutyCycleReg = _brakeResistor.MaxPwrDur * 1000;
                nominalPower = _brakeResistor.NomPwr;
                maximumPower = _brakeResistor.MaxPwr;
                resistance = _brakeResistor.Resistance;
            }

            _moves.MotorResults.QueueRmsReg.Clear();
            _moves.MotorResults.RmsRegSum = 0;
            _moves.MotorResults.RmsRegED = 0;
            _moves.MotorResults.PeakReg = 0;

            if (useInternal || useExternal)
            {
                foreach (SectorElement mp in _moves.ActiveSectorElementsList)
                {
                    double regPower = Math.Pow(mp.RegenerativePower, 2);

                    //Save the peak regenerative value
                    if (mp.RegenerativePower > _moves.MotorResults.PeakReg)
                        _moves.MotorResults.PeakReg = mp.RegenerativePower;

                    //Add the current speed value to the FIFO
                    _moves.MotorResults.QueueRmsReg.Enqueue(regPower);

                    //Add the result to a sum for all rms-values for the given duration
                    _moves.MotorResults.RmsRegSum += regPower;

                    //Check if the number of items in the FIFO exceeds the time duration
                    if (_moves.MotorResults.QueueRmsReg.Count > dutyCycleReg)
                    {
                        if (Math.Sqrt(_moves.MotorResults.RmsRegSum / dutyCycleReg) > _moves.MotorResults.RmsRegED)
                            _moves.MotorResults.RmsRegED = Math.Sqrt(_moves.MotorResults.RmsRegSum / dutyCycleReg);

                        //Remove the oldest value from the FIFO
                        _moves.MotorResults.RmsRegSum -= _moves.MotorResults.QueueRmsReg.Peek();
                        _moves.MotorResults.QueueRmsReg.Dequeue();
                    }
                }

                if (_moves.MotorResults.RmsRegED == 0)
                    _moves.MotorResults.RmsRegED = Math.Sqrt(_moves.MotorResults.RmsRegSum / _moves.MotorResults.QueueRmsReg.Count);

                nomPercent = FormatValue(_moves.MotorResults.RmsRegED / nominalPower) * 100;
                peakPercent = FormatValue(_moves.MotorResults.PeakReg / maximumPower) * 100;
            
                textBox_RegenerativePowerRms.Text = FormatValue(_moves.MotorResults.RmsRegED).ToString() + " (" + nomPercent + "%)";

                if (nomPercent > 100)
                    textBox_RegenerativePowerRms.BackColor = Color.Salmon;
                else
                    textBox_RegenerativePowerRms.BackColor = System.Drawing.SystemColors.Control;
                                
                textBox_RegenerativePowerPeak.Text = FormatValue(_moves.MotorResults.PeakReg).ToString() + " (" + peakPercent + "%)";

                if (peakPercent > 100)
                    textBox_RegenerativePowerPeak.BackColor = Color.Salmon;
                else
                    textBox_RegenerativePowerPeak.BackColor = System.Drawing.SystemColors.Control;

                truePower = (_motor.InvEff / 100) * (_moves.MotorResults.RmsRegED - (Settings.MainSettings.CapacitorCharge / (_moves.ActiveSectorElementsList.Count() / 1000)));
                truePercent = FormatValue(truePower / nominalPower) * 100;

                textBox_RegenerativePowerTrue.Text = FormatValue(truePower).ToString() + " (" + truePercent + "%)";

                if (truePercent > 100)
                    textBox_RegenerativePowerTrue.BackColor = Color.Salmon;
                else
                    textBox_RegenerativePowerTrue.BackColor = System.Drawing.SystemColors.Control;
            }
            else
            {
                textBox_RegenerativePowerRms.Text = "0 (0%)";
                textBox_RegenerativePowerRms.BackColor = System.Drawing.SystemColors.Control;

                textBox_RegenerativePowerPeak.Text = "0 (0%)";
                textBox_RegenerativePowerPeak.BackColor = System.Drawing.SystemColors.Control;

                textBox_RegenerativePowerTrue.Text = "0 (0%)";
                textBox_RegenerativePowerTrue.BackColor = System.Drawing.SystemColors.Control;
            }
        }

        public GroupBox GetRegenerativeGroupBox()
        {
            return groupBox_Regenerative;
        }
    }
}