using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.Windows.Forms.DataVisualization.Charting;

namespace DockSample
{
    public partial class FrmGraph : ToolWindow
    {
        private int _magnify;

        public FrmGraph()
        {
            InitializeComponent();
        }

        public int Magnify
        {
            get { return _magnify; }
            set
            {
                if (value != this._magnify)
                {
                    if (value > 40)
                        value = 40;
                    if (value < 2)
                        value = 2;

                    this._magnify = value;
                }
            }
        }

        public void ClearJerkAccAndVelDistGraph()
        {
            //Jerk & Acc
            while (chartJerkAndAcc.ChartAreas.Count > 0)
                chartJerkAndAcc.ChartAreas.RemoveAt(0);

            while (chartJerkAndAcc.Series.Count > 0)
                chartJerkAndAcc.Series.RemoveAt(0);

            //Vel o Dist
            while (chartVelAndDist.ChartAreas.Count > 0)
                chartVelAndDist.ChartAreas.RemoveAt(0);

            while (chartVelAndDist.Series.Count > 0)
                chartVelAndDist.Series.RemoveAt(0);
        }

        public void CreateJerkAccAndVelDistGraphs(IList<SectorElement> elementsList)
        {
            //Jerk & Acc
            while (chartJerkAndAcc.ChartAreas.Count > 0)
                chartJerkAndAcc.ChartAreas.RemoveAt(0);

            chartJerkAndAcc.ChartAreas.Add("JerkAndAccChartArea");

            chartJerkAndAcc.ChartAreas[0].AxisY.Title = "Jerk [m/s]";


            chartJerkAndAcc.ChartAreas[0].AxisY2.Title = "Acc [m/s]";

            while (chartJerkAndAcc.Series.Count > 0)
                chartJerkAndAcc.Series.RemoveAt(0);

            Series jerkTable = new Series();


            jerkTable = chartJerkAndAcc.Series.Add("Jerk");
            jerkTable.ChartType = SeriesChartType.Line;
            jerkTable.YAxisType = AxisType.Primary;
            jerkTable.Color = Color.Peru;

            chartJerkAndAcc.Series["Jerk"].BorderWidth = 2;
            //chartJerkAndAcc.Series["Jerk"].BorderDashStyle = ChartDashStyle.Dash;
            
            Series accTable = new Series();

            accTable = chartJerkAndAcc.Series.Add("Acceleration");
            accTable.ChartType = SeriesChartType.Line;
            accTable.YAxisType = AxisType.Secondary;
            accTable.Color = Color.Crimson;

            chartJerkAndAcc.Series["Acceleration"].BorderWidth = 2;
            
            //Vel o Dist
            while (chartVelAndDist.ChartAreas.Count > 0)
                chartVelAndDist.ChartAreas.RemoveAt(0);

            chartVelAndDist.ChartAreas.Add("VelAndDistChartArea");

            chartVelAndDist.ChartAreas[0].AxisY.Title = "Vel [m/s]";

            chartVelAndDist.ChartAreas[0].AxisY2.Title = "Dist [m]";

            while (chartVelAndDist.Series.Count > 0)
                chartVelAndDist.Series.RemoveAt(0);

            Series velTable = new Series();

            velTable = chartVelAndDist.Series.Add("Velocity");
            velTable.ChartType = SeriesChartType.Line;
            velTable.YAxisType = AxisType.Primary;
            velTable.Color = Color.OliveDrab;

            chartVelAndDist.Series["Velocity"].BorderWidth = 2;
            
            Series distTable = new Series();

            distTable = chartVelAndDist.Series.Add("Distance");
            distTable.ChartType = SeriesChartType.Line;
            distTable.YAxisType = AxisType.Secondary;
            distTable.Color = Color.RoyalBlue;

            chartVelAndDist.Series["Distance"].BorderWidth = 2;
                //chartVelAndDist.Series["Distance"].BorderDashStyle = ChartDashStyle.Dash;
            
            int inc = Convert.ToInt32(Math.Round(elementsList.Count / 10000.0, 0));
            if (inc < 1)
                inc = 1;
         
            //Add graph data
            for (int i = 0; i < elementsList.Count; i += inc)
            {
                jerkTable.Points.AddXY(elementsList[i].Time, elementsList[i].Jerk);

                accTable.Points.AddXY(elementsList[i].Time, elementsList[i].Acc);

                distTable.Points.AddXY(elementsList[i].Time, elementsList[i].Pos);

                velTable.Points.AddXY(elementsList[i].Time, elementsList[i].Vel);
            }
        }

        public void AddMotorSpeedToVelGraph(IList<SectorElement> elementsList, double scale, double nomSpd, double maxSpd)
        {
            //Clear any existing old data
            for (int i = 0; i < chartVelAndDist.Series.Count; i++)
                if (chartVelAndDist.Series[i].Name == "Velocity")
                    chartVelAndDist.Series.RemoveAt(i);

            chartVelAndDist.ChartAreas[0].AxisY.Title = "Vel [rpm]";

            Series velTable = new Series();

            velTable = chartVelAndDist.Series.Add("Velocity");
            velTable.ChartType = SeriesChartType.Line;
            velTable.YAxisType = AxisType.Primary;
            velTable.Color = Color.OliveDrab;

            chartVelAndDist.Series["Velocity"].BorderWidth = 2;
            

            double max = 0;
            double min = 0;

            int inc = Convert.ToInt32(Math.Round(elementsList.Count / 1000.0, 0));
            
            if (inc < 1)
                inc = 1;
            
            //Add graph data
            for (int j = 0; j < elementsList.Count; j += inc)
                {
                    velTable.Points.AddXY(elementsList[j].Time, elementsList[j].Vel * scale);
                    if (elementsList[j].Vel * scale > max)
                        max = elementsList[j].Vel * scale;

                    if (elementsList[j].Vel * scale < min)
                        min = elementsList[j].Vel * scale;
                }

            for (int k = 0; k < chartVelAndDist.ChartAreas[0].AxisY.StripLines.Count; k++)
                chartVelAndDist.ChartAreas[0].AxisY.StripLines.RemoveAt(k);

            chartVelAndDist.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[0].BackColor = Color.LightSalmon;
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[0].StripWidth = max;
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[0].IntervalOffset = maxSpd;

            chartVelAndDist.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[1].BackColor = Color.PapayaWhip;  //FromArgb(80, 252, 180, 65);
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[1].StripWidth = maxSpd - nomSpd;
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[1].IntervalOffset = nomSpd;


            int lineNr = chartVelAndDist.ChartAreas[0].AxisY.StripLines.Count;

            chartVelAndDist.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].BackColor = Color.LightSalmon;
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth = Math.Abs(min);
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].IntervalOffset =
                -maxSpd - chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth;

            lineNr++;

            chartVelAndDist.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].BackColor = Color.PapayaWhip;
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth = maxSpd - nomSpd;
            chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].IntervalOffset =
                -nomSpd - chartVelAndDist.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth;
        }

        public void CreateTorqueGraph(IList<SectorElement> elementsList, double nomTrq, double maxTrq)
        {
            //Torque
            while (chartTorqueAndRegenPwr.ChartAreas.Count > 0)
                chartTorqueAndRegenPwr.ChartAreas.RemoveAt(0);

            chartTorqueAndRegenPwr.ChartAreas.Add("TorqueAndRegenPwrChartArea");
            chartTorqueAndRegenPwr.ChartAreas[0].AxisY.Title = "Torque [Nm]";

            while (chartTorqueAndRegenPwr.Series.Count > 0)
                chartTorqueAndRegenPwr.Series.RemoveAt(0);

            Series torqueTable = chartTorqueAndRegenPwr.Series.Add("Torque");
            torqueTable.ChartType = SeriesChartType.Line;
            torqueTable.YAxisType = AxisType.Primary;
            torqueTable.Color = Color.Sienna;

            chartTorqueAndRegenPwr.Series[0].BorderWidth = 2;

            //RegenPwr
            chartTorqueAndRegenPwr.ChartAreas[0].AxisY2.Title = "RegenPwr [W]";

            Series regenPwrTable = chartTorqueAndRegenPwr.Series.Add("RegenPwr");
            regenPwrTable.ChartType = SeriesChartType.Line;
            regenPwrTable.YAxisType = AxisType.Secondary;
            regenPwrTable.Color = Color.MediumVioletRed;

            chartTorqueAndRegenPwr.Series[1].BorderWidth = 2;

            double max = 0;
            double min = 0;

            bool drawTrueMaxPos = false;
            bool drawTrueMaxNeg = false;

            int inc = Convert.ToInt32(Math.Round(elementsList.Count / 10000.0, 0));

            if (inc < 1)
                inc = 1;

            //Add graph data
            for (int i = 0; i < elementsList.Count; i += inc)
            {
                torqueTable.Points.AddXY(elementsList[i].Time, elementsList[i].Torque);

                regenPwrTable.Points.AddXY(elementsList[i].Time, elementsList[i].RegenerativePower);

                if (elementsList[i].Torque > max)
                    max = elementsList[i].Torque;

                if (max > elementsList[i].MotorTorqueLimit)
                    drawTrueMaxPos = true;

                if (elementsList[i].Torque < min)
                    min = elementsList[i].Torque;

                if (Math.Abs(min) > elementsList[i].MotorTorqueLimit)
                    drawTrueMaxNeg = true;
            }

            for (int j = 0; j < chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Count; j++)
                chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.RemoveAt(j);

            //Do not draw any strip lines if the max torque is undefined
            if (maxTrq != 0)
            {
                if (drawTrueMaxPos)
                {
                    Series trueMaxTable = new Series("trueMax+");
                    trueMaxTable.ChartType = SeriesChartType.Line;
                    trueMaxTable.Color = Color.Red;

                    chartTorqueAndRegenPwr.Series.Add(trueMaxTable);
                }

                if (drawTrueMaxNeg)
                {
                    Series trueMaxTable = new Series("trueMax-");
                    trueMaxTable.ChartType = SeriesChartType.Line;
                    trueMaxTable.Color = Color.Red;

                    chartTorqueAndRegenPwr.Series.Add(trueMaxTable);
                }

                foreach (SectorElement sElm in elementsList)
                {
                    if (drawTrueMaxPos)
                        chartTorqueAndRegenPwr.Series["trueMax+"].Points.AddXY(sElm.Time, sElm.MotorTorqueLimit);

                    if (drawTrueMaxNeg)
                        chartTorqueAndRegenPwr.Series["trueMax-"].Points.AddXY(sElm.Time, -1 * sElm.MotorTorqueLimit);
                }

            //    if (max > nomTrq)
            //    {
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[0].BackColor = Color.LightSalmon;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[0].StripWidth = max;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[0].IntervalOffset = maxTrq;

                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[1].BackColor = Color.PapayaWhip;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[1].StripWidth = maxTrq - nomTrq;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[1].IntervalOffset = nomTrq;
             //   }
            /*    else
                    if (max > nomTrq)
                    {
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[0].BackColor = Color.PapayaWhip;
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[0].StripWidth = max - nomTrq;
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[0].IntervalOffset = nomTrq;
                    }
            */
                int lineNr = chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Count;

              //  if (Math.Abs(min) > nomTrq)
              //  {
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].BackColor = Color.LightSalmon;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth = Math.Abs(min);
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].IntervalOffset =
                        -maxTrq - chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth;

                    lineNr++;

                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].BackColor = Color.PapayaWhip;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth = maxTrq - nomTrq;
                    chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].IntervalOffset =
                        -nomTrq - chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth;
               // }
            /*    else
                    if (Math.Abs(min) > nomTrq)
                    {
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Add(new StripLine());
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].BackColor = Color.PapayaWhip;
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth = Math.Abs(min) - nomTrq;
                        chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].IntervalOffset =
                            -nomTrq - chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[lineNr].StripWidth;
                    }
            */}
        }

        public void ClearTorqueAndRegenPwrGraph()
        {
            while (chartTorqueAndRegenPwr.ChartAreas.Count > 0)
                chartTorqueAndRegenPwr.ChartAreas.RemoveAt(0);

            chartTorqueAndRegenPwr.ChartAreas.Add("TorqueChartArea");
            chartTorqueAndRegenPwr.ChartAreas[0].AxisY.Title = "Torque [Nm]";

            while (chartTorqueAndRegenPwr.Series.Count > 0)
                chartTorqueAndRegenPwr.Series.RemoveAt(0);
        }

        public void CreateTorquePerComponentGraph(MotorResult motorResult, SortedList<int, LoadResult> loadResultsList)
        {
            //Motor usage
            while (chartTorquePerComponent.ChartAreas.Count > 0)
                chartTorquePerComponent.ChartAreas.RemoveAt(0);

            chartTorquePerComponent.ChartAreas.Add("TorquePerComponentChartArea");

            while (chartTorquePerComponent.Series.Count > 0)
                chartTorquePerComponent.Series.RemoveAt(0);

            if (loadResultsList.Count > 0 && motorResult.RmsTrqED > 0)
            {
                Series torquePerComponentTable = chartTorquePerComponent.Series.Add("TorquePerComponent");
                //chartTorquePerComponent.Series["TorquePerComponent"].Palette = ChartColorPalette.BrightPastel;
                torquePerComponentTable.ChartType = SeriesChartType.Pie;
                chartTorquePerComponent.ChartAreas["TorquePerComponentChartArea"].Area3DStyle.Enable3D = true;
                chartTorquePerComponent.Series["TorquePerComponent"].IsValueShownAsLabel = true;
                chartTorquePerComponent.Series["TorquePerComponent"]["PieLabelStyle"] = "Outside";
                chartTorquePerComponent.Series["TorquePerComponent"].Label = "#VALX #PERCENT{P1}";
                chartTorquePerComponent.Series["TorquePerComponent"].Font = new Font("Times New Roman", 8);
                
                List<string> componentsNameList = new List<string>();
                List<double> componentsTorqueList = new List<double>();

                foreach (var lr in loadResultsList)
                {
                    LoadResult loadResult = lr.Value;

                    if (loadResult.Active || !loadResult.Active)
                    {
                        componentsNameList.Add(loadResult.Name);
                        componentsTorqueList.Add(loadResult.RmsTrqED / loadResult.AccInpGearing);
                    }
                }

                chartTorquePerComponent.Series[0].Points.AddXY("Motor", motorResult.RmsTrqED - componentsTorqueList[0]);

                for (int i = 0; i < componentsNameList.Count; i++)
                {
                    if (i < componentsNameList.Count - 1)
                        chartTorquePerComponent.Series[0].Points.AddXY(componentsNameList[i], Math.Abs(componentsTorqueList[i] - componentsTorqueList[i + 1]));
                    else
                        chartTorquePerComponent.Series[0].Points.AddXY(componentsNameList[i], Math.Abs(componentsTorqueList[i]));
                }
            }
        }

        public void ClearTorquePerComponentsGraph()
        {
            while (chartTorquePerComponent.ChartAreas.Count > 0)
                chartTorquePerComponent.ChartAreas.RemoveAt(0);

            while (chartTorquePerComponent.Series.Count > 0)
                chartTorquePerComponent.Series.RemoveAt(0);
        }

        #region PrintMethods
        //Return the requested object to the main form for printing

        public Chart GetJerkAndAccChart()
        {
           /* Chart chartCopy = new Chart();

            for (int areaNr = 0; areaNr < chartJerkAndAcc.ChartAreas.Count; areaNr++)
                chartCopy.ChartAreas.Add(chartJerkAndAcc.ChartAreas[areaNr]);

            for (int legendNr = 0; legendNr < chartJerkAndAcc.Legends.Count; legendNr++)
                chartCopy.Legends.Add(chartJerkAndAcc.Legends[legendNr]);
            
            chartCopy.ChartAreas[0].AxisY.Title = chartJerkAndAcc.ChartAreas[0].AxisY.Title;
            chartCopy.ChartAreas[0].AxisY2.Title = chartJerkAndAcc.ChartAreas[0].AxisY2.Title;

            chartCopy.ChartAreas[0].AxisY.LabelAutoFitMaxFontSize = 8;
            chartCopy.ChartAreas[0].AxisY2.LabelAutoFitMaxFontSize = 8; 
            
            for (int seriesNr = 0; seriesNr < chartJerkAndAcc.Series.Count; seriesNr++)
                chartCopy.Series.Add(chartJerkAndAcc.Series[seriesNr]);

            for (int stripLineNr = 0; stripLineNr < chartJerkAndAcc.ChartAreas[0].AxisY.StripLines.Count; stripLineNr++)
                chartCopy.ChartAreas[0].AxisY.StripLines.Add(chartJerkAndAcc.ChartAreas[0].AxisY.StripLines[stripLineNr]);
            */
            return chartJerkAndAcc;
        }

        public Chart GetVelAndDistChart()
        {
            Chart chartCopy = new Chart();

           /* for (int areaNr = 0; areaNr < chartVelAndDist.ChartAreas.Count; areaNr++)
                chartCopy.ChartAreas.Add(chartVelAndDist.ChartAreas[areaNr]);

            for (int legendNr = 0; legendNr < chartVelAndDist.Legends.Count; legendNr++)
                chartCopy.Legends.Add(chartVelAndDist.Legends[legendNr]);

            chartCopy.ChartAreas[0].AxisY.Title = chartVelAndDist.ChartAreas[0].AxisY.Title;
            chartCopy.ChartAreas[0].AxisY2.Title = chartVelAndDist.ChartAreas[0].AxisY2.Title;

            chartCopy.ChartAreas[0].AxisY.LabelAutoFitMaxFontSize = 8;
            chartCopy.ChartAreas[0].AxisY2.LabelAutoFitMaxFontSize = 8; 

            for (int seriesNr = 0; seriesNr < chartVelAndDist.Series.Count; seriesNr++)
                chartCopy.Series.Add(chartVelAndDist.Series[seriesNr]);

            //for (int stripLineNr = 0; stripLineNr < chartVelAndDist.ChartAreas[0].AxisY.StripLines.Count; stripLineNr++)
                //chartCopy.ChartAreas[0].AxisY.StripLines.Add(chartVelAndDist.ChartAreas[0].AxisY.StripLines[stripLineNr]);
            */

            return chartVelAndDist;
        }

        public Chart GetTrqChart()
        {
            /*Chart chartCopy = new Chart();

            for (int areaNr = 0; areaNr < chartTorqueAndRegenPwr.ChartAreas.Count; areaNr++)
                chartCopy.ChartAreas.Add(chartTorqueAndRegenPwr.ChartAreas[areaNr]);

            for (int legendNr = 0; legendNr < chartTorqueAndRegenPwr.Legends.Count; legendNr++)
                chartCopy.Legends.Add(chartTorqueAndRegenPwr.Legends[legendNr]);

            chartCopy.ChartAreas[0].AxisY.Title = chartTorqueAndRegenPwr.ChartAreas[0].AxisY.Title;

            chartCopy.ChartAreas[0].AxisY.LabelAutoFitMaxFontSize = 8;
 
            for (int seriesNr = 0; seriesNr < chartTorqueAndRegenPwr.Series.Count; seriesNr++)
                chartCopy.Series.Add(chartTorqueAndRegenPwr.Series[seriesNr]);
            */
            //for (int stripLineNr = 0; stripLineNr < chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines.Count; stripLineNr++)
                //chartCopy.ChartAreas[0].AxisY.StripLines.Add(chartTorqueAndRegenPwr.ChartAreas[0].AxisY.StripLines[stripLineNr]); 
            
            return chartTorqueAndRegenPwr;
        }

        public Chart GetTrqPerComponentChart()
        {
            return chartTorquePerComponent;
        }

        #endregion

    }
}