﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.IO;
using System.Linq;
using System.Data.OleDb;
using System.Globalization;

using DGVPrinterHelper;
using System.Drawing.Printing;

namespace DockSample
{
    public partial class FrmMove : DockContent
    {
        private Move _move;
        private SortedList<int, ILoad> _iLoads;
        private Motor _motor; 

        //Form constructor
        public FrmMove(Move move, SortedList<int, ILoad> iLoads, Motor motor)
        {
            _move = move as Move;
            _iLoads = iLoads as SortedList<int, ILoad>;
            _motor = motor as Motor;
            _move.PropertyChanged += new PropertyChangedEventHandler(Move_PropertyChanged);

            InitializeComponent();
        }

        #region Events

        private void FrmMove_Load(object sender, EventArgs e)
        {
            //Configure the controls to be shown or hidden depending on MoveType
            if (_move.MoveType == MoveType.PreDefined)
            {
                checkBox_LockTime.Enabled = false;
                checkBox_LockTime.Checked = true;
                textBox_Time.Enabled = true;

                checkBox_LockDistance.Enabled = false;
                checkBox_LockDistance.Checked = true;

                if (_move.DistanceInDegrees)
                {
                    radioButton_DistanceUnitDegrees.Checked = true;
                    radioButton_DistanceUnitMeters.Checked = false;
                    textBox_DistanceInMeters.Enabled = false;
                    textBox_DistanceInDegrees.Enabled = true;
                    textBox_DistanceInDegrees.Text = _move.Degrees.ToString();
                }
                else
                {
                    radioButton_DistanceUnitDegrees.Checked = false;
                    radioButton_DistanceUnitMeters.Checked = true;
                    textBox_DistanceInMeters.Enabled = true;
                    textBox_DistanceInDegrees.Enabled = false;
                    textBox_DistanceInDegrees.Text = "0";
                }

                checkBox_LockFinalSpeed.Enabled = false;
                checkBox_LockAcc.Enabled = false;
            }
            else
            {
                groupBox_Profile.Enabled = false;
                radioButton_SymmetricProfile.Checked = false;
                radioButton_TriangularProfile.Checked = false;
                textBox_InitialSpeed.Enabled = true;

                if (_move.TimeLocked)
                {
                    checkBox_LockTime.Checked = true;
                    textBox_Time.Enabled = true;
                }
                if (_move.DistanceLocked)
                {
                    checkBox_LockDistance.Checked = true;

                    if (_move.DistanceInDegrees)
                    {
                        radioButton_DistanceUnitDegrees.Checked = true;
                        radioButton_DistanceUnitMeters.Checked = false;
                        textBox_DistanceInMeters.Enabled = false;
                        textBox_DistanceInDegrees.Enabled = true;
                    }
                    else
                    {
                        radioButton_DistanceUnitDegrees.Checked = false;
                        radioButton_DistanceUnitMeters.Checked = true;
                        textBox_DistanceInMeters.Enabled = true;
                        textBox_DistanceInDegrees.Enabled = false;
                    }
                }
                if (_move.FinalSpeedLocked)
                {
                    checkBox_LockFinalSpeed.Checked = false;
                    textBox_FinalSpeed.Enabled = true;
                }
                if (_move.AverageAccLocked)
                {
                    checkBox_LockAcc.Checked = true;
                    textBox_AverageAcc.Enabled = true;
                }
            }

            //Update the form textboxes
            textBox_Name.Text = _move.Name;

            textBox_DistanceInMeters.Text = Math.Round(_move.Distance, 3).ToString();
            textBox_Time.Text = Math.Round(_move.Time, 3).ToString();

            textBox_InitialSpeed.Text = Math.Round(_move.InitialSpeed, 3).ToString();
            textBox_FinalSpeed.Text = Math.Round(_move.FinalSpeed, 3).ToString();

            textBox_AverageAcc.Text = Math.Round(_move.AverageAcc, 3).ToString();

            textBox_PercentSineVel.Text = Math.Round(_move.PercentSineVel, 0).ToString();
            textBox_PercentSineAcc.Text = Math.Round(_move.PercentSineAcc, 0).ToString();

        }

        private void FrmMove_Activated(object sender, EventArgs e)
        {
            _move.Refresh = true; 
        }

        private void Move_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //Catch move events
            switch (e.PropertyName)
            {
                case "Distance":
                    textBox_DistanceInMeters.Text = formatValue(_move.Distance).ToString();
                    break;

                case "Time":
                    textBox_Time.Text = formatValue(_move.Time).ToString();
                    break;

                case "InitialSpeed":
                    textBox_InitialSpeed.Text = formatValue(_move.InitialSpeed).ToString();
                    break;

                case "FinalSpeed":
                    textBox_FinalSpeed.Text = formatValue(_move.FinalSpeed).ToString();
                    break;

                case "AverageAcc":
                    textBox_AverageAcc.Text = formatValue(_move.AverageAcc).ToString();
                    break;

                case "PercentSineVel":
                    textBox_PercentSineVel.Text = Math.Round(_move.PercentSineVel, 0).ToString();
                    break;

                case "PercentSineAcc":
                    textBox_PercentSineAcc.Text = Math.Round(_move.PercentSineAcc, 0).ToString();
                    break;
            }        
        }

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        #region DefineMove

        private void textBox_Name_Validated(object sender, EventArgs e)
        {
            if (_move.Name != textBox_Name.Text)
            {
                _move.Name = textBox_Name.Text;
                renameSector();
            }
        }

        private void textBox_Name_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                if (_move.Name != textBox_Name.Text)
                {
                    _move.Name = textBox_Name.Text;
                    renameSector();
                }
            }
        }

        private void radioButton_TriangularProfile_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_TriangularProfile.Checked == true)
            {
                //Remove the constant sector if it exists
                if (_move.SectorsList.Count == 3)
                    _move.SectorsList.RemoveAt(1);

                _move.Profile = Profile.Triangular;
                textBox_AverageAcc.Text = "0";
                textBox_AverageAcc.Enabled = false;
                checkBox_LockAcc.Checked = false;

                trackBar_Ramp.Enabled = false;
                trackBar_Ramp.Value = 50;

                radioButton_SymmetricProfile.Checked = false;
                radioButton_AsymmetricProfile.Checked = false;
            }
        }

        private void radioButton_SymmetricProfile_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_SymmetricProfile.Checked == true)
            {
                //Add the constant sector if it does not exist
                if (_move.SectorsList.Count == 2)
                {
                    Sector _sectorConst = new Sector(_move.Name + "_Const", _move.Name, 1);
                    _move.SectorsList.Insert(1, _sectorConst);
                }

                _move.Profile = Profile.Symmetric;
                textBox_AverageAcc.Text = "0";
                textBox_AverageAcc.Enabled = false;
                checkBox_LockAcc.Checked = false;

                trackBar_Ramp.Enabled = false;
                trackBar_Ramp.Value = 33;

                radioButton_TriangularProfile.Checked = false;
                radioButton_AsymmetricProfile.Checked = false;
            }
        }

        private void radioButton_AsymmetricProfile_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_AsymmetricProfile.Checked == true)
            {
                //Add the constant sector if it does not exist
                if (_move.SectorsList.Count == 2)
                {
                    Sector _sectorConst = new Sector(_move.Name + "_Const", _move.Name, 1);
                    _move.SectorsList.Insert(1, _sectorConst);
                }

                _move.Profile = Profile.Asymmetric;
                textBox_AverageAcc.Text = "0";
                textBox_AverageAcc.Enabled = false;
                checkBox_LockAcc.Checked = false;

                trackBar_Ramp.Enabled = true;

                _move.Ramp = trackBar_Ramp.Value;

                radioButton_TriangularProfile.Checked = false;
                radioButton_SymmetricProfile.Checked = false;
            }
        }

        private void checkBox_LockTime_Click(object sender, EventArgs e)
        {
            if (checkBox_LockTime.Checked == false)
            {
                int nrOfCheckedBoxes = 0;

                if (checkBox_LockDistance.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockFinalSpeed.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockAcc.Checked == true)
                    nrOfCheckedBoxes++;

                if (nrOfCheckedBoxes <= 1)
                {
                    checkBox_LockTime.Checked = true;
                    _move.TimeLocked = true;
                    textBox_Time.Enabled = true;
                }
            }
            else
            {
                checkBox_LockTime.Checked = false;
                _move.TimeLocked = false;
                textBox_Time.Enabled = false;
            }
        }

        private void textBox_Time_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double time = Math.Abs(Math.Round(validate_textBox(textBox_Time.Text), 3));

            //Update the textBox if the validate-function did any formatting
            textBox_Time.Text = time.ToString();

            if (_move.Time != time)
            {
                _move.Time = time;
            }
        }

        private void textBox_Time_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double time = Math.Abs(Math.Round(validate_textBox(textBox_Time.Text), 3));

                //Update the textBox if the validate-function did any formatting
                textBox_Time.Text = time.ToString();

                //Recalculate if the value has been changed
                if (_move.Time != time)
                    _move.Time = time;

            }
        }

        private void radioButton_DistanceUnitDegrees_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_DistanceUnitDegrees.Checked == true)
            {
                _move.DistanceInDegrees = true;
                radioButton_DistanceUnitMeters.Checked = false;
                textBox_DistanceInDegrees.Enabled = true;
                textBox_DistanceInMeters.Enabled = false;
            }
        }

        private void textBox_DistanceInDegrees_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double degrees = Math.Round(validate_textBox(textBox_DistanceInDegrees.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_DistanceInDegrees.Text = degrees.ToString();

            //Recalculate if the value has been changed
            if (_move.Degrees != degrees)
            {
                _move.Degrees = degrees;
                _move.Distance = degrees / 360 * _move.TotalLoadResults.InpMovePerRev * _move.TotalLoadResults.AccOutGearing;
                textBox_DistanceInMeters.Text = _move.Distance.ToString();
            }
        }

        private void textBox_DistanceInDegrees_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double degrees = Math.Round(validate_textBox(textBox_DistanceInDegrees.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_DistanceInDegrees.Text = degrees.ToString();

                //Recalculate if the value has been changed
                if (_move.Degrees != degrees)
                {
                    _move.Degrees = degrees;
                    _move.Distance = degrees / 360 * _move.TotalLoadResults.InpMovePerRev * _move.TotalLoadResults.AccOutGearing;
                    textBox_DistanceInMeters.Text = _move.Distance.ToString();
                }
            }
        }

        private void checkBox_LockDistance_Click(object sender, EventArgs e)
        {
            if (checkBox_LockDistance.Checked == false)
            {
                int nrOfCheckedBoxes = 0;

                if (checkBox_LockTime.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockFinalSpeed.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockAcc.Checked == true)
                    nrOfCheckedBoxes++;

                if (nrOfCheckedBoxes <= 1)
                {
                    checkBox_LockDistance.Checked = true;
                    _move.DistanceLocked = true;

                    if (radioButton_DistanceUnitMeters.Checked == true)
                        textBox_DistanceInMeters.Enabled = true;
                    else
                        textBox_DistanceInDegrees.Enabled = true;
                }
            }
            else
            {
                checkBox_LockDistance.Checked = false;
                _move.DistanceLocked = false;
                textBox_DistanceInDegrees.Enabled = false;
                textBox_DistanceInMeters.Enabled = false;
            }
        }

        private void radioButton_DistanceUnitMeters_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton_DistanceUnitMeters.Checked == true)
            {
                _move.DistanceInDegrees = false;
                radioButton_DistanceUnitDegrees.Checked = false;
                textBox_DistanceInDegrees.Enabled = false;
                textBox_DistanceInDegrees.Text = "0";
                textBox_DistanceInMeters.Enabled = true;
            }
        }

        private void textBox_DistanceInMeters_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double distance = Math.Round(validate_textBox(textBox_DistanceInMeters.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_DistanceInMeters.Text = distance.ToString();

            //Recalculate if the value has been changed
            if (_move.Distance != distance)
                _move.Distance = distance;

        }

        private void textBox_DistanceInMeters_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double distance = Math.Round(validate_textBox(textBox_DistanceInMeters.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_DistanceInMeters.Text = distance.ToString();

                //Recalculate if the value has been changed
                if (_move.Distance != distance)
                    _move.Distance = distance;
            }
        }

        private void textBox_InitialSpeed_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double initialSpeed = Math.Round(validate_textBox(textBox_InitialSpeed.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_InitialSpeed.Text = initialSpeed.ToString();

            //Recalculate if the value has been changed
            if (_move.InitialSpeed != initialSpeed)
                _move.InitialSpeed = initialSpeed;
        }

        private void textBox_InitialSpeed_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double initialSpeed = Math.Round(validate_textBox(textBox_InitialSpeed.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_InitialSpeed.Text = initialSpeed.ToString();

                //Recalculate if the value has been changed
                if (_move.InitialSpeed != initialSpeed)
                    _move.InitialSpeed = initialSpeed;
            }
        }

        private void checkBox_LockFinalSpeed_Click(object sender, EventArgs e)
        {
            if (checkBox_LockFinalSpeed.Checked == false)
            {
                int nrOfCheckedBoxes = 0;

                if (checkBox_LockTime.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockDistance.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockAcc.Checked == true)
                    nrOfCheckedBoxes++;

                if (nrOfCheckedBoxes <= 1)
                {
                    checkBox_LockFinalSpeed.Checked = true;
                    _move.FinalSpeedLocked = true;
                    textBox_FinalSpeed.Enabled = true;
                }
            }
            else
            {
                checkBox_LockFinalSpeed.Checked = false;
                _move.FinalSpeedLocked = false;
                textBox_FinalSpeed.Enabled = false;
            }
        }

        private void textBox_FinalSpeed_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double finalSpeed = Math.Round(validate_textBox(textBox_FinalSpeed.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_FinalSpeed.Text = finalSpeed.ToString();

            //Recalculate if the value has been changed
            if (_move.FinalSpeed != finalSpeed)
                _move.FinalSpeed = finalSpeed;
        }

        private void textBox_FinalSpeed_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double finalSpeed = Math.Round(validate_textBox(textBox_FinalSpeed.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_FinalSpeed.Text = finalSpeed.ToString();

                //Recalculate if the value has been changed
                if (_move.FinalSpeed != finalSpeed)
                    _move.FinalSpeed = finalSpeed;
            }
        }

        private void checkBox_LockAcc_Click(object sender, EventArgs e)
        {
            if (checkBox_LockAcc.Checked == false)
            {
                int nrOfCheckedBoxes = 0;

                if (checkBox_LockTime.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockDistance.Checked == true)
                    nrOfCheckedBoxes++;
                if (checkBox_LockFinalSpeed.Checked == true)
                    nrOfCheckedBoxes++;

                if (nrOfCheckedBoxes <= 1)
                {
                    checkBox_LockAcc.Checked = true;
                    _move.AverageAccLocked = true;
                    textBox_AverageAcc.Enabled = true;
                }
            }
            else
            {
                checkBox_LockAcc.Checked = false;
                _move.AverageAccLocked = false;
                textBox_AverageAcc.Enabled = false;
            }
        }

        private void textBox_AverageAcc_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double averageAcc = Math.Round(validate_textBox(textBox_AverageAcc.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_AverageAcc.Text = averageAcc.ToString();

            //Recalculate if the value has been changed
            if (_move.AverageAcc != averageAcc)
                _move.AverageAcc = averageAcc;
        }

        private void textBox_AverageAcc_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double averageAcc = Math.Round(validate_textBox(textBox_AverageAcc.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_AverageAcc.Text = averageAcc.ToString();

                //Recalculate if the value has been changed
                if (_move.AverageAcc != averageAcc)
                    _move.AverageAcc = averageAcc;
            }
        }

        private void textBox_PercentSineVel_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double percentSineVel = Math.Round(validate_textBox(textBox_PercentSineVel.Text), 0);

            //Update the textBox if the validate-function did any formatting
            textBox_PercentSineVel.Text = percentSineVel.ToString();

            //Recalculate if the value has been changed
            if (_move.PercentSineVel != percentSineVel)
                _move.PercentSineVel = percentSineVel;
        }

        private void textBox_PercentSineVel_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double percentSineVel = Math.Round(validate_textBox(textBox_PercentSineVel.Text), 0);

                //Update the textBox if the validate-function did any formatting
                textBox_PercentSineVel.Text = percentSineVel.ToString();

                //Recalculate if the value has been changed
                if (_move.PercentSineVel != percentSineVel)
                    _move.PercentSineVel = percentSineVel;
            }
        }

        private void textBox_PercentSineAcc_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            double percentSineAcc = Math.Round(validate_textBox(textBox_PercentSineAcc.Text), 0);

            //Update the textBox if the validate-function did any formatting
            textBox_PercentSineAcc.Text = percentSineAcc.ToString();

            //Recalculate if the value has been changed
            if (_move.PercentSineAcc != percentSineAcc)
                _move.PercentSineAcc = percentSineAcc;
        }

        private void textBox_PercentSineAcc_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                double percentSineAcc = Math.Round(validate_textBox(textBox_PercentSineAcc.Text), 0);

                //Update the textBox if the validate-function did any formatting
                textBox_PercentSineAcc.Text = percentSineAcc.ToString();

                //Recalculate if the value has been changed
                if (_move.PercentSineAcc != percentSineAcc)
                    _move.PercentSineAcc = percentSineAcc;
            }
        }

        #endregion

        void dataGridView_Loads_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            if (dataGridView_Loads.IsCurrentCellDirty)
            {
                dataGridView_Loads.CommitEdit(DataGridViewDataErrorContexts.Commit);
                if (dataGridView_Loads.CurrentCell.OwningColumn.Name == "Active")

                    foreach (var l in _move.LoadResultsList)
                    {
                        LoadResult loadResult = l.Value;

                        if (dataGridView_Loads.CurrentRow.Cells[0].Value.ToString() == loadResult.Name)
                        {
                            _move.LoadResultsList[l.Key].Active =
                        Convert.ToBoolean(dataGridView_Loads.CurrentCell.Value);

                            _move.Refresh = true;
                            break;
                        }
                    }
            }
        }

        #endregion

        #region Methods

        private void renameSector()
        {
            foreach (Sector s in _move.SectorsList)
            {
                s.Parent = _move.Name;
            }

            if (_move.MoveType == MoveType.PreDefined)
            {
                if (_move.Profile == Profile.Triangular)
                {
                    _move.SectorsList[0].Name = _move.Name + "_Acc";
                    _move.SectorsList[1].Name = _move.Name + "_Dec";
                }

                if (_move.Profile == Profile.Symmetric || _move.Profile == Profile.Asymmetric)
                {
                    _move.SectorsList[0].Name = _move.Name + "_Acc";
                    _move.SectorsList[1].Name = _move.Name + "_Const";
                    _move.SectorsList[2].Name = _move.Name + "_Dec";
                }
            }

            if (_move.MoveType == MoveType.UserDefined)
                _move.SectorsList[0].Name = _move.Name;

            UpdateMoveResults();
        }

        public void UpdateMoveResults()
        {
            //Display the recalculated textbox values
            textBox_DistanceInMeters.Text = formatValue(_move.Distance).ToString();
            textBox_Time.Text = formatValue(_move.Time).ToString();

            textBox_InitialSpeed.Text = formatValue(_move.InitialSpeed).ToString();
            textBox_FinalSpeed.Text = formatValue(_move.FinalSpeed).ToString();

            textBox_AverageAcc.Text = formatValue(_move.AverageAcc).ToString();

            textBox_PercentSineVel.Text = Math.Round(_move.PercentSineVel, 0).ToString();
            textBox_PercentSineAcc.Text = Math.Round(_move.PercentSineAcc, 0).ToString();

            if (dataGridView_Sectors.RowCount > 0)
                dataGridView_Sectors.Rows.Clear();

            dataGridView_Sectors.DataSource = null;

            double time = 0;
            double distance = 0;

            //Add data to the sector table
            foreach (Sector s in _move.SectorsList)
            {
                dataGridView_Sectors.Rows.Add(s.Name,
                    Math.Round(s.Time, 3),
                    Math.Round(s.Distance, 3),
                    Math.Round(s.InitialSpeed, 3),
                    Math.Round(s.FinalSpeed, 3),
                    Math.Round(s.AverageSpeed, 3),
                    Math.Round(s.AverageAcc, 3),
                    Math.Round(s.PeakAcc, 3),
                    Math.Round(s.ChangingAccTime, 3),
                    Math.Round(s.AverageJerk, 3),
                    Math.Round(s.PeakJerk, 3),
                    Math.Round(s.ChangingJerkTime, 3));
                    
                time += s.Time;
                distance += s.Distance;
            }
        }

        private double formatValue(double raw)
        {
            if (Math.Abs(raw) < 10)
                return Math.Round(raw, 2);

            if (Math.Abs(raw) < 100)
                return Math.Round(raw, 1);
            else
                return Math.Round(raw, 0);
        }

        public void CreateLoadResultsTable()
        {
            //Clear the table if any rows exists
            if (dataGridView_Loads.RowCount > 0)
                dataGridView_Loads.Rows.Clear();

            dataGridView_Loads.DataSource = null;

            //Set the default row height
            dataGridView_Loads.RowTemplate.Height = 20;

            //Calculate percentage values for the limiting speed and torque
            double maxSpdPercent = Math.Abs(Math.Round((_move.MotorResults.PeakSpd / _motor.MaxSpd) * 100, 0));
            double nomSpdPercent = Math.Abs(Math.Round((_move.MotorResults.RmsSpdED / _motor.NomSpd) * 100, 0));

            double liftTrqPercent = Math.Abs(Math.Round((_move.MotorResults.LiftTorque / _motor.NomTrq) * 100, 0));

            double maxTrqPercent = Math.Abs(Math.Round((_move.MotorResults.PeakTrq / _motor.MaxTrq) * 100, 0));
            double nomTrqPercent = Math.Abs(Math.Round((_move.MotorResults.RmsTrqED / _motor.NomTrq) * 100, 0));

            double inertiaPercent = Math.Abs(Math.Round((_move.MotorResults.Inertia - _motor.Inertia) / (_motor.Inertia * _motor.InertiaRatio) * 100, 0));

            //Check the existing torque values
            string accTrq = "-";
            string constTrq = "-";
            string decTrq = "-";

            //if (_move.MotorResults.AccTorqueExists)
                accTrq = formatValue(_move.MotorResults.AccTorque).ToString();

            //if (_move.MotorResults.ConstTorqueExists)
                constTrq = formatValue(_move.MotorResults.ConstTorque).ToString();

            //if (_move.MotorResults.DecTorqueExists)
                decTrq = formatValue(_move.MotorResults.DecTorque).ToString();

            //Add the motor results to the table
            dataGridView_Loads.Rows.Add(_motor.Name, _motor.Description, true, 1,
                formatValue(_move.MotorResults.PeakSpd) + " (" + maxSpdPercent + "%)",
                formatValue(_move.MotorResults.RmsSpdED) + " (" + nomSpdPercent + "%)",
                "360 °",
                "0",
                formatValue(_move.MotorResults.Efficiency * 100),
                formatValue(_move.MotorResults.Inertia * 10000) + " (" + inertiaPercent + "%)",
                formatValue(_move.MotorResults.ThrustTorque),
                formatValue(_move.MotorResults.LiftTorque) + " (" + liftTrqPercent + "%)",
                constTrq,
                accTrq,                
                decTrq,
                formatValue(_move.MotorResults.PeakTrq) + " (" + maxTrqPercent + "%)",
                formatValue(_move.MotorResults.RmsTrqED) + " (" + nomTrqPercent + "%)");                     

            //Get the handle of the last added row
            DataGridViewRow row = new DataGridViewRow();
            row = dataGridView_Loads.Rows[dataGridView_Loads.RowCount - 1];

            //Disable the "active" checkbox if it is a mechanical load type
            DataGridViewCell cell = row.Cells[2];
            DataGridViewCheckBoxCell chkCell = cell as DataGridViewCheckBoxCell;

            chkCell.Value = true;
            chkCell.FlatStyle = FlatStyle.Flat;
            chkCell.Style.ForeColor = Color.DarkGray;
            cell.ReadOnly = true;

            //Add the unrounded values as tooltips
            row.Cells[4].ToolTipText = _move.MotorResults.PeakSpd.ToString();
            row.Cells[5].ToolTipText = _move.MotorResults.RmsSpdED.ToString();
            row.Cells[8].ToolTipText = _move.MotorResults.Efficiency.ToString();
            row.Cells[9].ToolTipText = _move.MotorResults.Inertia.ToString();
            row.Cells[10].ToolTipText = _move.MotorResults.ThrustTorque.ToString();
            row.Cells[11].ToolTipText = _move.MotorResults.LiftTorque.ToString();
            row.Cells[12].ToolTipText = _move.MotorResults.ConstTorque.ToString();
            row.Cells[13].ToolTipText = _move.MotorResults.AccTorque.ToString();
            row.Cells[14].ToolTipText = _move.MotorResults.DecTorque.ToString();
            row.Cells[15].ToolTipText = _move.MotorResults.PeakTrq.ToString();
            row.Cells[16].ToolTipText = _move.MotorResults.RmsTrqED.ToString();

            //Change the background color if the motor is user defined
            if (_motor.AutoSelect == true)
                row.DefaultCellStyle.BackColor = Color.PowderBlue;
            else
                row.DefaultCellStyle.BackColor = Color.White;

            if (maxSpdPercent > 80 || maxSpdPercent == 0)
                if (maxSpdPercent > 100)
                    row.Cells[4].Style.BackColor = Color.Coral;
                else
                    row.Cells[4].Style.BackColor = Color.Gold;

            if (nomSpdPercent > 80 || nomSpdPercent == 0)
                if (nomSpdPercent > 100)
                    row.Cells[5].Style.BackColor = Color.Coral;
                else
                    row.Cells[5].Style.BackColor = Color.Gold;

            if (inertiaPercent > 80 || inertiaPercent == 0)
                if (inertiaPercent > 100)
                    row.Cells[9].Style.BackColor = Color.Coral;
                else
                    row.Cells[9].Style.BackColor = Color.Gold;

            if (liftTrqPercent > Settings.MainSettings.AllowedLiftTorque - 30)
                if (liftTrqPercent > Settings.MainSettings.AllowedLiftTorque)
                    row.Cells[11].Style.BackColor = Color.Coral;
                else
                    row.Cells[11].Style.BackColor = Color.Gold;

            if (maxTrqPercent > 80 || maxTrqPercent == 0)
                if (maxTrqPercent > 100)
                    row.Cells[15].Style.BackColor = Color.Coral;
                else
                    row.Cells[15].Style.BackColor = Color.Gold;

            if (nomTrqPercent > 80 || nomTrqPercent == 0)
                if (nomTrqPercent > 100)
                    row.Cells[16].Style.BackColor = Color.Coral;
                else
                    row.Cells[16].Style.BackColor = Color.Gold;

            //Go through each load result and add data to the LoadResults table
            //Read the specified torque and speed limits from the corresponding load objects
            foreach (var lR in _move.LoadResultsList)
            {
                LoadResult l = lR.Value;
                int loadNr = lR.Key;

                //Define the variables to be fetched from the LoadsList
                string descr = "";
                double maxSpd = 0;
                double nomSpd = 0;
                double maxTrq = 0;
                double nomTrq = 0;
                double movePerRev = 0;

                //Find the corresponding load in the loadslist
                ILoad iLoad = _iLoads[loadNr];

                //Check the load type and fetch the properties
                switch (iLoad.MainType)
                {
                    case MainType.BallScrew:
                        BallScrew ballScrew = iLoad as BallScrew;
                        descr = ballScrew.Description;
                        maxSpd = ballScrew.MaxSpd;
                        nomSpd = ballScrew.NomSpd;
                        maxTrq = ballScrew.MaxTrq;
                        nomTrq = ballScrew.NomTrq;
                        movePerRev = ballScrew.MovePerRev;
                        break;

                    case MainType.Coupling:
                        Coupling coupling = iLoad as Coupling;
                        descr = coupling.Description;
                        maxSpd = coupling.MaxSpd;
                        nomSpd = coupling.NomSpd;
                        maxTrq = coupling.MaxTrq;
                        nomTrq = coupling.NomTrq;
                        break;

                    case MainType.RackAndPinion:
                        RackAndPinion rackAndPinion = iLoad as RackAndPinion;
                        descr = rackAndPinion.Description;
                        maxSpd = rackAndPinion.MaxSpd;
                        nomSpd = rackAndPinion.NomSpd;
                        maxTrq = rackAndPinion.MaxTrq;
                        nomTrq = rackAndPinion.NomTrq;
                        movePerRev = rackAndPinion.MovePerRev;
                        break;

                    case MainType.TurnTable:
                        TurnTable turnTable = iLoad as TurnTable;
                        descr = turnTable.Description;
                        maxSpd = turnTable.MaxSpd;
                        nomSpd = turnTable.NomSpd;
                        maxTrq = turnTable.MaxTrq;
                        nomTrq = turnTable.NomTrq;
                        movePerRev = turnTable.MovePerRev;
                        break;

                    case MainType.ToothedBelt:
                        ToothedBelt toothedBelt = iLoad as ToothedBelt;
                        descr = toothedBelt.Description;
                        maxSpd = toothedBelt.MaxSpd;
                        nomSpd = toothedBelt.NomSpd;
                        maxTrq = toothedBelt.MaxTrq;
                        nomTrq = toothedBelt.NomTrq;
                        movePerRev = toothedBelt.MovePerRev;
                        break;

                    case MainType.GearBox:
                        GearBox gearBox = iLoad as GearBox;
                        descr = gearBox.Description;
                        maxSpd = gearBox.MaxSpd;
                        nomSpd = gearBox.NomSpd;
                        maxTrq = gearBox.MaxTrq / gearBox.Gearing;
                        nomTrq = gearBox.NomTrq / gearBox.Gearing;
                        break;

                   // case MainType.Mass:
                   //     LoadMass mass = iLoad as LoadMass;
                   //     descr = "";
                   //     break;
                }

                //Calculate percentage values for the limiting speed and torque
                maxSpdPercent = Math.Abs(Math.Round((l.PeakSpd / maxSpd) * 100, 0));
                nomSpdPercent = Math.Abs(Math.Round((l.RmsSpdED / nomSpd) * 100, 0));

                liftTrqPercent = Math.Abs(Math.Round((l.LiftTorque / nomTrq) * 100, 0));

                maxTrqPercent = Math.Abs(Math.Round((l.PeakTrq / maxTrq) * 100, 0));
                nomTrqPercent = Math.Abs(Math.Round((l.RmsTrqED / nomTrq) * 100, 0));

                //If no limit is defined then set the percentage to zero
                if (maxSpd == 0)
                    maxSpdPercent = 0;

                if (nomSpd == 0)
                    nomSpdPercent = 0;

                if (nomTrq == 0)
                    liftTrqPercent = 0;

                if (maxTrq == 0)
                    maxTrqPercent = 0;

                if (nomTrq == 0)
                    nomTrqPercent = 0;

                //Check the MovePerRev-unit
                string strUnit = "mm";

                if (l.InpMovePerRevUnitmm == false)
                {
                    l.InpMovePerRev *= 360;
                    strUnit = " °";
                }
                else
                    l.InpMovePerRev *= 1000;

                //Check the Backlash-unit
                string backlashUnit = " arcmin";

                if (l.InpBacklashUnitmm == true)
                {
                    l.AccInpBacklash = ((l.AccInpBacklash / 60) / 360) * (l.InpMovePerRev * l.AccOutGearing * 1000);
                    backlashUnit = " µm";
                }

                //Check the existing torque values
                accTrq = "-";
                constTrq = "-";
                decTrq = "-";

                //if (l.AccTorqueExists)
                    accTrq = formatValue(l.AccTorque).ToString();

                //if (l.ConstTorqueExists)
                    constTrq = formatValue(l.ConstTorque).ToString();

                //if (l.DecTorqueExists)
                    decTrq = formatValue(l.DecTorque).ToString();

                //Note: Convert the checkbox to a textbox for non-detachable objects
                //http://stackoverflow.com/questions/14124033/hide-some-datagridview-checkbox-cell?rq=1

                //Add the result to the table

                //Do not include the percent load if the load is detachable
                if (_iLoads[loadNr].Detachable == false)
                {
                    dataGridView_Loads.Rows.Add(l.Name, descr, l.Active,
                        formatValue(l.AccInpGearing),
                        formatValue(l.PeakSpd) + " (" + maxSpdPercent + "%)",
                        formatValue(l.RmsSpdED) + " (" + nomSpdPercent + "%)",
                        formatValue(l.InpMovePerRev) + strUnit,
                        formatValue(l.AccInpBacklash) + backlashUnit,
                        formatValue(l.Efficiency * 100),
                        formatValue(l.Inertia * 10000),
                        formatValue(l.ThrustTorque),
                        formatValue(l.LiftTorque) + " (" + liftTrqPercent + "%)",
                        constTrq,
                        accTrq,                        
                        decTrq,
                        formatValue(l.PeakTrq) + " (" + maxTrqPercent + "%)",
                        formatValue(l.RmsTrqED) + " (" + nomTrqPercent + "%)");
                }
                else
                {
                    dataGridView_Loads.Rows.Add(l.Name, descr, l.Active,
                        formatValue(l.AccInpGearing),
                        formatValue(l.PeakSpd),
                        formatValue(l.RmsSpdED),
                        formatValue(l.InpMovePerRev) + strUnit,
                        formatValue(l.AccInpBacklash) + backlashUnit,
                        formatValue(l.Efficiency * 100),
                        formatValue(l.Inertia * 10000),
                        formatValue(l.ThrustTorque),
                        formatValue(l.LiftTorque) + " (" + liftTrqPercent + "%)",
                        constTrq,
                        accTrq,
                        decTrq,
                        formatValue(l.PeakTrq),
                        formatValue(l.RmsTrqED),
                        formatValue(l.RegenEnergy));
                }

                //Format the table to the load properties
                //Get the handle of the last added row
                row = dataGridView_Loads.Rows[dataGridView_Loads.RowCount - 1];

                //Disable the "active" checkbox if it is a mechanical load type
                cell = row.Cells[2];
                chkCell = cell as DataGridViewCheckBoxCell;
                
                if (_iLoads[loadNr].Detachable == true)
                {
                    cell.ReadOnly = false;
                }
                else
                {
                    chkCell.Value = true;
                    chkCell.FlatStyle = FlatStyle.Flat;
                    chkCell.Style.ForeColor = Color.DarkGray;
                    cell.ReadOnly = true;
                }

                //Add the unrounded values as tooltips
                row.Cells[3].ToolTipText = l.AccInpGearing.ToString();
                row.Cells[4].ToolTipText = l.PeakSpd.ToString();
                row.Cells[5].ToolTipText = l.RmsSpdED.ToString();
                row.Cells[6].ToolTipText = l.InpMovePerRev.ToString();
                row.Cells[7].ToolTipText = l.AccInpBacklash.ToString();
                row.Cells[8].ToolTipText = l.Efficiency.ToString();
                row.Cells[9].ToolTipText = l.Inertia.ToString();
                row.Cells[10].ToolTipText = l.ThrustTorque.ToString();
                row.Cells[11].ToolTipText = l.LiftTorque.ToString();
                row.Cells[12].ToolTipText = l.ConstTorque.ToString(); 
                row.Cells[13].ToolTipText = l.AccTorque.ToString();
                row.Cells[14].ToolTipText = l.DecTorque.ToString();
                row.Cells[15].ToolTipText = l.PeakTrq.ToString();
                row.Cells[16].ToolTipText = l.RmsTrqED.ToString();                

                //Change the background color if the load is user defined
                if (_iLoads[loadNr].AutoSelect == true)
                    row.DefaultCellStyle.BackColor = Color.PowderBlue;
                else
                    row.DefaultCellStyle.BackColor = Color.White;

                //Set a yellow background if the percentage value is > 80
                //Set a red background if the percentage value is > 100
                if (_iLoads[loadNr].Detachable == false)
                {
                    if (maxSpdPercent > 80 || maxSpdPercent == 0)
                        if (maxSpdPercent > 100)
                            row.Cells[4].Style.BackColor = Color.Coral;
                        else
                            row.Cells[4].Style.BackColor = Color.Gold;

                    if (nomSpdPercent > 80 || nomSpdPercent == 0)
                        if (nomSpdPercent > 100)
                            row.Cells[5].Style.BackColor = Color.Coral;
                        else
                            row.Cells[5].Style.BackColor = Color.Gold;
                    
                    if (liftTrqPercent > Settings.MainSettings.AllowedLiftTorque - 30)
                        if (liftTrqPercent > Settings.MainSettings.AllowedLiftTorque)
                            row.Cells[11].Style.BackColor = Color.Coral;
                        else
                            row.Cells[11].Style.BackColor = Color.Gold;

                    if (maxTrqPercent > 80 || maxTrqPercent == 0)
                        if (maxTrqPercent > 100)
                            row.Cells[15].Style.BackColor = Color.Coral;
                        else
                            row.Cells[15].Style.BackColor = Color.Gold;

                    if (nomTrqPercent > 80 || nomTrqPercent == 0)
                        if (nomTrqPercent > 100)
                            row.Cells[16].Style.BackColor = Color.Coral;
                        else
                            row.Cells[16].Style.BackColor = Color.Gold;
                }

                //Output side

                //Check the MovePerRev-unit
                strUnit = " mm";

                if (l.OutMovePerRevUnitmm == false)
                {
                    l.OutMovePerRev *= 360;
                    strUnit = " °";
                }
                else
                    l.OutMovePerRev *= 1000;

                backlashUnit = " arcmin";

                if (l.OutBacklashUnitmm == true)
                {
                    l.AccOutBacklash = ((l.AccOutBacklash / 60) / 360) * (l.OutMovePerRev * l.AccOutGearing * 1000);
                    backlashUnit = " µm";
                }

                //Check the existing torque values
                accTrq = "-";
                constTrq = "-";
                decTrq = "-";

                //if (l.AccTorqueExists)
                    accTrq = formatValue(l.AccTorque * l.Gearing * l.Efficiency).ToString();

                //if (l.ConstTorqueExists)
                    constTrq = formatValue(l.ConstTorque * l.Gearing * l.Efficiency).ToString();

                //if (l.DecTorqueExists)
                    decTrq = formatValue(l.DecTorque * l.Gearing * l.Efficiency).ToString();

                //All physical components are displayed on both input and output side
                if (_iLoads[loadNr].Detachable == false)
                {
                    //Add the result to the table
                    dataGridView_Loads.Rows.Add(l.Name, "(loadSide)", l.Active,
                        formatValue(l.AccOutGearing),
                        formatValue(l.PeakSpd / l.Gearing),
                        formatValue(l.RmsSpdED / l.Gearing),
                        formatValue(l.OutMovePerRev) + strUnit,
                        formatValue(l.AccOutBacklash) + backlashUnit,
                        formatValue(l.Efficiency * 100),
                        formatValue((l.Inertia * Math.Pow(l.Gearing, 2)) * 10000),
                        formatValue(l.ThrustTorque * l.Gearing * l.Efficiency),
                        formatValue(l.LiftTorque * l.Gearing * l.Efficiency),
                        constTrq, 
                        accTrq,
                        decTrq,
                        formatValue(l.PeakTrq * l.Gearing * l.Efficiency),
                        formatValue(l.RmsTrqED * l.Gearing * l.Efficiency));

                    //Get the handle of the last added row
                    row = dataGridView_Loads.Rows[dataGridView_Loads.RowCount - 1];

                    //Disable the "active" checkbox if it is a mechanical load type
                    cell = row.Cells[2];
                    chkCell = cell as DataGridViewCheckBoxCell;
                    
                    chkCell.Value = true;
                    chkCell.FlatStyle = FlatStyle.Flat;
                    chkCell.Style.ForeColor = Color.DarkGray;
                    cell.ReadOnly = true;

                    //Add the unrounded values as tooltips
                    row.Cells[3].ToolTipText = l.AccOutGearing.ToString();
                    row.Cells[4].ToolTipText = (l.PeakSpd / l.Gearing).ToString();
                    row.Cells[5].ToolTipText = (l.RmsSpdED / l.Gearing).ToString();
                    row.Cells[6].ToolTipText = l.OutMovePerRev.ToString();
                    row.Cells[7].ToolTipText = l.AccOutBacklash.ToString();
                    row.Cells[8].ToolTipText = l.Efficiency.ToString();
                    row.Cells[9].ToolTipText = l.Inertia.ToString();
                    row.Cells[10].ToolTipText = (l.ThrustTorque * l.Gearing * l.Efficiency).ToString();
                    row.Cells[11].ToolTipText = (l.LiftTorque * l.Gearing * l.Efficiency).ToString();
                    row.Cells[12].ToolTipText = l.ConstTorque.ToString(); 
                    row.Cells[13].ToolTipText = l.AccTorque.ToString();                    
                    row.Cells[14].ToolTipText = l.DecTorque.ToString();
                    row.Cells[15].ToolTipText = (l.PeakTrq * l.Gearing * l.Efficiency).ToString();
                    row.Cells[16].ToolTipText = (l.RmsTrqED * l.Gearing * l.Efficiency).ToString();                    

                    //Set the row background color to light grey
                    row.DefaultCellStyle.BackColor = Color.Gainsboro;
                }
            }
        }

        private void ReadMotorFromdB(Motor motor, string articleNo)
        {
            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb"; //Path.GetFullPath
            // (@"C:\DriveSystems.mdb");

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {
                    //Read the data from the Motors table
                    var qryMotor =
                        from tblMotors myMotors in dbDriveSystemsContext.tblMotors
                        where myMotors.ArticleNo == articleNo
                        select myMotors;

                    //Load the motor object with its data
                    foreach (var dBMotor in qryMotor) //Should only be one item
                    {
                        motor.ArticleNo = dBMotor.ArticleNo;

                        motor.Brand = Brand.BSD;

                        motor.Description = dBMotor.Description;
                        // ToDo: motor. = dBMotor.Type;
                        motor.Price = dBMotor.Price;

                        //Main data
                        motor.Resolution = dBMotor.Resolution;
                        motor.InertiaRatio = dBMotor.InertiaRatio;
                        motor.Inertia = dBMotor.Inertia / 10000;
                        motor.Eff = dBMotor.Eff;
                        motor.FldWeakSpd = dBMotor.FldWeakSpd;

                        //Mechanical output
                        switch (dBMotor.OutType)
                        {
                            case "Flange":
                                motor.OutType = MechInterface.Flange;
                                break;

                            case "SmoothHollow":
                                motor.OutType = MechInterface.SmoothHollow;
                                break;

                            case "KeyedHollow":
                                motor.OutType = MechInterface.KeyedHollow;
                                break;

                            case "Keyed shaft":
                                motor.OutType = MechInterface.KeyedShaft;
                                break;

                            case "Smooth shaft":
                                motor.OutType = MechInterface.SmoothShaft;
                                break;

                            default:
                                motor.OutType = MechInterface.Undefined;
                                break;
                        }

                        motor.OutMinDia = dBMotor.OutMinDia;
                        motor.OutMaxDia = dBMotor.OutMaxDia;
                        motor.OutRadFrc = dBMotor.OutRadFrc;
                        motor.OutAxFrc = dBMotor.OutAxFrc;

                        //Torque data
                        motor.MaxTrq = dBMotor.MaxTrq;
                        motor.MaxTrqDur = dBMotor.MaxTrqDur;
                        motor.TrqED = dBMotor.TrqED;
                        motor.NomTrq = dBMotor.NomTrq;

                        //Speed data
                        motor.MaxSpd = dBMotor.MaxSpd;
                        motor.MaxSpdDur = dBMotor.MaxSpdDur;
                        motor.SpdED = dBMotor.SpdED;
                        motor.NomSpd = dBMotor.NomSpd;
                    }
                }
            }
        }

        #endregion

        public void CalcMotorResults()
        {
            //Calculate the motor results without accessing the database

            //If the torque ED-factor is not defined then set a conservative value
            if (_motor.MaxTrqDur <= 0)
                _motor.MaxTrqDur = 1;

            //If the speed ED-factor is not defined then set a conservative value
            if (_motor.MaxSpdDur <= 0)
                _motor.MaxSpdDur = 1;

            //If the field weakening speed is not defined then set conservative values
            if (_motor.FldWeakSpd == 0)
                _motor.FldWeakSpd = _motor.NomSpd * 0.75;
                //frmOutput... send message

            //Clear the motor results before calculating
            _move.MotorResults = null;
            _move.MotorResults = new MotorResult();

            //Queue<double> _debug = new Queue<double>();

            //Calculate the motor inertia and load torque
            _move.MotorResults.Inertia = _motor.Inertia + _move.TotalLoadResults.Inertia;
            _move.MotorResults.IdleTorque = _move.TotalLoadResults.IdleTorque;
            _move.MotorResults.LiftTorque = _move.TotalLoadResults.LiftTorque;
            _move.MotorResults.ThrustTorque = _move.TotalLoadResults.ThrustTorque;
            _move.MotorResults.Angle = _move.TotalLoadResults.Angle;
            _move.MotorResults.Efficiency = _move.TotalLoadResults.Efficiency;

            //the rms-values are based on the duty cycle. During this cycle the rms-value must not exceed the nominal value.
            double dutyCycleTrq = (_motor.MaxTrqDur / (_motor.TrqED / 100)) * Settings.MainSettings.Resolution;
            double dutyCycleSpd = (_motor.MaxSpdDur / (_motor.SpdED / 100)) * Settings.MainSettings.Resolution;

            //Go through the motionProfile to check the motor rms- and maximum torque.
            foreach (Sector sector in _move.SectorsList)
            {
                //Check movement direction
                bool forwardOrUp = (sector.Distance >= 0);

                if (Math.Abs(sector.FinalSpeed) > Math.Abs(sector.InitialSpeed))
                {
                    //Acceleration
                    _move.MotorResults.AccTorque = _move.MotorResults.LiftTorque + _move.MotorResults.ThrustTorque +
                        _move.MotorResults.Inertia * (sector.PeakAcc * 2 * Math.PI) / (_move.TotalLoadResults.InpMovePerRev * _move.MotorResults.Efficiency);
                    
                    if (forwardOrUp)
                        _move.MotorResults.AccTorque += _move.MotorResults.IdleTorque;
                    else
                        _move.MotorResults.AccTorque -= _move.MotorResults.IdleTorque;
                }

                if (sector.FinalSpeed == sector.InitialSpeed)
                {
                    //Constant speed
                    _move.MotorResults.ConstTorque = _move.MotorResults.LiftTorque + _move.MotorResults.ThrustTorque;

                    if (sector.Distance != 0)
                        if (forwardOrUp)
                            _move.MotorResults.ConstTorque += _move.MotorResults.IdleTorque;
                        else
                            _move.MotorResults.ConstTorque -= _move.MotorResults.IdleTorque;
                }

                if (Math.Abs(sector.FinalSpeed) < Math.Abs(sector.InitialSpeed))
                {
                    //Deceleration
                    _move.MotorResults.DecTorque = _move.MotorResults.LiftTorque + _move.MotorResults.ThrustTorque +
                        _move.MotorResults.Inertia * (sector.PeakAcc * 2 * Math.PI) / (_move.TotalLoadResults.InpMovePerRev * _move.MotorResults.Efficiency);

                    if (forwardOrUp)
                        _move.MotorResults.DecTorque += _move.MotorResults.IdleTorque;
                    else
                        _move.MotorResults.DecTorque -= _move.MotorResults.IdleTorque;
                }

                foreach (SectorElement mp in sector.SectorElementsList)
                {
                    //Todo! Cykeltider verkar inte vara defininerade!!! dutyCycle...
                    
                    //Check the available motor torque in relation to the motorspeed. If the motorspeed is
                    //above the field weakening point the available torque is assumed to drop at a linear rate.

                    //Recalculate the velocity from m/s to rpm
                    double scale = 1 / (_move.TotalLoadResults.InpMovePerRev / 60);
                    double spdRpm = mp.Vel * scale;

                    if (spdRpm > _motor.FldWeakSpd)
                        mp.MotorTorqueLimit = _motor.MaxTrq * _motor.FldWeakSpd / spdRpm;
                    else
                        mp.MotorTorqueLimit = _motor.MaxTrq;

                    //torque

                    //Calculate the total torque profile.
                    mp.Torque = _move.MotorResults.LiftTorque + _move.MotorResults.ThrustTorque +
                        _move.MotorResults.Inertia * (mp.Acc * 2 * Math.PI) / (_move.TotalLoadResults.InpMovePerRev * _move.MotorResults.Efficiency);

                    if (sector.Distance != 0)
                        if (forwardOrUp)
                            mp.Torque += _move.MotorResults.IdleTorque;
                        else
                            mp.Torque -= _move.MotorResults.IdleTorque;

                    //Save the peak torque value
                    if (Math.Abs(mp.Torque) > Math.Abs(_move.MotorResults.PeakTrq))
                        _move.MotorResults.PeakTrq = mp.Torque;

                    //Calculate the square of the momentary torque value
                    double temp = Math.Pow(mp.Torque, 2);

                    //Add the current torque value to the FIFO
                    _move.MotorResults.QueueRmsTrq.Enqueue(temp);

                    //Add the result to a sum for all rms-values for the given duration
                    _move.MotorResults.RmsTrqSum += temp;

                    //Check if the number of items in the FIFO exceeds the time duration
                    if (_move.MotorResults.QueueRmsTrq.Count > dutyCycleTrq)
                    {
                        //Save the rms value if it exceeds the previously stored value //_motor.MaxTrqDur * Settings.MainSettings.Resolution
                        if (Math.Sqrt(_move.MotorResults.RmsTrqSum / dutyCycleTrq) > _move.MotorResults.RmsTrqED)

                            _move.MotorResults.RmsTrqED = Math.Sqrt(_move.MotorResults.RmsTrqSum / dutyCycleTrq);

                        //Remove the oldest value from the FIFO
                        _move.MotorResults.RmsTrqSum -= _move.MotorResults.QueueRmsTrq.Peek();
                        _move.MotorResults.QueueRmsTrq.Dequeue();
                    }

                    //Speed
                    temp = 60 * (mp.Vel / _move.TotalLoadResults.InpMovePerRev);

                    //_debug.Enqueue(temp);

                    if (Math.Abs(temp) > Math.Abs(_move.MotorResults.PeakSpd))
                        _move.MotorResults.PeakSpd = temp;

                    temp = Math.Pow(temp, 2);

                    //Add the current speed value to the FIFO
                    _move.MotorResults.QueueRmsSpd.Enqueue(temp);

                    //Add the result to a sum for all rms-values for the given duration
                    _move.MotorResults.RmsSpdSum += temp;

                    //Check if the number of items in the FIFO exceeds the time duration  // (_motor.MaxSpdDur * Settings.MainSettings.Resolution)
                    if (_move.MotorResults.QueueRmsSpd.Count > dutyCycleSpd)
                    {
                        if (Math.Sqrt(_move.MotorResults.RmsSpdSum / dutyCycleSpd) > _move.MotorResults.RmsSpdED)
                            _move.MotorResults.RmsSpdED = Math.Sqrt(_move.MotorResults.RmsSpdSum / dutyCycleSpd);

                        //Remove the oldest value from the FIFO
                        _move.MotorResults.RmsSpdSum -= _move.MotorResults.QueueRmsSpd.Peek();
                        _move.MotorResults.QueueRmsSpd.Dequeue();
                    }
                }
            }

            //If the total duration for the move did not exceed the allowed ED duration then 
            //calculate the rms-values based on the number of samples
            if (_move.MotorResults.RmsTrqED == 0)
                _move.MotorResults.RmsTrqED = Math.Sqrt(_move.MotorResults.RmsTrqSum / _move.MotorResults.QueueRmsTrq.Count);
            /*
            //Dec torque
            _move.MotorResults.DecTorque = -(_move.MotorResults.AccTorque - _move.MotorResults.ConstTorque);
            _move.MotorResults.DecTorque += _move.MotorResults.ConstTorque;

            //Peak torque
            if (_move.MotorResults.AccTorque > _move.MotorResults.DecTorque)
                _move.MotorResults.PeakTrq = _move.MotorResults.AccTorque;
            else
                _move.MotorResults.PeakTrq = _move.MotorResults.DecTorque;
            */
            //If the total duration for the move did not exceed the allowed ED duration then 
            //calculate the rms-values based on the number of samples
            if (_move.MotorResults.RmsSpdED == 0)
                _move.MotorResults.RmsSpdED = Math.Sqrt(_move.MotorResults.RmsSpdSum / _move.MotorResults.QueueRmsSpd.Count);
        }

        #region PrintMethods
        //Return the requested object to the main form for printing
        
        public GroupBox GetGroupBoxMove()
        {
            return groupBox_MoveData;
        }

        public DataGridView GetDGVSectors()
        {
            return dataGridView_Sectors;
        }

        public DataGridView GetDGVComponents()
        {
            return dataGridView_Loads;
        }

        #endregion

        private void trackBar_Ramp_Scroll(object sender, EventArgs e)
        {
            if (_move.Profile == Profile.Asymmetric)
                _move.Ramp = trackBar_Ramp.Value;
        }
    }
}