using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;

namespace DockSample
{
    public partial class FrmNavigator : ToolWindow
    {
        public TreeView treeView_Navigator = new TreeView();

        ContextMenuStrip contextMenuStrip_DriveApplication = new ContextMenuStrip();

        //Context menus and event handlers for (Load/Move) Add or Insert 
        ContextMenuStrip contextMenuStrip_AddInsertLoad = new ContextMenuStrip();
        public event EventHandler<AddInsertLoadEventArgs> AddInsertLoadEvent;

        ContextMenuStrip contextMenuStrip_AddInsertMove = new ContextMenuStrip();
        public event EventHandler<AddInsertMoveEventArgs> AddInsertMoveEvent;

        //Context menus and event handlers for (Load/Move) Move or Remove
        ContextMenuStrip contextMenuStrip_MoveRemoveLoad = new ContextMenuStrip();
        public event EventHandler<MoveRemoveLoadEventArgs> MoveRemoveLoadEvent;

        ContextMenuStrip contextMenuStrip_MoveRemoveMove = new ContextMenuStrip();
        public event EventHandler<MoveRemoveMoveEventArgs> MoveRemoveMoveEvent;

        public FrmNavigator()
        {
            InitializeComponent();
        }

        public void CreateTreeView()
        {
            //Define and add the TreeView
            treeView_Navigator.Dock = DockStyle.Fill;
            treeView_Navigator.HotTracking = true;
            treeView_Navigator.Scrollable = true;

            //Create an event handler that selects the treeview node on mouse click
            //This is needed in order for the context menus to work

            treeView_Navigator.NodeMouseClick +=new TreeNodeMouseClickEventHandler(treeView_Navigator_NodeMouseClick);

            Controls.Add(treeView_Navigator);

            //Add base root node
            TreeNode rootNode = new TreeNode("DriveApplication");
            treeView_Navigator.Nodes.Add(rootNode);

            //Create contextmenustrip for "Drive application"
            string[] contextMenuStripItems = new string[] { };

            contextMenuStripItems = new string[] { "Expand all", "Collapse all" };

            foreach (string str in contextMenuStripItems)
            {
                ToolStripMenuItem newItem = new ToolStripMenuItem(str);
                newItem.Click += new EventHandler(ContextMenuStrip_DriveApplication_Click);
                contextMenuStrip_DriveApplication.Items.Add(newItem);
            }

            rootNode.ContextMenuStrip = contextMenuStrip_DriveApplication;

            //Create variables for the child nodes. 
            //NodeName is used for search and NodeText is used for display.
            List<string> childNodeNames = new List<string>();
            List<string> childNodeTexts = new List<string>();

            //Add childen to the "Drive application" node
            childNodeNames.Add("OverView");
            childNodeTexts.Add("OverView");

            childNodeNames.Add("Accessories");
            childNodeTexts.Add("Accessories");

            childNodeNames.Add("Results");
            childNodeTexts.Add("Results");
            
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Add child nodes for "OverView"
            childNodeNames.Add("ApplicationData");
            childNodeTexts.Add("ApplicationData");

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "OverView");

            //Add the node to the TreeView control
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Add child nodes for "Results"
            childNodeNames.Add("Moves");
            childNodeTexts.Add("Moves");

            childNodeNames.Add("Components");
            childNodeTexts.Add("Components");

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Results");

            //Add the node to the TreeView control
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Add child nodes for "Components"
            childNodeNames.Add("Motor");
            childNodeTexts.Add("Motor");

            childNodeNames.Add("Loads");
            childNodeTexts.Add("Loads");

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Components");

            //Add the node to the TreeView control
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Create contextmenustrip for "Loads"
            contextMenuStripItems = new string[] { "New load" };

            foreach (string str in contextMenuStripItems)
            {
                ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                newChildMenuItem.Click += new EventHandler(ContextMenuStrip_AddInsertLoad_Click);
                contextMenuStrip_AddInsertLoad.Items.Add(newChildMenuItem);
            }

            rootNode = FindNode(treeView_Navigator.Nodes[0], "Loads");
            rootNode.ContextMenuStrip = contextMenuStrip_AddInsertLoad;

            //subcontextmenustrip for "New load"
            contextMenuStripItems = new string[] { 
                MainType.BallScrew.ToString(), MainType.Coupling.ToString(), MainType.GearBox.ToString(), MainType.ToothedBelt.ToString(),
                MainType.RackAndPinion.ToString(), MainType.TurnTable.ToString(),
                MainType.Mass.ToString(), MainType.Inertia.ToString(), MainType.Torque.ToString(), MainType.Force.ToString() };

            for (int i = 0; i < contextMenuStrip_AddInsertLoad.Items.Count; i++)
            {
                ToolStripMenuItem parentMenuItem = (ToolStripMenuItem)contextMenuStrip_AddInsertLoad.Items[i];
                if (parentMenuItem.Text == "New load")
                {
                    foreach (string str in contextMenuStripItems)
                    {                        
                        ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                        newChildMenuItem.Click += new EventHandler(ContextMenuStrip_AddInsertLoad_Click);
                        parentMenuItem.DropDownItems.Add(newChildMenuItem);

                        if (str == MainType.TurnTable.ToString())
                            parentMenuItem.DropDownItems.Add(new ToolStripSeparator()); 
                    }
                }
            }

            //Create contextmenustrip for "Moves"
            contextMenuStripItems = new string[] { "New move" };

            foreach (string str in contextMenuStripItems)
            {
                ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                newChildMenuItem.Click += new EventHandler(ContextMenuStrip_AddInsertMove_Click);
                contextMenuStrip_AddInsertMove.Items.Add(newChildMenuItem);
            }

            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            rootNode.ContextMenuStrip = contextMenuStrip_AddInsertMove;

            //subcontextmenustrip for "New move"
            contextMenuStripItems = new string[] { MoveType.PreDefined.ToString(), MoveType.UserDefined.ToString() };

            for (int i = 0; i < contextMenuStrip_AddInsertMove.Items.Count; i++)
            {
                ToolStripMenuItem parentMenuItem = (ToolStripMenuItem)contextMenuStrip_AddInsertMove.Items[i];
                if (parentMenuItem.Text == "New move")
                {
                    foreach (string str in contextMenuStripItems)
                    {
                        ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                        newChildMenuItem.Click += new EventHandler(ContextMenuStrip_AddInsertMove_Click);
                        parentMenuItem.DropDownItems.Add(newChildMenuItem);
                    }
                }
            }

            //Create contextmenustrip for (Load/Move) "Move" and "Load"
            contextMenuStripItems = new string[] { MoveRemoveEvent.Insert.ToString(), MoveRemoveEvent.MoveUp.ToString(), 
                MoveRemoveEvent.MoveDown.ToString(), MoveRemoveEvent.Remove.ToString() };

            //Load
            foreach (string str in contextMenuStripItems)
            {
                ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                newChildMenuItem.Click += new EventHandler(ContextMenuStrip_MoveRemoveLoad_Click);
                contextMenuStrip_MoveRemoveLoad.Items.Add(newChildMenuItem);
            }

            //Move
            foreach (string str in contextMenuStripItems)
            {
                ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                newChildMenuItem.Click += new EventHandler(ContextMenuStrip_MoveRemoveMove_Click);
                contextMenuStrip_MoveRemoveMove.Items.Add(newChildMenuItem);
            }

            //Create sub contextmenustrip for "Insert Load"
            contextMenuStripItems = new string[] { 
                MainType.BallScrew.ToString(), MainType.Coupling.ToString(), MainType.GearBox.ToString(), MainType.ToothedBelt.ToString(),
                MainType.RackAndPinion.ToString(), MainType.TurnTable.ToString(),
                MainType.Mass.ToString(), MainType.Inertia.ToString(), MainType.Torque.ToString(), MainType.Force.ToString() };

            for (int i = 0; i < contextMenuStrip_MoveRemoveLoad.Items.Count; i++)
            {
                ToolStripMenuItem parentMenuItem = (ToolStripMenuItem)contextMenuStrip_MoveRemoveLoad.Items[i];
                if (parentMenuItem.Text == MoveRemoveEvent.Insert.ToString())
                {
                    foreach (string str in contextMenuStripItems)
                    {
                        ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                        newChildMenuItem.Click += new EventHandler(ContextMenuStrip_AddInsertLoad_Click);
                        parentMenuItem.DropDownItems.Add(newChildMenuItem);
                        
                        if (str == MainType.TurnTable.ToString())
                            parentMenuItem.DropDownItems.Add(new ToolStripSeparator()); 
                    }
                }
            }

            //Create sub contextmenustrip for "Insert Move"
            contextMenuStripItems = new string[] { MoveType.PreDefined.ToString(), MoveType.UserDefined.ToString() };

            for (int i = 0; i < contextMenuStrip_MoveRemoveMove.Items.Count; i++)
            {
                ToolStripMenuItem parentMenuItem = (ToolStripMenuItem)contextMenuStrip_MoveRemoveMove.Items[i];
                if (parentMenuItem.Text == MoveRemoveEvent.Insert.ToString())
                {
                    foreach (string str in contextMenuStripItems)
                    {
                        ToolStripMenuItem newChildMenuItem = new ToolStripMenuItem(str);
                        newChildMenuItem.Click += new EventHandler(ContextMenuStrip_AddInsertMove_Click);
                        parentMenuItem.DropDownItems.Add(newChildMenuItem);
                    }
                }
            }
        }

        public void AddApplicationDataNode(ApplicationData applicationData)
        {
            TreeNode rootNode = new TreeNode();

            //Create variables for the child nodes. 
            //NodeName is used for search and NodeText is used for display.
            List<string> childNodeNames = new List<string>();
            List<string> childNodeTexts = new List<string>();

            //Add children to the "Application data"-node
            childNodeNames.Add("AppName");
            if (applicationData.AppName != null)
                childNodeTexts.Add("AppName: " + applicationData.AppName);
            else
                childNodeTexts.Add("AppName: ");

            childNodeNames.Add("Date");
            if (applicationData.Date != null)
                childNodeTexts.Add("Date: " + applicationData.Date);
            else
                childNodeTexts.Add("Date: ");

            childNodeNames.Add("Customer");
            if (applicationData.Customer != null)
                childNodeTexts.Add("Customer: " + applicationData.Customer);
            else
                childNodeTexts.Add("Customer: ");

            childNodeNames.Add("OurRef");
            if (applicationData.OurRef != null)
                childNodeTexts.Add("OurRef: " + applicationData.OurRef);
            else
                childNodeTexts.Add("OurRef: ");

            childNodeNames.Add("YourRef");
            if (applicationData.YourRef != null)
                childNodeTexts.Add("YourRef: " + applicationData.YourRef);
            else
                childNodeTexts.Add("YourRef: ");

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "ApplicationData");

            //Add the node to the TreeView control
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            treeView_Navigator.ExpandAll();
        }
     
        public void AddLoadNode(int index, ILoad load, string node)
        {
            TreeNode rootNode = new TreeNode();

            rootNode = FindNode(treeView_Navigator.Nodes[0], "Loads");

            //Add child to the "Loads" node
            TreeNode childNode = new TreeNode(node);
            childNode.Name = node;
            childNode.Text = node;
            //Add the context menu
            childNode.ContextMenuStrip = contextMenuStrip_MoveRemoveLoad;

            rootNode.Nodes.Insert(index, childNode);

            treeView_Navigator.SelectedNode = childNode;
        }

        public void AddMoveNode(int index, Move move, string node)
        {
            TreeNode rootNode = new TreeNode();

            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");

            //Add child to the "Moves" node
            TreeNode childNode = new TreeNode(node);
            childNode.Name = node;
            childNode.Text = node;

            //Add the context menu
            childNode.ContextMenuStrip = contextMenuStrip_MoveRemoveMove;

            rootNode.Nodes.Insert(index, childNode);

            //Create variables for the child nodes. 
            //NodeName is used for search and NodeText is used for display.
            List<string> childNodeNames = new List<string>();
            List<string> childNodeTexts = new List<string>();

            childNodeNames.Add("MoveType");
            childNodeTexts.Add("MoveType: " + move.MoveType.ToString());

            childNodeNames.Add("Profile");
            childNodeTexts.Add("Profile: " + move.Profile.ToString());

            childNodeNames.Add("Time");
            childNodeTexts.Add("Time: " + move.Time.ToString());

            childNodeNames.Add("Distance");
            childNodeTexts.Add("Distance: " + move.Distance.ToString());

            childNodeNames.Add("Sectors");
            childNodeTexts.Add("Sectors");

            childNodeNames.Add("LoadResults");
            childNodeTexts.Add("LoadResults");

            //Find the node where the sub nodes should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            rootNode = FindNode(rootNode, childNode.Name);

            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Add children to the "LoadResults" node

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            childNodeNames.Add("TotalGearing");
            childNodeTexts.Add("TotalGearing: " + move.TotalLoadResults.AccOutGearing.ToString() + " :1");

            childNodeNames.Add("PeakSpeed");
            childNodeTexts.Add("PeakSpeed: " + move.TotalLoadResults.PeakSpd.ToString() + " m/s");

            childNodeNames.Add("RmsSpeed");
            childNodeTexts.Add("RmsSpeed: " + move.TotalLoadResults.RmsSpdED.ToString() + " m/s");

            string movePerRevUnit = " mm";

            if (move.TotalLoadResults.InpMovePerRevUnitmm == false)
                movePerRevUnit = " ";            

            childNodeNames.Add("MovePerRev");
            childNodeTexts.Add("MovePerRev: " + move.TotalLoadResults.InpMovePerRev.ToString() + movePerRevUnit);

            childNodeNames.Add("Backlash");
            childNodeTexts.Add("Backlash: " + move.TotalLoadResults.Backlash.ToString() + " arcmin");

            childNodeNames.Add("TotalEfficiency");
            childNodeTexts.Add("TotalEfficiency: " + move.TotalLoadResults.Efficiency.ToString() + " %"); 
            
            childNodeNames.Add("TotalInertia");
            childNodeTexts.Add("TotalInertia: " + move.TotalLoadResults.Inertia.ToString() + " kgcm");

            childNodeNames.Add("TotalAccTorque");
            childNodeTexts.Add("TotalAccTorque: " + move.TotalLoadResults.AccTorque.ToString() + " Nm");

            childNodeNames.Add("TotalConstTorque");
            childNodeTexts.Add("TotalConstTorque: " + move.TotalLoadResults.ConstTorque.ToString() + " Nm");

            childNodeNames.Add("TotalDecTorque");
            childNodeTexts.Add("TotalDecTorque: " + move.TotalLoadResults.DecTorque.ToString() + " Nm");

            childNodeNames.Add("TotalPeakTorque");
            childNodeTexts.Add("TotalPeakTorque: " + move.TotalLoadResults.PeakTrq.ToString() + " Nm");

            childNodeNames.Add("TotalRmsTorque");
            childNodeTexts.Add("TotalRmsTorque: " + move.TotalLoadResults.RmsTrqED.ToString() + " Nm");

            //Find the node where the sub nodes should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            rootNode = FindNode(rootNode, childNode.Name);
            rootNode = FindNode(rootNode, "LoadResults");

            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Expand the "Moves" node to display the added node
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            rootNode.Expand();
        }

        public void AddSectorNode(Sector sector, string parentNode, string node, int position)
        {
            TreeNode rootNode = new TreeNode();

            //Find the node where the sub nodes should be added
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            rootNode = FindNode(rootNode, parentNode);
            rootNode = FindNode(rootNode, "Sectors");

            //Add the new sector node
            TreeNode childNode = new TreeNode();
            childNode.Name = node;
            childNode.Text = node;

            rootNode.Nodes.Insert(position, childNode);

            //Find the added node
            rootNode = FindNode(rootNode, node);

            //Create variables for the child nodes. 
            //NodeName is used for search and NodeText is used for display.
            List<string> childNodeNames = new List<string>();
            List<string> childNodeTexts = new List<string>();

            //Add children to the new sector node
            childNodeNames.Add("Distance");
            childNodeTexts.Add("Distance: " + sector.Distance.ToString() + " m");

            childNodeNames.Add("Time");
            childNodeTexts.Add("Time: " + sector.Time.ToString() + " s");

            childNodeNames.Add("AverageAcc");
            childNodeTexts.Add("AverageAcc: " + sector.AverageAcc.ToString() + " m/s");

            childNodeNames.Add("InitialSpeed");
            childNodeTexts.Add("InitialSpeed: " + sector.InitialSpeed.ToString() + " m/s");

            childNodeNames.Add("FinalSpeed");
            childNodeTexts.Add("FinalSpeed: " + sector.FinalSpeed.ToString() + " m/s");

            childNodeNames.Add("PercentSineVel");
            childNodeTexts.Add("PercentSineVel: " + sector.PercentSineVel.ToString() + " m/s");

            childNodeNames.Add("PercentSineAcc");
            childNodeTexts.Add("PercentSineAcc: " + sector.PercentSineAcc.ToString() + " m/s");

            childNodeNames.Add("AverageSpeed");
            childNodeTexts.Add("AverageSpeed: " + sector.AverageSpeed.ToString() + " m/s");

            childNodeNames.Add("PeakAcc");
            childNodeTexts.Add("PeakAcc: " + sector.PeakAcc.ToString() + " m/s");

            childNodeNames.Add("ChangingAccTime");
            childNodeTexts.Add("ChangingAccTime: " + sector.ChangingAccTime.ToString() + " s");

            childNodeNames.Add("AverageJerk");
            childNodeTexts.Add("AverageJerk: " + sector.AverageJerk.ToString() + " m/s");

            childNodeNames.Add("PeakJerk");
            childNodeTexts.Add("PeakJerk: " + sector.PeakJerk.ToString() + " m/s");

            childNodeNames.Add("ChangingJerkTime");
            childNodeTexts.Add("ChangingJerkTime: " + sector.ChangingJerkTime.ToString() + " s");

            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);
        }

        public void RemoveLoadNode(string node)
        {
            TreeNode rootNode = new TreeNode();
            TreeNode childNode = new TreeNode();

            //Find the node where the sub nodes should be removed
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Loads");
            childNode = FindNode(rootNode, node);

            rootNode.Nodes.Remove(childNode);
        }

        public void SelectLoadNode(string node)
        {
            TreeNode rootNode = new TreeNode();
            TreeNode childNode = new TreeNode();

            //Find the node where the sub nodes should be removed
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Loads");
            childNode = FindNode(rootNode, node);

            treeView_Navigator.SelectedNode = childNode;
        }

        public void RemoveMoveNode(string node)
        {
            TreeNode rootNode = new TreeNode();
            TreeNode childNode = new TreeNode();

            //Find the node where the sub nodes should be removed
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            childNode = FindNode(rootNode, node);

            rootNode.Nodes.Remove(childNode);
        }

        public void RemoveSectorNode(string parentNode, int index)
        {
            TreeNode rootNode = new TreeNode();
            TreeNode childNode = new TreeNode();

            //Find the node where the sub nodes should be removed
            rootNode = FindNode(treeView_Navigator.Nodes[0], "Moves");
            rootNode = FindNode(rootNode, parentNode);
            rootNode = FindNode(rootNode, "Sectors");

            rootNode.Nodes.RemoveAt(index);
        }

        public TreeNode FindNode(TreeNode parentNode, string nodeName)
        {
            foreach (TreeNode node in parentNode.Nodes)
            {
                if (node.Name == nodeName)
                {
                    return node;
                }
                else
                {
                    TreeNode nodeChild = FindNode(node, nodeName);
                    if (nodeChild != null)
                        return nodeChild;
                }
            }
            return (TreeNode)null;
        }

        public void AddTreeNodes(TreeNode parentNode, IList<string> nodeNames, IList<string> nodeTexts)
        {
            //IList takes both "string[]" and "List<string>" as parameters and replaces two functions
            int count = 0;
            foreach (string childName in nodeNames)
            {
                TreeNode childNode = new TreeNode();
                childNode.Name = childName;
                childNode.Text = nodeTexts[count];

                count++;

                parentNode.Nodes.Add(childNode);
            }
        }

        private void treeView_Navigator_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                treeView_Navigator.SelectedNode = e.Node;
            }
        }

        private void ContextMenuStrip_DriveApplication_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem clickedItem = (ToolStripMenuItem)sender;

            if (clickedItem.ToString() == "Expand all")
                treeView_Navigator.ExpandAll();
            else
                treeView_Navigator.CollapseAll();
        }

        private void ContextMenuStrip_AddInsertLoad_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem clickedItem = (ToolStripMenuItem)sender;

            //Find the node from where the clicked event was raised
            TreeNode selectedNode = treeView_Navigator.SelectedNode;

            if (selectedNode != null)
            {
                if (clickedItem.ToString() == MainType.BallScrew.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.BallScrew);

                if (clickedItem.ToString() == MainType.ToothedBelt.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.ToothedBelt);

                if (clickedItem.ToString() == MainType.Coupling.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.Coupling);

                if (clickedItem.ToString() == MainType.GearBox.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.GearBox);

                if (clickedItem.ToString() == MainType.Mass.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.Mass);

                if (clickedItem.ToString() == MainType.TurnTable.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.TurnTable);

                if (clickedItem.ToString() == MainType.RackAndPinion.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.RackAndPinion);
                
                //ToDo:                
                if (clickedItem.ToString() == MainType.Inertia.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.Inertia);
                
                if (clickedItem.ToString() == MainType.Torque.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.Torque);
                
                if (clickedItem.ToString() == MainType.Force.ToString())
                    RaiseAddInsertLoadEvent(selectedNode.Text, MainType.Force);                 
            }
        }

        private void RaiseAddInsertLoadEvent(string name, MainType type)
        {
            EventHandler<AddInsertLoadEventArgs> handler = AddInsertLoadEvent;
            if (handler != null)
                handler(null, new AddInsertLoadEventArgs(name, type));
        }

        private void ContextMenuStrip_MoveRemoveLoad_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem clickedItem = (ToolStripMenuItem)sender;

            TreeNode selectedNode = treeView_Navigator.SelectedNode;

            if (selectedNode.Name != null)
            {
                if (clickedItem.ToString() == MoveRemoveEvent.MoveDown.ToString())
                    RaiseMoveRemoveLoadEvent(selectedNode.Text, MoveRemoveEvent.MoveDown);

                if (clickedItem.ToString() == MoveRemoveEvent.MoveUp.ToString())
                    RaiseMoveRemoveLoadEvent(selectedNode.Text, MoveRemoveEvent.MoveUp);

                if (clickedItem.ToString() == MoveRemoveEvent.Remove.ToString())
                    RaiseMoveRemoveLoadEvent(selectedNode.Text, MoveRemoveEvent.Remove);
            }
        }

        private void RaiseMoveRemoveLoadEvent(string name, MoveRemoveEvent nodeEvent)
        {
            EventHandler<MoveRemoveLoadEventArgs> handler = MoveRemoveLoadEvent;
            if (handler != null)
                handler(null, new MoveRemoveLoadEventArgs(name, nodeEvent));
        }

        private void ContextMenuStrip_AddInsertMove_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem clickedItem = (ToolStripMenuItem)sender;

            TreeNode selectedNode = treeView_Navigator.SelectedNode;

            if (selectedNode != null)
            {
                if (clickedItem.ToString() == MoveType.PreDefined.ToString())
                    RaiseAddInsertMoveEvent(selectedNode.Text, MoveType.PreDefined);

                if (clickedItem.ToString() == MoveType.UserDefined.ToString())
                    RaiseAddInsertMoveEvent(selectedNode.Text, MoveType.UserDefined);
            }
        }

        private void RaiseAddInsertMoveEvent(string name, MoveType moveType)
        {
            EventHandler<AddInsertMoveEventArgs> handler = AddInsertMoveEvent;
            if (handler != null)
                handler(null, new AddInsertMoveEventArgs(name, moveType));
        }

        private void ContextMenuStrip_MoveRemoveMove_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem clickedItem = (ToolStripMenuItem)sender;

            TreeNode selectedNode = treeView_Navigator.SelectedNode;

            if (selectedNode.Name != null)
            {
                if (clickedItem.ToString() == MoveRemoveEvent.Insert.ToString())
                    RaiseMoveRemoveMoveEvent(selectedNode.Text, MoveRemoveEvent.Insert);

                if (clickedItem.ToString() == MoveRemoveEvent.MoveDown.ToString())
                    RaiseMoveRemoveMoveEvent(selectedNode.Text, MoveRemoveEvent.MoveDown);

                if (clickedItem.ToString() == MoveRemoveEvent.MoveUp.ToString())
                    RaiseMoveRemoveMoveEvent(selectedNode.Text, MoveRemoveEvent.MoveUp);

                if (clickedItem.ToString() == MoveRemoveEvent.Remove.ToString())
                    RaiseMoveRemoveMoveEvent(selectedNode.Text, MoveRemoveEvent.Remove);
            }
        }

        private void RaiseMoveRemoveMoveEvent(string name, MoveRemoveEvent nodeEvent)
        {
            EventHandler<MoveRemoveMoveEventArgs> handler = MoveRemoveMoveEvent;
            if (handler != null)
                handler(null, new MoveRemoveMoveEventArgs(name, nodeEvent));
        }
    }
}