﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using System.IO;
using System.Data.OleDb;
using System.Linq;
using System.Text.RegularExpressions;

namespace DockSample
{
    public partial class FrmOverview : DockContent
    {
        private ApplicationData _applicationData;
        private Motor _motor;
        private SortedList<int, ILoad> _iLoads;
        private SortedList<int, IAccessory> _iAccessories;
        
        public FrmOverview(ApplicationData applicationData, Motor motor, SortedList<int, ILoad> iLoads, SortedList<int, IAccessory> iAccessories)
        {
            _applicationData = applicationData as ApplicationData;
            _applicationData.PropertyChanged += new PropertyChangedEventHandler(ApplicationData_PropertyChanged);

            _motor = motor as Motor;            
            _iLoads = iLoads as SortedList<int, ILoad>;
            _iAccessories = iAccessories as SortedList<int, IAccessory>;

            InitializeComponent();
        }

        private void FrmApplicationData_Load(object sender, EventArgs e) 
        {
           // treeView_LoadSide.DrawNode +=
             //   new DrawTreeNodeEventHandler(TreeView_DrawNode);
        }

        private void FrmOverview_Activated(object sender, EventArgs e)
        {
            textBox_AppName.Text = _applicationData.AppName;
            textBox_Date.Text = _applicationData.Date;
            textBox_Customer.Text = _applicationData.Customer;
            textBox_OurRef.Text = _applicationData.OurRef;
            textBox_YourRef.Text = _applicationData.YourRef;
            textBox_Description.Text = _applicationData.Description;

            CreateBillOfMaterial();
            CreateTreeView_MotorSide();
            CreateTreeView_LoadSide();
        }

        private void ApplicationData_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "AppName":
                    textBox_AppName.Text = _applicationData.AppName;
                    break;

                case "Date":
                    textBox_Date.Text = _applicationData.Date;
                    break;

                case "Customer":
                    textBox_Customer.Text = _applicationData.Customer;
                    break;

                case "OurRef":
                    textBox_OurRef.Text = _applicationData.OurRef;
                    break;

                case "YourRef":
                    textBox_YourRef.Text = _applicationData.YourRef;
                    break;

                case "Description":
                    textBox_Description.Text = _applicationData.Description;
                    break;
            }
        }

        private void textBox_ApplicationAppName_Validated(object sender, EventArgs e)
        {
            _applicationData.AppName = textBox_AppName.Text;
        }

        private void textBox_Date_Validated(object sender, EventArgs e)
        {
            _applicationData.Date = textBox_Date.Text;
        }

        private void textBox_Customer_Validated(object sender, EventArgs e)
        {
            _applicationData.Customer = textBox_Customer.Text;
        }

        private void textBox_OurRef_Validated(object sender, EventArgs e)
        {
            _applicationData.OurRef = textBox_OurRef.Text;
        }

        private void textBox_YourRef_Validated(object sender, EventArgs e)
        {
            _applicationData.YourRef = textBox_YourRef.Text;
        }

        private void textBox_Description_Validated(object sender, EventArgs e)
        {
        _applicationData.Description = textBox_Description.Text;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            // Create a Bitmap object from an image file.
            Bitmap myBitmap = new Bitmap(Properties.Resources.Beijerlogo);

            // Make backColor transparent for myBitmap.
            myBitmap.MakeTransparent();

            // Draw the transparent bitmap to the screen.
            e.Graphics.DrawImage(
                myBitmap, 740, 56, 180, 80);
        }

        public void CreateBillOfMaterial()
        {
            //Clear old data
            if (dataGridView_BillOfMaterial.Rows.Count > 0)
                dataGridView_BillOfMaterial.Rows.Clear();

            //Go through all components and add those with an existing article number

            //Create local component variables
            string articleNo = "";
            string mainType = "";
            string brand = "";
            string name = "";
            string condition = "";
            string description = "";

            if (_motor.Campaign)
                condition = "Campaign";
            else
                condition = "New";

            //Motor
            if (_motor.ArticleNo != null && _motor.ArticleNo != "")
                dataGridView_BillOfMaterial.Rows.Add(_motor.ArticleNo, _motor.MainType, _motor.Brand, _motor.Name, _motor.Description);

            //go through all loads                
            foreach (var lR in _iLoads)
            {
                ILoad iL = lR.Value;
                int loadNr = lR.Key;

                switch (iL.MainType)
                {
                    case MainType.GearBox:
                        var gearBox = iL as GearBox;

                        articleNo = gearBox.ArticleNo;
                        mainType = gearBox.MainType.ToString();
                        brand = gearBox.Brand.ToString();
                        name = gearBox.Name;

                        if (gearBox.Campaign)
                            condition = "Campaign";
                        else
                            condition = "New";

                        description = gearBox.Description;

                        break;

                    case MainType.Coupling:
                        var coupling = iL as Coupling;

                        articleNo = coupling.ArticleNo;
                        mainType = coupling.MainType.ToString();
                        brand = coupling.Brand.ToString();
                        name = coupling.Name;

                        if (coupling.Campaign)
                            condition = "Campaign";
                        else
                            condition = "New";

                        description = coupling.Description;

                        break;

                    case MainType.BallScrew:
                        var ballScrew = iL as BallScrew;

                        articleNo = ballScrew.ArticleNo;
                        mainType = ballScrew.MainType.ToString();
                        brand = ballScrew.Brand.ToString();
                        name = ballScrew.Name;

                        if (ballScrew.Campaign)
                            condition = "Campaign";
                        else
                            condition = "New";

                        description = ballScrew.Description;

                        break;

                    case MainType.RackAndPinion:
                        var rackAndPinion = iL as RackAndPinion;

                        articleNo = rackAndPinion.ArticleNo;
                        mainType = rackAndPinion.MainType.ToString();
                        brand = rackAndPinion.Brand.ToString();
                        name = rackAndPinion.Name;

                        if (rackAndPinion.Campaign)
                            condition = "Campaign";
                        else
                            condition = "New";

                        description = rackAndPinion.Description;

                        break;

                    case MainType.TurnTable:
                        var turnTable = iL as TurnTable;

                        articleNo = turnTable.ArticleNo;
                        mainType = turnTable.MainType.ToString();
                        brand = turnTable.Brand.ToString();
                        name = turnTable.Name;

                        if (turnTable.Campaign)
                            condition = "Campaign";
                        else
                            condition = "New";

                        description = turnTable.Description;

                        break;

                    case MainType.ToothedBelt:
                        var toothedBelt = iL as ToothedBelt;

                        articleNo = toothedBelt.ArticleNo;
                        mainType = toothedBelt.MainType.ToString();
                        brand = toothedBelt.Brand.ToString();
                        name = toothedBelt.Name;

                        if (toothedBelt.Campaign)
                            condition = "Campaign";
                        else
                            condition = "New";

                        description = toothedBelt.Description;

                        break;
                }

                if (articleNo != null && articleNo != "")
                    dataGridView_BillOfMaterial.Rows.Add(articleNo, mainType, brand, name, description);
            }

            //Go through all accessories

            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {

                    foreach (var iAcc in _iAccessories)
                    {
                        IAccessory iA = iAcc.Value;

                        //Check the load type and fetch the properties
                        switch (iA.MainType)
                        {
                            case (MainType.MotorCable):
                                Cable motorCable = iA as Cable;

                                //Find the item in the database
                                var qryMotorCable =
                                    from tblCables myMotorCable in dbDriveSystemsContext.tblCables
                                    where myMotorCable.ArticleNo == motorCable.ArticleNo
                                    select myMotorCable;

                                foreach (var dBMotorCable in qryMotorCable) //Should only be one item
                                {
                                    motorCable.Brand = Brand.BSD; 

                                    motorCable.Description = dBMotorCable.Description;
                                    //motorCable.Campaign = dBMotorCable.Campaign;

                                    articleNo = motorCable.ArticleNo;
                                    mainType = motorCable.MainType.ToString();
                                    brand = motorCable.Brand.ToString();
                                    name = motorCable.Name;

                                    if (motorCable.Campaign)
                                        condition = "Campaign";
                                    else
                                        condition = "New";

                                    description = motorCable.Description;
                                }
                                break;

                            case (MainType.EncoderCable):
                                Cable encoderCable = iA as Cable;

                                //Find the item in the database
                                var qryEncoderCable =
                                    from tblCables myEncoderCable in dbDriveSystemsContext.tblCables
                                    where myEncoderCable.ArticleNo == encoderCable.ArticleNo
                                    select myEncoderCable;

                                foreach (var dBEncoderCable in qryEncoderCable) //Should only be one item
                                {
                                    encoderCable.Brand = Brand.BSD;

                                    encoderCable.Description = dBEncoderCable.Description;
                                    //motorCable.Campaign = dBMotorCable.Campaign;

                                    articleNo = encoderCable.ArticleNo;
                                    mainType = encoderCable.MainType.ToString();
                                    brand = encoderCable.Brand.ToString();
                                    name = encoderCable.Name;

                                    if (encoderCable.Campaign)
                                        condition = "Campaign";
                                    else
                                        condition = "New";

                                    description = encoderCable.Description;
                                }
                                break;

                            case (MainType.BrakeCable):
                                Cable brakeCable = iA as Cable;

                                //Find the item in the database
                                var qryBrakeCable =
                                    from tblCables myBrakeCable in dbDriveSystemsContext.tblCables
                                    where myBrakeCable.ArticleNo == brakeCable.ArticleNo
                                    select myBrakeCable;

                                foreach (var dBBrakeCable in qryBrakeCable) //Should only be one item
                                {
                                    brakeCable.Brand = Brand.BSD;

                                    brakeCable.Description = dBBrakeCable.Description;
                                    //motorCable.Campaign = dBMotorCable.Campaign;

                                    articleNo = brakeCable.ArticleNo;
                                    mainType = brakeCable.MainType.ToString();
                                    brand = brakeCable.Brand.ToString();
                                    name = brakeCable.Name;

                                    if (brakeCable.Campaign)
                                        condition = "Campaign";
                                    else
                                        condition = "New";

                                    description = brakeCable.Description;
                                }
                                break;


                            case (MainType.Drive):
                                Drive drv = iA as Drive;

                                //Find the item in the database
                                var qryDrive =
                                    from tblDrives myDrive in dbDriveSystemsContext.tblDrives
                                    where myDrive.ArticleNo == drv.ArticleNo
                                    select myDrive;

                                foreach (var dBDrive in qryDrive) //Should only be one item
                                {

                                    articleNo = drv.ArticleNo;
                                    mainType = drv.MainType.ToString();
                                    brand = drv.Brand.ToString();
                                    name = drv.Name;

                                    if (drv.Campaign)
                                        condition = "Campaign";
                                    else
                                        condition = "New";

                                    description = drv.Description;

                                    drv.CapCharge = dBDrive.CapacitorCharge;
                                    drv.IntBrkRes.Resistance = dBDrive.Resistance;
                                    drv.IntBrkRes.MaxPwr = dBDrive.IntBrakeResMaxPwr;
                                    drv.IntBrkRes.MaxPwrDur = dBDrive.IntBrakeResDur;
                                    drv.IntBrkRes.PwrED = dBDrive.IntBrakeResED;
                                }
                                break;

                            case (MainType.EMCFilter):
                                Filter emc = iA as Filter;

                                articleNo = emc.ArticleNo;
                                mainType = emc.MainType.ToString();
                                brand = emc.Brand.ToString();
                                name = emc.Name;

                                if (emc.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = emc.Description;

                                break;

                            case (MainType.DCFilter):
                                Filter dc = iA as Filter;

                                articleNo = dc.ArticleNo;
                                mainType = dc.MainType.ToString();
                                brand = dc.Brand.ToString();
                                name = dc.Name;

                                if (dc.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = dc.Description;

                                break;

                            case (MainType.BrakeResistor):
                                BrakeResistor bRes = iA as BrakeResistor;

                                articleNo = bRes.ArticleNo;
                                mainType = bRes.MainType.ToString();
                                brand = bRes.Brand.ToString();
                                name = bRes.Name;

                                if (bRes.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = bRes.Description;

                                break;

                            case (MainType.Safety):
                                Option safe = iA as Option;

                                articleNo = safe.ArticleNo;
                                mainType = safe.MainType.ToString();
                                brand = safe.Brand.ToString();
                                name = safe.Name;

                                if (safe.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = safe.Description;

                                break;

                            case (MainType.Battery):
                                Option bat = iA as Option;

                                articleNo = bat.ArticleNo;
                                mainType = bat.MainType.ToString();
                                brand = bat.Brand.ToString();
                                name = bat.Name;

                                if (bat.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = bat.Description;

                                break;

                            case (MainType.IOCard):
                                Option io = iA as Option;

                                articleNo = io.ArticleNo;
                                mainType = io.MainType.ToString();
                                brand = io.Brand.ToString();
                                name = io.Name;

                                if (io.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = io.Description;

                                break;

                            case (MainType.IOCable):
                                Option ioc = iA as Option;

                                articleNo = ioc.ArticleNo;
                                mainType = ioc.MainType.ToString();
                                brand = ioc.Brand.ToString();
                                name = ioc.Name;

                                if (ioc.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = ioc.Description;

                                break;

                            case (MainType.BusCable):
                                Option bus = iA as Option;

                                articleNo = bus.ArticleNo;
                                mainType = bus.MainType.ToString();
                                brand = bus.Brand.ToString();
                                name = bus.Name;

                                if (bus.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = bus.Description;

                                break;

                            case (MainType.Controller):
                                Option con = iA as Option;

                                articleNo = con.ArticleNo;
                                mainType = con.MainType.ToString();
                                brand = con.Brand.ToString();
                                name = con.Name;

                                if (con.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = con.Description;

                                break;

                            case (MainType.ExtensionCard):
                                Option ext = iA as Option;

                                articleNo = ext.ArticleNo;
                                mainType = ext.MainType.ToString();
                                brand = ext.Brand.ToString();
                                name = ext.Name;

                                if (ext.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = ext.Description;

                                break;

                            case (MainType.ExtensionCable):
                                Option extCab = iA as Option;

                                articleNo = extCab.ArticleNo;
                                mainType = extCab.MainType.ToString();
                                brand = extCab.Brand.ToString();
                                name = extCab.Name;

                                if (extCab.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = extCab.Description;

                                break;

                            case (MainType.ExtensionTerminal):
                                Option extTerm = iA as Option;

                                articleNo = extTerm.ArticleNo;
                                mainType = extTerm.MainType.ToString();
                                brand = extTerm.Brand.ToString();
                                name = extTerm.Name;

                                if (extTerm.Campaign)
                                    condition = "Campaign";
                                else
                                    condition = "New";

                                description = extTerm.Description;

                                break;
                        }

                        if (articleNo != null && articleNo != "")
                            dataGridView_BillOfMaterial.Rows.Add(articleNo, mainType, brand, name, description);

                        //reset articleNo
                        articleNo = "";
                    }
                }
            }
        }

        public void OldCreateTreeView_MotorSide()
        {
            //Clear old data
            treeView_MotorSide.Nodes.Clear();

            //Add base root node
            TreeNode rootNode = new TreeNode("ComponentOverView_MotorSide");
            treeView_MotorSide.Nodes.Add(rootNode);

            //Create variables for the child nodes. 
            //NodeName is used for search and NodeText is used for display.
            List<string> childNodeNames = new List<string>();
            List<string> childNodeTexts = new List<string>();

            //Create motor node with main part data
            childNodeNames.Add("Motor");
            childNodeTexts.Add("Motor (ArticleNo: " + _motor.ArticleNo + ", Brand: " + _motor.Brand + ", SubType: " + _motor.SubType +
                ", Description: " + _motor.Description + ", Campaign: " + _motor.Campaign + ")");

            //Add child to the root node
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Create motor child nodes

            //Specific data that does not fit into any general category
            childNodeNames.Add("Specific");
            childNodeTexts.Add("Specific (Braked: " + _motor.Braked + ", Efficiency: " + FormatValue(_motor.Eff) + " %, Inertia: " + FormatValue(_motor.Inertia * 10000) +
                " kg•cm², MaxAllowedInertiaRatio: " + FormatValue(_motor.InertiaRatio) + " times, Resolution: " + _motor.Resolution + " ppr, InverseEfficiency: " + _motor.InvEff + "  %)");
            
            //Speed data
            childNodeNames.Add("Speed");
            childNodeTexts.Add("Speed (Nominal: " + FormatValue(_motor.NomSpd) + ", Maximum: " + FormatValue(_motor.MaxSpd) + ", WithFullTorque: " + FormatValue(_motor.FldWeakSpd) +
                ") [rpm], DurationAtMax: " + _motor.MaxSpdDur + " s, ED-factor: " + _motor.SpdED + " %)");

            //Torque data
            childNodeNames.Add("Torque");
            childNodeTexts.Add("Torque (Nominal: " + FormatValue(_motor.NomTrq) + ", Maximum: " + FormatValue(_motor.MaxTrq) + ") [Nm], DurationAtMax: " + FormatValue(_motor.MaxTrqDur) +
                 " s, ED-factor: " + FormatValue(_motor.TrqED) + " %, )");

            //Mechanical data
            childNodeNames.Add("MechanicalOutput");
            childNodeTexts.Add("MechanicalOutput (Type: " + _motor.OutType.ToString() + ", ShaftDiameter: " + FormatValue(_motor.OutMaxDia) + " mm, MaxForce(Axial: " +
                 FormatValue(_motor.OutAxFrc) + ", Radial: " + FormatValue(_motor.OutRadFrc) + ") [N])");

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_MotorSide.Nodes[0], "Motor");

            //Add child to the root node
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Create a node for each accessory
            TreeNode powerCable = new TreeNode();
            TreeNode encoderCable = new TreeNode();
            TreeNode brakeCable = new TreeNode();
            TreeNode drive = new TreeNode();
            TreeNode emcFilter = new TreeNode();
            TreeNode dcFilter = new TreeNode();
            TreeNode brakeResistor = new TreeNode();
            TreeNode safety = new TreeNode();
            TreeNode battery = new TreeNode();
            TreeNode iOCard = new TreeNode();
            TreeNode iOCable = new TreeNode();
            TreeNode busCable = new TreeNode();
            TreeNode controller = new TreeNode();
            TreeNode extensionCard = new TreeNode();
            TreeNode extensionCable = new TreeNode();
            TreeNode extensionTerminal = new TreeNode();

            foreach (var iAcc in _iAccessories)
            {
                IAccessory iA = iAcc.Value;

                //Check the load type and fetch the properties
                switch (iA.MainType)
                {
                    case (MainType.MotorCable):
                        Cable pC = iA as Cable;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            powerCable.Text = "PowerCable (ArticleNo: " + pC.ArticleNo + ", Brand: " + pC.Brand + ", SubType: " + pC.MainType +
                                ", Description: " + pC.Description + ", Campaign: " + pC.Campaign + ")";
                        else
                            powerCable.Text = "";
                        break;

                    case (MainType.EncoderCable):
                        Cable eC = iA as Cable;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            encoderCable.Text = "EncoderCable (ArticleNo: " + eC.ArticleNo + ", Brand: " + eC.Brand + ", SubType: " + eC.MainType +
                                ", Description: " + eC.Description + ", Campaign: " + eC.Campaign + ")";
                        else
                            encoderCable.Text = "";
                        break;

                    case (MainType.BrakeCable):
                        Cable bC = iA as Cable;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            brakeCable.Text = "BrakeCable (ArticleNo: " + bC.ArticleNo + ", Brand: " + bC.Brand + ", SubType: " + bC.MainType +
                                ", Description: " + bC.Description + ", Campaign: " + bC.Campaign + ")";
                        else
                            brakeCable.Text = "";
                        break;

                    case (MainType.Drive):
                        Drive drv = iA as Drive;
                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                        {
                            drive.Text = "Drive (ArticleNo: " + drv.ArticleNo + ", Brand: " + drv.Brand + ", SubType: " + drv.MainType +
                                ", Description: " + drv.Description + ", Campaign: " + drv.Campaign + ")";
                        }
                        else
                            drive.Text = "";
                        break;

                    case (MainType.EMCFilter):
                        Filter emc = iA as Filter;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            emcFilter.Text = "EMCFilter (ArticleNo: " + emc.ArticleNo + ", Brand: " + emc.Brand + ", SubType: " + emc.MainType +
                                ", Description: " + emc.Description + ", Campaign: " + emc.Campaign + ")";
                        break;

                    case (MainType.DCFilter):
                        Filter dc = iA as Filter;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            dcFilter.Text = "DCFilter (ArticleNo: " + dc.ArticleNo + ", Brand: " + dc.Brand + ", SubType: " + dc.MainType +
                                ", Description: " + dc.Description + ", Campaign: " + dc.Campaign + ")";
                        else
                            dcFilter.Text = "";
                        break;

                    case (MainType.BrakeResistor):
                        BrakeResistor bRes = iA as BrakeResistor;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            brakeResistor.Text = "BrakeResistor (ArticleNo: " + bRes.ArticleNo + ", Brand: " + bRes.Brand + ", SubType: " + bRes.MainType +
                                ", Description: " + bRes.Description + ", Campaign: " + bRes.Campaign + ")";
                        else
                            brakeResistor.Text = "";
                        break;

                    case (MainType.Safety):
                        Option safe = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            safety.Text = "Safety (ArticleNo: " + safe.ArticleNo + ", Brand: " + safe.Brand + ", SubType: " + safe.MainType +
                                ", Description: " + safe.Description + ", Campaign: " + safe.Campaign + ")";
                        else
                            safety.Text = "";
                        break;

                    case (MainType.Battery):
                        Option bat = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            battery.Text = "Battery (ArticleNo: " + bat.ArticleNo + ", Brand: " + bat.Brand + ", SubType: " + bat.MainType +
                                ", Description: " + bat.Description + ", Campaign: " + bat.Campaign + ")";
                        else
                            battery.Text = "";
                        break;

                    case (MainType.IOCard):
                        Option io = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            iOCard.Text = "IOCard (ArticleNo: " + io.ArticleNo + ", Brand: " + io.Brand + ", SubType: " + io.MainType +
                                ", Description: " + io.Description + ", Campaign: " + io.Campaign + ")";
                        else
                            iOCard.Text = "";
                        break;

                    case (MainType.IOCable):
                        Option ioc = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            iOCable.Text = "IOCable (ArticleNo: " + ioc.ArticleNo + ", Brand: " + ioc.Brand + ", SubType: " + ioc.MainType +
                                ", Description: " + ioc.Description + ", Campaign: " + ioc.Campaign + ")";
                        else
                            iOCable.Text = "";
                        break;

                    case (MainType.BusCable):
                        Option bus = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            busCable.Text = "BusCable (ArticleNo: " + bus.ArticleNo + ", Brand: " + bus.Brand + ", SubType: " + bus.MainType +
                                ", Description: " + bus.Description + ", Campaign: " + bus.Campaign + ")";
                        else
                            busCable.Text = "";
                        break;

                    case (MainType.Controller):
                        Option con = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            controller.Text = "Controller (ArticleNo: " + con.ArticleNo + ", Brand: " + con.Brand + ", SubType: " + con.MainType +
                                ", Description: " + con.Description + ", Campaign: " + con.Campaign + ")";
                        else
                            controller.Text = "";
                        break;

                    case (MainType.ExtensionCard):
                        Option ext = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            extensionCard.Text = "ExtensionCard (ArticleNo: " + ext.ArticleNo + ", Brand: " + ext.Brand + ", SubType: " + ext.MainType +
                                ", Description: " + ext.Description + ", Campaign: " + ext.Campaign + ")";
                        else
                            extensionCard.Text = "";
                        break;

                    case (MainType.ExtensionCable):
                        Option extCab = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            extensionCable.Text = "ExtensionCable (ArticleNo: " + extCab.ArticleNo + ", Brand: " + extCab.Brand + ", SubType: " + extCab.MainType +
                                ", Description: " + extCab.Description + ", Campaign: " + extCab.Campaign + ")";
                        else
                            extensionCable.Text = "";
                        break;

                    case (MainType.ExtensionTerminal):
                        Option extTerm = iA as Option;

                        if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            extensionTerminal.Text = "ExtensionTerminal (ArticleNo: " + extTerm.ArticleNo + ", Brand: " + extTerm.Brand + ", SubType: " + extTerm.MainType +
                                ", Description: " + extTerm.Description + ", Campaign: " + extTerm.Campaign + ")";
                        else
                            extensionTerminal.Text = "";
                        break;
                }
            }

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Add 'Cables' node
            childNodeNames.Add("MotorCables");
            childNodeTexts.Add("MotorCables");

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_MotorSide.Nodes[0], "Motor");

            //Add child to the root node
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Add data for each motor cable type
            if (powerCable.Text != "")
            {
                childNodeNames.Add("PowerCable");
                childNodeTexts.Add(powerCable.Text);
            }

            if (encoderCable.Text != "")
            {
                childNodeNames.Add("EncoderCable");
                childNodeTexts.Add(encoderCable.Text);
            }

            if (brakeCable.Text != "")
            {
                childNodeNames.Add("BrakeCable");
                childNodeTexts.Add(brakeCable.Text);
            }

            //Find the node where the sub node should be added
            rootNode = FindNode(treeView_MotorSide.Nodes[0], "MotorCables");

            //Add child to the root node
            AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

            //Clear old childnode data
            childNodeNames.Clear();
            childNodeTexts.Clear();

            //Add 'Drive' node
            if (drive.Text != "")
            {
                childNodeNames.Add("Drive");
                childNodeTexts.Add(drive.Text);

                //Find the node where the sub node should be added
                rootNode = FindNode(treeView_MotorSide.Nodes[0], "Drive");

                //Add child to the root node
                AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

                //Find the node where the sub node should be added
                rootNode = FindNode(treeView_MotorSide.Nodes[0], "Motor");

                //Add child to the root node
                AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

                //Clear old childnode data
                childNodeNames.Clear();
                childNodeTexts.Clear();

                //Add 'Accessories' node
                childNodeNames.Add("Accessories");
                childNodeTexts.Add("Accessories");

                //Find the node where the sub node should be added
                rootNode = FindNode(treeView_MotorSide.Nodes[0], "Drive");

                //Add child to the root node
                AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

                //Clear old childnode data
                childNodeNames.Clear();
                childNodeTexts.Clear();

                //Add accessories
                //Add data for each motor cable type
                if (emcFilter.Text != "")
                {
                    childNodeNames.Add("EMCFilter");
                    childNodeTexts.Add(emcFilter.Text);
                }

                if (dcFilter.Text != "")
                {
                    childNodeNames.Add("DCFilter");
                    childNodeTexts.Add(dcFilter.Text);
                }

                if (brakeResistor.Text != "")
                {
                    childNodeNames.Add("BrakeResistor");
                    childNodeTexts.Add(brakeResistor.Text);
                }

                if (safety.Text != "")
                {
                    childNodeNames.Add("Safety");
                    childNodeTexts.Add(safety.Text);
                }

                if (battery.Text != "")
                {
                    childNodeNames.Add("Battery");
                    childNodeTexts.Add(battery.Text);
                }

                if (iOCard.Text != "")
                {
                    childNodeNames.Add("IOCard");
                    childNodeTexts.Add(iOCard.Text);
                }

                if (iOCable.Text != "")
                {
                    childNodeNames.Add("IOCable");
                    childNodeTexts.Add(iOCable.Text);
                }

                if (busCable.Text != "")
                {
                    childNodeNames.Add("BusCable");
                    childNodeTexts.Add(busCable.Text);
                }

                if (controller.Text != "")
                {
                    childNodeNames.Add("Controller");
                    childNodeTexts.Add(controller.Text);
                }

                if (extensionCard.Text != "")
                {
                    childNodeNames.Add("ExtensionCard");
                    childNodeTexts.Add(extensionCard.Text);
                }

                if (extensionCable.Text != "")
                {
                    childNodeNames.Add("ExtensionCable");
                    childNodeTexts.Add(extensionCable.Text);
                }

                if (extensionTerminal.Text != "")
                {
                    childNodeNames.Add("ExtensionTerminal");
                    childNodeTexts.Add(extensionTerminal.Text);
                }


                //Find the node where the sub node should be added
                rootNode = FindNode(treeView_MotorSide.Nodes[0], "Accessories");

                //Add child to the root node
                if (childNodeNames.Count > 0)
                    AddTreeNodes(rootNode, childNodeNames, childNodeTexts);
            }

            treeView_MotorSide.ExpandAll();
        }

        public void CreateTreeView_MotorSide()
        {
            //Go through all accessories

            //Connect to the database
            string dbPath = Application.StartupPath + @"\Database\DriveSystems.mdb";

            using (OleDbConnection dbDriveSystemsConn = new OleDbConnection(
                "Provider=Microsoft.Jet.OLEDB.4.0;" +
                "Data Source=" + dbPath + ";" +
                "Jet OLEDB:Database Password=BBSD;"))
            {
                using (dbDriveSystemsDataClassesDataContext dbDriveSystemsContext =
                    new dbDriveSystemsDataClassesDataContext(dbDriveSystemsConn))
                {

                    //Clear old data
                    treeView_MotorSide.Nodes.Clear();

                    //Add base root node
                    TreeNode rootNode = new TreeNode("ComponentOverView_MotorSide");
                    treeView_MotorSide.Nodes.Add(rootNode);

                    //Create variables for the child nodes. 
                    //NodeName is used for search and NodeText is used for display.
                    List<string> childNodeNames = new List<string>();
                    List<string> childNodeTexts = new List<string>();

                    //Create motor node with main part data
                    childNodeNames.Add("Motor");
                    childNodeTexts.Add("Motor (ArticleNo: " + _motor.ArticleNo + ", Brand: " + _motor.Brand + ", SubType: " + _motor.SubType +
                        ", Description: " + _motor.Description + ")");

                    //Add child to the root node
                    AddTreeNodes(rootNode, childNodeNames, childNodeTexts);

                    //Clear old childnode data
                    childNodeNames.Clear();
                    childNodeTexts.Clear();

                    //Create motor child nodes

                    //Specific data that does not fit into any general category
                    childNodeNames.Add("Specific");
                    childNodeTexts.Add("Specific (Braked: " + _motor.Braked + ", Efficiency: " + FormatValue(_motor.Eff) + " %, Inertia: " + FormatValue(_motor.Inertia * 10000) +
                        " kg•cm², MaxAllowedInertiaRatio: " + FormatValue(_motor.InertiaRatio) + " times, Resolution: " + _motor.Resolution + " ppr, InverseEfficiency: " + _motor.InvEff + "  %)");

                    //Speed data
                    childNodeNames.Add("Speed");
                    childNodeTexts.Add("Speed (Nominal: " + FormatValue(_motor.NomSpd) + ", Maximum: " + FormatValue(_motor.MaxSpd) + ", WithFullTorque: " + FormatValue(_motor.FldWeakSpd) +
                        ") [rpm], DurationAtMax: " + _motor.MaxSpdDur + " s, ED-factor: " + _motor.SpdED + " %)");

                    //Torque data
                    childNodeNames.Add("Torque");
                    childNodeTexts.Add("Torque (Nominal: " + FormatValue(_motor.NomTrq) + ", Maximum: " + FormatValue(_motor.MaxTrq) + ") [Nm], DurationAtMax: " + FormatValue(_motor.MaxTrqDur) +
                         " s, ED-factor: " + FormatValue(_motor.TrqED) + " %, )");

                    //Mechanical data
                    childNodeNames.Add("MechanicalOutput");
                    childNodeTexts.Add("MechanicalOutput (Type: " + _motor.OutType.ToString() + ", ShaftDiameter: " + FormatValue(_motor.OutMaxDia) + " mm, MaxForce(Axial: " +
                         FormatValue(_motor.OutAxFrc) + ", Radial: " + FormatValue(_motor.OutRadFrc) + ") [N])");

                    //Create variable for the parent node of each load component
                    List<string> parentNodeNames = new List<string>();
                    List<string> parentNodeTexts = new List<string>();

                    TreeNode parentNode = new TreeNode();
                    // TreeNode lastParentNode = new TreeNode();

                    //Find the node where the sub node should be added
                    parentNode = FindNode(treeView_MotorSide.Nodes[0], "Motor");

                    //Add child to the root node
                    AddTreeNodes(parentNode, childNodeNames, childNodeTexts);

                    foreach (var iAcc in _iAccessories)
                    {
                        //Clear old childnode data
                        parentNodeNames.Clear();
                        parentNodeTexts.Clear();

                        //Clear old childnode data
                        childNodeNames.Clear();
                        childNodeTexts.Clear();

                        IAccessory iA = iAcc.Value;

                        //Check the load type and fetch the properties
                        switch (iA.MainType)
                        {

                            case (MainType.MotorCable):

                                Cable pC = iA as Cable;

                                if (iA.ArticleNo != "" && iA.ArticleNo != null)
                                {
                                    parentNodeNames.Add("PowerCable");
                                    parentNodeTexts.Add("PowerCable (ArticleNo: " + pC.ArticleNo + ", Brand: " + pC.Brand + ", SubType: " + pC.MainType +
                                        ", Description: " + pC.Description + ")");
                                }
                                break;

                            case (MainType.EncoderCable):
                                Cable eC = iA as Cable;

                                if (iA.ArticleNo != "" && iA.ArticleNo != null)
                                {
                                    parentNodeNames.Add("EncoderCable");
                                    parentNodeTexts.Add("EncoderCable (ArticleNo: " + eC.ArticleNo + ", Brand: " + eC.Brand + ", SubType: " + eC.MainType +
                                        ", Description: " + eC.Description + ")");
                                }
                                break;

                            case (MainType.BrakeCable):
                                Cable bC = iA as Cable;

                                if (iA.ArticleNo != "" && iA.ArticleNo != null)
                                {
                                    parentNodeNames.Add("BrakeCable");
                                    parentNodeTexts.Add("BrakeCable (ArticleNo: " + bC.ArticleNo + ", Brand: " + bC.Brand + ", SubType: " + bC.MainType +
                                        ", Description: " + bC.Description + ")");
                                }
                                break;

                            case (MainType.Drive):
                                Drive drv = iA as Drive;
                                      
                                if (iA.ArticleNo != "" && iA.ArticleNo != null)
                                {
                                    //Find the item in the database
                                    var qryDrive =
                                        from tblDrives myDrive in dbDriveSystemsContext.tblDrives
                                        where myDrive.ArticleNo == drv.ArticleNo
                                        select myDrive;

                                    foreach (var dBDrive in qryDrive) //Should only be one item
                                    {
                                    parentNodeNames.Add("Drive");
                                    parentNodeTexts.Add("Drive (ArticleNo: " + drv.ArticleNo + ", Brand: " + drv.Brand + ", SubType: " + drv.MainType +
                                        ", Description: " + drv.Description + ")");

                                    //Mechanical data
                                    childNodeNames.Add("Regenerative");
                                    childNodeTexts.Add("Regenerative (CapacitorCharge: " + dBDrive.CapacitorCharge + ", Internal brake resistor data: Resistance: " + dBDrive.Resistance +
                                        " Ω, Maximum Power: " + dBDrive.IntBrakeResMaxPwr + " W, Maximum power duration: " + dBDrive.IntBrakeResDur + " s, ED-factor: " + dBDrive.IntBrakeResED + " %, Nominal power: " + dBDrive.IntBrakeResNomPwr + " W)");
                                    }
                                }
                                break;
                            
                        case (MainType.EMCFilter):
                            Filter emc = iA as Filter;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("EMCFilter");
                                parentNodeTexts.Add("EMCFilter (ArticleNo: " + emc.ArticleNo + ", Brand: " + emc.Brand + ", SubType: " + emc.MainType +
                                    ", Description: " + emc.Description + ")");
                            }
                            break;

                        case (MainType.DCFilter):
                            Filter dc = iA as Filter;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("DCFilter");
                                parentNodeTexts.Add("DCFilter (ArticleNo: " + dc.ArticleNo + ", Brand: " + dc.Brand + ", SubType: " + dc.MainType +
                                    ", Description: " + dc.Description + ")");
                            }
                            break;

                        case (MainType.BrakeResistor):
                            BrakeResistor bRes = iA as BrakeResistor;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                //Find the item in the database
                                var qryBrakeResistor =
                                    from tblBrakeResistors myBrakeResistor in dbDriveSystemsContext.tblBrakeResistors
                                    where myBrakeResistor.ArticleNo == bRes.ArticleNo
                                    select myBrakeResistor;

                                foreach (var dBBres in qryBrakeResistor) //Should only be one item
                                {
                                    parentNodeNames.Add("BrakeResistor");
                                    parentNodeTexts.Add("BrakeResistor (ArticleNo: " + bRes.ArticleNo + ", Brand: " + bRes.Brand + ", SubType: " + bRes.MainType +
                                        ", Description: " + bRes.Description + ")");

                                    //Mechanical data
                                    childNodeNames.Add("Regenerative");
                                    childNodeTexts.Add("Regenerative (Brake resistor data: Resistance: " + dBBres.Resistance +
                                        " Ω, Maximum Power: " + dBBres.MaxPwr + " W, Maximum power duration: " + dBBres.MaxPwrDur + " s, ED-factor: " + dBBres.PwrED + " %, Nominal power: " + dBBres.NomPwr + " W)");
                                }
                            }
                            break;

                        case (MainType.Safety):
                            Option safe = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("Safety");
                                parentNodeTexts.Add("Safety (ArticleNo: " + safe.ArticleNo + ", Brand: " + safe.Brand + ", SubType: " + safe.MainType +
                                    ", Description: " + safe.Description + ")");
                            }
                            break;

                        case (MainType.Battery):
                            Option bat = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("Battery");
                                parentNodeTexts.Add("Battery (ArticleNo: " + bat.ArticleNo + ", Brand: " + bat.Brand + ", SubType: " + bat.MainType +
                                    ", Description: " + bat.Description + ")");
                            }
                            break;

                        case (MainType.IOCard):
                            Option io = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("IOCard");
                                parentNodeTexts.Add("IOCard (ArticleNo: " + io.ArticleNo + ", Brand: " + io.Brand + ", SubType: " + io.MainType +
                                    ", Description: " + io.Description + ")");
                            }
                            break;

                        case (MainType.IOCable):
                            Option ioc = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("IOCable");
                                parentNodeTexts.Add("IOCable (ArticleNo: " + ioc.ArticleNo + ", Brand: " + ioc.Brand + ", SubType: " + ioc.MainType +
                                    ", Description: " + ioc.Description + ")");
                            }
                            break;

                        case (MainType.BusCable):
                            Option bus = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("BusCable");
                                parentNodeTexts.Add("BusCable (ArticleNo: " + bus.ArticleNo + ", Brand: " + bus.Brand + ", SubType: " + bus.MainType +
                                    ", Description: " + bus.Description + ")");
                            }
                            break;

                        case (MainType.Controller):
                            Option con = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("Controller");
                                parentNodeTexts.Add("Controller (ArticleNo: " + con.ArticleNo + ", Brand: " + con.Brand + ", SubType: " + con.MainType +
                                    ", Description: " + con.Description + ")");
                            }
                            break;

                        case (MainType.ExtensionCard):
                            Option ext = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("ExtensionCard");
                                parentNodeTexts.Add("ExtensionCard (ArticleNo: " + ext.ArticleNo + ", Brand: " + ext.Brand + ", SubType: " + ext.MainType +
                                    ", Description: " + ext.Description + ")");
                            }
                            break;

                        case (MainType.ExtensionCable):
                            Option extCab = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("ExtensionCable");
                                parentNodeTexts.Add("ExtensionCable (ArticleNo: " + extCab.ArticleNo + ", Brand: " + extCab.Brand + ", SubType: " + extCab.MainType +
                                    ", Description: " + extCab.Description + ")");
                            }
                            break;

                        case (MainType.ExtensionTerminal):
                            Option extTerm = iA as Option;

                            if (iA.ArticleNo != "" && iA.ArticleNo != null)
                            {
                                parentNodeNames.Add("ExtensionTerminal");
                                parentNodeTexts.Add("ExtensionTerminal (ArticleNo: " + extTerm.ArticleNo + ", Brand: " + extTerm.Brand + ", SubType: " + extTerm.MainType +
                                    ", Description: " + extTerm.Description + ")");
                            }
                            break;                            
                        }

                        //Find the node where the sub node should be added
                        if (parentNodeNames.Count > 0)
                        {
                            //rootNode = FindNode(treeView_LoadSide.Nodes[0], "ComponentOverView_MotorSide");

                            AddTreeNodes(rootNode, parentNodeNames, parentNodeTexts);

                            parentNode = FindNode(treeView_MotorSide.Nodes[0], parentNodeNames[0]);

                            //Add child to the root node
                            AddTreeNodes(parentNode, childNodeNames, childNodeTexts);

                            //lastParentNode.Name = parentNodeNames[0];
                            //lastParentNode.Text = parentNodeTexts[0];

                        }

                    }
                }

                treeView_MotorSide.ExpandAll();
            }
        }
        public void CreateTreeView_LoadSide()
        {
            //Clear old data
            treeView_LoadSide.Nodes.Clear();

            //Add base root node
            TreeNode rootNode = new TreeNode();
            
            rootNode.Name = "ComponentOverView_LoadSide";
            rootNode.Text = "ComponentOverView_LoadSide";

            treeView_LoadSide.Nodes.Add(rootNode);

            //Create variable for the parent node of each load component
            List<string> parentNodeNames = new List<string>();
            List<string> parentNodeTexts = new List<string>();

            TreeNode parentNode = new TreeNode();
            TreeNode lastParentNode = new TreeNode();

            //Create variables for the child nodes. 
            //NodeName is used for search and NodeText is used for display.
            List<string> childNodeNames = new List<string>();
            List<string> childNodeTexts = new List<string>();

            //Add a node for each components 
            foreach (var iL in _iLoads)
            {
                //Clear old childnode data
                parentNodeNames.Clear();
                parentNodeTexts.Clear();

                //Clear old childnode data
                childNodeNames.Clear();
                childNodeTexts.Clear();

                ILoad iLoad = iL.Value;

                //Check the load type and fetch the properties
                switch (iLoad.MainType)
                {
                    case (MainType.BallScrew):
                        BallScrew bS = iLoad as BallScrew;
                        parentNodeNames.Add("BallScrew");
                        parentNodeTexts.Add("BallScrew " + bS.Name + " (ArticleNo: " + bS.ArticleNo + ", Brand: " + bS.Brand + ", SubType: " + bS.MainType +
                            ", Description: " + bS.Description + ", Campaign: " + bS.Campaign + ")");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (MovePerRev: " + FormatValue(bS.MovePerRev * 1000) + " mm, Angle: " + FormatValue(bS.Angle) + " °, Inertia: " + FormatValue(bS.Inertia * 10000) +
                            " kg•cm², ManualInertiaCalc: " + bS.BallScrewGeometry.ManualInertiaCalc + " , Backlash: " + FormatValue(bS.Backlash) + " arcmin, Efficiency: " + FormatValue(bS.Eff) + " %)");

                        //Speed data
                        childNodeNames.Add("Speed");
                        childNodeTexts.Add("Speed (Nominal: " + FormatValue(bS.NomSpd) + ", Maximum: " + FormatValue(bS.MaxSpd) + ") [rpm], DurationAtMax: "
                            + FormatValue(bS.MaxSpdDur) + " s, ED-factor: " + FormatValue(bS.SpdED) + " %)");

                        //Torque data
                        childNodeNames.Add("Torque");
                        childNodeTexts.Add("Torque (Nominal: " + FormatValue(bS.NomTrq) + ", Maximum: " + FormatValue(bS.MaxTrq) + ", IdleTorque: " + FormatValue(bS.IdleTrq) +
                            ") [Nm], DurationAtMax: " + FormatValue(bS.MaxTrqDur) + " s, ED-factor: " + FormatValue(bS.TrqED) + " %, )");

                        //Geometry data
                        if (bS.BallScrewGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("Geometry");
                            childNodeTexts.Add("Geometry (BodyDimensions (Length: " + FormatValue(bS.BallScrewGeometry.CylinderHeight) + ", Inner: " + FormatValue(bS.BallScrewGeometry.InnerDiameter) + 
                                ", Outer: " + FormatValue(bS.BallScrewGeometry.OuterDiameter) + ") [mm], Material: " + bS.BallScrewGeometry.Material + ", Weight: " + FormatValue(bS.BallScrewGeometry.PartMass) + " kg,)");
                        }
                    
                        //Mechanical data
                        childNodeNames.Add("MechanicalInput");
                        childNodeTexts.Add("MechanicalInput (Type: " + bS.InpType.ToString() + ", ShaftDiameter: " + FormatValue(bS.InpMaxDia) + " mm, MaxForce(Axial: " +
                             FormatValue(bS.InpAxFrc) + ", Radial: " + FormatValue(bS.InpRadFrc) + ") [N])");

                        break;

                    case (MainType.Coupling):
                        Coupling c = iLoad as Coupling;
                        parentNodeNames.Add("Coupling");
                        parentNodeTexts.Add("Coupling " + c.Name + " (ArticleNo: " + c.ArticleNo + ", Brand: " + c.Brand + ", SubType: " + c.MainType +
                            ", Description: " + c.Description + ", Campaign: " + c.Campaign + ")");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (Inertia: " + FormatValue(c.Inertia * 10000) + " kg•cm², ManualInertiaCalc: " + c.CouplingGeometry.ManualInertiaCalc + " , Backlash: " + FormatValue(c.Backlash) + ")");

                        //Speed data
                        childNodeNames.Add("Speed");
                        childNodeTexts.Add("Speed (Nominal: " + FormatValue(c.NomSpd) + ", Maximum: " + FormatValue(c.MaxSpd) + ") [rpm], DurationAtMax: "
                            + FormatValue(c.MaxSpdDur) + " s, ED-factor: " + FormatValue(c.SpdED) + " %)");

                        //Torque data
                        childNodeNames.Add("Torque");
                        childNodeTexts.Add("Torque (Nominal: " + FormatValue(c.NomTrq) + ", Maximum: " + FormatValue(c.MaxTrq) + ") [Nm], DurationAtMax: " 
                            + FormatValue(c.MaxTrqDur) + " s, ED-factor: " + FormatValue(c.TrqED) + " %, )");

                        //Geometry data
                        if (c.CouplingGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("Geometry");
                            childNodeTexts.Add("Geometry (BodyDimensions (Length: " + FormatValue(c.CouplingGeometry.CylinderHeight) + ", Inner: " + FormatValue(c.CouplingGeometry.InnerDiameter) +
                                ", Outer: " + FormatValue(c.CouplingGeometry.OuterDiameter) + ") [mm], Material: " + c.CouplingGeometry.Material + ", Weight: " + FormatValue(c.CouplingGeometry.PartMass) + " kg,)");
                        }

                        //Mechanical data
                        childNodeNames.Add("MechanicalInput");
                        childNodeTexts.Add("MechanicalInput (Type: " + c.InpType.ToString() + ", Diameter (Min: " + FormatValue(c.InpMinDia) + " Max: " + FormatValue(c.InpMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(c.InpAxFrc) + ", Radial: " + FormatValue(c.InpRadFrc) + ") [N])");

                        childNodeNames.Add("MechanicalOutput");
                        childNodeTexts.Add("MechanicalOutput (Type: " + c.OutType.ToString() + ", Diameter (Min: " + FormatValue(c.OutMinDia) + " Max: " + FormatValue(c.OutMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(c.OutAxFrc) + ", Radial: " + FormatValue(c.OutRadFrc) + ") [N])");
                        break;

                    case (MainType.ToothedBelt):
                        ToothedBelt tB = iLoad as ToothedBelt;
                        parentNodeNames.Add("ToothedBelt");
                        parentNodeTexts.Add("ToothedBelt " + tB.Name + " (ArticleNo: " + tB.ArticleNo + ", Brand: " + tB.Brand + ", SubType: " + tB.MainType +
                            ", Description: " + tB.Description + ", Used: " + tB.Campaign + ")");
                                                
                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (MovePerRev: " + FormatValue(tB.MovePerRev * 1000) + " mm, Angle: " + FormatValue(tB.Angle) + " °, Inertia: " + FormatValue(tB.Inertia * 10000) +
                            " kg•cm², ManualInertiaCalc (DriveWheel: " + tB.DriveWheelGeometry.ManualInertiaCalc + ", LoadWheel: " + tB.LoadWheelGeometry.ManualInertiaCalc + ") , Backlash: " + FormatValue(tB.Backlash) + " arcmin, Efficiency: " + FormatValue(tB.Eff) + " %)");

                        //Speed data
                        childNodeNames.Add("Speed");
                        childNodeTexts.Add("Speed (Nominal: " + FormatValue(tB.NomSpd) + ", Maximum: " + FormatValue(tB.MaxSpd) + ") [rpm], DurationAtMax: "
                            + FormatValue(tB.MaxSpdDur) + " s, ED-factor: " + FormatValue(tB.SpdED) + " %)");

                        //Torque data
                        childNodeNames.Add("Torque");
                        childNodeTexts.Add("Torque (Nominal: " + FormatValue(tB.NomTrq) + ", Maximum: " + FormatValue(tB.MaxTrq) + ") [Nm], DurationAtMax: "
                            + FormatValue(tB.MaxTrqDur) + " s, ED-factor: " + FormatValue(tB.TrqED) + " %, )");

                        if (tB.DriveWheelGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("DriveWheelGeometry");
                            childNodeTexts.Add("DriveWheelGeometry (BodyDimensions (Length: " + FormatValue(tB.DriveWheelGeometry.CylinderHeight) + ", Inner: " + FormatValue(tB.DriveWheelGeometry.InnerDiameter) +
                                ", Outer: " + tB.DriveWheelGeometry.OuterDiameter + ") [mm], Material: " + tB.DriveWheelGeometry.Material + ", Inertia: " + FormatValue(tB.DriveWheelGeometry.PartInertia) +
                                " kg•cm², Weight: " + FormatValue(tB.DriveWheelGeometry.PartMass) + " kg,)");
                        }

                        if (tB.LoadWheelGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("LoadWheelGeometry");
                            childNodeTexts.Add("LoadWheelGeometry (BodyDimensions (Length: " + FormatValue(tB.LoadWheelGeometry.CylinderHeight) + ", Inner: " + FormatValue(tB.LoadWheelGeometry.InnerDiameter) +
                                ", Outer: " + FormatValue(tB.LoadWheelGeometry.OuterDiameter) + ") [mm], Material: " + tB.LoadWheelGeometry.Material + ", Inertia: " + FormatValue(tB.LoadWheelGeometry.PartInertia) +
                                " kg•cm², Weight: " + FormatValue(tB.LoadWheelGeometry.PartMass) + " kg,)");
                        }

                        //Mechanical data
                        childNodeNames.Add("MechanicalInput");
                        childNodeTexts.Add("MechanicalInput (Type: " + tB.InpType.ToString() + ", Diameter (Min: " + FormatValue(tB.InpMinDia) + " Max: " + FormatValue(tB.InpMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(tB.InpAxFrc) + ", Radial: " + FormatValue(tB.InpRadFrc) + ") [N])");

                        childNodeNames.Add("MechanicalOutput");
                        childNodeTexts.Add("MechanicalOutput (Type: " + tB.OutType.ToString() + ", Diameter (Min: " + FormatValue(tB.OutMinDia) + " Max: " + FormatValue(tB.OutMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(tB.OutAxFrc) + ", Radial: " + FormatValue(tB.OutRadFrc) + ") [N])");
                        break;

                    case (MainType.TurnTable):
                        TurnTable tT = iLoad as TurnTable;
                        parentNodeNames.Add("TurnTable");
                        parentNodeTexts.Add("TurnTable " + tT.Name + " (ArticleNo: " + tT.ArticleNo + ", Brand: " + tT.Brand + ", SubType: " + tT.MainType +
                            ", Description: " + tT.Description + ", Used: " + tT.Campaign + ")");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (MovePerRev: " + FormatValue(tT.MovePerRev * 1000) + " mm, Angle: " + FormatValue(tT.Angle) + " °, Inertia: " + FormatValue(tT.Inertia * 10000) +
                            " kg•cm², ManualInertiaCalc (DriveWheel: " + tT.TurnTableGeometry.ManualInertiaCalc + ", LoadWheel: " + tT.TurnTableGeometry.ManualInertiaCalc + ") , Backlash: " + FormatValue(tT.Backlash) + " arcmin, Efficiency: " + FormatValue(tT.Eff) + " %)");

                        //Speed data
                        childNodeNames.Add("Speed");
                        childNodeTexts.Add("Speed (Nominal: " + FormatValue(tT.NomSpd) + ", Maximum: " + FormatValue(tT.MaxSpd) + ") [rpm], DurationAtMax: "
                            + FormatValue(tT.MaxSpdDur) + " s, ED-factor: " + FormatValue(tT.SpdED) + " %)");

                        //Torque data
                        childNodeNames.Add("Torque");
                        childNodeTexts.Add("Torque (Nominal: " + FormatValue(tT.NomTrq) + ", Maximum: " + FormatValue(tT.MaxTrq) + ") [Nm], DurationAtMax: "
                            + FormatValue(tT.MaxTrqDur) + " s, ED-factor: " + FormatValue(tT.TrqED) + " %, )");

                        if (tT.TurnTableGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("TurnTableGeometry");
                            childNodeTexts.Add("TurnTableGeometry (BodyDimensions (Length: " + FormatValue(tT.TurnTableGeometry.CylinderHeight) + ", Inner: " + FormatValue(tT.TurnTableGeometry.InnerDiameter) +
                                ", Outer: " + tT.TurnTableGeometry.OuterDiameter + ") [mm], Material: " + tT.TurnTableGeometry.Material + ", Inertia: " + FormatValue(tT.TurnTableGeometry.PartInertia) +
                                " kg•cm², Weight: " + FormatValue(tT.TurnTableGeometry.PartMass) + " kg,)");
                        }

                        //Mechanical data
                        childNodeNames.Add("MechanicalInput");
                        childNodeTexts.Add("MechanicalInput (Type: " + tT.InpType.ToString() + ", Diameter (Min: " + FormatValue(tT.InpMinDia) + " Max: " + FormatValue(tT.InpMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(tT.InpAxFrc) + ", Radial: " + FormatValue(tT.InpRadFrc) + ") [N])");
                        break;

                    case (MainType.RackAndPinion):
                        RackAndPinion rP = iLoad as RackAndPinion;
                        parentNodeNames.Add("RackAndPinion");
                        parentNodeTexts.Add("RackAndPinion " + rP.Name + " (ArticleNo: " + rP.ArticleNo + ", Brand: " + rP.Brand + ", SubType: " + rP.MainType +
                            ", Description: " + rP.Description + ", Used: " + rP.Campaign + ")");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (MovePerRev: " + FormatValue(rP.MovePerRev * 1000) + " mm, Angle: " + FormatValue(rP.Angle) + " °, Inertia: " + FormatValue(rP.Inertia * 10000) +
                            " kg•cm², ManualInertiaCalc: " + rP.RackAndPinionGeometry.ManualInertiaCalc + " , Backlash: " + FormatValue(rP.Backlash) + " arcmin, Efficiency: " + FormatValue(rP.Eff) + " %)");

                        //Speed data
                        childNodeNames.Add("Speed");
                        childNodeTexts.Add("Speed (Nominal: " + FormatValue(rP.NomSpd) + ", Maximum: " + FormatValue(rP.MaxSpd) + ") [rpm], DurationAtMax: "
                            + FormatValue(rP.MaxSpdDur) + " s, ED-factor: " + FormatValue(rP.SpdED) + " %)");

                        //Torque data
                        childNodeNames.Add("Torque");
                        childNodeTexts.Add("Torque (Nominal: " + FormatValue(rP.NomTrq) + ", Maximum: " + FormatValue(rP.MaxTrq) + ", IdleTorque: " + FormatValue(rP.IdleTrq) +
                            ") [Nm], DurationAtMax: " + FormatValue(rP.MaxTrqDur) + " s, ED-factor: " + FormatValue(rP.TrqED) + " %, )");

                        //Geometry data
                        if (rP.RackAndPinionGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("Geometry");
                            childNodeTexts.Add("Geometry (BodyDimensions (Length: " + FormatValue(rP.RackAndPinionGeometry.CylinderHeight) + ", Inner: " + FormatValue(rP.RackAndPinionGeometry.InnerDiameter) +
                                ", Outer: " + FormatValue(rP.RackAndPinionGeometry.OuterDiameter) + ") [mm], Material: " + rP.RackAndPinionGeometry.Material + ", Weight: " + FormatValue(rP.RackAndPinionGeometry.PartMass) + " kg,)");
                        }

                        //Mechanical data
                        childNodeNames.Add("MechanicalInput");
                        childNodeTexts.Add("MechanicalInput (Type: " + rP.InpType.ToString() + ", ShaftDiameter: " + FormatValue(rP.InpMaxDia) + " mm, MaxForce(Axial: " +
                             FormatValue(rP.InpAxFrc) + ", Radial: " + FormatValue(rP.InpRadFrc) + ") [N])");

                        break;

                    case (MainType.GearBox):
                        GearBox gB = iLoad as GearBox;

                        parentNodeNames.Add("GearBox");
                        parentNodeTexts.Add("GearBox " + gB.Name + " (ArticleNo: " + gB.ArticleNo + ", Brand: " + gB.Brand + ", SubType: " + gB.MainType +
                            ", Description: " + gB.Description + ", Used: " + gB.Campaign + ")");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (Gearing: " + FormatValue(gB.Gearing) + ":1, Inertia: " + FormatValue(gB.Inertia * 10000) + " kg•cm², Backlash: " + FormatValue(gB.Backlash) + ")");

                        //Speed data
                        childNodeNames.Add("Speed");
                        childNodeTexts.Add("Speed (Nominal: " + FormatValue(gB.NomSpd) + ", Maximum: " + FormatValue(gB.MaxSpd) + ") [rpm], DurationAtMax: "
                            + FormatValue(gB.MaxSpdDur) + " s, ED-factor: " + FormatValue(gB.SpdED) + " %)");

                        //Torque data
                        childNodeNames.Add("Torque");
                        childNodeTexts.Add("Torque (Nominal: " + FormatValue(gB.NomTrq) + ", Maximum: " + FormatValue(gB.MaxTrq) + ", IdleTorque: " + FormatValue(gB.IdleTrq) + ") [Nm], DurationAtMax: "
                            + FormatValue(gB.MaxTrqDur) + " s, ED-factor: " + FormatValue(gB.TrqED) + " %, )");

                        //Mechanical data
                        childNodeNames.Add("MechanicalInput");
                        childNodeTexts.Add("MechanicalInput (Type: " + gB.InpType.ToString() + ", Diameter (Min: " + FormatValue(gB.InpMinDia) + " Max: " + FormatValue(gB.InpMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(gB.InpAxFrc) + ", Radial: " + FormatValue(gB.InpRadFrc) + ") [N])");

                        childNodeNames.Add("MechanicalOutput");
                        childNodeTexts.Add("MechanicalOutput (Type: " + gB.OutType.ToString() + ", Diameter (Min: " + FormatValue(gB.OutMinDia) + " Max: " + FormatValue(gB.OutMaxDia) + ") [mm], MaxForce(Axial: " +
                             FormatValue(gB.OutAxFrc) + ", Radial: " + FormatValue(gB.OutRadFrc) + ") [N])");
                        break;

                    case (MainType.Mass):
                        LoadMass m = iLoad as LoadMass;

                        parentNodeNames.Add("Mass");
                        parentNodeTexts.Add("Mass " + m.Name + " (Linear (non rotating mass)");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (Mass: " + FormatValue(m.Mass) + " kg)");
                        break;

                    case (MainType.Force):
                        LoadForce f = iLoad as LoadForce;

                        parentNodeNames.Add("Force");
                        parentNodeTexts.Add("Force " + f.Name + " (Thrust force (signed)");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (Force: " + FormatValue(f.Force) + " N)");
                        break;

                    case (MainType.Inertia):
                        LoadInertia i = iLoad as LoadInertia;
                        parentNodeNames.Add("Inertia " + i.Name);
                        parentNodeTexts.Add("Inertia " + i.Name + " (Rotating disc with specified geometry)");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (" + FormatValue(i.Inertia * 10000) + " kg•cm², ManualInertiaCalc: " + i.InertiaGeometry.ManualInertiaCalc + ")");
                        
                        //Geometry data
                        if (i.InertiaGeometry.ManualInertiaCalc)
                        {
                            childNodeNames.Add("Geometry");
                            childNodeTexts.Add("Geometry (BodyDimensions (Length: " + FormatValue(i.InertiaGeometry.CylinderHeight) + ", Inner: " + FormatValue(i.InertiaGeometry.InnerDiameter) +
                                ", Outer: " + FormatValue(i.InertiaGeometry.OuterDiameter) + ") [mm], Material: " + i.InertiaGeometry.Material + ", Weight: " + FormatValue(i.InertiaGeometry.PartMass) + " kg,)");
                        }
                        break;

                    case (MainType.Torque):
                        LoadTorque t = iLoad as LoadTorque;

                        parentNodeNames.Add("StaticTorque");
                        parentNodeTexts.Add("StaticTorque " + t.Name + " (Static torque (signed)");

                        //Specific data that does not fit into any general category
                        childNodeNames.Add("Specific");
                        childNodeTexts.Add("Specific (Torque: " + FormatValue(t.Torque) + " Nm)");
                        break;
                }

               //Find the node where the sub node should be added
               //if (lastParentNode.Text != null && lastParentNode.Text != "")
               //rootNode = FindNode(treeView_LoadSide.Nodes[0], "ComponentOverView_LoadSide");

                AddTreeNodes(rootNode, parentNodeNames, parentNodeTexts);

                parentNode = FindNode(treeView_LoadSide.Nodes[0], parentNodeNames[0]);

                //Add child to the root node
                AddTreeNodes(parentNode, childNodeNames, childNodeTexts);

                //lastParentNode.Name = parentNodeNames[0];
                //lastParentNode.Text = parentNodeTexts[0];
            }

            treeView_LoadSide.ExpandAll();
        }

        private TreeNode FindNode(TreeNode parentNode, string nodeName)
        {
            foreach (TreeNode node in parentNode.Nodes)
            {
                if (node.Name == nodeName)
                {
                    return node;
                }
                else
                {
                    TreeNode nodeChild = FindNode(node, nodeName);
                    if (nodeChild != null)
                        return nodeChild;
                }
            }
            return (TreeNode)null;
        }

        private void AddTreeNodes(TreeNode parentNode, IList<string> nodeNames, IList<string> nodeTexts)
        {
            //IList takes both "string[]" and "List<string>" as parameters and replaces two functions
            int count = 0;
            foreach (string childName in nodeNames)
            {
                TreeNode childNode = new TreeNode();
                childNode.Name = childName;
                childNode.Text = nodeTexts[count];

                count++;

                parentNode.Nodes.Add(childNode);
            }
        }

        private double FormatValue(double raw)
        {
            //Hm........?
            if (Math.Abs(raw) < 0.1)
                return Math.Round(raw, 5);

            if (Math.Abs(raw) < 1)
                return Math.Round(raw, 4);

            if (Math.Abs(raw) < 10)
                return Math.Round(raw, 3);

            if (Math.Abs(raw) < 100)
                return Math.Round(raw, 2);

            if (Math.Abs(raw) < 1000)
                return Math.Round(raw, 1);
            else
                return Math.Round(raw, 0);
        }

        private void TreeView_DrawNode(object sender, DrawTreeNodeEventArgs e)
        {
            //string regex = @"^.*\s+\[\d+\]$";
            //if (Regex.IsMatch(e.Node.Text, regex, RegexOptions.Compiled))
            {
                string[] parts = e.Node.Text.Split(' ');
                if (parts.Length > 1)
                {
                    string count = parts[0]; //parts.Length - 1];
                    string text = " " + string.Join(" ", parts, 0, parts.Length - 1);
                    Font normalFont = e.Node.TreeView.Font;

                    float textWidth = e.Graphics.MeasureString(text, normalFont).Width;
                    e.Graphics.DrawString(text,
                                          normalFont,
                                          SystemBrushes.WindowText,
                                          e.Bounds);

                    using (Font boldFont = new Font(normalFont, FontStyle.Bold))
                    {
                        e.Graphics.DrawString(count,
                                              boldFont,
                                              SystemBrushes.WindowText,
                                              e.Bounds.Left + textWidth,
                                              e.Bounds.Top);
                    }
                }
            }
            //else
           // {
            //    e.DrawDefault = true;
            //}
        }

        #region PrintMethods
        //Return the requested object to the main form for printing

        public DataGridView GetDGVBillOfMaterial()
        {            
            return dataGridView_BillOfMaterial;
        }

        public int BillOfMaterialRowCount()
        {
            CreateBillOfMaterial(); 
            return dataGridView_BillOfMaterial.Rows.Count;
        }

        public TreeView GetTreeViewMotorSide()
        {
            CreateTreeView_MotorSide(); 
            return treeView_MotorSide;
        }

        public TreeView GetTreeViewLoadSide()
        {
            CreateTreeView_LoadSide(); 
            return treeView_LoadSide;
        }

        #endregion PrintMethods
    }
}

