using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;

using System.Reflection;

namespace DockSample
{
    public partial class FrmProperties : ToolWindow
    {
        //The Properties form can display 'load'- and 'motor'-objects

        private SortedList<int, ILoad> _iLoads;
        private SortedList<int, IAccessory> _iAccessories;
        private Motor _motor;

        public FrmProperties(SortedList<int, ILoad> iLoads, SortedList<int, IAccessory> iAccessories, Motor motor)
        {
            _iLoads = iLoads as SortedList<int, ILoad>;
            _iAccessories = iAccessories as SortedList<int, IAccessory>;
            _motor = motor as Motor;

            InitializeComponent();

            //Set the motor object as default
            comboBox.Items.Add(_motor.Name); 

            comboBox.SelectedItem = _motor.Name;
            propertyGrid.SelectedObject = _motor;
        }

        public void Refresh()
        {
            propertyGrid.Refresh();
        }

        public void Clear()
        {
            comboBox.SelectedItem = null;
            propertyGrid.SelectedObject = null;
        }

        public void ComboBox_SelectLoad(string loadName)
        {
            comboBox.SelectedItem = loadName;
            FindLoad(loadName);
        }
        
        public void ComboBox_AddLoad(string loadName)
        {
            //Add a new load loadname to the ComboBox
            ComboBox_ReLoad();
            comboBox.SelectedItem = loadName;
            FindLoad(loadName);
        }

        public void ComboBox_ReLoad()
        {
            //A load has been removed. Reload the ComboBox
            //Remove all items from the combobox
            comboBox.Items.Clear();

            //Add each load in order
            foreach (KeyValuePair<int, ILoad> pair in _iLoads)
            {
                comboBox.Items.Add(pair.Value.Name);
            }

//            //Add each accessory in order
//            foreach (KeyValuePair<int, IAccessories> pair in _iAccessories)
//            {
//                comboBox.Items.Add(pair.Value.Name);
//            }

            //Finally add the motor
            comboBox.Items.Add(_motor.Name);
        }

        private void FindLoad(string loadName)
        {
            //Check if it is a motor
            if (loadName == _motor.Name)
                _motor.SetOrResetReadOnlyAttributes(_motor);

            //Create a variable to store the list key
            int loadIndex = -1;

            //Create temporary move variables
            ILoad tempLoad = null;

            //Find the node in the sorted list
            foreach (KeyValuePair<int, ILoad> pair in _iLoads)
            {
                if (pair.Value.Name == loadName)
                    loadIndex = pair.Key;
            }

            //Copy the found item to an ILoad variable
            if (loadIndex > -1)
            {
                _iLoads.TryGetValue(loadIndex, out tempLoad);

                switch (tempLoad.MainType)
                {
                    case MainType.GearBox:
                        var gearBox = tempLoad as GearBox;
                        gearBox.SetOrResetReadOnlyAttributes(gearBox);
                        propertyGrid.SelectedObject = gearBox;
                        break;

                    case MainType.Coupling:
                        var coupling = tempLoad as Coupling;
                        coupling.SetOrResetReadOnlyAttributes(coupling);
                        coupling.SetAutoSelectTrue();
                        propertyGrid.SelectedObject = coupling;
                        break;

                    case MainType.BallScrew:
                        var ballScrew = tempLoad as BallScrew;
                        ballScrew.SetOrResetReadOnlyAttributes(ballScrew);
                        ballScrew.SetAutoSelectTrue();
                        propertyGrid.SelectedObject = ballScrew;
                        break;

                    case MainType.RackAndPinion:
                        var rackAndPinion = tempLoad as RackAndPinion;
                        rackAndPinion.SetOrResetReadOnlyAttributes(rackAndPinion);
                        rackAndPinion.SetAutoSelectTrue();
                        propertyGrid.SelectedObject = rackAndPinion;
                        break;

                    case MainType.TurnTable:
                        var turnTable = tempLoad as TurnTable;
                        turnTable.SetOrResetReadOnlyAttributes(turnTable);
                        turnTable.SetAutoSelectTrue();
                        propertyGrid.SelectedObject = turnTable;
                        break;

                    case MainType.ToothedBelt:
                        var toothedBelt = tempLoad as ToothedBelt;
                        toothedBelt.SetOrResetReadOnlyAttributes(toothedBelt);
                        toothedBelt.SetAutoSelectTrue();
                        propertyGrid.SelectedObject = toothedBelt;
                        break;

                    case MainType.Mass:
                        var mass = tempLoad as LoadMass;
                        propertyGrid.SelectedObject = mass;
                        break;

                    case MainType.Force:
                        var force = tempLoad as LoadForce;
                        propertyGrid.SelectedObject = force;
                        break;

                    case MainType.Torque:
                        var torque = tempLoad as LoadTorque;
                        propertyGrid.SelectedObject = torque;
                        break;

                    case MainType.Inertia:
                        var inertia = tempLoad as LoadInertia;
                        propertyGrid.SelectedObject = inertia;
                        break;
                }
            }

            //Create a variable to store the list key
            int accIndex = -1;

            //Create temporary move variables
            IAccessory tempAccLoad = null;

            //Find the node in the sorted list
            foreach (KeyValuePair<int, IAccessory> pair in _iAccessories)
            {
                if (pair.Value.Name == loadName)
                    accIndex = pair.Key;
            }

            //Copy the found item to an ILoad variable
            if (accIndex > -1)
            {
                _iAccessories.TryGetValue(accIndex, out tempAccLoad);

                switch (tempAccLoad.MainType)
                {
                    case MainType.Drive:
                        var driveUnit = tempAccLoad as Drive;
                        propertyGrid.SelectedObject = driveUnit;
                        break;
                }
            }

        }

        private void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            foreach (var l in _iLoads)
            {
                ILoad load = l.Value;
                if ((string)comboBox.SelectedItem == load.Name)
                {                    
                    FindLoad(load.Name);
                }
            }

            if ((string)comboBox.SelectedItem == _motor.Name)
                propertyGrid.SelectedObject = _motor;
        }
    }
}