﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DockSample
{
    public partial class UIDialogDefineGearing : Form
    {
        double _input = 0; 
        double _output = 0;
        public double Gearing = 0;

        public UIDialogDefineGearing()
        {
            InitializeComponent();
        }

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        private void textBox_Input_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _input = Math.Round(validate_textBox(textBox_Input.Text), 3);
            
            //Update the textBox if the validate-function did any formatting
            textBox_Input.Text = _input.ToString();

            Gearing = _output / _input;

            textBox_GearingInverse.Text = Gearing.ToString();

            if ((Gearing == 0) || double.IsNaN(Gearing) || double.IsInfinity(Gearing))
                Gearing = 1;
        }

        private void textBox_Input_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;

                //Check if the textbox string is a valid double
                _input = Math.Round(validate_textBox(textBox_Input.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_Input.Text = _input.ToString();

                Gearing = _output / _input;

                textBox_Gearing.Text = Gearing.ToString();
                textBox_GearingInverse.Text = (1 / Gearing).ToString();

                if ((Gearing == 0) || double.IsNaN(Gearing) || double.IsInfinity(Gearing))
                    Gearing = 1;
            }
        }

        private void textBox_Output_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _output = Math.Round(validate_textBox(textBox_Output.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_Output.Text = _output.ToString();

            Gearing = _output / _input;

            textBox_Gearing.Text = Gearing.ToString();
            textBox_GearingInverse.Text = (1 / Gearing).ToString();

            if ((Gearing == 0) || double.IsNaN(Gearing) || double.IsInfinity(Gearing))
                Gearing = 1;
        }

        private void textBox_Output_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                e.Handled = true;
                //Check if the textbox string is a valid double
                _output = Math.Round(validate_textBox(textBox_Output.Text), 3);

                //Update the textBox if the validate-function did any formatting
                textBox_Output.Text = _output.ToString();
                                
                Gearing = _output / _input;

                textBox_Gearing.Text = Gearing.ToString(); 
                textBox_GearingInverse.Text = (1 / Gearing).ToString();

                if ((Gearing == 0) || double.IsNaN(Gearing) || double.IsInfinity(Gearing))
                    Gearing = 1;
            }
        }

        private void button_OK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            Close();
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
