﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DockSample
{
    public partial class UIDialogDefineInertia : Form
    {
        double _length = 0;
        double _outerDiameter = 0;
        double _innerDiameter = 0;
        double _density = 0;
        double _mass = 0;
        public double Inertia = 0;

        public UIDialogDefineInertia()
        {
            InitializeComponent();
        }

        private double validate_textBox(string str)
        {
            var culture = CultureInfo.GetCultureInfo("sv-SE");

            var format = (NumberFormatInfo)culture.NumberFormat.Clone();

            double returnValue;

            //Check if it can be parsed with the standard number separator
            if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
            {
                //if not then try with the 'other' (dot or comma) separator
                if (format.NumberDecimalSeparator == ",")
                    format.NumberDecimalSeparator = ".";
                else
                    format.NumberDecimalSeparator = ".";

                if (Double.TryParse(str, NumberStyles.Float, format, out returnValue) == false)
                    //If it still can not be parsed then reset the value and return
                    return returnValue = 0;
            }

            //Re-read the current culture format
            format = (NumberFormatInfo)culture.NumberFormat.Clone();

            //Format the return value according to the current culture
            return returnValue = Convert.ToDouble(returnValue, CultureInfo.InvariantCulture);
        }

        private void textBox_Length_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _length = Math.Round(validate_textBox(textBox_Length.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_Length.Text = _length.ToString();
        }

        private void textBox_OuterDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _outerDiameter = Math.Round(validate_textBox(textBox_OuterDiameter.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_OuterDiameter.Text = _outerDiameter.ToString();
        }

        private void textBox_InnerDiameter_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _innerDiameter = Math.Round(validate_textBox(textBox_InnerDiameter.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_InnerDiameter.Text = _innerDiameter.ToString();
        }

        private void textBox_Density_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _density = Math.Round(validate_textBox(textBox_Density.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_Density.Text = _density.ToString();
        }

        private void textBox_Mass_Validated(object sender, EventArgs e)
        {
            //Check if the textbox string is a valid double
            _mass = Math.Round(validate_textBox(textBox_Mass.Text), 3);

            //Update the textBox if the validate-function did any formatting
            textBox_Mass.Text = _mass.ToString(); 
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            Close();
        }

        private void button_CalcMass_Click(object sender, EventArgs e)
        {
            double totalVolume  = Math.PI * Math.Pow(_outerDiameter/2000, 2) * _length/1000;
            double hollowVolume = Math.PI * Math.Pow(_innerDiameter/2000, 2) * _length/1000;

            double realVolume = totalVolume - hollowVolume; 
            _mass =  realVolume * _density;

            textBox_Mass.Text = Math.Round(_mass, 3).ToString();

            Inertia = _mass * (Math.Pow(_outerDiameter/2000, 2) + Math.Pow(_innerDiameter/2000, 2)) * 0.5;
            textBox_Inertia.Text = Inertia.ToString();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
